(function ($) {

    'use strict';

    var Elements;

    function ElementContent(html, css, params) {

        var parent = this;
        var selfHtml, selfCss;
        var selfParams = {
            classes: '',
            droppable: false
        };

        if (!html) {
            html = '';
        }

        if (!css) {
            css = '';
        }

        if (!params) {
            params = {};
        }

        this.html = function (html) {
            if (!html) {
                return selfHtml;
            }
            selfHtml = html;
        };

        this.css = function (css) {
            if (!css) {
                return selfCss;
            }
            selfCss = css;
        };

        this.setParams = function (paramsArray) {
            var param;
            for (param in paramsArray) {
                if (paramsArray.hasOwnProperty(param)) {
                    selfParams[param] = paramsArray[param];
                }
            }
        };

        this.getParams = function (param) {

            return param ? selfParams[param] : selfParams;
        };

        this.clone = function () {

            return new ElementContent(selfHtml, selfCss, selfParams);

        };

        (function construct() {

            parent.html(html);
            parent.css(css);
            parent.setParams(params);

        })();

    }

    /**
     * Element object
     * @return Element Object
     */
    function Element(params, content) {

        var parent = this;

        var selfParams = {
            id: null,
            structure: false, /* is content structure */
            classes: '',
            title: null,
            iconClass: '',
            limit: null,
            padding: '0 0 0 0',
            margin: '0 0 10px 0',
            required: false,
            ismust: false
        };

        var selfContent = [];

        if (!params) {
            params = {};
        }

        if (!content) {
            content = [];
        }

        this.addContent = function (content) {
            if (!(content instanceof ElementContent)) {
                throw 'The content is instance of ElementContent';
            }
            selfContent.push(content);
        };

        this.getContent = function () {
            return selfContent;
        };

        this.setParams = function (paramsArray) {
            var param;
            for (param in paramsArray) {
                if (paramsArray.hasOwnProperty(param)) {
                    selfParams[param] = paramsArray[param];
                }
            }
        };

        this.getParams = function (param) {

            return param ? selfParams[param] : selfParams;
        };

        this.clone = function () {

            var element = new Element();

            element.setParams(selfParams);

            $.each(selfContent, function () {
                element.addContent(this.clone());
            });

            return element;

        };

        (function construct() {

            parent.setParams(params);

            $.each(content, function () {
                parent.addContent(this);
            });

        })();
    }

    Elements = {
        selfElements: {},

        add: function (element) {

            if (!(element instanceof  Element)) {
                throw 'The element is not instance of Element';
            }

            this.selfElements[element.getParams('id')] = element;
        },

        get: function (id) {

            if (typeof this.selfElements[id] === 'undefined') {
                throw 'Element ' + id + ' does not exist';
            }

            return this.selfElements[id];
        }
    };

    window.stecBuilderElement = Element;
    window.stecBuilderElementContent = ElementContent;
    window.stecBuilderElements = Elements;

})(window.jQuery);