(function ($) {

    'use strict';

    function History() {

        var index = 0;
        var selfElements = [];

        this.add = function (plotElement) {

            var html = [];
            var elementObject = plotElement.data('elementObject');
            elementObject.setParams({hid: ++index});

            selfElements.push(plotElement);

            html.push('<li>');
            html.push(elementObject.getParams('title'));
            html.push('   <span class="delete"><i class="fas fa-trash"></i></span>');
            html.push('   <span class="settings"><i class="fas fa-cog"></i></span>');
            html.push('   <span class="duplicate"><i class="fas fa-clone"></i></span>');
            html.push('   <span class="visibility"><i class="far fa-eye"></i></span>');
            html.push('</li>');

            $(html.join(''))
                    .data('plotElement', plotElement)
                    .appendTo($('.stec-builder-history ul'));
        };

        this.remove = function (plotElement) {

            var hid = $(plotElement).data('elementObject').getParams('hid');

            selfElements = $.grep(selfElements, function (plotElement) {
                return plotElement.data('elementObject').getParams('hid') !== hid;
            });

            $('.stec-builder-history ul').children().each(function () {
                var ehid = $(this).data('plotElement').data('elementObject').getParams('hid');
                if (hid === ehid) {
                    $(this).remove();
                }
            });
        };

        this.get = function (plotElementHid) {
            var found = null;
            $('.stec-builder-plot-element').each(function () {
                if (plotElementHid === $(this).data('elementObject').getParams('hid')) {
                    found = this;
                    return false; // break;
                }
            });
            return found;
        };

        function controller() {

            $('.stec-builder-history').draggable({
                snap: '.stec-builder-modal'
            });

            $('.stec-builder-history').on('click', '> ul > li', function () {
                var el = $(this).data('plotElement');
                $('html, body').animate({
                    scrollTop: $(el).offset().top - 80 - $('#wpadminbar').height() - 2
                }, {
                    duration: 250,
                    easing: "linear"
                });
            });

            $('.stec-builder-history').on('click', '> ul > li .visibility', function () {
                var plotElement = $(this).parents('li').first().data('plotElement');
                plotElement.toggleClass('stec-builder-plot-element-invisible');
                $(this).parents('li').first().toggleClass('stec-builder-history-element-invisible');
            });

            $('.stec-builder-history').on('click', '> ul > li .duplicate', function (e) {
                var elementPlot = $(this).parents('li').first().data('plotElement');
                e.stopPropagation();
                window.stecBuilderPlot.duplicateElement(elementPlot);
            });

            $('.stec-builder-history').on('click', '> ul > li .settings', function (e) {
                var plotElement = $(this).parents('li').first().data('plotElement');
                var elementObject = plotElement.data('elementObject');
                e.stopPropagation();
                window.stecBuilderSettings.openSettings(plotElement, elementObject);
            });

            $('.stec-builder-history').on('click', '> ul > li .delete', function (e) {
                var elementPlot = $(this).parents('li').first().data('plotElement');
                var elementObject = elementPlot.data('elementObject');
                e.stopPropagation();
                window.stecBuilderPlot.deleteElement(elementPlot, elementObject);
            });

            $('.stec-builder-history').on('mouseover', '> ul > li', function () {
                var element = $(this).data('plotElement');
                $(element).addClass('stec-builder-history-item-outline');
            });

            $('.stec-builder-history').on('mouseleave', '> ul > li', function () {
                var element = $(this).data('plotElement');
                $(element).removeClass('stec-builder-history-item-outline');
            });

            $(document).on('keydown', function (e) {

                switch (e.which) {
                    case 104:
                    case 72:
                        if (e.altKey) {
                            $('.stec-builder-history').toggle();
                        }
                        break;
                }
            });
        }

        (function construct() {
            $('.stec-builder-history ul').empty();
            controller();
        })();
    }

    $(function () {
        window.stecBuilderHistory = History;
    });


})(window.jQuery);