(function ($) {

    'use strict';

    function Main(projectType, id) {

        var parent = this;

        this.project_type = projectType;
        this.id = id;

        this.projectListWindow = {

            getLIhtml: function (project) {
                var html = [];

                html.push('<li data-id="' + project.id + '">');
                html.push('   <label>');
                html.push('       <input type="checkbox" ' + (project.id !== 'default' ? '' : 'disabled') + '/>');
                html.push('       <a href="?page=stec_menu__builder&view=builder&project_type=event_submission_form&project_id=' + project.id + '">' + project.title + '</a>');
                html.push('   </label>');
                html.push('   <div>');
                html.push('       <a href="?page=stec_menu__builder&view=builder&project_type=event_submission_form&project_id=' + project.id + '"><i class="fas fa-pen-square"></i></a>');
                html.push('       <a data-duplicate="' + project.id + '" href=""><i class="fas fa-clone"></i></a>');
                if (project.id !== 'default') {
                    html.push('       <a data-delete="' + project.id + '" href=""><i class="fas fa-trash"></i></a>');
                }
                html.push('   </div>');
                html.push('</li>');

                return html.join('');
            },

            bulkDelete: function (type, idArray) {

                var DB = new window.StecDB();

                var request = DB.newRequest();
                request.setData({
                    action: 'stec_ajax_action',
                    task: 'stec_builder_bulk_delete',
                    project_type: type,
                    project_ids: idArray
                });

                request.setBefore(function () {
                    $('.stec-builder-home .stec-builder-project-list li').each(function () {
                        if ($.inArray($(this).data('id'), idArray) !== -1) {
                            $(this).css('opacity', 0.5);
                        }
                    });
                });

                request.setCallback(function (result) {
                    if (result.data.success === 1) {
                        $('.stec-builder-home .stec-builder-project-list li').each(function () {
                            if ($.inArray($(this).data('id'), idArray) !== -1) {
                                $(this).remove();
                            }
                        });
                        window.stecBuilderProjects = result.data.projects;
                    }
                });

                DB.query(request);
            },

            duplicateProject: function (type, id) {

                var DB = new window.StecDB();

                var request = DB.newRequest();
                request.setData({
                    action: 'stec_ajax_action',
                    task: 'stec_builder_duplicate',
                    project_type: type,
                    project_id: id
                });

                request.setBefore(function () {
                    $('.stec-builder-home .stec-builder-project-list li[data-id="' + id + '"]').css('opacity', 0.5);
                });

                request.setCallback(function (result) {
                    if (result.data.success === 1) {
                        $('.stec-builder-home .stec-builder-project-list li[data-id="' + id + '"]').css('opacity', 1);
                        $(parent.projectListWindow.getLIhtml(result.data.project)).appendTo($('.stec-builder-list').find('.stec-builder-project-list'));
                        window.stecBuilderProjects = result.data.projects;
                    }
                });

                DB.query(request);
            },

            deleteProject: function (type, id) {

                var DB = new window.StecDB();

                var request = DB.newRequest();
                request.setData({
                    action: 'stec_ajax_action',
                    task: 'stec_builder_delete',
                    project_type: type,
                    project_id: id
                });

                request.setBefore(function () {
                    $('.stec-builder-home .stec-builder-project-list li[data-id="' + id + '"]').css('opacity', 0.5);
                });

                request.setCallback(function (result) {
                    if (result.data.success === 1) {
                        $('.stec-builder-home .stec-builder-project-list li[data-id="' + id + '"]').remove();
                        window.stecBuilderProjects = result.data.projects;
                    }
                });

                DB.query(request);
            },

            list: function (projects) {
                $.each(projects, function () {
                    if (this.id === 'default') {
                        $(parent.projectListWindow.getLIhtml(this)).prependTo($('.stec-builder-list').find('.stec-builder-project-list'));
                    } else {
                        $(parent.projectListWindow.getLIhtml(this)).appendTo($('.stec-builder-list').find('.stec-builder-project-list'));
                    }
                });
            }
        };

        this.saveProject = function () {

            var DB = new window.StecDB();

            var request = DB.newRequest();
            request.setData({
                action: 'stec_ajax_action',
                task: 'stec_builder_save',
                elements: window.stecBuilderPlot.stringifyElements(),
                plot: window.stecBuilderPlot.stringifyPlot(),
                project_type: parent.project_type,
                project_id: parent.id,
                project_title: $('#project-title').text()
            });

            request.setBefore(function () {
                parent.notification.show(window.stecLang.saving);
            });

            request.setCallback(function (response) {
                if (response.error === 0) {
                    parent.id = response.data.id;
                    parent.notification.show(window.stecLang.projectSaved);
                } else {
                    parent.notification.show(window.stecLang.projectSavedError);
                }
            });

            DB.query(request);

        };

        this.loadProject = function (project) {

            var packedElements = JSON.parse(project.elements);
            var plot = JSON.parse(project.plot);

            $('#project-title').html(project.title);

            window.stecBuilderPlot.loadPlot(plot);
            window.stecBuilderPlot.unpackElements(packedElements);
        };

        this.notification = {

            html: function (msg) {

                var html = [];

                html.push('<div class="stec-builder-notification">');
                html.push('   <span><i class="fa fa-times"></i></span>');
                html.push('   <div>');
                html.push('       <p>' + msg + '</p>');
                html.push('   </div>');
                html.push('   <a href="javascript:void(0);">' + window.stecLang.close + '</a>');
                html.push('</div>');

                return html.join('');
            },

            show: function (msg) {

                var html = this.html(msg);

                this.hide();

                $(html).appendTo('.stec-builder');

            },

            hide: function () {
                $('.stec-builder-notification').remove();
            }
        };

        function controller() {

            /**
             * Home screen binds
             */

            // Close projects list
            $('.stec-builder-home .stec-builder-project-list-page-head a').on('click', function (e) {
                e.preventDefault();
                $('.stec-builder-list').hide();
            });

            // Open projects list
            $('.stec-builder-home .stec-builder-open-project-list').on('click', function () {
                var projects = window.stecBuilderProjects[$(this).data('type')];

                $('.stec-builder-list').find('.stec-builder-project-list').empty();
                $('.stec-builder-list').data('type', $(this).data('type'));
                $('.stec-builder-project-control input[type="checkbox"]').prop('checked', false);
                $('.stec-builder-project-list-page-head h1').text($(this).data('title'));
                $('.stec-builder-list').show();

                parent.projectListWindow.list(projects);

            });

            // Select All
            $('.stec-builder-home .stec-builder-project-control input[type="checkbox"]').on('change', function () {
                $('.stec-builder-home .stec-builder-project-list')
                        .children('li')
                        .find('input[type="checkbox"]').not(':disabled')
                        .prop('checked', $(this).prop('checked'));
            });

            // Check project
            $('.stec-builder-home .stec-builder-project-list').on('change', 'input[type="checkbox"]', function () {

                var itemsLength = $('.stec-builder-home .stec-builder-project-list')
                        .children('li').length;

                var checkedLength = $('.stec-builder-home .stec-builder-project-list')
                        .children('li')
                        .find('input[type="checkbox"]').not(':disabled')
                        .filter(function () {
                            return true === $(this).prop('checked');
                        }).length;

                if (itemsLength - 1 === checkedLength) {
                    $('.stec-builder-home .stec-builder-project-control input[type="checkbox"]').prop('checked', true);
                } else {
                    $('.stec-builder-home .stec-builder-project-control input[type="checkbox"]').prop('checked', false);
                }

            });

            // Duplicate project
            $('.stec-builder-home .stec-builder-list').on('click', 'li a[data-duplicate]', function (e) {
                e.preventDefault();
                parent.projectListWindow.duplicateProject($('.stec-builder-list').data('type'), $(this).data('duplicate'));
            });

            // Delete project
            $('.stec-builder-home .stec-builder-list').on('click', 'li a[data-delete]', function (e) {
                e.preventDefault();
                parent.projectListWindow.deleteProject($('.stec-builder-list').data('type'), $(this).data('delete'));
            });

            // Bulk delete projects
            $('.stec-builder-home .bulk-delete').on('click', function (e) {
                var ids = [];

                e.preventDefault();

                $('.stec-builder-home .stec-builder-project-list').children('li').each(function () {
                    var id = $(this).data('id');
                    if (true === $(this).find('input[type="checkbox"]').prop('checked')) {
                        ids.push(id);
                    }
                });

                if (ids.length <= 0) {
                    return;
                }

                parent.projectListWindow.bulkDelete($('.stec-builder-list').data('type'), ids);
            });


            /**
             * Modal windows
             */
            $('.stec-builder-modal').resizable({
                minWidth: 240,
                minHeight: 240,
                resize: function (e, ui) {
                    $(this).children('.panel-sections, ul').css({
                        height: ui.size.height - 27
                    });
                }
            });
            $('.stec-builder-modal > div:first-child').on('dblclick', function () {
                $(this).parent().toggleClass('retract');
                if (!$(this).parent().hasClass('retract')) {
                    $(this).parent().css('height', 'auto');
                }
            });
            $('.stec-builder-modal').on('mousedown', function () {
                $('.stec-builder-modal.last-active').removeClass('last-active');
                $(this).addClass('last-active');
            });

            $('.stec-builder-modal > div:first-child span').on('click', function () {

                var modal = $(this).parents('.stec-builder-modal');

                if (modal.hasClass('stec-builder-settings')) {
                    window.stecBuilderSettings.closeSettings();
                    return;
                }

                if (modal.hasClass('stec-builder-plot-settings')) {
                    window.stecBuilderSettings.closePlotSettings();
                    return;
                }

                modal.hide();

            });

            $('.stec-builder').on('click', '.stec-builder-notification a, .stec-builder-notification span', function () {
                parent.notification.hide();
            });

        }

        this.init = function () {

            if (true === (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent))) {
                parent.notification.show(window.stecLang.notMobileOptimized);
            }

            if (parent.project_type) {

                window.stecBuilderSettings = new window.stecBuilderSettings();
                window.stecBuilderHistory = new window.stecBuilderHistory();
                window.stecBuilderPlot = new window.stecBuilderPlot();
                window.stecBuilderPanel = new window.stecBuilderPanel();
            }

            controller();
        };
    }

    window.stecBuilderMain = Main;

})(window.jQuery);