(function ($) {

    'use strict';

    function Plot() {

        var parent = this;
        var elementWrapperClass = 'stec-builder-plot-element';
        var elementDroppableClass = 'stec-builder-plot-element-droppable';
        var elementOutlineClass = 'stec-builder-plot-element-outline';

        function getElementWrapperHTML() {

            var tooltipHTML = [];
            tooltipHTML.push('<div class="tooltip">');
            tooltipHTML.push('    <span class="title"><i class="fas fa-arrows-alt"></i><span>{{elementTitle}}</span></span>');
            tooltipHTML.push('    <span class="duplicate"><i class="fas fa-clone"></i></span>');
            tooltipHTML.push('    <span class="settings"><i class="fas fa-cog"></i></span>');
            tooltipHTML.push('    <span class="remove"><i class="fas fa-trash"></i></span>');
            tooltipHTML.push('</div>');

            return '<div class="' + elementWrapperClass + '">' + tooltipHTML.join('') + '{{elementContents}}</div>';
        }

        function getElementPlotHTML(element) {

            var contents = element.getContent();

            var HTML = '';

            $.each(contents, function (i) {

                var $content = $(this.html());

                if (this.getParams('text')) {
                    $content.text(this.getParams('text'));
                }

                $content.addClass(elementOutlineClass + ' ' + this.getParams('classes'));

                if (true === this.getParams('droppable')) {
                    $content.addClass(elementDroppableClass);
                }

                $content.css(this.css());
                $content.attr('cid', i);

                HTML += $content.get(0).outerHTML;

            });

            HTML = getElementWrapperHTML()
                    .replace('{{elementTitle}}', element.getParams('title'))
                    .replace('{{elementContents}}', HTML);

            return HTML;
        }

        this.clear = function () {

            $('.stec-builder-plot-main').children('.stec-builder-plot-element').each(function () {
                var eo = $(this).data('elementObject');
                parent.deleteElement(this, eo);
            });
        };

        this.stringifyPlot = function () {

            return JSON.stringify($('.stec-builder-plot').data('elementObject').getParams());
        };

        this.stringifyElements = function () {

            return JSON.stringify(this.packElements());
        };

        this.loadPlot = function (plot) {
            var plotObject = $('.stec-builder-plot').data('elementObject');
            plotObject.setParams(plot);

            $('.stec-builder-plot-main').css({
                padding: plotObject.getParams('padding'),
                margin: plotObject.getParams('margin')
            });
        };

        this.unpackElements = function (elements, container) {

            if (!container) {
                container = '.stec-builder-plot-main';
            }

            $.each(elements, function () {

                var parentElement;
                var element = this;
                var elementObject = window.stecBuilderElements.get(this.params.id).clone();

                elementObject.setParams(element.params);

                $.each(elementObject.getContent(), function (i) {

                    if (!element.content[i]) {
                        console.warn('Mismatch content in element:', element.params.id);
                        return true;
                    }

                    this.css(element.content[i].css);
                    this.setParams(element.content[i].params);
                });

                parentElement = window.stecBuilderPlot.placeElement(elementObject, container);

                $.each(element.children, function (cid) {

                    parent.unpackElements(this,
                            parentElement
                            .find('.stec-builder-plot-element-droppable')
                            .eq(cid));
                });
            });
        };

        this.packElements = function (container) {

            var elements = [];

            if (!container) {
                container = '.stec-builder-plot-main';
            }

            $(container).children('.stec-builder-plot-element').each(function () {

                var elementObject = $(this).data('elementObject');
                var elementContent = elementObject.getContent();
                var packElement;

                var content = [];

                $.each(elementContent, function () {
                    content.push({
                        params: this.getParams(),
                        css: this.css()
                    });
                });

                packElement = {
                    params: elementObject.getParams(),
                    content: content,
                    children: []
                };

                $(this).find('.stec-builder-plot-element-droppable').each(function (i) {
                    var children = parent.packElements(this);
                    if (children.length > 0) {
                        packElement.children[i] = children;
                    }
                });

                elements.push(packElement);

            });

            return elements;
        };

        /**
         * Checks if element has reached create limit
         * @param {type} element
         * @return {Boolean}
         */
        function hasReachedLimit(element) {

            var limit = element.getParams('limit');

            if (limit === null || limit > 0) {
                return false;
            }

            return true;

        }

        /**
         * Places element on the plot
         * @param {elementObject} element is REFERENCE copy! Make sure it's cloned beforehand
         * @param {class/object} the append container
         * @todo Clone inside this function?
         */
        this.placeElement = function (element, container) {

            var protoElement = window.stecBuilderElements.get(element.getParams('id'));
            var plotElement;
            var limit;

            if (true === hasReachedLimit(protoElement)) {
                window.stecBuilderMain.notification.show('You have reached the maximum number of instances for this element');
                return;
            }

            limit = protoElement.getParams('limit');

            if (null !== limit) {
                protoElement.setParams({limit: Math.max(0, --limit)});

                // timeout fixes drag data lost
                setTimeout(function () {
                    if (limit === 0) {
                        window.stecBuilderPanel.disableElement(protoElement);
                    }
                }, 0);
            }

            plotElement = $(getElementPlotHTML(element))
                    .data('elementObject', element)
                    .css({
                        margin: element.getParams('margin'),
                        padding: element.getParams('padding'),
                        textAlign: element.getParams('textAlign'),
                        height: element.getParams('height')
                    })
                    .appendTo(container);

            window.stecBuilderHistory.add(plotElement);

            $(document).trigger('stec-builder-plot-reset-droppable');
            $(document).trigger('stec-builder-plot-reset-sortable');

            return plotElement;
        };

        this.duplicateElement = function (plotElement) {

            var elementObject = plotElement.data('elementObject');
            var $container = '';

            if (false === elementObject.getParams('structure')) {
                // Clone sub element into the same container
                $container = $(plotElement).parents('.stec-builder-plot-element-droppable').first();
                parent.placeElement(elementObject.clone(), $container);
            } else {
                // Clone structure elements with its subelements
                $container = '.stec-builder-plot-main';
                parent.placeElement(elementObject.clone(), $container);
                plotElement.find('.stec-builder-plot-element').each(function () {
                    var subElement = $(this).data('elementObject').clone();

                    var dropIndex = $(this).parents('.stec-builder-plot-element-droppable').first().index();
                    parent.placeElement(subElement, $('.stec-builder-plot-main > .stec-builder-plot-element').last().find('.stec-builder-plot-element-droppable').eq(dropIndex - 1));
                });
            }

        };

        /**
         * Deletes element and all its sub elements and corrects limits
         */
        this.deleteElement = function (plotElement, elementObject) {

            var protoElement;
            var limit;

            // Refresh sub elements limit
            $(plotElement).find('.stec-builder-plot-element').each(function () {
                var protoElement = window.stecBuilderElements.get($(this).data('elementObject').getParams('id'));
                var limit = protoElement.getParams('limit');
                if (limit !== null) {
                    protoElement.setParams({limit: ++limit});

                    if (limit > 0) {
                        window.stecBuilderPanel.enableElement(protoElement);
                    }
                }

                // Delete sub element from history
                window.stecBuilderHistory.remove(this);
                protoElement = null;
            });

            protoElement = window.stecBuilderElements.get(elementObject.getParams('id'));

            // Check main element limit
            limit = protoElement.getParams('limit');

            if (limit !== null) {
                protoElement.setParams({limit: ++limit});

                if (limit > 0) {
                    window.stecBuilderPanel.enableElement(protoElement);
                }
            }

            // Delete from history
            window.stecBuilderHistory.remove(plotElement);

            // Delete
            $(plotElement).remove();

        };

        function controller() {

            $(document).on('keypress', function (e) {
                switch (e.which) {
                    case 104:
                    case 72:

                        if ('true' === e.srcElement.contentEditable) {
                            break;
                        }

                        $('.stec-builder-plot').toggleClass('stec-builder-hide-outlines');
                        break;
                }
            });

            $(document).on('focus', '.stec-builder-plot-element [contenteditable="true"]', function () {
                var $this = $(this);

                $('.stec-builder .ui-sortable').sortable("disable"); // temp disable sortable while editing
                $this.data('before', $this.html());
                
            }).on('blur', '.stec-builder-plot-element [contenteditable="true"]', function () {
                var $this = $(this);
                if ($this.data('before') !== $this.html()) {
                    $this.data('before', $this.html());
                    $this.trigger('change');
                }

                $('.stec-builder .ui-sortable').sortable("enable"); // enable sortable after done editing
            });

            $(document).on('change', '.stec-builder-plot-element [contenteditable="true"]', function () {

                var childrenIndex;
                var element;
                var childrens;

                try {
                    childrenIndex = $(this).attr('cid');
                    element = $(this).parents('.stec-builder-plot-element').first().data('elementObject');
                    childrens = element.getContent();
                    childrens[childrenIndex].setParams({text: $(this).text()});
                } catch (err) {
                    console.log(err.message);
                }

            });

            $(document).on('click', '.stec-builder-plot-element [contenteditable="true"]', function (e) {
                e.stopPropagation();
                $(this).focus();
            });

            $(document).on('mouseover', '.stec-builder-plot-element', function (e) {
                e.stopPropagation();

                if ($('.stec-builder-plot-main-droppable-valid-item').length > 0 ||
                        $('.stec-builder-plot-element-droppable-valid-item').length > 0) {
                    return;
                }

                $('.stec-builder-plot-element').not(this).removeClass('show-tooltip');
                $(this).addClass('show-tooltip');
            });

            $(document).on('mouseleave', '.stec-builder-plot-element', function () {

                if ($('.stec-builder-plot-main-droppable-valid-item').length > 0 ||
                        $('.stec-builder-plot-element-droppable-valid-item').length > 0) {
                    return;
                }

                $(this).removeClass('show-tooltip');
            });

            /**
             * Make droppable elements to accept sub-elements from Panel
             */
            $(document).on('stec-builder-plot-reset-droppable', function () {

                $('.stec-builder-plot-element-droppable').droppable({
                    tolerance: "pointer",
                    greedy: true,
                    accept: '.section-element-draggable',
                    out: function (event, ui) {
                    },
                    over: function (event, ui) {
                    },
                    create: function () {
                    },
                    drop: function (event, ui) {

                        var element = $(ui.draggable).data('elementObject');

                        // Do not allow structure elements to be stacked
                        if (true === element.getParams('structure')) {
                            return;
                        }

                        parent.placeElement(element.clone(), this);

                    }
                });
            });

            /**
             * Make sub elements sortable
             */
            $(document).on('stec-builder-plot-reset-sortable', function () {

                $('.stec-builder-plot-element-droppable').sortable({
                    cancel: "",
                    items: '.stec-builder-plot-element',
                    tolerance: "pointer",
                    connectWith: '.stec-builder-plot-element-droppable',
                    scroll: false,
                    start: function (e, ui) {
                        $(this).parents('.stec-builder-plot-element').first().css('zIndex', '1');
                        $('.stec-builder-plot-element-droppable').addClass('stec-builder-plot-element-droppable-valid-item');
                    },
                    stop: function (e, ui) {
                        $(this).parents('.stec-builder-plot-element').first().css('zIndex', '0');
                        $('.stec-builder-plot-element-droppable').removeClass('stec-builder-plot-element-droppable-valid-item');
                    }
                });
            });

            /**
             * Make main plot droppable to accept structure elements like 100, 50/50
             */
            $('.stec-builder-plot-main-droppable').droppable({
                greedy: true,
                accept: '.section-element-draggable-structure',
                out: function (event, ui) {
                },
                over: function (event, ui) {
                },
                drop: function (event, ui) {

                    var protoElement = $(ui.helper.prevObject).data('elementObject');
                    var newElement = protoElement.clone();

                    parent.placeElement(newElement, '.stec-builder-plot-main');

                }
            });

            /**
             * Make plot elements sortable
             */
            $('.stec-builder-plot-main').sortable({
                items: '> .stec-builder-plot-element',
                tolerance: "pointer",
                zIndex: 100
            });

            /**
             * Duplicate element
             */
            $(document).on('click', '.stec-builder-plot-element > .tooltip .duplicate', function () {
                var plotElement = $(this).parents('.stec-builder-plot-element').first();
                parent.duplicateElement(plotElement);
            });

            /**
             * Delete element
             */
            $(document).on('click', '.stec-builder-plot-element > .tooltip .remove', function (e) {
                var plotElement, elementObject;
                e.stopPropagation();
                plotElement = $(this).parents('.stec-builder-plot-element').first();
                elementObject = plotElement.data('elementObject');
                parent.deleteElement(plotElement, elementObject);
            });

            /**
             * Open element settings
             */
            $(document).on('click', '.stec-builder-plot-element > .tooltip .settings', function (e) {
                var plotElement, elementObject;
                e.stopPropagation();
                plotElement = $(this).parents('.stec-builder-plot-element').first();
                elementObject = plotElement.data('elementObject');
                window.stecBuilderSettings.openSettings(plotElement, elementObject);
            });

            /**
             * Open plot settings
             */
            $(document).on('click', '.stec-builder-plot-settings-menu > div:last', function (e) {
                e.stopPropagation();

                if ($('.stec-builder-plot-settings').is(':visible')) {
                    window.stecBuilderSettings.closePlotSettings();
                } else {
                    window.stecBuilderSettings.openPlotSettings();
                }
            });
        }

        (function __construct() {
            $('.stec-builder-plot').data('elementObject', window.stecBuilderElements.get('plot').clone());
            controller();
        })();
    }

    $(function () {
        window.stecBuilderPlot = Plot;
    });

})(window.jQuery);