(function ($) {

    'use strict';

    var Sections, contentStructure, customContent, formSection;

    function Section(id, title, elements) {

        var parent = this;
        var selfElements = [];
        var selfParams = {
            id: null,
            title: null
        };

        this.setParams = function (paramsArray) {
            var param;
            for (param in (paramsArray)) {

                if (paramsArray.hasOwnProperty(param)) {
                    selfParams[param] = paramsArray[param];
                }

            }
        };

        this.getParams = function (param) {

            return param ? selfParams[param] : selfParams;
        };

        this.setElements = function (elementsArray) {
            selfElements = elementsArray;
        };

        this.getElements = function () {

            var elements = {};

            $.each(selfElements, function () {
                elements[this] = window.stecBuilderElements.get(this);
            });

            return elements;

        };

        (function construct() {

            if (id) {
                parent.setParams({id: id});
            }

            if (title) {
                parent.setParams({title: title});
            }

            if (elements) {
                parent.setElements(elements);
            }

        })();

    }

    Sections = {
        selfSections: {},

        add: function (section) {

            if (!(section instanceof Section)) {
                throw 'The section is not instance of Section';
            }

            this.selfSections[section.getParams('id')] = section;
        },

        get: function (id) {

            if (!id) {
                return this.selfSections;
            }

            if (typeof this.selfSections[id] === 'undefined') {
                throw 'Section ' + id + ' does not exist';
            }

            return this.selfSections[id];
        }
    };

    contentStructure = new Section('content-structure', 'Content Structure', ['full-width', 'half-width']);
    Sections.add(contentStructure);

    customContent = new Section('custom-content', 'Custom Content', ['headers', 'paragraph']);
    Sections.add(customContent);

    formSection = new Section('submit-form', 'Form',
            [
                'title',
                'description',
                'short-description',
                'image',
                'attachments',
                'location',
                'guests',
                'rsvp',
                'attendance',
                'organizers',
                'schedule',
                'website-url',
                'icon',
                'calendar',
                'category',
                'color',
                'start-date',
                'end-date',
                'all-day-check',
                'repeater',
                'keywords',
                'featured',
                'counter',
                'comments',
                'email',
                'booking',
                'notes',
                'submit',
                'captcha',
                'privacy'
            ]);
    Sections.add(formSection);




    window.stecBuilderSections = Sections;



})(window.jQuery);