(function ($) {

    'use strict';

    function Settings() {

        var parent = this;
        var activePlotElement = null;
        var activeElementObject = null;

        this.updateSettings = function () {

            var padding, margin, align, height, required;

            if (null === activeElementObject) {
                return;
            }

            padding = [];
            margin = [];
            align = $('.stec-builder-settings').find('span[class^="img-align-"]').filter('.active').data('align');
            height = $('.stec-builder-settings').find('input[name="height"]').val();

            padding.push($('.stec-builder-settings').find('input[name="padding_top"]').val());
            padding.push($('.stec-builder-settings').find('input[name="padding_right"]').val());
            padding.push($('.stec-builder-settings').find('input[name="padding_bottom"]').val());
            padding.push($('.stec-builder-settings').find('input[name="padding_left"]').val());

            margin.push($('.stec-builder-settings').find('input[name="margin_top"]').val());
            margin.push($('.stec-builder-settings').find('input[name="margin_right"]').val());
            margin.push($('.stec-builder-settings').find('input[name="margin_bottom"]').val());
            margin.push($('.stec-builder-settings').find('input[name="margin_left"]').val());

            required = $('.stec-builder-settings').find('input[name="required"]').prop('checked');

            padding = padding.join('px ') + 'px';
            margin = margin.join('px ') + 'px';

            activeElementObject.setParams({
                padding: padding,
                margin: margin,
                required: required,
                textAlign: align ? align : null,
                height: height ? height + 'px' : ''
            });

            activePlotElement.css({
                padding: padding,
                margin: margin,
                textAlign: align ? align : null,
                height: height ? height + 'px' : ''
            });

        };

        this.openSettings = function (plotElement, elementObject) {

            var padding, margin, required, ismust, textAlign, height;

            parent.clearSettings();

            activePlotElement = plotElement;
            activeElementObject = elementObject;

            if (plotElement.children('input, select, textarea').length <= 0) {
                $('.stec-builder-settings').find('li[data-setting="conditions"]').hide();
            } else {
                $('.stec-builder-settings').find('li[data-setting="conditions"]').show();
            }

            if (plotElement.children('select[multiple], textarea').length > 0) {
                $('.stec-builder-settings').find('li[data-setting="height"]').show();
            } else {
                $('.stec-builder-settings').find('li[data-setting="height"]').hide();
            }

            if (plotElement.children('p, h1, textarea, input').length <= 0 && 'submit' !== activeElementObject.getParams('id')) {
                $('.stec-builder-settings').find('li[data-setting="align"]').hide();
            } else {
                $('.stec-builder-settings').find('li[data-setting="align"]').show();
            }

            $('.stec-builder-plot').find('.stec-builder-plot-element-settings-item').removeClass('stec-builder-plot-element-settings-item');
            activePlotElement.addClass('stec-builder-plot-element-settings-item');

            padding = activeElementObject.getParams('padding');
            margin = activeElementObject.getParams('margin');
            required = activeElementObject.getParams('required');
            ismust = activeElementObject.getParams('ismust');
            textAlign = activeElementObject.getParams('textAlign');
            height = activeElementObject.getParams('height');

            padding = padding.split(' ');
            margin = margin.split(' ');

            $('.stec-builder-settings').find('.img-align-' + textAlign).addClass('active');
            $('.stec-builder-settings').find('input[name="padding_top"]').val(parseInt(padding[0], 10));
            $('.stec-builder-settings').find('input[name="padding_right"]').val(parseInt(padding[1], 10));
            $('.stec-builder-settings').find('input[name="padding_bottom"]').val(parseInt(padding[2], 10));
            $('.stec-builder-settings').find('input[name="padding_left"]').val(parseInt(padding[3], 10));
            $('.stec-builder-settings').find('input[name="margin_top"]').val(parseInt(margin[0], 10));
            $('.stec-builder-settings').find('input[name="margin_right"]').val(parseInt(margin[1], 10));
            $('.stec-builder-settings').find('input[name="margin_bottom"]').val(parseInt(margin[2], 10));
            $('.stec-builder-settings').find('input[name="margin_left"]').val(parseInt(margin[3], 10));
            $('.stec-builder-settings').find('input[name="height"]').val(parseInt(height, 10));
            $('.stec-builder-settings').find('input[name="required"]').prop('checked', required);

            if (true === ismust) {
                $('.stec-builder-settings').find('input[name="required"]').prop('disabled', true);
            } else {
                $('.stec-builder-settings').find('input[name="required"]').prop('disabled', false);
            }

            $('.stec-builder-modal.last-active').removeClass('last-active');
            $('.stec-builder-settings').addClass('last-active');
            $('.stec-builder-settings').show();


        };

        this.clearSettings = function () {
            $('.stec-builder-plot').find('.stec-builder-plot-element-settings-item').removeClass('stec-builder-plot-element-settings-item');
            $('.stec-builder-settings').find('input').val(null);
            $('.stec-builder-settings').find('span[class^="img-align-"]').removeClass('active');
            $('.stec-builder-settings').removeClass('last-active');
        };

        this.closeSettings = function () {
            activePlotElement = null;
            activeElementObject = null;

            parent.clearSettings();

            if ($('.stec-builder-settings').is(':visible')) {
                $('.stec-builder-settings').hide();
            }

        };

        this.openPlotSettings = function () {

            var plotObject = $('.stec-builder-plot').data('elementObject');

            var padding = plotObject.getParams('padding');
            var margin = plotObject.getParams('margin');

            padding = padding.split(' ');
            margin = margin.split(' ');

            $('.stec-builder-plot-settings').find('input[name="padding_top"]').val(parseInt(padding[0], 10));
            $('.stec-builder-plot-settings').find('input[name="padding_right"]').val(parseInt(padding[1], 10));
            $('.stec-builder-plot-settings').find('input[name="padding_bottom"]').val(parseInt(padding[2], 10));
            $('.stec-builder-plot-settings').find('input[name="padding_left"]').val(parseInt(padding[3], 10));
            $('.stec-builder-plot-settings').find('input[name="margin_top"]').val(parseInt(margin[0], 10));
            $('.stec-builder-plot-settings').find('input[name="margin_right"]').val(parseInt(margin[1], 10));
            $('.stec-builder-plot-settings').find('input[name="margin_bottom"]').val(parseInt(margin[2], 10));
            $('.stec-builder-plot-settings').find('input[name="margin_left"]').val(parseInt(margin[3], 10));
            $('.stec-builder-plot').addClass('stec-builder-plot-settings-item');
            $('.stec-builder-modal.last-active').removeClass('last-active');
            $('.stec-builder-plot-settings').addClass('last-active');
            $('.stec-builder-plot-settings').show();
        };

        this.closePlotSettings = function () {
            $('.stec-builder-plot').removeClass('stec-builder-plot-settings-item');
            $('.stec-builder-plot-settings').hide();
        };

        this.updatePlotSettings = function () {

            var padding = [];
            var margin = [];

            var plotObject = $('.stec-builder-plot').data('elementObject');

            padding.push($('.stec-builder-plot-settings').find('input[name="padding_top"]').val());
            padding.push($('.stec-builder-plot-settings').find('input[name="padding_right"]').val());
            padding.push($('.stec-builder-plot-settings').find('input[name="padding_bottom"]').val());
            padding.push($('.stec-builder-plot-settings').find('input[name="padding_left"]').val());
            margin.push($('.stec-builder-plot-settings').find('input[name="margin_top"]').val());
            margin.push($('.stec-builder-plot-settings').find('input[name="margin_right"]').val());
            margin.push($('.stec-builder-plot-settings').find('input[name="margin_bottom"]').val());
            margin.push($('.stec-builder-plot-settings').find('input[name="margin_left"]').val());

            padding = padding.join('px ') + 'px';
            margin = margin.join('px ') + 'px';

            plotObject.setParams({
                padding: padding,
                margin: margin
            });

            $('.stec-builder-plot-main').css({
                padding: padding,
                margin: margin
            });
        };

        function controller() {

            $('.stec-builder-settings').find('span[class^="img-align-"]').on('click', function () {
                $('.stec-builder-settings').find('span[class^="img-align-"]').removeClass('active');
                $(this).addClass('active');
                parent.updateSettings();
            });

            $('.stec-builder-settings').find('input').on('keyup keypress blur change paste cut', function () {
                parent.updateSettings();
            });

            $('.stec-builder-plot-settings').find('input').on('keyup keypress blur change paste cut', function () {
                parent.updatePlotSettings();
            });

            $('.stec-builder-plot-settings, .stec-builder-settings').draggable({
                snap: '.stec-builder-modal'
            });

            $(document).on('keydown', function (e) {

                switch (e.which) {
                    case 83:
                        if (e.altKey) {
                            $('.stec-builder-settings').toggle();

                            if (!$('.stec-builder-settings').is(':visible')) {
                                parent.closeSettings();
                            } else {
                                $('.stec-builder-modal.last-active').removeClass('last-active');
                                $('.stec-builder-settings').addClass('last-active');
                            }
                        }
                        break;
                    case 87:
                        if (e.altKey) {
                            $('.stec-builder-plot-settings').toggle();

                            if (!$('.stec-builder-plot-settings').is(':visible')) {
                                parent.closePlotSettings();
                            } else {
                                parent.openPlotSettings();
                            }
                        }
                        break;
                }

            });

        }

        (function construct() {
            $('.stec-builder-history ul').empty();
            controller();
        })();

    }

    $(function () {
        window.stecBuilderSettings = Settings;
    });


})(window.jQuery);