(function ($) {

    'use strict';

    var proto, rProto;

    /**
     * Request Object
     */
    var Request = function () {
        this.base = window.ajaxurl;
        this.data = null;
        this.dataType = 'json';
        this.type = 'POST';
        this.cache = false;
        this.callback = null;
        this.onError = null;
        this.beforeSend = null;
    };

    rProto = Request.prototype;

    rProto.setCache = function (bool) {
        this.cache = bool;
    };

    rProto.setDataType = function (type) {
        this.dataType = type;
    };

    rProto.setData = function (data) {
        this.data = data;
    };

    rProto.setType = function (type) {

        var allowedTypes = ['GET', 'POST'];

        if (-1 === $.inArray(type.toUpperCase(), allowedTypes)) {
            console.warn('Invalid method type: ' + type);
        }

        this.type = type;

    };

    rProto.setCallback = function (callback) {
        this.callback = callback;
    };

    rProto.setOnError = function (fn) {
        this.onError = fn;
    };

    rProto.setBefore = function (fn) {
        this.beforeSend = fn;
    };



    /**
     * Database query functions
     */
    window.StecDB = function () {
        this.ajax = null;
    };

    proto = window.StecDB.prototype;

    proto.query = function (request) {

        var parent = this, isForm;

        if (!(request instanceof Request)) {
            console.warn('request is not instance of Request!');
        }

        if (!window.stecApiNonce || !window.stecApiNonce.nonce) {
            console.warn('stecApiNonce is missing');
        }

        isForm = request.data instanceof FormData ? true : false;

        if (isForm) {
            request.data.append('security', window.stecApiNonce.nonce);
        } else {
            request.data.security = window.stecApiNonce.nonce;
        }

        parent.ajax = $.ajax({
            dataType: request.dataType,
            type: 'POST',
            cache: true,
            url: request.base,
            data: request.data,

            beforeSend: function () {
                if (parent.ajax !== null) {
                    parent.ajax.abort();
                }

                if (typeof request.beforeSend === 'function') {
                    request.beforeSend();
                }
            },
            success: function (data) {
                if (typeof request.callback === "function") {
                    request.callback(data);
                }
            },
            error: function (xhr, status, thrown) {
                if (typeof request.onError === "function") {
                    request.onError(xhr, status, thrown);
                } else {
                    console.log(xhr, status, thrown);
                }
            },
            complete: function () {
                parent.ajax = null;
            },
            processData: isForm ? false : true,
            contentType: isForm ? false : 'application/x-www-form-urlencoded; charset=UTF-8'
        });

    };

    proto.newRequest = function () {
        return new Request();
    };


})(window.jQuery);