(function ($) {

    "use strict";

    var DB = new window.StecDB();
    var moment = window.moment;
    moment.locale('en');

    $(document).on('onEventToggleOpen', function (e, data) {

        var helper,
                glob,
                $container,
                $inner,
                event,
                eventOccurrenceDate,
                template,
                invitedUser,
                status = 0,
                $avatar,
                nowDate,
                startDate,
                availableSlots;

        if (data.event.attendance.length <= 0) {
            return;
        }

        helper = data.helper;
        glob = data.glob;
        $container = data.$container;
        $inner = $container.find('.stec-layout-event-inner-attendance');
        event = data.event;
        eventOccurrenceDate = moment(data.event.start_date).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');

        if ($inner.length <= 0 || !$inner.find('.stec-layout-event-inner-attendance-attendee-template')[0]) {
            return;
        }

        template = $inner.find('.stec-layout-event-inner-attendance-attendee-template')[0].outerHTML;
        $inner.find('.stec-layout-event-inner-attendance-attendee-template').remove();

        availableSlots = helper.attendanceHasFreeSlots(event, eventOccurrenceDate);

        if (event.attendance.length > 0) {

            $(event.attendance).each(function (i) {

                var th = this, $attendee;

                // Don't display private attendee
                if (th.email === false && th.userid === 0) {
                    return true; // continue;
                }

                $attendee = $(template).html(function (index, html) {

                    var statusKlass = '';

                    switch (parseInt(th.status[eventOccurrenceDate], 10)) {
                        case 1:
                            statusKlass = 'fa fa-check';
                            break;

                        case 2:
                            statusKlass = 'fa fa-times';
                            break;

                        default:
                            statusKlass = 'fa fa-question';
                    }

                    return html
                            .replace(/stec_replace_people/g, th.people)
                            .replace(/stec_replace_name/g, th.name)
                            .replace(/#stec_replace_avatar/g, th.avatar)
                            .replace(/stec_replace_userid/g, th.userid ? th.userid : '')
                            .replace(/stec_replace_status/g, '<li class=""><i class="' + statusKlass + '"></i></li>');

                })
                        .removeClass('stec-layout-event-inner-attendance-attendee-template')
                        .appendTo($inner.find('.stec-layout-event-inner-attendance-attendees'));

                if (parseInt(th.status[eventOccurrenceDate], 10) === 1) {
                    $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').show();
                }
            });

        }

        invitedUser = false;

        if (!isNaN(glob.options.userid)) {

            // check if user is invited
            $(event.attendance).each(function () {
                if (this.userid === glob.options.userid) {
                    invitedUser = true;
                    return false; // break
                }
            });
        }

        if (invitedUser === true) {

            $(event.attendance).each(function () {
                if (this.userid === glob.options.userid && this.status[eventOccurrenceDate]) {
                    status = parseInt(this.status[eventOccurrenceDate], 10);
                    return false; // break
                }
            });

            $avatar = $inner.find('.stec-layout-event-inner-attendance-attendee-avatar')
                    .filter('[data-userid="' + glob.options.userid + '"]');

            if (9999 === availableSlots) {
                $inner.find('.stec-layout-event-inner-attendance-invited')
                        .find('.stec-layout-event-inner-attendance-invited-slots-count')
                        .parent().hide();
            }

            $inner.find('.stec-layout-event-inner-attendance-invited')
                    .find('.stec-layout-event-inner-attendance-invited-slots-count')
                    .text(availableSlots);

            switch (status) {
                case 1:
                    // accepted
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').removeClass('active');

                    $avatar.find('li i').attr('class', 'fa fa-check');

                    break;

                case 2:
                    // declined
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').removeClass('active');
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');

                    $avatar.find('li i').attr('class', 'fa fa-times');

                    break;
            }
        } else {
            $inner.find('.stec-layout-event-inner-attendance-invited').remove();
        }

        // Attend / Decline

        function ajaxAttendance(status) {

            // status
            // 0 - no decision
            // 1 - accept 
            // 2 - decline

            var request = DB.newRequest();

            request.setData({
                action: 'stec_public_ajax_action',
                task: 'set_user_event_attendance',
                event_id: event.id,
                date: eventOccurrenceDate,
                status: status
            });

            request.setBefore(function () {
                $inner.find('.stec-layout-event-inner-attendance-invited-buttons').hide();
                $(glob.template.preloader).appendTo($inner.find('.stec-layout-event-inner-attendance-invited'));
            });

            request.setCallback(function (rtrn) {

                var $avatar;
                var $attendee;

                $container.find('.stec-layout-event-inner-intro-attendance li').removeClass('active');
                $inner.find('.stec-layout-event-inner-attendance-invited-buttons li').removeClass('active');

                $avatar = $inner.find('.stec-layout-event-inner-attendance-attendee-avatar')
                        .filter('[data-userid="' + glob.options.userid + '"]');

                $attendee = $avatar.parent();

                switch (parseInt(rtrn.status, 10)) {
                    case 1 :
                        $container.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                        $avatar.find('li i').attr('class', 'fa fa-check');
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').show();
                        break;
                    case 2 :
                        $container.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');
                        $avatar.find('li i').attr('class', 'fa fa-times');
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();
                        break;
                    default:
                        $avatar.find('li i').attr('class', 'fa fa-question');
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();

                }

                $(event.attendance).each(function () {

                    if (this.userid === glob.options.userid) {
                        this.status[eventOccurrenceDate] = status;
                        return false; // break
                    }

                });

                $inner.find('.stec-layout-event-inner-attendance-invited-buttons').css('display', 'flex');
                $inner.find('.stec-layout-event-inner-attendance-invited').find('.stec-preloader').remove();

                $inner.find('.stec-layout-event-inner-attendance-invited')
                        .find('.stec-layout-event-inner-attendance-invited-slots-count')
                        .text(rtrn.slots_left);

            });

            DB.query(request);
        }

        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').on(helper.clickHandle(), function (e) {

            var rsvp = data.glob.rsvp;

            e.preventDefault();

            if ($(this).hasClass('active')) {
                ajaxAttendance(0);
            } else {
                rsvp.settings.id = data.event.id;
                rsvp.settings.offset = data.repeatOffset;
                rsvp.settings.event = data.event;
                rsvp.settings.date = moment(data.event.start_date).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
                rsvp.settings.update = true;
                rsvp.popOpen();
            }
        });

        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').on(helper.clickHandle(), function (e) {
            var status;
            e.preventDefault();
            status = $(this).hasClass('active') ? 0 : 2;
            ajaxAttendance(status);
        });

        // Check if event is in progress
        nowDate = moment();
        startDate = moment.tz(event.start_date, event.timezone).add(data.repeatOffset, 'seconds');

        if (nowDate >= startDate) {
            $inner.find('.stec-layout-event-inner-intro-attendance').hide();
        }

        if (nowDate >= startDate) {
            $inner.find('.stec-layout-event-inner-attendance-invited').hide();
        }

        // Remove tab preloaders
        $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
        $inner.find('.stec-layout-event-inner-preload-wrap').remove();
        $inner.find('.stec-preloader').remove();

    });


})(window.jQuery);