(function ($) {

    "use strict";

    var DB = new window.StecDB();
    var moment = window.moment;
    moment.locale('en');
    
    const __ = new window.StecHelper().lang;

    $(document).on('onBeforeEventWeatherDataAjax', function (e, data) {

        var glob = data.glob;
        var container;
        var $tab;

        if (data.glob.options.general_settings.open_event_in === 'modal') {
            container = '#stec-layout-event-inner-' + data.glob.options.id;
        } else {
            container = '#' + data.glob.options.id + ' .stec-layout-event.active .stec-layout-event-inner';
        }

        $tab = $(container).find('.stec-layout-event-inner-forecast');
        $tab.wrapInner('<div class="stec-layout-event-inner-preload-wrap" />');
        $(glob.template.preloader).appendTo($tab);
    });

    $(document).on('onEventToggleOpen', function (e, data) {

        var glob, $instance, helper, $container, $inner, template, isLoaded, getWeather, fillData, chart, fillError;

        if (!data.event.location.coordinates || !data.glob.options.general_settings.weather_api_key) {
            return;
        }

        glob = data.glob;
        $instance = data.$instance;
        helper = data.helper;
        $container = data.$container;
        $inner = $container.find('.stec-layout-event-inner-forecast');

        if ($inner.length <= 0 || !$inner.find('.stec-layout-event-inner-forecast-details-left-forecast-day-template')[0]) {
            return;
        }

        template = $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-day-template')[0].outerHTML;
        $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-day-template').remove();

        function floorFigure(figure, decimals) {
            var d;
            if (!decimals) {
                decimals = 2;
            }

            d = Math.pow(10, decimals);
            return (parseInt(figure * d) / d).toFixed(decimals);
        }

        function getWindDir(bearing) {

            var val, arr;

            while (bearing < 0) {
                bearing += 360;
            }

            while (bearing >= 360) {
                bearing -= 360;
            }

            val = Math.round((bearing - 11.25) / 22.5);
            arr = [
                __('N'),
                __('NNE'),
                __('NE'),
                __('ENE'),
                __('E'),
                __('ESE'),
                __('SE'),
                __('SSE'),
                __('S'),
                __('SSW'),
                __('SW'),
                __('WSW'),
                __('W'),
                __('WNW'),
                __('NW'),
                __('NNW')
            ];
            return arr[ Math.abs(val) ];

        }

        function iconToText(icon) {

            switch (icon) {

                case ('clear-day') :
                case ('clear-night') :
                    return __('clear_sky');

                case ('partly-cloudy-day') :
                case ('partly-cloudy-night') :
                    return __('partly_cloudy');

                case ('cloudy') :
                    return __('cloudy');

                case ('fog') :
                    return __('fog');

                case ('rain') :
                    return __('rain');

                case ('sleet') :
                    return __('sleet');

                case ('snow') :
                    return __('snow');

                case ('wind') :
                    return __('wind');

            }
        }

        function iconToiconHTML(icon, forceday) {

//          clear-day, clear-night, rain, snow, sleet, wind, fog, cloudy, partly-cloudy-day, or partly-cloudy-night

            switch (icon) {

                case ('clear-day') :
                    return '<div class="stec-forecast-icon-clear-day"></div>';

                case ('clear-night') :
                    return forceday ? '<div class="stec-forecast-icon-clear-day"></div>' : '<div class="stec-forecast-icon-clear-night"></div>';

                case ('partly-cloudy-day') :
                    return '<div class="stec-forecast-icon-cloudy-day"></div>';

                case ('partly-cloudy-night') :
                    return forceday ? '<div class="stec-forecast-icon-cloudy-day"></div>' : '<div class="stec-forecast-icon-cloudy-night"></div>';

                case ('cloudy') :
                    return '<div class="stec-forecast-icon-cloudy"></div>';

                case ('fog') :
                    return '<div class="stec-forecast-icon-mist"></div>';

                case ('rain') :
                    return  '<div class="stec-forecast-icon-rain"></div>';

                case ('sleet') :
                    return '<div class="stec-forecast-icon-sleet"></div>';

                case ('snow') :
                    return '<div class="stec-forecast-icon-snow"></div>';

                case ('wind') :
                    return '<div class="stec-forecast-icon-cloudy"></div>';

            }
        }

        getWeather = function () {

            var location, request;

            if (data.event.forecast) {

                if (!data.event.forecast.error) {
                    fillData();
                } else {
                    fillError();
                }
                return;
            }

            location = data.event.location.coordinates;
            request = DB.newRequest();

            request.setData({
                action: 'stec_public_ajax_action',
                task: 'get_weather_data',
                location: function () {
                    location = location.split(',');
                    location[0] = floorFigure(location[0]);
                    location[1] = floorFigure(location[1]);
                    location = location.join(',');
                    return location;

                }
            });

            request.setBefore(function () {
                $instance.trigger('onBeforeEventWeatherDataAjax', {
                    $instance: $instance,
                    glob: glob
                });
            });

            request.setCallback(function (rtrn) {
                if (rtrn) {

                    if (!data.event.forecast) {
                        data.event.forecast = rtrn;
                    }

                    // error ?
                    if (data.event.forecast.error || !data.event.forecast) {
                        fillError();
                        return;
                    }

                    fillData();
                } else {
                    fillError();
                }

                // Remove tabs preloaders
                $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                $inner.find('.stec-preloader').remove();

            });

            DB.query(request);

        };

        fillData = function () {

            var fiveDays = [];
            var i = 0;
            var forecast = data.event.forecast;
            var ch;

            $(forecast.daily.data).each(function () {

                var th, icon, d, niceDay;

                if (i > 4) {
                    return false;
                }

                th = this;
                icon = iconToiconHTML(th.icon, true);

                // Localtime
                d = helper.treatAsUTC(new Date(this.time * 1000));
                d.setHours(d.getHours() + forecast.offset);

                niceDay = helper.beautifyTimespan(d, d, 1);

                fiveDays[i] = $(template).html(function (index, html) {

                    var tempFmin = Math.round(th.temperatureMin);
                    var tempCmin = Math.round((tempFmin - 32) * 5 / 9);

                    var tempFmax = Math.round(th.temperatureMax);
                    var tempCmax = Math.round((tempFmax - 32) * 5 / 9);

                    return html
                            .replace(/\bstec_replace_date\b/g, niceDay)
                            .replace(/\bstec_replace_min\b/g, glob.options.general_settings.temp_units === "C" ? tempCmin : tempFmin)
                            .replace(/\bstec_replace_max\b/g, glob.options.general_settings.temp_units === "C" ? tempCmax : tempFmax)
                            .replace(/\bstec_replace_temp_units\b/g, glob.options.general_settings.temp_units === "C" ? "C" : "F")
                            .replace(/\bstec_replace_icon_div\b/g, icon);
                })[0].outerHTML;

                i++;
            });

            fiveDays = fiveDays.join('');

            $inner.html(function (index, html) {

                var icon = iconToiconHTML(forecast.currently.icon);
                var tempF = Math.round(forecast.currently.temperature);
                var tempC = Math.round((tempF - 32) * 5 / 9);
                var apTempF = Math.round(forecast.currently.apparentTemperature);
                var apTempC = Math.round((tempF - 32) * 5 / 9);
                var windMPH = Math.round(forecast.currently.windSpeed);
                var windKPH = Math.round(windMPH * 1.609344);
                var niceDay;

                // Local time
                var d = helper.treatAsUTC(new Date(forecast.currently.time * 1000));
                d.setHours(d.getHours() + forecast.offset);

                niceDay = helper.beautifyTimespan(d, d, 1);

                return html
                        .replace(/\bstec_replace_current_summary_text\b/g, iconToText(forecast.currently.icon))
                        .replace(/\bstec_replace_today_date\b/g, niceDay)
                        .replace(/\bstec_replace_location\b/g, helper.capitalizeFirstLetter(data.event.location.full_address))
                        .replace(/\bstec_replace_current_temp\b/g, glob.options.general_settings.temp_units === "C" ? tempC : tempF)
                        .replace(/\bstec_replace_current_feels_like\b/g, glob.options.general_settings.temp_units === "C" ? apTempC : apTempF)
                        .replace(/\bstec_replace_current_humidity\b/g, (forecast.currently.humidity * 100).toFixed(0))
                        .replace(/\bstec_replace_current_temp_units/g, glob.options.general_settings.temp_units === "C" ? "C" : "F")
                        .replace(/\bstec_replace_current_wind\b/g, glob.options.general_settings.wind_units === "MPH" ? windMPH : windKPH)
                        .replace(/\bstec_replace_current_wind_units\b/g, glob.options.general_settings.wind_units === "MPH" ? "MPH" : "KPH")
                        .replace(/\bstec_replace_current_wind_direction\b/g, getWindDir(forecast.currently.windBearing))
                        .replace(/\bstec_replace_today_icon_div\b/g, icon)
                        .replace(/\bstec_replace_5days\b/g, fiveDays);

            });

            // Chart instance
            setTimeout(function () {

                var th,
                        i,
                        humidity = [],
                        tempf,
                        tempc,
                        tempC = [],
                        tempF = [],
                        rain = [],
                        j = -1,
                        timeFormat,
                        d;


                var charTimeLabels = [];

                for (i = 0; i < 8; i++) {

                    j = j + 3;

                    th = forecast.hourly.data[j];

                    tempf = Math.round(th.temperature);
                    tempc = Math.round((tempf - 32) * 5 / 9);

                    tempC[i] = tempc;
                    tempF[i] = tempf;
                    humidity[i] = Math.round(th.humidity * 100).toFixed(0);
                    rain[i] = th.precipProbability * 100;

                    timeFormat = 'HH:mm';

                    if (parseInt(glob.options.general_settings.time_format, 10) === 12) {
                        timeFormat = 'hh:mma';
                    }

                    // Local time
                    d = helper.treatAsUTC(new Date(th.time * 1000));
                    charTimeLabels.push(moment(d).format(timeFormat));

                }

                ch = new chart();

                ch.setCanvas($inner.find('.stec-layout-event-inner-forecast-details-chart canvas'));

                ch.setChartData({
                    labels: charTimeLabels,
                    datasets: [
                        {
                            label: __('humidity_percents'),
                            data: humidity,
                            backgroundColor: "rgba(200,200,200,0.1)",
                            borderColor: "rgba(200,200,200,1)",
                            pointBackgroundColor: "rgba(200,200,200,1)",
                            fill: true,
                            lineTension: 0.3,
                            pointHoverRadius: 5,
                            pointHitRadius: 10,
                            borderWidth: 1
                        },
                        {
                            label: __('rain_chance_percents'),
                            data: rain,
                            backgroundColor: "rgba(70,129,195,0.1)",
                            borderColor: "rgba(70,129,195,1)",
                            pointBackgroundColor: "rgba(70,129,195,1)",
                            fill: true,
                            lineTension: 0.3,
                            pointHoverRadius: 5,
                            pointHitRadius: 10,
                            borderWidth: 1
                        },
                        {
                            label: __('temperature') + ' ' + '\u00B0' + (glob.options.general_settings.temp_units === "C" ? 'C' : 'F'),
                            data: glob.options.general_settings.temp_units === "C" ? tempC : tempF,
                            backgroundColor: "rgba(252,183,85,0.3)",
                            borderColor: "rgba(252,183,85,1)",
                            pointBackgroundColor: "rgba(252,183,85,1)",
                            fill: true,
                            lineTension: 0.3,
                            pointHoverRadius: 5,
                            pointHitRadius: 10,
                            borderWidth: 1
                        }
                    ]
                });

                ch.build();

                helper.onResizeEnd(function () {

                    if ($instance.hasClass('stec-media-small')) {
                        ch.chart.options.legend.display = false;
                    } else {
                        ch.chart.options.legend.display = true;
                    }

                    ch.chart.update();

                }, 50, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

            }, 0);

        };

        chart = function () {

            this.ctx = null;
            this.chartData = null;
            this.chart = null;
            this.setCanvas = function ($canvas) {

                var canvas = $canvas.get(0);

                var w = parseInt($inner.find('.stec-layout-event-inner-forecast-details-chart').width(), 10);
                var h = parseInt($inner.find('.stec-layout-event-inner-forecast-details-chart').height(), 10);

                canvas.width = w;
                canvas.height = h;

                this.ctx = $canvas.get(0).getContext("2d");

            };
            this.setChartData = function (chartData) {
                this.chartData = chartData;
            };
            this.destroy = function () {
                this.chart.destroy();
            };
            this.build = function () {

                var parent = this;
                var generalTextColor,
                        generalFontFamily,
                        displayLegend;

                if (this.chart) {
                    this.destroy();
                }

                generalTextColor = $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-top p').css('color');
                generalFontFamily = $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-top p').css('font-family');
                displayLegend = true;

                if ($instance.hasClass('stec-media-small')) {
                    displayLegend = false;
                }

                this.chart = new window.Chart(this.ctx, {
                    type: 'line',
                    data: parent.chartData,
                    options: {
                        maintainAspectRatio: false,
                        responsive: true,
                        defaultFontFamily: generalFontFamily,
                        defaultFontColor: generalTextColor,
                        legend: {
                            display: displayLegend,
                            labels: {
                                fontFamily: generalFontFamily,
                                fontColor: generalTextColor,
                                fontSize: 12
                            }
                        },
                        scales: {
                            xAxes: [{
                                    ticks: {
                                        fontFamily: generalFontFamily,
                                        fontSize: 11,
                                        fontColor: generalTextColor
                                    },
                                    gridLines: {
                                        color: 'rgba(0,0,0,0.1)',
                                        zeroLineColor: 'rgba(0,0,0,0)'
                                    }
                                }],
                            yAxes: [{
                                    ticks: {
                                        fontFamily: generalFontFamily,
                                        fontSize: 11,
                                        fontColor: generalTextColor
                                    },
                                    gridLines: {
                                        color: 'rgba(0,0,0,0.1)',
                                        zeroLineColor: 'rgba(0,0,0,0)'
                                    }
                                }]
                        },
                        tooltips: {
                            titleFontColor: '#fff',
                            titleFontStyle: generalFontFamily,
                            bodyFontFamily: generalFontFamily,
                            bodyFontColor: '#fff'
                        }

                    }
                });
            };

        };

        fillError = function () {
            $inner.find('.stec-layout-event-inner-forecast-content').remove();
            $inner.find('.errorna').show();
        };

        isLoaded = false;

        $(window).on('stec-tab-click-' + glob.options.id, function () {
            if (!$(this).hasClass('active') && isLoaded === false) {
                getWeather();
                isLoaded = true;
            }
        });

        if ($inner.hasClass('active')) {
            getWeather();
            isLoaded = true;
        }

    });

})(window.jQuery);