(function ($) {

    "use strict";

    var moment = window.moment;
    moment.locale('en');

    $(document).on('onEventToggleOpen', function (e, data) {

        var glob = data.glob;
        var $instance = data.$instance;
        var $container = data.$container;
        var $inner = $container.find('.stec-layout-event-inner-intro');
        var helper, event, DB, eventOccurrenceDate, url, nowDate, startDate, endDate, repeatOffset, googleCalImportLink, categoryTagsHtml = [];
        var slider, organizersTemplate, clock;
        var invitedUser, status, availableSlots;
        var attachmentsTemplate;

        // Init once
        if ($inner.length <= 0 || $inner.data('init') === false) {
            return;
        }

        $inner.data('init', false);

        helper = data.helper;
        event = data.event;
        repeatOffset = parseInt(data.repeatOffset, 10);
        DB = new window.StecDB();
        eventOccurrenceDate = moment(data.event.start_date).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
        url = helper.getPermalink(event.permalink, data.repeatOffset);

        // add the repeat offset
        startDate = helper.dbDateOffset(event.start_date, data.repeatOffset);
        endDate = helper.dbDateOffset(event.end_date, data.repeatOffset);

        googleCalImportLink = helper.eventToGoogleCalImportLink(event.id, data.repeatOffset);

        if (event.category.length > 0 && parseInt(glob.options.general_settings.show_event_tags, 10) === 1) {

            // Sort categories alphabetically
            event.category.sort(function (a, b) {
                return a.title.localeCompare(b.title, undefined, {numeric: true, sensitivity: 'base'});
            });

            $.each(event.category, function () {
                categoryTagsHtml.push('<span class="stec-layout-event-inner-intro-tag" style="background-color: ' + this.color + '">' + this.title + '</span>');
            });
        } else {
            data.$container.find('.stec-layout-event-inner-intro-tags').remove();
        }

        $inner.html(function (index, html) {
            return html

                    .replace(/stec_replace_summary/g, event.title)
                    .replace(/stec_replace_tags/g, (categoryTagsHtml.join('')))
                    .replace(/stec_replace_description/g, (event.description))
                    .replace(/#stec_replace_link/g, event.link)
                    .replace(/#stec_replace_googlecal_import/g, googleCalImportLink)
                    .replace(/stec_replace_event_id/g, event.id)
                    .replace(/stec_replace_calendar_id/g, event.calid)
                    .replace(/#stec_replace_event_single_url/g, url)
                    .replace(/stec_replace_event_single_url/g, url);
        });

        if (!event.edit_link) {
            $inner.find('.stec-layout-event-inner-intro-edit-link').remove();
        }

        slider = {
            cslide: 0,
            offset: 0,
            total: 0,
            blockAction: false,
            visCount: 0,
            visCountSmall: 3,
            visCountBig: 4,
            init: function () {

                var parent = this;

                if (!event.images_meta || event.images_meta.length <= 0) {

                    $inner.find('.stec-layout-event-inner-intro-media').remove();

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();

                    return;
                }

                if (event.images_meta.length === 1) {
                    $inner.find('.stec-layout-event-inner-intro-media-controls').remove();
                } else {
                    if (event.images_meta.length < this.visCountBig) {
                        this.visCountBig = event.images_meta.length;
                    }

                    if (event.images_meta.length < this.visCountSmall) {
                        this.visCountSmall = event.images_meta.length;
                    }
                }

                this.html();

                helper.imgLoaded($inner.find('.stec-layout-event-inner-intro-media-content img'), function () {

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();

                    setTimeout(function () {
                        parent.controlsDimensions();
                        parent.showImage();

                        $inner.find('.stec-layout-event-inner-intro-media').fadeTo(1000, 1);
                    }, 100);


                });

                this.bindControls();

            },
            bindControls: function () {

                var parent = this;

                // resize on tab click
                $(document).on('stec-tab-click-' + glob.options.id, function () {
                    parent.controlsDimensions();
                });

                helper.onResizeEnd(function () {
                    parent.controlsDimensions();
                }, 100, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

                $inner.find('.stec-layout-event-inner-intro-media-controls-next').on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.slideNext();
                });

                $inner.find('.stec-layout-event-inner-intro-media-controls-prev').on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.slidePrev();
                });

                $inner.find('.stec-layout-event-inner-intro-media-controls li').on(helper.clickHandle(), function (e) {

                    e.preventDefault();

                    if (parent.cslide === $(this).index()) {
                        return;
                    }

                    parent.cslide = $(this).index();

                    parent.showImage();

                });

            },
            html: function () {

                var html = [];

                $(event.images_meta).each(function () {

                    html.push('<div style="background-image:url(' + this.src + ');">');
                    html.push('   <img src="' + this.src + '" alt="' + this.alt + '">');

                    if (this.caption !== '' || this.description !== '') {
                        html.push('   <div>');

                        if (this.caption !== '') {
                            html.push('       <p>' + this.caption + '</p>');
                        }

                        if (this.description !== '') {
                            html.push('       <span>' + this.description + '</span>');
                        }

                        html.push('   </div>');
                    }

                    html.push('</div>');

                });

                $(html.join('')).appendTo($inner.find('.stec-layout-event-inner-intro-media-content'));

                html = [];

                $(event.images_meta).each(function () {
                    html.push('<li style="background-image:url(' + this.thumb + ')">');
                    html.push('</li>');
                });

                $(html.join('')).appendTo($inner.find('.stec-layout-event-inner-intro-media-controls-list'));

            },
            controlsDimensions: function () {

                var parent = this;
                var maxWidth, $li, liWidth, listWidth, left;

                if (!$inner.is(':visible')) {
                    return;
                }

                if ($instance.hasClass('stec-media-small')) {
                    parent.visCount = parent.visCountSmall;
                } else {
                    parent.visCount = parent.visCountBig;
                }

                if ($inner.find('.stec-layout-event-inner-intro-media-controls-list li').length === parent.visCount) {
                    $inner.find('.stec-layout-event-inner-intro-media-controls').addClass('no-side-controls');
                } else {
                    $inner.find('.stec-layout-event-inner-intro-media-controls').removeClass('no-side-controls');
                }

                maxWidth = $inner.find('.stec-layout-event-inner-intro-media-controls-list-wrap').width();
                $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');

                $inner.find('.stec-layout-event-inner-intro-media-content').height($inner.find('.stec-layout-event-inner-intro-media img').first().height());

                // ~'calc( (100% - 2*10px) / 3 )';
                liWidth = (maxWidth - ((this.visCount - 1) * 10)) / this.visCount;

                listWidth = ($li.length * liWidth) + ($li.length * 10) - 10;

                $inner.find('.stec-layout-event-inner-intro-media-controls-list').width(listWidth);
                $li.width(liWidth);

                this.offset = 0;

                left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().css({
                    left: left
                });


            },
            showImage: function () {

                var $old, $new, $textContent, height;

                $inner.find('.stec-layout-event-inner-intro-media-controls-list .active-thumb').removeClass('active-thumb');
                $inner.find('.stec-layout-event-inner-intro-media-controls-list li').eq(this.cslide).addClass('active-thumb');

                $old = $inner.find('.stec-layout-event-inner-intro-media-content .active-image');
                $new = $inner.find('.stec-layout-event-inner-intro-media-content > div').eq(this.cslide);

                $textContent = $new.find('div');

                if ($textContent.length > 0) {

                    $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 0, function () {

                        var caption = $textContent.find('p').text();
                        var desc = $textContent.find('span').text();

                        $inner.find('.stec-layout-event-inner-intro-media-content-subs p').text(caption);
                        $inner.find('.stec-layout-event-inner-intro-media-content-subs span').text(desc);

                        height = $inner.find('.stec-layout-event-inner-intro-media-content-subs p').height() + $inner.find('.stec-layout-event-inner-intro-media-content-subs span').height();

                        if (height > 0) {
                            height = height + 40;
                        }

                        $inner.find('.stec-layout-event-inner-intro-media-content-subs').stop().animate({
                            height: height
                        }, {
                            duration: 400,
                            easing: 'stecExpo',
                            complete: function () {
                                $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 1);
                            }
                        });
                    });


                } else {

                    $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 0, function () {

                        $inner.find('.stec-layout-event-inner-intro-media-content-subs').stop().animate({
                            height: 0
                        }, {
                            duration: 400,
                            easing: 'stecExpo'
                        });

                    });

                }

                $new.addClass('fade-in');

                setTimeout(function () {

                    $old.removeClass('active-image');
                    $new.removeClass('fade-in').addClass('active-image');

                }, 250);
            },
            slideNext: function () {

                var $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');
                var left;

                if (this.offset + this.visCount >= $li.length) {
                    this.offset = 0;
                } else {
                    this.offset = this.offset + this.visCount;

                    if (this.offset > $li.length - this.visCount) {
                        this.offset = $li.length - this.visCount;
                    }
                }

                left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().animate({
                    left: left
                }, {
                    duration: 750,
                    easing: 'stecExpo'
                });

            },
            slidePrev: function () {

                var $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');
                var left;

                if (this.offset <= 0) {
                    this.offset = $li.length - this.visCount;
                } else {
                    this.offset = this.offset - this.visCount;

                    if (this.offset < 0) {
                        this.offset = 0;
                    }
                }

                left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().animate({
                    left: left
                }, {
                    duration: 750,
                    easing: 'stecExpo'
                });

            }

        };

        slider.init();

        if (event.organizers.length > 0 && $inner.find('.stec-layout-event-inner-intro-organizers-organizer-template').length > 0) {
            organizersTemplate = $inner.find('.stec-layout-event-inner-intro-organizers-organizer-template')[0].outerHTML;
            $inner.find('.stec-layout-event-inner-intro-organizers-organizer-template').remove();

            $.each(event.organizers, function () {

                var th = this;
                var social = '';
                var image = th.photo_src ? 'style="background-image: url(' + th.photo_src + ')"' : "";

                if (th.social.length > 0) {
                    $.each(th.social, function () {
                        var link = this.link;
                        if ('fas fa-envelope' === this.ico) {
                            link = 'mailto:' + link;
                        }
                        social += '<a target="_blank" href="' + link + '"><i class="' + this.ico + '"></i></a>';
                    });
                }

                $(organizersTemplate)
                        .removeClass('stec-layout-event-inner-intro-organizers-organizer-template')
                        .html(function (index, html) {

                            var organizerName = th.name;

                            if (th.permalink) {
                                organizerName = '<a href="' + th.permalink + '">' + th.name + '</a>';
                            }

                            return html
                                    .replace(/stec_replace_title/g, organizerName)
                                    .replace(/stec_replace_event/g, event.title)
                                    .replace(/stec_replace_about/g, th.details)
                                    .replace(/stec_replace_social/g, social)
                                    .replace(/stec_replace_image/g, image);

                        }).appendTo($inner.find('.stec-layout-event-inner-intro-organizers'));


            });
        } else {
            $inner.find('.stec-layout-event-inner-intro-organizers').remove();
        }

        clock = {
            days: 0,
            hours: 0,
            minutes: 0,
            seconds: 0,
            daysLabel: '',
            hoursLabel: '',
            minutesLabel: '',
            secondsLabel: '',
            interval: '',
            init: function () {

                var nowDate, startDate, timeLeft;

                // Check counter disabled
                if (parseInt(event.counter, 10) !== 1) {
                    $inner.find('.stec-layout-event-inner-intro-counter').hide();
                    return;
                }

                nowDate = moment();
                startDate = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');
                timeLeft = Math.floor((startDate.unix() - nowDate.unix()));

                this.days = Math.floor(timeLeft / 86400);
                this.hours = Math.floor(timeLeft % 86400 / 3600);
                this.minutes = Math.floor(timeLeft % 86400 % 3600 / 60);
                this.seconds = Math.floor(timeLeft % 86400 % 3600 % 60);

                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(this.days);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(this.hours);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(this.minutes);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(this.seconds);

                this.daysLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(0);
                this.hoursLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(1);
                this.minutesLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(2);
                this.secondsLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(3);

                this.daysLabel.text(this.days === 1 ? this.daysLabel.attr('data-singular-label') : this.daysLabel.attr('data-plural-label'));
                this.hoursLabel.text(this.hours === 1 ? this.hoursLabel.attr('data-singular-label') : this.hoursLabel.attr('data-plural-label'));
                this.minutesLabel.text(this.minutes === 1 ? this.minutesLabel.attr('data-singular-label') : this.minutesLabel.attr('data-plural-label'));
                this.secondsLabel.text(this.seconds === 1 ? this.secondsLabel.attr('data-singular-label') : this.secondsLabel.attr('data-plural-label'));

                if (timeLeft < 0) {
                    this.complete();
                    return;
                }

                this.count();
            },
            count: function () {

                var parent = this;

                if (parent.interval) {
                    clearInterval(parent.interval);
                }

                parent.interval = setInterval(function () {

                    parent.seconds--;

                    if (parent.seconds < 0) {
                        parent.seconds = 59;
                        parent.minutes--;
                        if (parent.minutes < 0) {
                            parent.minutes = 59;
                            parent.hours--;
                            if (parent.hours < 0) {
                                parent.hours = 23;
                                if (parent.days > 0) {
                                    parent.days--;
                                }
                                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(parent.days);
                                parent.daysLabel.text(parent.days === 1 ? parent.daysLabel.attr('data-singular-label') : parent.daysLabel.attr('data-plural-label'));
                            }
                            $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(parent.hours);
                            parent.hoursLabel.text(parent.hours === 1 ? parent.hoursLabel.attr('data-singular-label') : parent.hoursLabel.attr('data-plural-label'));
                        }
                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(parent.minutes);
                        parent.minutesLabel.text(parent.minutes === 1 ? parent.minutesLabel.attr('data-singular-label') : parent.minutesLabel.attr('data-plural-label'));
                    }
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(parent.seconds);
                    parent.secondsLabel.text(parent.seconds === 1 ? parent.secondsLabel.attr('data-singular-label') : parent.secondsLabel.attr('data-plural-label'));


                    if (parent.days === 0 && parent.hours === 0 && parent.minutes === 0 && parent.seconds === 0) {
                        clearInterval(parent.interval);
                        parent.complete();
                    }

                }, 1000);
            },
            complete: function () {

                var now, endDate;

                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(0);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(0);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(0);
                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(0);

                this.daysLabel.text(this.days === 1 ? this.daysLabel.attr('data-singular-label') : this.daysLabel.attr('data-plural-label'));
                this.hoursLabel.text(this.hours === 1 ? this.hoursLabel.attr('data-singular-label') : this.hoursLabel.attr('data-plural-label'));
                this.minutesLabel.text(this.minutes === 1 ? this.minutesLabel.attr('data-singular-label') : this.minutesLabel.attr('data-plural-label'));
                this.secondsLabel.text(this.seconds === 1 ? this.secondsLabel.attr('data-singular-label') : this.secondsLabel.attr('data-plural-label'));

                $inner.find('.stec-layout-event-inner-intro-counter').hide();

                now = moment();
                endDate = moment.tz(event.end_date, event.timezone).add(repeatOffset, 'seconds');

                if (now >= endDate) {
                    $inner.find('.stec-layout-event-inner-intro-event-status').show();
                    $inner.find('.stec-layout-event-inner-intro-event-status-text.event-expired').show();
                } else {
                    $inner.find('.stec-layout-event-inner-intro-event-status').show();
                    $inner.find('.stec-layout-event-inner-intro-event-status-text.event-inprogress').show();
                }
            }

        };

        clock.init();


        // Attend / Decline
        function ajaxAttendance(status) {

            // status
            // 0 - no decision
            // 1 - accept 
            // 2 - decline

            var request = DB.newRequest();

            request.setData({
                action: 'stec_public_ajax_action',
                task: 'set_user_event_attendance',
                event_id: event.id,
                date: eventOccurrenceDate,
                status: status
            });

            request.setBefore(function () {
                $inner.find('.stec-layout-event-inner-intro-attendance').children().hide();

                $('<li>' + glob.template.preloader + '</li>').addClass('intro-attendance')
                        .appendTo($inner.find('.stec-layout-event-inner-intro-attendance'));
            });

            request.setCallback(function (rtrn) {
                var status = parseInt(rtrn.status, 10);
                var $avatar, $attendee;

                $inner.find('.stec-layout-event-inner-intro-attendance li').removeClass('active');
                $container.find('.stec-layout-event-inner-attendance-invited-buttons li').removeClass('active');

                $avatar = $container.find('.stec-layout-event-inner-attendance-attendee-avatar')
                        .filter('[data-userid="' + glob.options.userid + '"]');

                $attendee = $avatar.parent();

                switch (status) {
                    case 1 :
                        $inner.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                        $container.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                        $avatar.find('li i').attr('class', 'fa fa-check');
                        break;
                    case 2 :
                        $inner.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                        $container.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');
                        $avatar.find('li i').attr('class', 'fa fa-times');
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();
                        break;
                    default:
                        $avatar.find('li i').attr('class', 'fa fa-question');
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();

                }

                $(event.attendance).each(function () {

                    if (this.userid === glob.options.userid) {
                        this.status[eventOccurrenceDate] = status;
                        return false; // break
                    }

                });

                $inner.find('.stec-layout-event-inner-intro-attendance').children().show();
                $inner.find('.stec-layout-event-inner-intro-attendance').children().last().remove();

                availableSlots = rtrn.slots_left;

                $container.find('.stec-layout-event-inner-attendance-invited-slots-count').text(availableSlots);
            });

            DB.query(request);

        }

        $inner.find('.stec-layout-event-inner-intro-attendance-attend').on(helper.clickHandle(), function (e) {

            var rsvp = data.glob.rsvp;

            e.preventDefault();

            if ($(this).hasClass('active')) {
                ajaxAttendance(0);
            } else {
                rsvp.settings.id = data.event.id;
                rsvp.settings.offset = data.repeatOffset;
                rsvp.settings.event = data.event;
                rsvp.settings.date = moment(data.event.start_date).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
                rsvp.settings.update = true;
                rsvp.popOpen();
            }

        });

        $inner.find('.stec-layout-event-inner-intro-attendance-decline').on(helper.clickHandle(), function (e) {
            var status;
            e.preventDefault();
            status = $(this).hasClass('active') ? 0 : 2;
            ajaxAttendance(status);
        });

        invitedUser = false;
        availableSlots = helper.attendanceHasFreeSlots(event, eventOccurrenceDate);

        if (!isNaN(glob.options.userid)) {

            // check if user is invited
            $(event.attendance).each(function () {
                if (this.userid === glob.options.userid) {
                    invitedUser = true;
                    return false; // break
                }
            });
        }

        if (invitedUser !== false) {

            status = 0;

            $(event.attendance).each(function () {
                if (this.userid === glob.options.userid && this.status[eventOccurrenceDate]) {
                    status = parseInt(this.status[eventOccurrenceDate], 10);
                    return false; // break
                }
            });

            if (availableSlots <= 0 && status !== 1) {
                $inner.find('.stec-layout-event-inner-intro-attendance').hide();
            }

            switch (status) {
                case 1:
                    $inner.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                    $inner.find('.stec-layout-event-inner-intro-attendance-decline').removeClass('active');
                    break;

                case 2:
                    $inner.find('.stec-layout-event-inner-intro-attendance-attend').removeClass('active');
                    $inner.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                    break;
            }
        } else {
            $inner.find('.stec-layout-event-inner-intro-attendance').hide();
        }

        nowDate = moment();
        startDate = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');

        // RSVP button
        if (true === invitedUser || parseInt(event.rsvp, 10) !== 1 || nowDate >= startDate) {
            $inner.find('.stec-layout-event-inner-intro-rsvp').remove();
        }

        // Booking button
        if (event.bookable_products.length <= 0 || nowDate >= startDate) {
            $inner.find('button[data-action="open-tickets"]').remove();
        } else {
            $inner.find('button[data-action="open-tickets"]').data({
                event: event,
                offset: data.repeatOffset
            });
        }

        // External link button
        if (event.link !== "") {
            $inner.find('.stec-layout-event-inner-intro-external-link').show();
        }

        // Remove RSVP and Link section
        if (!event.link && $inner.find('.stec-layout-event-inner-intro-rsvp').length <= 0) {
            $inner.find('.stec-layout-event-inner-intro-external-link').parent().remove();
        }

        // Check if event is in progress
        if (nowDate >= startDate) {
            $inner.find('.stec-layout-event-inner-intro-attendance').hide();
        }

        // Attachments
        if (event.attachments.length > 0) {
            attachmentsTemplate = $inner.find('.stec-layout-event-inner-intro-attachment-template')[0].outerHTML;
            $inner.find('.stec-layout-event-inner-intro-attachment-template').remove();

            $(event.attachments).each(function () {

                var th = this;

                $(attachmentsTemplate).html(function (index, html) {
                    return html
                            .replace(/stec_replace_filename/g, th.filename)
                            .replace(/stec_replace_desc/g, th.description)
                            .replace(/\#stec_replace_url/g, th.link)
                            .replace(/stec_replace_size/g, th.size);

                }).appendTo($inner.find('.stec-layout-event-inner-intro-attachments-list'));
            });

            $inner.find('.stec-layout-event-inner-intro-attachments-toggle').on(helper.clickHandle(), function (e) {

                e.preventDefault();

                $(this).toggleClass('active');

                $inner.find('.stec-layout-event-inner-intro-attachments-list').toggleClass('active');

            });
        } else {
            $inner.find('.stec-layout-event-inner-intro-attachments').remove();
        }

        $inner.find('.stec-layout-event-inner-intro-exports-toggle').on(helper.clickHandle(), function (e) {

            e.preventDefault();

            $(this).toggleClass('active');

            $inner.find('.stec-layout-event-inner-intro-exports-options').toggleClass('active');

        });

        $inner.on(helper.clickHandle(), '.fa-link', function (e) {

            var eventId = data.$container.data('event-id');
            var eventOffset = data.$container.data('repeat-time-offset');
            var linkURL = $(this).parent().attr('href');
            var sharer = new window.stecSharer($instance, glob, helper, linkURL, eventId, eventOffset);

            e.stopPropagation();
            e.preventDefault();

            sharer.init();

        });

        $inner.find('.stec-layout-event-inner-intro-share-and-export').on(helper.clickHandle(), '.fa-envelope', function (e) {

            var eventId = $(this).parents('.stec-layout-event').attr('data-id');
            var eventOffset = $(this).parents('.stec-layout-event').attr('data-repeat-time-offset');
            var linkURL = $(this).parent().attr('href');
            var emailForm = new window.stecEmailEvent($instance, glob, helper, linkURL, eventId, eventOffset);

            e.stopPropagation();
            e.preventDefault();

            emailForm.init();

        });


    });

})(window.jQuery);