(function ($) {

    "use strict";

    $(document).on('onEventToggleOpen', function (e, data) {

        var glob, helper, event, $container, $inner;

        if (!data.event.location.full_address) {
            return;
        }

        glob = data.glob;
        helper = data.helper;

        event = data.event;
        $container = data.$container;
        $inner = $container.find('.stec-layout-event-inner-location');

        $inner.html(function (index, html) {

            return html
                    .replace(/stec_replace_location/g, function () {
                        if (event.location.title) {
                            return [event.location.title, event.location.full_address].join(', ');
                        }

                        return event.location.full_address;
                    })
                    .replace(/stec_replace_details/g, event.location.details ? event.location.details : '');
        });

        if ($.trim($inner.find('.stec-layout-event-inner-location-details').text()) === "") {
            $inner.find('.stec-layout-event-inner-location-optional-details').hide();
        }

        /**
         * @todo Instance or Single?
         */
        function gmap() {

            this.$tabCont = "";
            this.directionsService = "";
            this.directionsDisplay = "";
            this.map = "";
            this.geocoder = "";
            this.mapDiv = "";
            this.marker = "";
            this.address = "";

            this.initNoMap = function ($el) {
                var parent = this;
                var $start, $end;

                parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                parent.address = data.event.location.full_address;

                $start = parent.$tabCont.find('input[name="start"]');
                $end = parent.$tabCont.find('input[name="end"]');

                if ($.trim($end.val()) === "") {
                    $end.val(parent.$tabCont.find(".stec-layout-event-inner-location-address").text());
                }

                if ($.trim($start.val()) === "") {
                    parent.fillMyLocation($start);
                }

                parent.$tabCont.find('.stec-layout-event-inner-location-directions-buttons .stec-layout-event-inner-location-left-button').remove();

                parent.bindControls();

                // Remove tab preloaders
                $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                $inner.find('.stec-preloader').remove();
            };

            this.init = function ($el) {

                var parent = this;
                var $start, $end;

                if (!window.google || !window.google.maps) {
                    this.initNoMap($el);
                    return;
                }

                parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                parent.address = data.event.location.full_address;

                parent.mapDiv = $el.get(0);
                parent.map = new window.google.maps.Map(parent.mapDiv, {zoom: 15});
                parent.geocoder = new window.google.maps.Geocoder();
                parent.directionsService = new window.google.maps.DirectionsService();
                parent.directionsDisplay = new window.google.maps.DirectionsRenderer();
                parent.directionsDisplay.setMap(parent.map);
                parent.setLocation(parent.address);

                if (glob.options.gmap_theme && glob.options.gmap_theme !== 'standard') {
                    parent.map.mapTypes.set('Custom_Style', helper.getMapStyles(glob.options.gmap_theme));
                    parent.map.setMapTypeId('Custom_Style');
                }

                // start end directions
                $start = parent.$tabCont.find('input[name="start"]');
                $end = parent.$tabCont.find('input[name="end"]');

                if ($.trim($end.val()) === "") {
                    $end.val(event.location.full_address);
                }

                if ($.trim($start.val()) === "") {
                    parent.fillMyLocation($start);
                }

                this.bindControls();

                // Remove tab preloaders
                $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                $inner.find('.stec-preloader').remove();

            };

            this.bindControls = function () {

                var parent = this;

                helper.onResizeEnd(function () {
                    parent.refresh();
                }, 150, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

                parent.$tabCont
                        .parents('.stec-layout-event-inner')
                        .find('[data-tab="stec-layout-event-inner-location"]')
                        .on(helper.clickHandle(), function (e) {

                            parent.refresh(false);
                        });


                parent.$tabCont.find(".stec-layout-event-inner-location-open-gmap").on(helper.clickHandle(), function (e) {

                    var $tabCont = $(this).parents(".stec-layout-event-inner-location");
                    var $start = $tabCont.find('input[name="start"]');
                    var $end = $tabCont.find('input[name="end"]');
                    var eventLocation = $end.val();
                    var latlng, href;

                    if (event.location.coordinates) {
                        latlng = event.location.coordinates.split(',');

                        eventLocation = {
                            lat: parseFloat($.trim(latlng[0])),
                            lng: parseFloat($.trim(latlng[1]))
                        };

                        href = 'https://www.google.com/maps/dir/?api=1&origin=' + $start.val() + '&destination=' + eventLocation.lat + ',' + eventLocation.lng + '&travelmode=driving';
                        $(this).attr('href', href);
                    } else {
                        e.preventDefault();
                    }

                });
                parent.$tabCont.find(".stec-layout-event-inner-location-left-button").on(helper.clickHandle(), function (e) {

                    var $tabCont = $(this).parents(".stec-layout-event-inner-location");
                    var $start = $tabCont.find('input[name="start"]');
                    var $end = $tabCont.find('input[name="end"]');
                    var eventLocation = $end.val();
                    var latlng;

                    if (event.location.coordinates) {
                        latlng = event.location.coordinates.split(',');

                        eventLocation = {
                            lat: parseFloat($.trim(latlng[0])),
                            lng: parseFloat($.trim(latlng[1]))
                        };
                    }

                    if ($start.val() && eventLocation) {
                        e.preventDefault();
                        parent.getDirection($start.val(), eventLocation);
                    }
                });

            };

            this.refresh = function (centerOnLocation) {

                var parent = this;

                setTimeout(function () {
                    window.google.maps.event.trigger(parent.mapDiv, 'resize');

                    if (centerOnLocation === true) {
                        parent.setLocation(parent.address);
                    }
                }, 10); // timeout fixes resize bug

            };

            this.fillMyLocation = function ($el) {

                var parent = this;

                if (glob.options.myLocation) {
                    $el.val(glob.options.myLocation);
                    return;
                }

                if (navigator.geolocation) {

                    navigator.geolocation.getCurrentPosition(
                            function (position) {

                                var pos = [position.coords.latitude, position.coords.longitude];

                                if (!parent.geocoder || !parent.geocoder.geocode) {
                                    glob.options.myLocation = pos.join(', ');
                                    $el.val(glob.options.myLocation);
                                    return;
                                }

                                parent.geocoder.geocode({'address': pos.join(' ')}, function (results, status) {
                                    if (results.length > 0) {
                                        glob.options.myLocation = (results[0].formatted_address);
                                        $el.val(glob.options.myLocation);
                                    }
                                });
                            },
                            function (a, b, c) {
                                console.log('Navigator Geolocation Error');
                            }
                    );
                }
            };

            this.setLocation = function (address) {

                var parent = this, latlng, pos;

                if (event.location.coordinates) {

                    latlng = event.location.coordinates.split(',');
                    pos = {
                        lat: parseFloat($.trim(latlng[0])),
                        lng: parseFloat($.trim(latlng[1]))
                    };

                    parent.map.setCenter(pos);
                    parent.marker = new window.google.maps.Marker({
                        map: parent.map,
                        position: pos,
                        title: address
                    });

                } else {
                    parent.geocoder.geocode({'address': address}, function (results, status) {
                        if (status === window.google.maps.GeocoderStatus.OK) {
                            parent.map.setCenter(results[0].geometry.location);
                            parent.marker = new window.google.maps.Marker({
                                map: parent.map,
                                position: results[0].geometry.location,
                                title: address
                            });

                        } else {
                            console.log("Geocoder error: " + status);
                        }
                    });
                }


                parent.refresh();
            };

            this.getDirection = function (a, b) {

                var parent = this;

                parent.directionsService.route({
                    origin: a,
                    destination: b ? b : parent.marker.position,
                    travelMode: window.google.maps.TravelMode.DRIVING
                }, function (response, status) {

                    if (status === window.google.maps.DirectionsStatus.OK) {
                        parent.directionsDisplay.setDirections(response);
                    } else {
                        console.log("Direction Service Error");

                        $inner.find('.stec-layout-event-inner-location-direction-error').stop().fadeTo(250, 1, function () {

                            setTimeout(function () {

                                $inner.find('.stec-layout-event-inner-location-direction-error').fadeTo(250, 0);

                            }, 3000);

                        });
                    }

                });
            };
        }

        function loadMap() {

            var $event, $mapCont;

            if ($inner.is(':visible')) {
                if (glob.options.general_settings.open_event_in === 'modal') {
                    $event = $inner.parents('.stec-layout-event-inner');
                } else {
                    $event = $inner.parents('.stec-layout-event');
                }
            } else {
                return;
            }

            $mapCont = $event.find(".stec-layout-event-inner-location-right-gmap");

            // init once
            if ($mapCont.children().length <= 0) {
                new gmap().init($mapCont);
            }
        }

        $(document).on('stec-tab-click-' + glob.options.id, function () {
            loadMap();
        });

        loadMap();

        // Remove preloader
        $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
        $inner.find('.stec-layout-event-inner-preload-wrap').remove();
        $inner.find('.stec-preloader').remove();

    });

})(window.jQuery);