(function ($) {

    "use strict";

    $.stecExtend(function (master) {

        var $instance = master.$instance;
        var helper = master.helper;
        var glob = master.glob;
        var layout = master.layout;
        var calData = master.calData;

        /**
         * Top Search form handling
         */
        var search = {

            init: function () {
                this.bindControls();
            },

            bindControls: function () {

                var parent = this;

                var suggestionTimeout;

                $instance.$top.find(".stec-top-search-dropdown").on("keyup", function (e) {

                    var value, $lis, $selected;

                    clearTimeout(suggestionTimeout);

                    value = $instance.$top.find(".stec-top-search-form input").val();
                    $lis = $instance.$top.find(".stec-top-search-results li");

                    switch (e.which) {

                        // esc
                        case 27 :
                            parent.closeSearch();
                            break;

                            // enter 
                        case 13 :

                            if ($lis.filter(".active").length <= 0) {
                                parent.getResults(value);
                                return;
                            }

                            $selected = $lis.filter(".active");

                            // Jump date check
                            if (typeof $selected.attr("data-jumpdate") !== "undefined") {
                                parent.jumpToDate($selected);
                                parent.closeSearch();
                            }

                            break;

                            // up arrow
                        case 38 :
                            if ($lis.filter(".active").length > 0) {
                                $lis.filter(".active")
                                        .removeClass("active")
                                        .prev()
                                        .addClass("active");
                            } else {
                                $lis.filter(":last").addClass("active");
                            }
                            break;

                            // down arrow
                        case 40 :
                            if ($lis.filter(".active").length > 0) {
                                $lis.filter(".active")
                                        .removeClass("active")
                                        .next()
                                        .addClass("active");
                            } else {
                                $lis.filter(":first").addClass("active");
                            }
                            break;

                        default:
                            suggestionTimeout = setTimeout(function () {
                                parent.getResults(value);
                            }, 250);

                    }

                });

                // Result list click handle
                $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-search-results li", function (e) {

                    var $lis = $instance.$top.find(".stec-top-search-results li");
                    var $selected;

                    $lis.removeClass("active");
                    $(this).addClass("active");

                    $selected = $lis.filter(".active");

                    if (typeof $selected.attr("data-jumpsingle") === "undefined") {
                        e.preventDefault();
                    }

                    // Jump date check
                    if (typeof $selected.attr("data-jumpdate") !== "undefined") {
                        parent.jumpToDate($selected);
                        parent.closeSearch();
                    }

                });

                // Search button click handle
                $instance.$top.find(".stec-top-search-form a").on(helper.clickHandle(), function (e) {

                    var value;

                    e.preventDefault();

                    value = $instance.$top.find("input").val();
                    parent.getResults(value);
                });

                // Search main button toggle show/hide
                $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-search", function (e) {
                    var $dropdown;

                    e.preventDefault();
                    e.stopPropagation();

                    // fix for left offset since today button is not fixed width
                    $dropdown = $(this).find('.stec-top-search-dropdown');

                    $dropdown.css({
                        left: -1 * Math.round($(this).position().left)
                    });

                    $(this).toggleClass("active");
                });

                // Block search toggle for inner content
                $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-search-dropdown", function (e) {
                    // preventDefault blocks mobile keyboard
                    e.stopPropagation();
                });

            },

            /**
             * jump to $selected data-jumpdate attribute
             * data-jumpdate="yyyy-mm-dd"
             * @param {object} $selected $(element)
             */
            jumpToDate: function ($selected) {

                var date = helper.getDateFromData($selected.attr("data-jumpdate"));

                glob.options.year = date.getFullYear();
                glob.options.month = date.getMonth();
                glob.options.day = date.getDate();

                layout.set();

            },

            closeSearch: function () {
                this.resetResults();
                $instance.$top.find(".stec-top-menu-search").removeClass("active");
            },

            resetResults: function () {
                $instance.$top.find('.stec-top-search-dropdown-noresult').hide();
                $instance.$top.find(".stec-top-search-results").empty();
            },

            escapeHtml: function (string) {

                var entityMap = {
                    "&": "&amp;",
                    "<": "&lt;",
                    ">": "&gt;",
                    '"': '&quot;',
                    "'": '&#39;',
                    "/": '&#x2F;'
                };

                return String(string).replace(/[&<>"'\/]/g, function (s) {
                    return entityMap[s];
                });
            },

            getResults: function (keyword) {

                var parent = this;
                var keywordArray, WORD, html;
                var dateData, searchDate, dateString;
                var uniqueResults;
                var result;

                parent.resetResults();

                keyword = $.trim(keyword);

                if (keyword === "") {
                    return;
                }

                keywordArray = keyword.split(" ");

                WORD = {
                    generic: false,
                    date: {
                        year: false,
                        month: false,
                        day: false
                    }
                };

                keywordArray = $.grep(keywordArray, function (el, index) {
                    return index === $.inArray(el, keywordArray);
                });

                $(keywordArray).each(function () {

                    // Start as Generic word
                    var generic = true;
                    var monthName;

                    if (parent.isYear(this)) {
                        WORD.date.year = this;
                        generic = false;
                    }

                    if (parent.isMonthString(this)) {
                        monthName = this;

                        $(glob.options.monthLabels).each(function (monthNumber) {
                            if (this.toLowerCase() === monthName.toLowerCase()) {
                                WORD.date.month = monthNumber;
                                generic = false;
                            }
                        });

                        if (WORD.date.month === false) {
                            // check shortname
                            $(glob.options.monthLabelsShort).each(function (monthNumber) {
                                if (this.toLowerCase() === monthName.toLowerCase()) {
                                    WORD.date.month = monthNumber;
                                    generic = false;
                                }
                            });
                        }
                    }

                    if (parent.isDay(this)) {
                        WORD.date.day = this;
                        generic = false;
                    }

                    // NO Func Match Found
                    if (generic === true) {
                        WORD.generic = true;
                    }

                });

                html = '';

                // Not a generic word
                if (WORD.generic === false) {
                    if (WORD.date.year !== false || WORD.date.month !== false || WORD.date.day !== false) {

                        if (WORD.date.year === false) {
                            WORD.date.year = glob.options.year;
                        }
                        if (WORD.date.month === false) {
                            WORD.date.month = glob.options.month;
                        }
                        if (WORD.date.day === false) {
                            WORD.date.day = glob.options.day;
                        }

                        // Suggest navigate to Date

                        dateData = WORD.date.year + "-" + WORD.date.month + "-" + WORD.date.day;
                        searchDate = new Date(WORD.date.year, WORD.date.month, WORD.date.day);

                        dateString = helper.beautifyTimespan(searchDate, searchDate, 1);

                        html = '<li data-jumpDate="' + dateData + '"><i class="fas fa-calendar-check"></i> <p>' + dateString + '</p></li>';
                        $(html).appendTo($instance.$top.find(".stec-top-search-results"));

                    }

                } else {
                    // Generic
                    // Search for keywords

                    result = [];

                    $(keywordArray).each(function () {

                        var word = this;
                        var eventsPool;

                        if (word.length <= 2) {
                            return true;
                        }

                        word = word.toLowerCase();

                        eventsPool = [];

                        eventsPool = calData.eventsPool;

                        $(eventsPool).each(function () {

                            var summary, location, keyWords, closest;

                            if (calData.calendarFilter.indexOf(parseInt(this.calid, 10)) === -1) {
                                return; // continue loop
                            }

                            summary = this.title.toLowerCase();
                            location = this.location.full_address ? this.location.full_address.toLowerCase() : '';
                            keyWords = this.keywords.toLowerCase();

                            if (keyWords.indexOf(word) > -1 || summary.indexOf(word) > -1 || location.indexOf(word) > -1) {
                                
                                if (this.rrule) {
                                    closest = calData.repeaterGetClosest(this);
                                    result.push(closest[0]);
                                } else {
                                    result.push(this);
                                }
                            }
                        });

                    });

                    if (result.length > 0) {
                        // found match

                        html = [];

                        uniqueResults = [];

                        $.each(result, function () {
                            var th = this;
                            var found = false;
                            $.each(uniqueResults, function () {
                                if (th.id === this.id) {
                                    found = true;
                                    return false; // break;
                                }
                            });

                            if (false === found) {
                                uniqueResults.push(th);
                            }
                        });

                        result = uniqueResults;

                        $(result).each(function () {

                            var permalink = helper.getPermalink(this.permalink, this.repeat_time_offset);

                            html.push('<li data-jumpsingle><a href="' + permalink + '"><i class="' + this.icon + '"></i><p>' + this.title + '</p></a></li>');
                        });

                        $(html.join('')).appendTo($instance.$top.find(".stec-top-search-results"));

                    } else {
                        // no match found
                        $instance.$top.find('.stec-top-search-dropdown-noresult').show();
                    }
                }
            },

            isMonthString: function (keyword) {
                var found = false;
                if (isNaN(keyword)) {

                    if ($.inArray(keyword.toLowerCase(), glob.options.monthLabels) !== -1) {
                        found = true;
                    }

                    if ($.inArray(keyword.toLowerCase(), glob.options.monthLabelsShort) !== -1) {
                        found = true;
                    }

                }

                return found;
            },

            isDay: function (keyword) {
                var found = false;
                if (!isNaN(keyword)) {
                    // numbers
                    if (keyword.length <= 2) {
                        if (keyword <= 31 && keyword > 0) {
                            // use as day
                            found = true;
                        }
                    }
                }
                return found;
            },

            isYear: function (keyword) {
                var found = false;
                if (!isNaN(keyword)) {
                    // numbers
                    if (keyword.length === 4 && keyword >= 1800 && keyword <= 2200) {
                        // use as year
                        found = true;
                    }
                }
                return found;
            }
        };

        search.init();

    });

})(window.jQuery);