(function ($) {

    "use strict";

    var moment = window.moment;
    moment.locale('en');

    $(document).on('onEventToggleOpen', function (e, data) {

        var $container, $inner, template;

        if (data.event.products.length <= 0) {
            return;
        }

        $container = data.$container;
        $inner = $container.find('.stec-layout-event-inner-shop');

        if ($inner.length <= 0 || !$inner.find('.stec-layout-event-inner-shop-product-template')[0]) {
            return;
        }

        template = $inner.find('.stec-layout-event-inner-shop-product-template')[0].outerHTML;
        $inner.find('.stec-layout-event-inner-shop-product-template').remove();

        $(data.event.products).each(function () {

            var th = this;
            var $product, $link;

            $(template).html(function (index, html) {

                var price = th.html_price;
                var quantity = th.stock_quantity === null ? '-' : th.stock_quantity;

                if (th.has_child) {
                    price += '<a href="' + th.url + '">' + window.stecLang.select_options + '</a>';
                }

                return html
                        .replace(/stec_replace_product_image/g, th.image)
                        .replace(/stec_replace_product_title/g, '<a href="' + th.url + '">' + th.title + '</a>')
                        .replace(/stec_replace_product_short_desc/g, th.post_data.excerpt)
                        .replace(/stec_replace_product_quantity/g, quantity)
                        .replace(/stec_replace_product_price/g, price);

            })
                    .removeClass('stec-layout-event-inner-shop-product-template')
                    .appendTo($inner.find('.stec-layout-event-inner-shop-products'));

            // get last appended
            $product = $inner.find('.stec-layout-event-inner-shop-product').last();

            $link = $product.find('.stec-layout-event-inner-shop-product-buy-addtocart');

            $link
                    .attr('data-pid', th.id)
                    .attr('data-quantity', '1')
                    .attr('data-sku', th.sku);

            if (th.is_in_stock !== true || th.purchesable !== true) {
                $product.find('.stec-layout-event-inner-shop-product-about-outofstock').css('display', 'inline-block');
                $product.find('.stec-layout-event-inner-shop-product-buy-addtocart').hide();
            }

            if (th.is_on_sale === true) {
                $product.find('.stec-layout-event-inner-shop-product-about-sale').css('display', 'inline-block');
            }

            if (th.is_featured === true) {
                $product.find('.stec-layout-event-inner-shop-product-about-featured').css('display', 'inline-block');
            }


        });

        // Remove tab preloaders
        $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
        $inner.find('.stec-layout-event-inner-preload-wrap').remove();
        $inner.find('.stec-preloader').remove();

    });

    $.stecExtend(function (master) {

        var container;
        var DB = new window.StecDB();

        if (master.glob.options.general_settings.open_event_in === 'modal') {
            container = '#stec-layout-event-inner-' + master.glob.options.id;
        } else {
            container = master.instance + ' .stec-layout-event.active .stec-layout-event-inner';
        }


        function addToCart(product, $button) {

            var request = DB.newRequest();

            request.base = master.glob.options.siteurl + '/?wc-ajax=add_to_cart';
            request.setData({
                product_id: product.id,
                product_sku: product.sku,
                quantity: product.quantity,
                stec_event_start_date: product.event_start_date
            });

            request.setBefore(function () {
                $button.find('.stec-layout-event-inner-shop-product-buy-addtocart').hide();
                $button.find('.stec-layout-event-inner-shop-product-buy-ajax-status').empty();
                $(master.glob.template.preloader).appendTo($button.find('.stec-layout-event-inner-shop-product-buy-ajax-status'));
            });

            request.setCallback(function (data) {
                
                var key, quantity;
                
                $button.find('.stec-layout-event-inner-shop-product-buy-ajax-status').empty();

                if (!data || data === null || (data.error && data.error === true)) {

                    // add success icon
                    $('<i class="fa fa-times"></i>').appendTo($button.find('.stec-layout-event-inner-shop-product-buy-ajax-status'));

                    // error handle
                    console.log('Error adding product to cart');

                } else {

                    // update fragments 
                    if (data.fragments) {
                        for (key in data.fragments) {
                            if (data.fragments.hasOwnProperty(key)) {
                                $(key).replaceWith(data.fragments[key]);
                            }
                        }
                    }

                    // add success icon
                    $('<i class="fa fa-check"></i>').appendTo($button.find('.stec-layout-event-inner-shop-product-buy-ajax-status'));

                    // decrement quantity

                    quantity = $button.parent()
                            .find('.stec-layout-event-inner-shop-product-quantity span').last().text();

                    if (!isNaN(quantity) && quantity > 0) {
                        quantity--;

                        $button.parent()
                                .find('.stec-layout-event-inner-shop-product-quantity span').last().text(quantity);
                    }

                    setTimeout(function () {
                        $button.find('.stec-layout-event-inner-shop-product-buy-ajax-status i').stop().fadeTo(1000, 0, function () {

                            $(this).remove();

                            if (!isNaN(quantity) && quantity > 0 || quantity === '-') {
                                $button.find('.stec-layout-event-inner-shop-product-buy-addtocart').show();
                            }
                        });
                    }, 3000);
                }
            });

            DB.query(request);
        }


        $(document).on(master.helper.clickHandle(), container + ' .stec-layout-event-inner-shop-product-buy-addtocart', function (e) {

            var eventId, repeatOffset, event, start, product;

            e.preventDefault();

            eventId = $(container).data('event-id');
            repeatOffset = $(container).data('repeat-time-offset');
            event = master.calData.getEventById(eventId);
            start = moment(event.start_date).
                    add(repeatOffset, 'seconds')
                    .utcOffset(event.timezone_utc_offset / 60, true)
                    .format('YYYY-MM-DD HH:mm:ss');

            if (event.start_date_timestamp_tz) {
                start = moment(event.start_date)
                        .utcOffset(moment().utcOffset(), true)
                        .add(repeatOffset, 'seconds')
                        .utcOffset(event.timezone_utc_offset / 60)
                        .format('YYYY-MM-DD HH:mm:ss');
            }

            product = {
                id: $(this).data('pid'),
                sku: $(this).data('sku'),
                quantity: $(this).data('quantity'),
                ajaxurl: $(this).data('ajaxurl'),
                event_start_date: start + ' ' + event.calendar.timezone
            };

            addToCart(product, $(this).parent());

        });

    });
})(window.jQuery);