(function ($) {

    "use strict";

    var moment = window.moment;
    moment.locale('en');

    $.stecExtend(function (master) {

        var helper = master.helper;
        var glob = master.glob;
        var instance = master.instance;
        var $instance = master.$instance;
        var calData = master.calData;
        var event;
        var repeatTimeOffset = 0;


        var tooltip = {

            init: function () {

                if (parseInt(glob.options.general_settings.tooltip_display, 10) !== 1) {
                    return;
                }

                if (helper.isMobile() === true) {
                    // don't bind on mobile
                    return;
                }

                this.iconImageSlider();

                this.bindControls();

            },

            bindControls: function () {

                var parent = this;

                var eventClass = [];
                eventClass.push(instance + ' .stec-layout-month-daycell-event');
                eventClass.push(instance + ' .stec-layout-week-daycell-event');

                if (parseInt(master.glob.options.agenda_tooltip, 10) === 1) {
                    eventClass.push(instance + ' .stec-layout-agenda-daycell-event');
                }

                eventClass = eventClass.join(',');

                $(document).on('mouseenter', eventClass, function (e) {

                    var th = this;

                    $('#tooltip-' + glob.options.id).remove();

                    if (helper.animate !== false) {
                        helper.animate.tooltip.clear();
                    }

                    $(glob.template.tooltip).attr('id', 'tooltip-' + glob.options.id).appendTo('body');

                    event = calData.getEventById($(th).data('id'));
                    repeatTimeOffset = $(th).data('repeatTimeOffset');

                    parent.fillHTML(event);

                    parent.position(th);

                    if (helper.animate !== false) {

                        helper.animate.tooltip.show($('#tooltip-' + glob.options.id));

                    } else {

                        $('#tooltip-' + glob.options.id).fadeTo(0, 1);

                    }

                });

                $(document).on('mouseleave', instance + eventClass, function (e) {

                    if (helper.animate !== false) {
                        helper.animate.tooltip.hide($('#tooltip-' + glob.options.id), function () {

                            $('#tooltip-' + glob.options.id).remove();

                        });

                    } else {

                        $('#tooltip-' + glob.options.id).fadeTo(0, 0);

                    }

                    parent.clock.clear();
                });

            },

            iconImageSlider: function () {

                setInterval(function () {

                    $('#tooltip-' + glob.options.id).find('.stec-layout-event-preview-left-icon-images').filter(':visible').each(function () {

                        var currentImage, nextImage;

                        if ($(this).children().length <= 1) {
                            return;
                        }

                        currentImage = $(this).children().filter('.active');
                        nextImage = currentImage.next();

                        if (nextImage.length === 0) {
                            nextImage = $(this).children().first();
                        }

                        nextImage.addClass('transition');

                        setTimeout(function () {
                            currentImage.removeClass('active');
                            nextImage.removeClass('transition').addClass('active');

                        }, 1000);

                    });

                }, 6000);
            },

            position: function (th) {

                var klass = '';

                var $tooltip = $('#tooltip-' + glob.options.id);

                $tooltip.css({
                    left: function () {

                        var x = 0;

                        if ($(th).hasClass('stec-layout-agenda-daycell-event')) {
                            x = $(th).width() + $(th).offset().left + 15;
                        } else if ($(th).parents('td').first().index() > 3) {
                            x = $(th).offset().left - $tooltip.width() - 5;
                            klass += ' stec-tooltip-pos-left';
                        } else {
                            x = $(th).width() + $(th).offset().left + 5;
                        }

                        return x;
                    },
                    top: function () {

                        var y = 0;

                        if ($(th).hasClass('stec-layout-agenda-daycell-event')) {

                            y = $(th).offset().top - 23;

                        } else if ($(th).parents('tr').first().index() > 3) {

                            klass += ' stec-tooltip-pos-top';

                            y = $instance.hasClass('stec-media-small') ? $(th).offset().top - $tooltip.height() + 29 : $(th).offset().top - $tooltip.height() + 40;

                        } else {

                            y = $instance.hasClass('stec-media-small') ? $(th).offset().top - 27 : $(th).offset().top - 15;

                        }


                        return y;
                    }
                });

                $tooltip.addClass(klass);
            },

            fillHTML: function (event) {

                var $tooltip = $('#tooltip-' + glob.options.id);
                var icon = '';
                var d1, iconHtml, startDate, endDate, date, imageHtml;
                var eventState = helper.getEventState(event, repeatTimeOffset);

                switch (event.icon_type) {

                    case 'image' :

                        if (event.icon_images.length > 0) {

                            icon += '<div class="stec-layout-event-preview-left-icon-images">';

                            $.each(event.icon_images_meta, function (i) {
                                icon += '<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>';
                            });


                            icon += '</div>';

                        } else {
                            icon = '<i class="' + event.icon + '"></i>';
                        }

                        break;

                    case 'date' :
                        d1 = moment(event.start_date);

                        icon = '<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + d1.format('MMM') + '</span></span>';

                        break;

                    default:
                        icon = '<i class="' + event.icon + '"></i>';

                }

                iconHtml = '<div class="stec-tooltip-icon" style="background:' + event.color + ' ">' + icon + '</div>';

                // original date + the repeat offset
                startDate = moment.tz(event.start_date, event.timezone).add(repeatTimeOffset, 'seconds');

                // original date + the repeat offset
                endDate = moment.tz(event.end_date, event.timezone).add(repeatTimeOffset, 'seconds');

                date = helper.beautifyTimespan(startDate, parseInt(event.hide_end, 10) === 1 ? false : endDate, event.all_day);

                imageHtml = '';

                if (event.images_meta && event.images_meta.length > 0) {
                    imageHtml = '<div style="background-image: url(' + event.images_meta[0].src + ');"  ></div>';
                }

                $tooltip.html(function (index, html) {

                    return html
                            .replace(/stec_replace_image/g, imageHtml)
                            .replace(/stec_replace_summary/g, event.title)
                            .replace(/stec_replace_desc_short/g, event.description_short)
                            .replace(/stec_replace_icon/g, iconHtml)
                            .replace(/stec_replace_location/g, function () {
                                if (event.location.title) {
                                    return [event.location.title, event.location.full_address].join(', ');
                                }

                                return event.location.full_address;
                            })
                            .replace(/stec_replace_timespan/g, date);

                });

                if (!event.location.full_address) {
                    $tooltip.find('.stec-tooltip-location').hide();
                }

                if (imageHtml === '') {
                    $tooltip.find('.stec-tooltip-image').hide();
                    $tooltip.find('.stec-tooltip-icon').css({
                        top: 0,
                        position: 'static',
                        marginTop: 20
                    });
                }


                if (eventState === 2) {
                    $tooltip.find('.stec-tooltip-expired').css('display', 'inline-block');
                }

                if (eventState === 1) {
                    $tooltip.find('.stec-tooltip-progress').css('display', 'inline-block');
                }

                // Check counter
                if (parseInt(event.counter, 10) !== 0 && eventState === 0) {
                    $tooltip.find('.stec-tooltip-counter').css('display', 'inline-block');
                    this.clock.init(startDate);
                }

                helper.extendBind("stachethemes_ec_extend", "filterTooltipHtml", {
                    event: event,
                    repeat_offset: repeatTimeOffset,
                    tooltip: $tooltip
                });

            },

            clock: {
                days: 0,
                hours: 0,
                minutes: 0,
                seconds: 0,
                daysLabel: window.stecLang.DaysAbbr,
                hoursLabel: window.stecLang.HoursAbbr,
                minutesLabel: window.stecLang.MinutesAbbr,
                secondsLabel: window.stecLang.SecondsAbbr,
                interval: '',
                init: function (momentDate) {

                    var nowDate, timeLeft;

                    this.clear();

                    nowDate = moment();
                    timeLeft = Math.floor((momentDate.unix() - nowDate.unix()));
                    this.days = Math.floor(timeLeft / 86400);
                    this.hours = Math.floor(timeLeft % 86400 / 3600);
                    this.minutes = Math.floor(timeLeft % 86400 % 3600 / 60);
                    this.seconds = Math.floor(timeLeft % 86400 % 3600 % 60);

                    if (timeLeft < 0) {
                        this.complete();
                        return;
                    }

                    this.count();
                },
                setTimer: function () {

                    var $tooltip = $('#tooltip-' + glob.options.id);
                    var countText = this.days + this.daysLabel + ' ' + this.hours + this.hoursLabel + ' ' + this.minutes + this.minutesLabel + ' ' + this.seconds + this.secondsLabel;
                    $tooltip.find('.stec-tooltip-counter span').text(countText);

                },
                count: function () {

                    var parent = this;

                    parent.setTimer();

                    parent.interval = setInterval(function () {

                        parent.seconds--;

                        if (parent.seconds < 0) {
                            parent.seconds = 59;
                            parent.minutes--;
                            if (parent.minutes < 0) {
                                parent.minutes = 59;
                                parent.hours--;
                                if (parent.hours < 0) {
                                    parent.hours = 23;
                                    if (parent.days > 0) {
                                        parent.days--;
                                    }
                                }
                            }
                        }

                        parent.setTimer();

                        if (parent.days === 0 && parent.hours === 0 && parent.minutes === 0 && parent.seconds === 0) {
                            parent.clear();
                            parent.complete();
                        }

                    }, 1000);
                },

                complete: function () {
                    this.clear();
                },

                clear: function () {

                    clearInterval(this.interval);
                }

            }
        };

        tooltip.init();

    });


})(window.jQuery);