(function ($) {

    'use strict';
    var proto;
    var moment = window.moment;
    var DB = new window.StecDB();
    const __ = new window.StecHelper().lang;

    moment.locale('en');

    window.StecBooking = function (id) {

        this.settings = {
            popup: '#' + id,
            offset: null,
            event: null,
            glob: null,
            helper: null,
            $instance: null
        };

        this.blockAction = false;

    };

    proto = window.StecBooking.prototype;

    proto.set = function (obj, data) {

        if (typeof this.settings[obj] !== 'undefined') {
            this.settings[obj] = data;
        }

        return this;

    };

    proto.getFormattedPrice = function (value) {

        var generalSettings, priceFormat;

        if (window.stecSingleOptions) {
            generalSettings = window.stecSingleOptions;
        } else {
            generalSettings = this.settings.glob.options.general_settings;
        }

        priceFormat = generalSettings.woocommerce.price_format;

        value = Math.max(0, value).toFixed(generalSettings.woocommerce.decimals);
        value = value.toString().replace('.', generalSettings.woocommerce.decimals_separator);

        value = value.replace(/\B(?=(\d{3})+(?!\d))/g, generalSettings.woocommerce.thousands_separator);
        value = priceFormat
                .replace('%2$s', value)
                .replace('%1$s', generalSettings.woocommerce.symbol);

        return value;


    };

    proto.updateSummary = function () {

        var $popup = $(this.settings.popup);
        var total = 0;

        $.each($popup.find('.stec-booking-item'), function () {
            var quantity = parseFloat($(this).find('.stec-booking-item-quantity').val());
            var value = parseFloat($(this).find('.stec-booking-item-price').data('price'));

            if (isNaN(quantity) || isNaN(value)) {
                return true; // continue
            }

            total += Math.max(0, quantity * value);

        });

        $popup.find('.stec-booking-summary-total span').html(this.getFormattedPrice(total));
    };

    proto.getProductAttributesHtml = function (attribs) {

        var html = [];
        if (attribs.length > 0) {
            html.push('     <div class="stec-style-full-width">');
            html.push('         <span class="stec-booking-item-additional-information">Additional information</span>');
            html.push('     </div>');
            $.each(attribs, function () {
                var attributeOptions = [];
                attributeOptions = this.value.split('|');
                html.push('     <div class="stec-style-full-width stec-style-flexbox">');
                html.push('         <div class="stec-style-full-width">');
                html.push('             <span>' + this.name + '</span>');
                html.push('         </div>');
                html.push('         <div class="stec-style-full-width">');
                html.push('             <span>' + attributeOptions.join(', ') + '</span>');
                html.push('         </div>');
                html.push('     </div>');
            });
        }

        return html.join('');
    };

    proto.getTicketsLeft = function (product) {

        var parent = this;
        var ticketsLeft = 0;
        var ticketsSold = 0;
        var qtyPerEvent = parseInt(product.qty_limit_per_event, 10);
        var stockQty = parseInt(product.stock_quantity, 10);
        var availableQty = 0;

        if (product.tickets_sold[parent.settings.offset]) {
            ticketsSold = parseInt(product.tickets_sold[parent.settings.offset], 10);
        }

        if (false === isNaN(qtyPerEvent) && false === isNaN(stockQty)) {
            availableQty = Math.min(stockQty, qtyPerEvent);
        } else if (false === isNaN(qtyPerEvent) && true === isNaN(stockQty)) {
            availableQty = qtyPerEvent;
        } else if (true === isNaN(qtyPerEvent) && false === isNaN(stockQty)) {
            availableQty = stockQty;
        } else if (true === isNaN(qtyPerEvent) && true === isNaN(stockQty)) {
            availableQty = 9999;
        }

        if (9999 === availableQty) {
            ticketsLeft = 9999;
        } else {
            ticketsLeft = Math.max(0, availableQty - (qtyPerEvent ? ticketsSold : 0));
        }

        return ticketsLeft;
    };

    proto.getProductsHtml = function (event) {
        var parent = this;
        var html = [];

        var nowDate = moment();
        var startDate = moment.tz(event.start_date, event.timezone)
                .add(this.settings.offset, 'seconds');

        $.each(event.bookable_products, function () {

            var ticketsLeft = parent.getTicketsLeft(this);
            var ticketsLeftDisplay = ticketsLeft >= 9999 ? __('Unlimited') : ticketsLeft;
            var stopPurchase = false;
        
            var salesDate = startDate.subtract(this.stop_before, 'seconds');
            var salesDateDisplay = parent.settings.helper.beautifyDate(salesDate);

            if (nowDate.unix() > salesDate.unix()) {
                stopPurchase = true;
            }
            
            if (stopPurchase) {
                ticketsLeft = 0;
                ticketsLeftDisplay = 0;
            }

            html.push('<div class="stec-booking-item" data-id="' + this.id + '">');
            html.push('     <div class="stec-style-full-width stec-style-flexbox">');
            html.push('         <div class="stec-style-full-width">');
            html.push('             <span class="stec-booking-item-title stec-style-title">' + this.title + '</span>');

            if (parseInt(this.price, 10) === 0) {
                this.html_price = __('Free');
            }

            html.push('             <span class="stec-booking-item-price stec-style-title" data-price="' + this.price + '">' + this.html_price + '</span>');
            html.push('         </div>');
            html.push('         <div>');
            if (true === this.is_in_stock && true === this.is_purchesable && false === stopPurchase) {
                html.push('         <input class="stec-booking-item-quantity stec-style-input" type="number" value="0" name="quantity" min="0" max="' + ticketsLeft + '" />');
            } else {
                html.push('         <span class="stec-style-text stec-booking-item-out-of-stock">' + __('SOLD OUT') + '</span>');
            }
            html.push('         </div>');
            html.push('     </div>');

            html.push('     <div class="stec-style-full-width">');
            html.push('         <span class="stec-booking-item-until stec-style-text">' + __('Sales end on') + ' ' + salesDateDisplay + '</span>');
            html.push('     </div>');

            html.push('     <div class="stec-style-full-width">');
            html.push('         <span class="stec-booking-item-tickets-left">' + __('Tickets left:') + ' <span>' + ticketsLeftDisplay + '</span></span>');
            html.push('     </div>');

            html.push('     <div class="stec-booking-item-desc stec-style-text">' + this.short_description + '</div>');
            html.push(parent.getProductAttributesHtml(this.attributes));
            html.push('</div>');
        });

        return html.join('');
    };

    proto.popHtml = function () {

        var backgroundImage = '',
                html = [],
                event = this.settings.event,
                startDate,
                endDate;

        startDate = moment.tz(event.start_date, event.timezone).add(this.settings.offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
        endDate = moment.tz(event.end_date, event.timezone).add(this.settings.offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');

        if (event.images_meta.length > 0) {
            backgroundImage = event.images_meta[0].src;
        }

        html.push('<div id="' + this.settings.popup.replace('#', '') + '" class="stec-booking-popup-background stec-modal">');
        html.push('     <div class="stec-booking-popup">');
        if (backgroundImage) {
            html.push('     <div class="stec-booking-popup-header" style="background-image:url(' + backgroundImage + ')"></div>');
        }
        html.push('         <div class="stec-booking-popup-wrapper">');
        html.push('         <div class="stec-booking-popup-wrapper-block-action"></div>');
        html.push('             <div class="stec-booking-section">');
        html.push('                 <span class="stec-style-title">' + event.title + '</span>');
        html.push('                 <span class="stec-booking-event-date stec-style-text">' + this.settings.helper.beautifyTimespan(startDate, endDate, event.all_day) + '</span>');
        html.push('             </div>');
        html.push('             <input class="stec-style-input" type="text" name="discount" value="" placeholder="' + __('Discount Coupon') + '" />');

        html.push('             <div class="stec-scrollable">');
        html.push('                 <div class="stec-booking-items">');
        html.push(this.getProductsHtml(event));

        html.push('                 </div>');
        html.push('             </div>');
        html.push('             <div class="stec-style-flexbox stec-booking-popup-summary">');
        html.push('                 <span class="stec-booking-summary-total stec-style-title">' + __('Total') + ': <span>' + this.getFormattedPrice(0) + '</span></span>');
        html.push('                 <button class="stec-style-button" type="button">' + __('Checkout') + '</button>');
        html.push('             </div>');
        html.push('         </div>');
        html.push('         <i class="stec-booking-popup-close fa fa-times"></i>');
        html.push('     </div>');
        html.push('</div>');

        return html.join('');

    };

    proto.popOpen = function () {
        var popup,
                scroller;

        $('body').addClass('stec-modal-opened stec-no-scroll');

        $('#stec-layout-event-inner-out-wrapper-' + this.settings.glob.options.id).hide();
        popup = $(this.popHtml()).appendTo('body');

        if (!popup.find('.stec-scrollable').data('stec-scroll')) {
            scroller = new window.StecScroll();
            scroller.create(popup.find('.stec-scrollable'));
            $(this).find('.stec-scrollable').data('stec-scroll', scroller);
        }
    };

    proto.popClose = function () {

        this.settings.offset = null;
        this.settings.event = null;
        this.settings.date = null;
        this.blockAction = false;

        $(this.settings.popup).remove();

        $('body').removeClass('stec-no-scroll');

        if ($('body > .stec-modal').length <= 0) {
            $('body').removeClass('stec-modal-opened');
        }

        $('#stec-layout-event-inner-out-wrapper-' + this.settings.glob.options.id).show();

    };

    proto.addToCart = function (items, coupon, callback) {
        var parent = this;
        var $popup = $(this.settings.popup);
        var request = DB.newRequest();
        request.setData({
            action: 'stec_public_ajax_action',
            task: 'booking_add_to_cart',
            data: {
                items: items,
                coupon: coupon
            }
        });

        request.setBefore(function () {
            $popup.find('.stec-booking-popup-error').remove();
            $popup.find('.stec-booking-popup-wrapper-block-action').show();
            $(parent.settings.glob.template.preloader).appendTo($popup.find('.stec-booking-popup-wrapper'));
        });

        request.setCallback(function (result) {
            $popup.find('.stec-preloader, .stec-layout-single-preloader').remove();
            $popup.find('.stec-booking-popup-wrapper-block-action').hide();

            if (typeof callback === 'function') {
                callback(result);
            }
        });

        DB.query(request);
    };

    proto.displayErrorMessage = function (msg) {
        var $popup = $(this.settings.popup);
        $popup.find('.stec-booking-popup-error').remove();
        $('<span class="stec-booking-popup-error">' + msg + '</span>').insertAfter($popup.find('.stec-booking-popup-summary'));
    };

    proto.checkOut = function () {

        var parent = this;
        var helper = this.settings.helper;
        var event = this.settings.event;
        var offset = this.settings.offset;
        var eventStartDate = helper.beautifyDate(moment.tz(event.start_date, event.timezone).add(offset, 'seconds'));
        var eventEndDate = helper.beautifyDate(moment.tz(event.end_date, event.timezone).add(offset, 'seconds'));
        var $popup = $(this.settings.popup);
        var items = [];
        var coupon = $.trim($popup.find('input[name="discount"]').val());

        $.each($popup.find('.stec-booking-item'), function () {
            var quantity = Math.max(0, parseInt($(this).find('.stec-booking-item-quantity').val(), 10));
            var item;

            if (quantity > 0) {
                item = {
                    id: $(this).data('id'),
                    quantity: quantity,
                    variables: {
                        event_id: event.id,
                        event_title: event.title,
                        event_offset: offset,
                        event_start: eventStartDate,
                        event_end: eventEndDate,
                        event_permalink: helper.getPermalink(event.permalink, parent.settings.offset)
                    }
                };

                items.push(item);
            }

        });

        if (items.length > 0) {
            this.addToCart(items, coupon, function (result) {

                if (result.error) {
                    if (result.msg) {
                        parent.displayErrorMessage(result.msg);
                    }
                } else {
                    if (result.cart) {
                        window.open(result.cart, '_self');
                    }
                }
            });
        } else {
            this.displayErrorMessage(__('No tickets selected'));
        }

    };

    proto.controller = function () {

        var parent = this;
        var bookingSelectors = [];

        $(document).on(this.settings.helper.clickHandle(), this.settings.popup + ' .stec-booking-popup', function (e) {
            e.stopPropagation();
        });

        $(document).on(this.settings.helper.clickHandle(), this.settings.popup, function (e) {
            parent.popClose();
        });

        $(document).on(this.settings.helper.clickHandle(), this.settings.popup + ' .stec-booking-popup-close', function (e) {
            parent.popClose();
        });

        $(document).on('input', parent.settings.popup + ' .stec-booking-item-quantity', function (e) {

            if ($(this).val() < 0) {
                $(this).val(0);
            }

            if ($(this).val() > parseInt($(this).prop('max'), 10)) {
                $(this).val(parseInt($(this).prop('max'), 10));
            }

            parent.updateSummary();
        });

        bookingSelectors.push(parent.settings.$instance.selector + ' button[data-action="open-tickets"]');
        bookingSelectors.push('#stec-layout-event-inner-' + parent.settings.glob.options.id + ' button[data-action="open-tickets"]');

        // Booking open
        $(document).on(this.settings.helper.clickHandle(), bookingSelectors.join(','), function (e) {
            e.preventDefault();

            parent.settings.event = $(this).data('event');
            parent.settings.offset = parseInt($(this).data('offset'), 10);
            parent.settings.date = moment(parent.settings.event.start_date).add(parent.settings.offset, 'seconds').format('YYYY-MM-DD');

            parent.popOpen();
        });

        $(document).on(this.settings.helper.clickHandle(), parent.settings.popup + ' .stec-booking-popup-summary button', function (e) {
            e.preventDefault();
            parent.checkOut();
        });
    };


})(window.jQuery);