(function ($) {

    "use strict";

    var __ = new window.StecHelper().lang;
    var moment = window.moment;
    moment.locale('en');

    function StachethemesBuilderSubmissionForm() {

        var DB = null;
        var helper = null;
        var captchaContainer = null;
        var instance = '',
                $instance = '',
                parent = this;

        var dtPickerFormat = null;

        var glob = {
            options: {}
        };

        function dtInFormat(dt) {
            var dtFormat = '';

            switch (glob.options.date_format) {
                case 'dd-mm-yy' :
                    dtFormat = 'DD-MM-Y';
                    break;
                case 'mm-dd-yy' :
                    dtFormat = 'MM-DD-Y';
                    break;
                case 'yy-mm-dd' :
                    dtFormat = 'Y-MM-DD';
                    break;
                case 'dd.mm.yyyy' :
                    dtFormat = 'DD.MM.Y';
                    break;
                default:
                    dtFormat = glob.options.date_format;
            }

            return moment(dt).format(dtFormat);
        }

        this.captcha = {
            valid: false,

            set: function () {
                // Check privacy Agreement enabled
                if (parseInt(glob.options.display_privacy_agreement, 10) === 0 || $instance.find('.stec-builder-element-content-privacy-style').children().length <= 0) {
                    $instance.find('.stec-builder-element-content-privacy-style').parent().remove();
                }

                // Check captcha enabled
                if (parseInt(glob.options.captcha.enabled, 10) === 0 || $instance.find('.stec-builder-element-content-captcha-style').length <= 0) {
                    $instance.find('.stec-builder-element-content-captcha-style').parent().remove();
                    parent.captcha.valid = true;
                    return;
                }

                this.load();
            },

            load: function () {
                if ($instance.find('.stec-builder-element-content-captcha-style div').children().length <= 0) {
                    if (typeof window.grecaptcha.render === 'undefined') {
                        setTimeout(function () {
                            parent.captcha.load();
                        }, 100);
                        return;
                    }

                    captchaContainer = window.grecaptcha.render($instance.find('.stec-builder-element-content-captcha-style div').get(0), {
                        sitekey: glob.options.captcha.site_key,
                        callback: function (response) {
                            if (response.length !== 0) {
                                parent.captcha.valid = true;
                            }
                        }
                    });
                }
            }
        };

        this.colorPicker = {

            set: function () {

                // Remove old colorpickers
                $('.colorpicker-' + glob.options.id).remove();

                $instance.find('.stec-builder-element-content-colorpicker-style').each(function (i) {

                    var th = this;

                    var color = $(th).val();

                    $(th).css({
                        backgroundColor: color
                    });

                    $(th).StecColorPicker({
                        klass: 'colorpicker-' + glob.options.id,
                        id: 'colorpicker-' + i + '-' + glob.options.id,
                        color: color,
                        onShow: function (colpkr) {
                            $(colpkr).show();
                            return false;
                        },
                        onHide: function (colpkr) {
                            $(colpkr).hide();
                            return false;
                        },
                        onChange: function (hsb, hex, rgb) {
                            $(th).attr("title", "#" + hex);

                            $(th).css({
                                backgroundColor: "#" + hex
                            });

                            $(th).val("#" + hex);
                        }
                    });
                });
            }
        };

        this.datePicker = {

            set: function () {

                var now = [
                    glob.options.year,
                    glob.options.month + 1,
                    glob.options.day
                ].join('-');

                var startDate = window.moment(now, 'YYYY-M-D');
                var endDate = window.moment(now, 'YYYY-M-D');

                var minDate = window.moment();
                var hoursAndMinutesSelectors;

                if ($instance.find('[name="start_date"]').val() !== '') {
                    startDate = window.moment($instance.find('[name="start_date"]').val());
                }

                if ($instance.find('[name="end_date"]').val() !== '') {
                    endDate = window.moment($instance.find('[name="end_date"]').val());
                }

                if (parseInt(glob.options.create_event_form_allow_expired, 10) === 1 || glob.options.edit_event) {
                    minDate = null;
                }

                $instance.find('[name="start_date"]')
                        .datepicker("destroy")
                        .datepicker({
                            showAnim: 0,
                            minDate: minDate ? minDate.toDate() : null,
                            dateFormat: dtPickerFormat,
                            altFormat: "yy-mm-dd",
                            altField: $instance.find('[name="start_date_alt"]').selector,
                            onSelect: function () {
                                $instance.find('[name="end_date"]').datepicker('option', 'minDate', $(this).val());
                                parent.datePicker.generalTimeFix();
                            }
                        })
                        .datepicker("setDate", startDate.toDate());

                $instance.find('[name="end_date"]')
                        .datepicker("destroy")
                        .datepicker({
                            showAnim: 0,
                            minDate: minDate ? minDate.toDate() : null,
                            dateFormat: dtPickerFormat,
                            altFormat: "yy-mm-dd",
                            altField: $instance.find('[name="end_date_alt"]').selector,
                            onSelect: function () {
                                $instance.find('[name="start_date"]').datepicker('option', 'maxDate', $(this).val());
                                parent.datePicker.generalTimeFix();
                            }
                        })
                        .datepicker("setDate", endDate.toDate());

                $instance.find('[name="repeat_ends_on_date"]')
                        .datepicker("destroy")
                        .datepicker({
                            dateFormat: dtPickerFormat,
                            altFormat: "yy-mm-dd",
                            minDate: $instance.find('[name="end_date"]').val(),
                            altField: $instance.find('[name="repeat_ends_on_date_alt"]').selector
                        });

                $instance.find('[name="repeat_exdate_datepicker"]')
                        .datepicker("destroy")
                        .datepicker({
                            dateFormat: dtPickerFormat,
                            altFormat: "yy-mm-dd",
                            altField: $instance.find('[name="repeat_exdate_datepicker_alt"]').selector
                        });

                hoursAndMinutesSelectors = [
                    '[name="start_time_hours"]',
                    '[name="end_time_hours"]',
                    '[name="start_time_minutes"]',
                    '[name="end_time_minutes"]']
                        .join(',');

                $instance.find(hoursAndMinutesSelectors).on('change', function () {
                    parent.datePicker.generalTimeFix();
                });
            },

            generalTimeFix: function () {

                var $startDate = $instance.find('[name="start_date"]');
                var $endDate = $instance.find('[name="end_date"]');

                var $startHours = $instance.find('[name="start_time_hours"]');
                var $endHours = $instance.find('[name="end_time_hours"]');

                var $startMinutes = $instance.find('[name="start_time_minutes"]');
                var $endMinutes = $instance.find('[name="end_time_minutes"]');
                var start, end;

                start = new Date($startDate.val());
                start.setHours($startHours.val());
                start.setMinutes($startMinutes.val());

                end = new Date($endDate.val());
                end.setHours($endHours.val());
                end.setMinutes($endMinutes.val());

                if (start.getTime() > end.getTime()) {
                    $endHours.val($startHours.val());
                    $endMinutes.val($startMinutes.val());
                }

                $instance.find('[name="repeat_ends_on_date"]')
                        .datepicker('option', 'minDate', $endDate.val());

                $instance.find('.stec-builder-submission-schedule')
                        .find('.stec-builder-element-content-datetime-style')
                        .datepicker('option', {
                            minDate: $startDate.val(),
                            maxDate: $endDate.val()
                        });
            }
        };

        this.repeater = {
            init: function () {

                var exdateTemplate = $instance.find('.stec-repeater-content-exdate-datelist-template')
                        .removeClass('stec-repeater-content-exdate-datelist-template');

                this.template.exdate = exdateTemplate.get(0);

                exdateTemplate.remove();

                $instance.find('.stec-repeater-content-exdate-datelist-template').remove();

                this.controller();
            },

            template: {
                exdate: null
            },

            setRrule: function () {

                var freq, byWeekDay, interval, count, until, rrOptions = {}, theRule = '', rrule, exdate = [];

                if ($instance.find('select[name="repeater"]').length <= 0) {
                    return;
                }

                freq = $instance.find('select[name="repeater"]').val();
                byWeekDay = [];

                switch (freq) {
                    case '0' :
                        freq = false;
                        break;
                    case '1' :
                        freq = window.rrule.RRule.DAILY;
                        break;
                    case '2' :
                        freq = window.rrule.RRule.WEEKLY;

                        $instance.find('.stec-repeater-content-repeat-on')
                                .find('input[type="checkbox"]')
                                .filter(':checked')
                                .each(function () {
                                    byWeekDay.push(window.rrule.RRule[$(this).attr('name')]);
                                });

                        break;
                    case '3' :
                        freq = window.rrule.RRule.MONTHLY;
                        break;
                    case '4' :
                        freq = window.rrule.RRule.YEARLY;
                        break;
                }

                interval = $instance.find('select[name="repeat_gap"]').val();
                count = false;
                until = false;

                switch ($instance.find('input[name="repeat_endson"]').filter(":checked").val()) {

                    case '0' :
                        //never
                        count = false;
                        until = false;
                        break;

                    case '1' :
                        // after n
                        count = $instance.find('input[name="repeat_occurences"]').val();
                        until = false;

                        break;

                    case '2' :
                        // on date
                        count = false;
                        until = new Date($instance.find('input[name="repeat_ends_on_date_alt"]').val());
                        break;
                }

                rrOptions.freq = freq;

                if (count > 0) {
                    rrOptions.count = count;
                }

                if (until) {
                    rrOptions.until = until;
                }

                if (interval > 0) {
                    rrOptions.interval = interval;
                }

                if (byWeekDay.length > 0) {
                    rrOptions.byweekday = byWeekDay;
                }

                if (freq !== false) {
                    // Set rule
                    theRule = new window.rrule.RRule(rrOptions);


                } else {
                    // Empty rrule
                    theRule = new window.rrule.RRule();

                }

                rrule = theRule.toString();

                $instance.find('input[name="rrule"]').val(rrule);

                $instance.find('.stec-repeater-content-exdate-datelist').children().each(function () {
                    exdate.push($(this).data('exdate'));
                });

                $instance.find('input[name="exdate"]').val(exdate.join(','));

            },

            addExdate: function (date) {
                var exists = false, html;

                $instance.find('.stec-repeater-content-exdate-datelist').children().each(function () {
                    if ($(this).data('exdate') === date.format('YYYYMMDD')) {
                        exists = true;
                        return false; // break;
                    }
                });

                if (true === exists) {
                    return;
                }

                html = $(this.template.exdate).clone().html(function (index, html) {
                    return html.replace('stec_replace_date', dtInFormat(date));
                });

                $(html).data('exdate', date.format('YYYYMMDD'))
                        .appendTo($instance.find('.stec-repeater-content-exdate-datelist'));

            },

            controller: function () {

                var selectors = [
                    'select[name="start_time_hours"]',
                    'select[name="end_time_hours"]',
                    'select[name="start_time_minutes"]',
                    'select[name="end_time_minutes"]'
                ].join(', ');

                $instance.find('input[name="all_day"]').on('change', function () {

                    if ($(this).is(':checked')) {
                        $instance.find(selectors)
                                .val('00')
                                .css('opacity', 0.5)
                                .prop('disabled', true);
                    } else {
                        $instance.find(selectors)
                                .css('opacity', 1)
                                .prop('disabled', false);
                    }
                });

                $instance.find('.stec-repeater-content-exdate-datelist').on(helper.clickHandle(), '.stec-remove-exdate', function () {
                    $(this).parents('li').first().remove();
                });

                $instance.find('.stec-repeater-content-add-exdate-button').on(helper.clickHandle(), function () {
                    var date = moment($instance.find('[name="repeat_exdate_datepicker_alt"]').val(), 'YYYY-MM-DD');

                    if (true !== date.isValid()) {
                        return;
                    }

                    parent.repeater.addExdate(date);
                });

                $instance.find('input[name="repeat_endson"]').on('change', function () {
                    var val = parseInt($(this).val(), 10);

                    switch (val) {
                        case 0:
                            $instance.find('input[name="repeat_occurences"]').prop('disabled', true);
                            $instance.find('input[name="repeat_ends_on_date"]').prop('disabled', true);
                            break;

                        case 1:
                            $instance.find('input[name="repeat_occurences"]').prop('disabled', false);
                            $instance.find('input[name="repeat_ends_on_date"]').prop('disabled', true);
                            break;
                        case 2:
                            $instance.find('input[name="repeat_occurences"]').prop('disabled', true);
                            $instance.find('input[name="repeat_ends_on_date"]').prop('disabled', false);
                            break;

                    }
                });

                $instance.find('select[name="repeater"]').on('change', function () {
                    var val = parseInt($(this).val(), 10);

                    switch (val) {
                        case 0:
                            $instance.find('.stec-repeater-content').hide();
                            $instance.find('.stec-repeater-content-weekdays').hide();
                            break;
                        case 2:
                            $instance.find('.stec-repeater-content').show();
                            $instance.find('.stec-repeater-content-weekdays').show();
                            break;
                        default:
                            $instance.find('.stec-repeater-content').show();
                            $instance.find('.stec-repeater-content-weekdays').hide();
                    }
                });
            }
        };

        this.organizers = {
            template: null,
            blockAction: false,

            init: function () {
                if ($instance.find('.stec-builder-element-content-organizers-style').length <= 0) {
                    return;
                }

                this.template = $instance.find('.stec-builder-element-content-organizers-style')
                        .find('.stec-builder-submission-organizer-template')
                        .clone()
                        .removeClass('stec-builder-submission-organizer-template')
                [0].outerHTML;

                $instance.find('.stec-builder-element-content-organizers-style')
                        .find('.stec-builder-submission-organizer-template').remove();

            },

            arrangeSocLinks: function ($el) {
                $el.find('.stec-builder-submission-organizer-social').each(function (i) {
                    $(this).find("input, select").each(function () {
                        this.name = this.name.replace(/social\[\d\]/g, function (str, p1) {
                            return 'social[' + i + ']';
                        });
                    });
                });
            },

            validateFields: function () {
                var valid = true;

                var $container = $instance.find('.stec-builder-submission-organizer');

                var name = $.trim($container.find('input[name="name"]').val());

                if (!name) {
                    valid = false;
                }

                return valid;
            },

            insertNewOrganizer: function () {

                var $container, formData, request;

                if (false === this.validateFields() || true === this.blockAction) {
                    return;
                }

                $container = $instance.find('.stec-builder-submission-organizer');
                formData = new FormData();

                $container.find('input,select,textarea').each(function () {
                    formData.append($(this).attr('name'), $(this).val());
                });

                formData.append('action', 'stec_public_ajax_action');
                formData.append('task', 'insert_new_organizer');

                request = DB.newRequest();
                request.setData(formData);

                request.setBefore(function () {
                    parent.organizers.blockAction = true;
                    $container.find('.status').find('p').text($container.find('.status').data('default-text'));
                    $container.find('.status').show();
                    $container.find('[data-action="insert-organizer"]').hide();
                });

                request.setOnError(function (err) {
                    parent.organizers.blockAction = false;
                });

                request.setCallback(function (result) {

                    var $selector, html;

                    parent.organizers.blockAction = false;

                    $container.find('.status').hide();
                    $container.find('[data-action="insert-organizer"]').show();

                    if (result.error) {
                        $container.find('.status').find('p').text(result.error_msg);
                        $container.find('.status').show();
                        return;
                    }

                    if (result.data.success && result.data.organizer) {
                        $selector = $instance.find('select[name="organizers[]"]');
                        html = '<option selected value="' + result.data.organizer.id + '">' + result.data.organizer.name + '</option>';

                        $(html).appendTo($selector);

                        $selector.focus();

                        $container.find('.delete').trigger(helper.clickHandle());
                    }
                });

                DB.query(request);

            },

            controller: function () {

                parent.media.addImageController('.stec-builder-submission-organizer', {
                    multiple: false,
                    inputName: 'photo',
                    callback: function () {
                    }
                });

                $instance.find('.stec-builder-element-content-organizers-style').on(helper.clickHandle(),
                        '.stec-builder-submission-organizer-head button.collapse', function () {
                            var $organizer = $(this).parents('.stec-builder-submission-organizer').first();
                            $organizer.addClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-organizers-style').on(helper.clickHandle(),
                        '.stec-builder-submission-organizer-head button.expand', function () {
                            var $organizer = $(this).parents('.stec-builder-submission-organizer').first();
                            $organizer.removeClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-organizers-style').on(helper.clickHandle(),
                        '.stec-builder-submission-organizer-head button.delete', function () {
                            var $organizer = $(this).parents('.stec-builder-submission-organizer').first();
                            $organizer.remove();

                            $instance.find('.stec-builder-submission-add-organizer').show();
                        });

                $instance.find('.stec-builder-submission-add-organizer').on(helper.clickHandle(), function () {
                    $(parent.organizers.template).insertBefore(this);
                    $(this).hide();
                });

                $instance.find('.stec-builder-element-content-organizers-style').on(helper.clickHandle(),
                        '.add-organizers-soclink', function () {
                            var $organizer = $(this).parents('.stec-builder-submission-organizer').first();
                            var social = $organizer.find('.stec-builder-submission-organizer-social').first().clone()[0].outerHTML;
                            var $social = $(social).insertBefore($organizer.find('.add-organizers-soclink'));
                            $social.find('select, input').val(null);

                            parent.organizers.arrangeSocLinks($organizer);
                        });

                $instance.find('.stec-builder-element-content-organizers-style').on(helper.clickHandle(),
                        '.stec-builder-submission-organizer-social-remove', function () {
                            var $organizer = $(this).parents('.stec-builder-submission-organizer').first();
                            var $this = $(this).parents('.stec-builder-submission-organizer-social').first();

                            if ($organizer.find('.stec-builder-submission-organizer-social').length <= 1) {
                                $this.find('select, input').val(null);
                            } else {
                                $this.remove();
                            }

                            parent.organizers.arrangeSocLinks($organizer);
                        });

                $instance.on(helper.clickHandle(), '.stec-builder-submission-organizer [data-action="insert-organizer"]', function () {

                    parent.organizers.insertNewOrganizer();

                });
            }
        };

        this.guests = {
            template: null,

            init: function () {
                if ($instance.find('.stec-builder-element-content-guests-style').length <= 0) {
                    return;
                }

                this.template = $instance.find('.stec-builder-element-content-guests-style')
                        .find('.stec-builder-submission-guest-template')
                        .clone()
                        .removeClass('stec-builder-submission-guest-template')
                [0].outerHTML;

                $instance.find('.stec-builder-element-content-guests-style')
                        .find('.stec-builder-submission-guest-template').remove();

            },

            arrangeGuests: function () {
                $instance.find('.stec-builder-submission-guest').each(function (i) {
                    $(this).find("input, select, textarea").each(function () {
                        this.name = this.name.replace(/guests\[\d]/g, function (str, p1) {
                            return 'guests[' + i + ']';
                        });
                    });

                    $(this).find("[data-name]").each(function () {
                        $(this).attr('data-name', $(this).attr('data-name').replace(/guests\[\d]/g, function (str, p1) {
                            return 'guests[' + i + ']';
                        }));
                    });
                });
            },

            arrangeSocLinks: function ($el) {

                $el.find('.stec-builder-submission-guest-social').each(function (i) {
                    $(this).find("input, select").each(function () {
                        this.name = this.name.replace(/\[social]\[\d]/g, function (str, p1) {
                            return '[social][' + i + ']';
                        });
                    });
                });
            },

            addGuest: function (guest) {

                var $guest = $(this.template).insertBefore($instance.find('.stec-builder-submission-add-guest'));

                $guest.find("input[name^='guests['][name$='[name]']").val(guest.name);
                $guest.find("textarea[name^='guests['][name$='[about]']").val(guest.about);

                if (guest.photo) {
                    parent.media.fillImage($guest.get(0), {
                        inputName: 'guests[0][photo]',
                        image: {
                            id: guest.photo,
                            src: guest.photo_full
                        }
                    });
                }

                if (guest.links.length > 0) {
                    $.each(guest.links, function () {

                        var social, $social;

                        if (this.link !== '') {
                            social = $guest.find('.stec-builder-submission-guest-social').first().clone()[0].outerHTML;
                            $social = $(social).insertBefore($guest.find('.add-guests-soclink'));
                            $social.find('select').val(this.ico);
                            $social.find('input').val(this.link);

                            parent.guests.arrangeSocLinks($guest);
                        }
                    });

                    $guest.find('.stec-builder-submission-guest-social').first().remove();
                }

                this.arrangeGuests();
            },

            controller: function () {

                parent.media.addImageController('.stec-builder-submission-guest', {
                    multiple: false,
                    inputName: 'guests[0][photo]',
                    callback: function () {
                        parent.guests.arrangeGuests();
                    }
                });

                $instance.find('.stec-builder-element-content-guests-style').on(helper.clickHandle(),
                        '.stec-builder-submission-guest-head button.collapse', function () {
                            var $guest = $(this).parents('.stec-builder-submission-guest').first();
                            $guest.addClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-guests-style').on(helper.clickHandle(),
                        '.stec-builder-submission-guest-head button.expand', function () {
                            var $guest = $(this).parents('.stec-builder-submission-guest').first();
                            $guest.removeClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-guests-style').on(helper.clickHandle(),
                        '.stec-builder-submission-guest-head button.delete', function () {
                            var $guest = $(this).parents('.stec-builder-submission-guest').first();
                            $guest.remove();

                            parent.guests.arrangeGuests();
                        });

                $instance.find('.stec-builder-submission-add-guest').on(helper.clickHandle(), function () {
                    $(parent.guests.template).insertBefore(this);
                    parent.guests.arrangeGuests();
                });

                $instance.find('.stec-builder-element-content-guests-style').on(helper.clickHandle(),
                        '.add-guests-soclink', function () {
                            var $guest = $(this).parents('.stec-builder-submission-guest').first();
                            var social = $guest.find('.stec-builder-submission-guest-social').first().clone()[0].outerHTML;
                            var $social = $(social).insertBefore($guest.find('.add-guests-soclink'));
                            $social.find('select, input').val(null);

                            parent.guests.arrangeSocLinks($guest);
                        });

                $instance.find('.stec-builder-element-content-guests-style').on(helper.clickHandle(),
                        '.stec-builder-submission-guest-social-remove', function () {
                            var $guest = $(this).parents('.stec-builder-submission-guest').first();
                            var $this = $(this).parents('.stec-builder-submission-guest-social').first();

                            if ($guest.find('.stec-builder-submission-guest-social').length <= 1) {
                                $this.find('select, input').val(null);
                            } else {
                                $this.remove();
                            }

                            parent.guests.arrangeSocLinks($guest);
                        });
            }
        };

        this.attendance = {
            template: null,

            init: function () {
                if ($instance.find('.stec-builder-element-content-attendance-style').length <= 0) {
                    return;
                }

                this.template = $instance.find('.stec-builder-element-content-attendance-style')
                        .find('.stec-builder-submission-attendance-template')
                        .clone()
                        .removeClass('stec-builder-submission-attendance-template')
                [0].outerHTML;

                $instance.find('.stec-builder-element-content-attendance-style')
                        .find('.stec-builder-submission-attendance-template').remove();

            },

            arrangeAttendees: function () {
                $instance.find('.stec-builder-submission-attendance').each(function (i) {
                    $(this).find("input, select, textarea").each(function () {
                        this.name = this.name.replace(/attendees\[\d]/g, function (str, p1) {
                            return 'attendees[' + i + ']';
                        });
                    });

                    $(this).find("[data-name]").each(function () {
                        $(this).attr('data-name', $(this).attr('data-name').replace(/attendees\[\d]/g, function (str, p1) {
                            return 'attendees[' + i + ']';
                        }));
                    });
                });
            },

            addAttendee: function (attendee) {

                var $attendee = $(this.template).insertBefore($instance.find('.stec-builder-submission-add-attendee'));

                $attendee.find("input[name^='attendees['][name$='[email]']").val(attendee.email);
                $attendee.find("input[name^='attendees['][name$='[people]']").val(attendee.people);
                $attendee.find("input[name^='attendees['][name$='[send_email]']").prop('checked', attendee.mail_sent ? false : true);

                this.arrangeAttendees();
            },

            controller: function () {

                $instance.find('.stec-builder-element-content-attendance-style').on(helper.clickHandle(),
                        '.stec-builder-submission-attendee-head button.collapse', function () {
                            var $attendee = $(this).parents('.stec-builder-submission-attendance').first();
                            $attendee.addClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-attendance-style').on(helper.clickHandle(),
                        '.stec-builder-submission-attendee-head button.expand', function () {
                            var $attendee = $(this).parents('.stec-builder-submission-attendance').first();
                            $attendee.removeClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-attendance-style').on(helper.clickHandle(),
                        '.stec-builder-submission-attendee-head button.delete', function () {
                            var $attendee = $(this).parents('.stec-builder-submission-attendance').first();
                            $attendee.remove();

                            parent.attendance.arrangeAttendees();
                        });

                $instance.find('.stec-builder-submission-add-attendee').on(helper.clickHandle(), function () {
                    $(parent.attendance.template).insertBefore(this);
                    parent.attendance.arrangeAttendees();
                });

            }
        };

        this.schedules = {
            template: null,

            init: function () {
                if ($instance.find('.stec-builder-element-content-schedules-style').length <= 0) {
                    return;
                }

                this.template = $instance.find('.stec-builder-element-content-schedules-style')
                        .find('.stec-builder-submission-schedule-template')
                        .clone()
                        .removeClass('stec-builder-submission-schedule-template')
                [0].outerHTML;

                $instance.find('.stec-builder-element-content-schedules-style')
                        .find('.stec-builder-submission-schedule-template').remove();

            },

            arrangeSchedules: function () {
                $instance.find('.stec-builder-submission-schedule').each(function (i) {
                    $(this).find("input, select, textarea").each(function () {
                        this.name = this.name.replace(/schedule\[\d]/g, function (str, p1) {
                            return 'schedule[' + i + ']';
                        });
                    });

                    $(this).find("[data-name]").each(function () {
                        $(this).attr('data-name', $(this).attr('data-name').replace(/schedule\[\d]/g, function (str, p1) {
                            return 'schedules[' + i + ']';
                        }));
                    });
                });
            },

            addSchedule: function (schedule) {

                var $schedule = $(this.template).insertBefore($instance.find('.stec-builder-submission-add-schedule'));
                var datetime, time;

                this.arrangeSchedules();

                $schedule.find("input[name^='schedule['][name$='[schedule_title]']").val(schedule.title);
                $schedule.find("textarea[name^='schedule['][name$='[schedule_details]']").val(schedule.details);
                datetime = schedule.start_date.split(' ');
                time = datetime[1].split(':');

                $schedule.find('.stec-builder-element-content-datetime-style').first().val(dtInFormat(datetime[0]));
                $schedule.find('[name$="[schedule_time_hours_from]"]').val(time[0]);
                $schedule.find('[name$="[schedule_time_minutes_from]"]').val(time[1]);

                $schedule.find('.stec-builder-element-content-colorpicker-style').val(schedule.icon_color);
                $schedule.find('[name$="[schedule_icon]"]').val(schedule.icon);

                this.addDatePicker($schedule);
                this.addColorPicker($schedule);
            },

            addDatePicker: function ($schedule) {

                var dtDate = new Date(glob.options.year, glob.options.month, glob.options.day);

                if ($schedule.find('.stec-builder-element-content-datetime-style').first().val() !== '') {
                    dtDate = $schedule.find('.stec-builder-element-content-datetime-style').first().val();
                }

                $schedule.find('.stec-builder-element-content-datetime-style').first()
                        .datepicker("destroy")
                        .datepicker({
                            setDate: glob.options.year + '-' + glob.options.month + '-' + glob.options.day,
                            showAnim: 0,
                            minDate: $instance.find('[name="start_date"]').val(),
                            maxDate: $instance.find('[name="end_date"]').val(),
                            dateFormat: dtPickerFormat,
                            altFormat: "yy-mm-dd",
                            altField: $schedule.find('.stec-builder-element-content-datetime-style').first().next(),
                            onSelect: function () {
                            }
                        })
                        .datepicker("setDate", dtDate);
            },

            addColorPicker: function ($schedule) {

                $schedule.find('.stec-builder-element-content-colorpicker-style').each(function (i) {

                    var th = this, color = $(th).val();

                    if ($('#colorpicker-schedule-' + i + '-' + glob.options.id).length > 0) {
                        $('#colorpicker-schedule-' + i + '-' + glob.options.id).remove();
                    }

                    $(th).css({
                        backgroundColor: color
                    });

                    $(th).StecColorPicker({
                        klass: 'colorpicker-schedule-' + glob.options.id,
                        id: 'colorpicker-schedule-' + i + '-' + glob.options.id,
                        color: color,
                        onShow: function (colpkr) {
                            $(colpkr).show();
                            return false;
                        },
                        onHide: function (colpkr) {
                            $(colpkr).hide();
                            return false;
                        },
                        onChange: function (hsb, hex, rgb) {
                            $(th).attr("title", "#" + hex);

                            $(th).css({
                                backgroundColor: "#" + hex
                            });

                            $(th).val("#" + hex);
                        }
                    });
                });

            },

            controller: function () {

                $instance.find('.stec-builder-element-content-schedules-style').on(helper.clickHandle(),
                        '.stec-builder-submission-schedule-head button.collapse', function () {
                            var $schedule = $(this).parents('.stec-builder-submission-schedule').first();
                            $schedule.addClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-schedules-style').on(helper.clickHandle(),
                        '.stec-builder-submission-schedule-head button.expand', function () {
                            var $schedule = $(this).parents('.stec-builder-submission-schedule').first();
                            $schedule.removeClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-schedules-style').on(helper.clickHandle(),
                        '.stec-builder-submission-schedule-head button.delete', function () {
                            var $schedule = $(this).parents('.stec-builder-submission-schedule').first();
                            $schedule.remove();

                            parent.schedules.arrangeSchedules();
                        });

                $instance.find('.stec-builder-submission-add-schedule').on(helper.clickHandle(), function () {
                    var $schedule = $(parent.schedules.template).insertBefore(this);
                    parent.schedules.addDatePicker($schedule);
                    parent.schedules.addColorPicker($schedule);
                    parent.schedules.arrangeSchedules();
                });

            }
        };

        this.locations = {

            init: function () {

                this.checkState();
            },

            checkCountryState: function () {
                var $form = $instance.find('.stec-builder-submission-new-address');
                var $citySelector = $form.find('select[name="location_city"]');
                var country = $form.find('select[name="location_country"]').val();
                $citySelector.children().not(':disabled').hide();
                $citySelector.children().filter('[data-country="' + country + '"]').show();
                $citySelector.val($citySelector.children().first().val());
            },

            checkState: function () {

                switch ($instance.find('.stec-builder-element-content-select-style[name="location"]').val()) {
                    case 'custom_address':
                        $instance.find('.stec-builder-submission-new-address').show();
                        $instance.find('.stec-builder-submission-new-location').hide();
                        this.checkCountryState();
                        break;

                    case 'custom':
                        $instance.find('.stec-builder-submission-new-address').hide();
                        $instance.find('.stec-builder-submission-new-location').show();
                        break;

                    default:
                        $instance.find('.stec-builder-submission-new-location').hide();
                        $instance.find('.stec-builder-submission-new-address').hide();
                }
            },

            controller: function () {

                $instance.find('.stec-builder-element-content-select-style[name="location"]').on('change', function () {
                    parent.locations.checkState();
                });

                $instance.find('.stec-builder-submission-new-address select[name="location_country"]').on('change', function () {
                    parent.locations.checkCountryState();
                });

            }
        };

        this.booking = {
            template: null,

            init: function () {

                if ($instance.find('.stec-builder-element-content-booking-style')
                        .find('.stec-builder-submission-ticket-template').length <= 0) {
                    return;
                }

                this.template = $instance.find('.stec-builder-element-content-booking-style')
                        .find('.stec-builder-submission-ticket-template')
                        .clone()
                        .removeClass('stec-builder-submission-ticket-template')
                [0].outerHTML;

                $instance.find('.stec-builder-element-content-booking-style')
                        .find('.stec-builder-submission-ticket-template').remove();

            },

            arrangeTickets: function () {
                $instance.find('.stec-builder-submission-ticket').each(function (i) {
                    $(this).find("input, select, textarea").each(function () {
                        this.name = this.name.replace(/tickets\[\d]/g, function (str, p1) {
                            return 'tickets[' + i + ']';
                        });
                    });

                    $(this).find("[data-name]").each(function () {
                        $(this).attr('data-name', $(this).attr('data-name').replace(/tickets\[\d]/g, function (str, p1) {
                            return 'tickets[' + i + ']';
                        }));
                    });
                });
            },

            arrangeAttributes: function ($el) {

                $el.find('.stec-builder-submission-ticket-attributes').each(function (i) {
                    $(this).find("input").each(function () {
                        this.name = this.name.replace(/\[attr]\[\d]/g, function (str, p1) {
                            return '[attr][' + i + ']';
                        });
                    });
                });
            },

            addTicket: function (ticket) {
                var stopBeforeValue, stopBeforeUnits = 'hours';
                var $ticket = $(this.template).insertBefore($instance.find('.stec-builder-submission-add-ticket'));
                $ticket.find("input[name^='tickets['][name$='[id]']").val(ticket.id ? parseInt(ticket.id, 10) : 0);
                $ticket.find("input[name^='tickets['][name$='[name]']").val(ticket.title);
                $ticket.find("input[name^='tickets['][name$='[price]']").val(ticket.price);
                $ticket.find("input[name^='tickets['][name$='[qty]']").val(ticket.stock_quantity);
                $ticket.find("input[name^='tickets['][name$='[qty_per_event]']").val(ticket.qty_limit_per_event);
                $ticket.find("textarea[name^='tickets['][name$='[about]']").val(ticket.short_description);

                stopBeforeValue = parseInt(ticket.stop_before, 10);

                if (isNaN(stopBeforeValue)) {
                    stopBeforeValue = 0;
                }

                if (stopBeforeValue > 0) {
                    stopBeforeValue /= 3600;

                    if (0 === stopBeforeValue % 24) {
                        stopBeforeUnits = 'days';
                        stopBeforeValue /= 24;
                    }

                    $ticket.find("select[name^='tickets['][name$='[wc_book_create_ticket_stop_purchase][units]']").val(stopBeforeUnits);
                    $ticket.find("input[name^='tickets['][name$='[wc_book_create_ticket_stop_purchase][value]']").val(stopBeforeValue);
                }

                if (ticket.attributes.length > 0) {
                    $.each(ticket.attributes, function () {

                        var attribElement;

                        if (this.name !== '') {
                            attribElement = $ticket.find('.stec-builder-submission-ticket-attributes').first().clone()[0].outerHTML;
                            attribElement = $(attribElement).insertBefore($ticket.find('.stec-builder-submission-ticket-attributes').first());
                            attribElement.find('input[name$="[attr_name]"]').first().val(this.name);
                            attribElement.find('input[name$="[attr_value]"]').first().val(this.value);
                            parent.booking.arrangeAttributes($ticket);
                        }
                    });

                    $ticket.find('.stec-builder-submission-ticket-social').first().remove();
                }

                this.arrangeTickets();
            },

            controller: function () {

                $instance.find('.stec-builder-element-content-booking-style').on(helper.clickHandle(),
                        '.stec-builder-submission-ticket-head button.collapse', function () {
                            var $ticket = $(this).parents('.stec-builder-submission-ticket').first();
                            $ticket.addClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-booking-style').on(helper.clickHandle(),
                        '.stec-builder-submission-ticket-head button.expand', function () {
                            var $ticket = $(this).parents('.stec-builder-submission-ticket').first();
                            $ticket.removeClass('collapsed');
                        });

                $instance.find('.stec-builder-element-content-booking-style').on(helper.clickHandle(),
                        '.stec-builder-submission-ticket-head button.delete', function () {
                            var $ticket = $(this).parents('.stec-builder-submission-ticket').first();
                            $ticket.remove();

                            parent.booking.arrangeTickets();
                        });

                $instance.find('.stec-builder-submission-add-ticket').on(helper.clickHandle(), function () {
                    $(parent.booking.template).insertBefore(this);
                    parent.booking.arrangeTickets();
                });

                $instance.find('.stec-builder-element-content-booking-style').on(helper.clickHandle(),
                        '.add-ticket-attribute', function () {
                            var $ticket = $(this).parents('.stec-builder-submission-ticket').first();
                            var attrib = $ticket.find('.stec-builder-submission-ticket-attributes').first().clone()[0].outerHTML;
                            var $attrib = $(attrib).insertAfter($ticket.find('.stec-builder-submission-ticket-attributes').last());
                            $attrib.find('input').val(null);

                            parent.booking.arrangeAttributes($ticket);
                        });

                $instance.find('.stec-builder-element-content-booking-style').on(helper.clickHandle(),
                        '.stec-builder-submission-ticket-attrib-remove', function () {
                            var $ticket = $(this).parents('.stec-builder-submission-ticket').first();
                            var $this = $(this).parents('.stec-builder-submission-ticket-attributes').first();

                            if ($ticket.find('.stec-builder-submission-ticket-attributes').length <= 1) {
                                $this.find('input').val(null);
                            } else {
                                $this.remove();
                            }

                            parent.booking.arrangeAttributes($ticket);
                        });
            }
        };

        this.modal = {

            html: function (status, msg, confirmAction) {

                var id, klass, icon, html;

                id = [glob.options.id, 'modal'].join('-');

                switch (status) {

                    case 2:
                        icon = 'far fa-clock';
                        klass = 'stec-wait';
                        break;

                    case 0:
                        klass = 'stec-error';
                        icon = 'far fa-frown';

                        if (!msg) {
                            msg = __('submitError');
                        }

                        break;
                    case 1:
                    default:
                        klass = 'stec-success';
                        icon = 'far fa-smile';

                        if (!msg) {
                            msg = __('submitSuccess');
                        }

                }

                html = [];
                html.push('<div id="' + id + '" class="stec-builder-form-modal stec-modal ' + klass + '">');
                html.push('   <div class="stec-builder-form-modal-block">');
                html.push('           <div class="stec-builder-form-modal-block-top">');
                html.push('               <div>');
                html.push('                   <i class="' + icon + '"></i>');
                html.push('               </div>');
                html.push('           </div>');
                html.push('           <div class="stec-builder-form-modal-wrap">');
                html.push('               <p>' + msg + '</p>');

                if (confirmAction) {
                    html.push('               <button data-action="confirm" class="stec-style-button">Yes</button>');
                    html.push('               <button class="stec-style-button">No</button>');
                } else {
                    html.push('               <button class="stec-style-button">' + (status === 2 ? __('close') : __('ok')) + '</button>');
                }

                html.push('           </div>');
                html.push('   </div>');
                html.push('</div>');

                return html.join('');
            },

            close: function () {
                $('body').removeClass('stec-modal-opened');
                $('#' + glob.options.id + '-modal').remove();
            },

            open: function (status, msg, confirmAction) {
                $('body').addClass('stec-modal-opened');
                $(this.html(status, msg, confirmAction)).appendTo('body');

                if (typeof confirmAction === 'function') {
                    $('#' + glob.options.id + '-modal').find('[data-action="confirm"]').one(helper.clickHandle(), function () {
                        confirmAction();
                    });
                }
            }

        };

        this.media = {
            limit: 10,
            init: function () {
                this.controller();
            },
            fillImage: function (parentElement, params) {

                var html;

                if (params.multiple === true && $(parentElement).find('.stec-builder-media-photo').length >= parent.media.limit) {
                    $(parentElement).find('.stec-builder-add-media-photo').hide();
                    return;
                }

                html = [];
                html.push('<div class="stec-builder-media-photo">');
                html.push('   <i class="fa fa-times"></i>');
                html.push('   <img data-id="' + params.image.id + '" src="' + params.image.src + '" />');
                html.push('   <input type="hidden" name="' + params.inputName + '" value="' + params.image.id + '">');
                html.push('</div>');

                $(html.join('')).insertBefore($(parentElement).find('.stec-builder-add-media-photo'));

                if (params.multiple !== true || $(parentElement).find('.stec-builder-media-photo').length >= parent.media.limit) {
                    $(parentElement).find('.stec-builder-add-media-photo').hide();
                }

                if (typeof params.callback === 'function') {
                    params.callback();
                }
            },

            fillAttachment: function (parentElement, params) {

                var html;

                if (params.multiple === true && $(parentElement).find('.stec-builder-media-attachment').length >= parent.media.limit) {
                    $(parentElement).find('.stec-builder-add-media-attachments').hide();
                    return;
                }

                html = [];
                html.push('<div class="stec-builder-media-attachment">');
                html.push('   <span class="stec-builder-element-content-p-style">' + params.file.title + '</span>');
                html.push('   <i class="fa fa-times"></i>');
                html.push('   <input type="hidden" name="' + params.inputName + '" value="' + params.file.id + '">');
                html.push('</div>');

                $(html.join('')).insertBefore($(parentElement).find('.stec-builder-add-media-attachments'));

                if (params.multiple !== true || $(parentElement).find('.stec-builder-media-attachment').length >= parent.media.limit) {
                    $(parentElement).find('.stec-builder-add-media-attachments').hide();
                }

                if (typeof params.callback === 'function') {
                    params.callback();
                }
            },

            addFileController: function (parentElement, params) {

                $instance.on('click', parentElement + ' .stec-builder-add-media-attachments', function (e) {

                    var th = this, mediaFrame;

                    e.preventDefault();

                    mediaFrame = window.wp.media({
                        library: {
                        },
                        multiple: params.multiple
                    });

                    mediaFrame.open();

                    mediaFrame.on('select', function () {

                        var attachments = mediaFrame.state().get('selection').toJSON();

                        $(attachments).each(function (i) {

                            var att = this, html;

                            if (att.type === "image") {

                                if (params.multiple === true && $(parentElement).find('.stec-builder-media-photo').length >= parent.media.limit) {
                                    $(th).hide();
                                    return;
                                }

                                html = [];
                                html.push('<div class="stec-builder-media-attachment">');
                                html.push('   <span class="stec-builder-element-content-p-style">' + att.title + '</span>');
                                html.push('   <i class="fa fa-times"></i>');
                                html.push('   <input type="hidden" name="' + params.inputName + '" value="' + att.id + '">');
                                html.push('</div>');

                                $(html.join('')).insertBefore(th);

                                if (params.multiple !== true || $(parentElement).find('.stec-builder-media-attachment').length >= parent.media.limit) {
                                    $(parentElement).find('.stec-builder-add-media-attachments').hide();
                                }

                            }
                        });

                        if (attachments.length > 0 && typeof params.callback === 'function') {
                            params.callback();
                        }
                    });

                });

                $instance.on('click', parentElement + ' .stec-builder-media-attachment i', function () {
                    var $parent = $(this).parents(parentElement).first();
                    $(this).parents('.stec-builder-media-attachment').first().remove();
                    $parent.find('.stec-builder-add-media-attachments').show();
                });


            },
            addImageController: function (parentElement, params) {

                $instance.on('click', parentElement + ' .stec-builder-add-media-photo', function (e) {
                    var th = this, mediaFrame;

                    e.preventDefault();

                    mediaFrame = window.wp.media({
                        library: {
                            type: ['image']
                        },
                        multiple: params.multiple
                    });

                    mediaFrame.open();

                    mediaFrame.on('select', function () {

                        var attachments = mediaFrame.state().get('selection').toJSON();

                        $(attachments).each(function (i) {

                            var att = this, src, html;

                            if (att.type === "image") {

                                if (params.multiple === true && $(parentElement).find('.stec-builder-media-photo').length >= parent.media.limit) {
                                    $(th).hide();
                                    return;
                                }

                                src = att.sizes.medium ? att.sizes.medium.url : att.sizes.full.url;

                                html = [];
                                html.push('<div class="stec-builder-media-photo">');
                                html.push('   <i class="fa fa-times"></i>');
                                html.push('   <img data-id="' + att.id + '" src="' + src + '" alt="' + att.alt + '" />');
                                html.push('   <input type="hidden" name="' + params.inputName + '" value="' + att.id + '">');
                                html.push('</div>');

                                $(html.join('')).insertBefore(th);

                                if (params.multiple !== true || $(parentElement).find('.stec-builder-media-photo').length >= parent.media.limit) {
                                    $(parentElement).find('.stec-builder-add-media-photo').hide();
                                }

                            }
                        });

                        if (attachments.length > 0 && typeof params.callback === 'function') {
                            params.callback();
                        }
                    });

                });

                $instance.on('click', parentElement + ' .stec-builder-media-photo i', function () {

                    var $parent = $(this).parents(parentElement).first();

                    $(this).parents('.stec-builder-media-photo').first().remove();

                    $parent.find('.stec-builder-add-media-photo').show();
                });

            },
            controller: function () {
            }
        };

        /**
         * TinyMCE editor for description field
         */
        this.tinymce = {

            id: null,
            $editor: null,
            init: function () {

                var th = this, $textarea;
                var settings = {};

                try {

                    this.id = 'stec-editor-id-' + Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
                    $textarea = $instance.find('textarea[name="description"]').attr('id', this.id);

                    /**
                     * @see wp-before-quicktags-init, wp-before-tinymce-init jQuery triggers for filtering the settings
                     */
                    settings = {
                        init_instance_callback: function (editor) {
                            th.$editor = editor;
                        },
                        wpautop: true,
                        resize: false,
                        height: $textarea.height() - 74,
                        toolbar1: ''
                    };

                    window.wp.editor.initialize(this.id, {
                        tinymce: settings,
                        quicktags: true
                    });

                } catch (e) {
                    this.id = null;
                    this.$editor = null;
                }

            },

            getContent: function () {
                try {
                    return window.tinymce.editors[this.id].getContent();
                } catch (e) {
                    $instance.find('textarea[name="description"]').val();
                }
            },

            setContent: function (content) {
                try {
                    this.$editor.setContent(content);
                } catch (e) {
                    $instance.find('textarea[name="description"]').val(content);
                }
            }
        };

        function fillEditEventData() {

            var event = glob.options.edit_event;
            var rruleString;
            var rrule;
            var weekdays;
            var exdates;

            // Fill input type texts
            $.each($instance.find('input[type="text"], input[type="number"], input[type="email"], input[type="url"]'), function () {
                var th = this;
                var name = $(this).attr('name');
                var startDate, endDate;
                if (name) {
                    if (event[name]) {
                        switch (name) {
                            case 'start_date':
                                startDate = event.start_date.split(' ');
                                $(th).datepicker('setDate', moment(startDate[0]).toDate());
                                break;
                            case 'end_date':
                                endDate = event.end_date.split(' ');
                                $(th).datepicker('setDate', moment(endDate[0]).toDate());
                                break;
                            case 'rsvp[collect]]':
                                break;

                            default:
                                $(th).val(event[name]);
                        }
                    }
                }
            });

            // Fill textarea
            $.each($instance.find('textarea'), function () {
                var th = this;
                var name = $(this).attr('name');
                if (name) {
                    if (event[name]) {
                        $(th).html(event[name]);
                    }
                }
            });

            // Fill description content to the tinymce editor
            parent.tinymce.setContent(event.description);

            // Fill selectors
            $.each($instance.find('select'), function () {
                var th = this;
                var name = $(this).attr('name');
                var startDate, endDate, hoursMinutes;
                var cats;

                if (name) {
                    switch (name) {
                        case 'start_time_hours':
                            startDate = event.start_date.split(' ');
                            hoursMinutes = startDate[1].split(':');
                            $(th).val(hoursMinutes[0]);
                            break;

                        case 'end_time_hours':
                            endDate = event.end_date.split(' ');
                            hoursMinutes = endDate[1].split(':');
                            $(th).val(hoursMinutes[0]);
                            break;

                        case 'start_time_minutes':
                            startDate = event.start_date.split(' ');
                            hoursMinutes = startDate[1].split(':');
                            $(th).val(hoursMinutes[1]);
                            break;

                        case 'end_time_minutes':
                            endDate = event.end_date.split(' ');
                            hoursMinutes = endDate[1].split(':');
                            $(th).val(hoursMinutes[1]);
                            break;

                        case 'calendar_id' :
                            $(th).val(event.calid);
                            break;

                        case 'event_categories[]' :
                            if (event.category) {
                                cats = [];
                                $.each(event.category, function () {
                                    cats.push(this.id);
                                });
                                $(th).val(cats);
                            }
                            break;

                        case 'location' :
                            if (event.location.id) {
                                $(this).val(event.location.id);
                            }
                            break;

                        default:
                            if (event[name]) {
                                $(th).val(event[name]);
                            }
                    }

                }
            });

            // Fill colorpicker 
            $instance.find('input[name="event_color"]').ColorPickerSetColor(event.color);
            $instance.find('input[name="event_color"]').val(event.color);
            $instance.find('input[name="event_color"]').css('background-color', event.color);

            // Fill checkbox
            $.each($instance.find('input[type="checkbox"]'), function () {
                var th = this;
                var name = $(this).attr('name');
                var isChecked;
                if (name) {
                    if (event[name]) {
                        isChecked = parseInt(event[name], 10) === 1;
                        if ($(th).prop('checked') !== isChecked) {
                            $(th).prop('checked', true);
                            $(th).trigger("change");
                        }
                    }
                }
            });

            // Fill repeater
            if (event.rrule) {

                // Basic repeater

                rruleString = event.rrule;

                if (rruleString === '') {
                    $('select[name="repeater"]').val(0);
                    return;
                }

                rrule = new window.rrule.RRule.fromString(rruleString);

                // set freq
                $('select[name="repeater"]').val(Math.abs(4 - rrule.origOptions.freq)); // invert order 4,3,2,1 to 1,2,3,4...

                if (rrule.origOptions.byweekday) {

                    weekdays = ['MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SU'];
                    $('.stec-repeater-content-repeat-on input[type="checkbox"]').prop('checked', '');
                    $(rrule.origOptions.byweekday).each(function (a, b) {
                        var val = (weekdays[b.weekday]);
                        $('.stec-repeater-content-repeat-on input[name="' + val + '"]').prop('checked', 'checked');
                    });
                }

                if (rrule.origOptions.interval) {
                    $('select[name="repeat_gap"]').val(rrule.origOptions.interval);
                } else {
                    $('select[name="repeat_gap"]').val(0);
                }

                if (rrule.origOptions.count) {
                    $('input[name="repeat_occurences"]').val(rrule.origOptions.count);
                    $('input[name="repeat_occurences"]').parents('label').first().find('input[type="radio"]').trigger('click');
                }

                if (rrule.origOptions.until) {
                    $('input[name="repeat_ends_on_date"]').datepicker("setDate", rrule.origOptions.until);
                    $('input[name="repeat_ends_on_date"]').parents('label').first().find('input[type="radio"]').trigger('click');
                }

                $instance.find('select[name="repeater"]').trigger('change');

                // Exdates
                if (event.exdate) {
                    exdates = event.exdate.split(',');
                    $.each(exdates, function () {
                        var date = moment(this);
                        parent.repeater.addExdate(date);
                    });
                }
            }

            // Fill guests
            if (event.guests.length > 0) {
                $.each(event.guests, function () {
                    parent.guests.addGuest(this);
                });
            }

            // Fill rsvp settings
            $instance.find('input[name="rsvp[limit]"]').val(event.rsvp_limit);
            $instance.find('select[name="rsvp[collect]"]').val(event.rsvp);
            $instance.find('select[name="rsvp[visibility]"]').val(event.attendance_visibility);

            // Fill attendees
            if (event.attendance.length > 0) {
                $.each(event.attendance, function () {
                    parent.attendance.addAttendee(this);
                });
            }

            // Fill organizers
            if (event.organizers.length > 0) {
                $.each(event.organizers, function () {
                    $instance.find('select[name="organizers[]"] option[value="' + this.id + '"]').prop('selected', true);
                });
            }

            // Fill schedules
            if (event.schedule.length > 0) {
                $.each(event.schedule, function () {
                    parent.schedules.addSchedule(this);
                });
            }

            // Fill tickets
            if (event.bookable_products.length > 0) {
                $.each(event.bookable_products, function () {
                    parent.booking.addTicket(this);
                });
            }

            // Specify this is an edit to an event
            $('<input type="hidden" name="event_id" value="' + event.id + '" />').appendTo($instance.find('form'));

            // Fill slider images
            if (event.images.length > 0) {
                $.each(event.images_meta, function () {
                    parent.media.fillImage('.stec-builder-element-content-media-images-style', {
                        inputName: 'images[]',
                        multiple: true,
                        image: {
                            id: this.id,
                            src: this.src
                        }
                    });
                });
            }

            // Fill attachments
            if (event.attachments.length > 0) {
                $.each(event.attachments, function () {
                    parent.media.fillAttachment('.stec-builder-element-content-media-attachments-style', {
                        inputName: 'attachments[]',
                        multiple: true,
                        file: {
                            id: this.id,
                            title: this.filename
                        }
                    });
                });
            }

            // Add delete button
            $('<input data-action="delete-event" type="button" name="delete" value="' + __('Delete Event') + '">').appendTo($instance.find('.stec-builder-element-content-submit-style'));
        }

        function submitForm() {

            var formData, request;

            if (true !== parent.captcha.valid) {
                return false;
            }

            parent.repeater.setRrule();

            formData = new FormData($instance.find('form')[0]);
            formData.append('action', 'stec_public_ajax_action');
            formData.append('description', parent.tinymce.getContent());
            formData.append('task', 'front_create_event');

            request = DB.newRequest();
            request.setData(formData);

            request.setBefore(function () {
                $instance.find('.stec-builder-element-content-submit-style').hide();
                parent.modal.open(2, __('submittingEvent'));
            });

            request.setOnError(function (err) {
                parent.modal.open(0, err.statusText);
            });

            request.setCallback(function (data) {
                parent.modal.close();

                if (data.error === 0) {
                    parent.modal.open(1);
                    if (glob.options.master) {
                        glob.options.master.calData.addToEventsPool([data.event]);
                        glob.options.master.layout.set();
                    }
                } else {
                    parent.modal.open(0, data.msg);
                }

                $instance.find('.stec-builder-element-content-submit-style').show();

                if (typeof window.grecaptcha !== 'undefined') {
                    window.grecaptcha.reset(captchaContainer);
                    parent.captcha.valid = false;
                }
            });

            DB.query(request);

        }

        function controller() {

            parent.captcha.set();
            parent.datePicker.set();
            parent.colorPicker.set();
            parent.repeater.init();
            parent.guests.controller();
            parent.attendance.controller();
            parent.booking.controller();
            parent.organizers.controller();
            parent.schedules.controller();
            parent.locations.controller();

            parent.media.addImageController('.stec-builder-element-content-media-images-style', {
                multiple: true,
                inputName: 'images[]'
            });

            parent.media.addFileController('.stec-builder-element-content-media-attachments-style', {
                multiple: true,
                inputName: 'attachments[]'
            });

            $('body').on(helper.clickHandle(), '#' + glob.options.id + '-modal  button', function () {
                parent.modal.close();
            });

            $instance.on('submit', function (e) {
                e.preventDefault();

                // Reset organizer
                $instance.find('.stec-builder-submission-organizer').remove();
                $instance.find('.stec-builder-submission-add-organizer').show();

                submitForm();
            });

            // change color on calendar selection
            $instance.find('select[name="calendar_id"]').on('change', function () {
                var color = $(this).children().filter(':selected').data('color');
                $instance.find('input[name="event_color"]')
                        .css('background-color', color)
                        .val(color)
                        .ColorPickerSetColor(color);
            });

            // Non logged add image
            $instance.find('.stec-builder-submission-add-image').on(helper.clickHandle(), function () {
                $instance.find('input[name="stec_public_file_image"]').trigger('click');
            });

            $instance.find('input[name="stec_public_file_image"]').on('change', function (e) {
                if (e.target.files.length > 0) {
                    $instance.find('.stec-builder-submission-add-image-src').text(e.target.files[0].name);
                } else {
                    $instance.find('.stec-builder-submission-add-image-src').text('');
                }
            });

            // Non logged add attachment
            $instance.find('.stec-builder-upload-attachments').on(helper.clickHandle(), function () {
                $instance.find('input[name="stec_public_file_attachment"]').trigger('click');
            });

            $instance.find('input[name="stec_public_file_attachment"]').on('change', function (e) {
                if (e.target.files.length > 0) {
                    $instance.find('.stec-builder-submission-add-attachment-filename').text(e.target.files[0].name);
                } else {
                    $instance.find('.stec-builder-submission-add-attachment-filename').text('');
                }
            });

            $instance.on('stec-day-change', function (e, dt) {

                var now, newDate;

                glob.options.day = dt.day;
                glob.options.month = dt.month;
                glob.options.year = dt.year;

                now = [
                    glob.options.year,
                    glob.options.month + 1,
                    glob.options.day
                ].join('-');

                newDate = window.moment(now, 'YYYY-M-D');

                $instance.find('[name="start_date"]')
                        .datepicker("setDate", newDate.toDate());

                $instance.find('[name="end_date"]')
                        .datepicker("setDate", newDate.toDate());

                parent.datePicker.generalTimeFix();
            });

            $instance.on(helper.clickHandle(), 'input[data-action="delete-event"]', function () {

                if (true !== parent.captcha.valid) {
                    parent.modal.open(0, __('Verify Captcha'));
                    return;
                }

                parent.modal.open(0, __('Are you sure'), function () {
                    parent.deleteEvent();
                });
            });
        }

        this.deleteEvent = function () {
            var eventId = glob.options.edit_event.id;
            var request;

            request = DB.newRequest();
            request.setData({
                action: 'stec_public_ajax_action',
                'g-recaptcha-response': $instance.find('.g-recaptcha-response').val(),
                task: 'front_delete_event',
                id: eventId
            });

            request.setBefore(function () {
                parent.modal.close();
                parent.modal.open(2, __('deleting'));
            });

            request.setOnError(function (err) {
            });

            request.setCallback(function (result) {

                parent.modal.close();
                if (result) {
                    if (1 === result.error) {
                        parent.modal.open(0, result.msg);
                    } else {
                        parent.modal.open(1, result.msg);
                    }
                } else {
                    parent.modal.open(0, __('submitError'));
                }

                if (typeof window.grecaptcha !== 'undefined') {
                    parent.captcha.valid = false;
                    window.grecaptcha.reset(captchaContainer);
                }
            });

            DB.query(request);

        };

        this.init = function (options) {
            var now = new Date();
            var writableCalendars;
            var writableCategories;

            $.extend(glob.options, options);

            instance = "#" + glob.options.id;
            $instance = $(instance);

            DB = new window.StecDB();

            helper = new window.StecHelper();

            if (glob.options.master) {
                glob.options.day = glob.options.master.glob.options.day;
                glob.options.month = glob.options.master.glob.options.month;
                glob.options.year = glob.options.master.glob.options.year;
            } else {
                glob.options.day = now.getDate();
                glob.options.month = now.getMonth();
                glob.options.year = now.getFullYear();
            }

            switch (glob.options.date_format) {
                case 'dd.mm.yyyy' :
                    dtPickerFormat = 'dd.mm.yy';
                    break;
                default:
                    dtPickerFormat = glob.options.date_format;
            }

            if (glob.options.cal) {
                writableCalendars = $.map(glob.options.cal.split(','), function (value) {
                    return parseInt(value, 10);
                });
                $instance.find('select[name="calendar_id"]').children().each(function () {
                    if ($.inArray(parseInt($(this).val(), 10), writableCalendars) === -1) {
                        $(this).remove();
                    }
                });
            }

            if (glob.options.cat) {
                writableCategories = $.map(glob.options.cat.split(','), function (value) {
                    return parseInt(value, 10);
                });
                $instance.find('select[name="event_categories[]"]').children().each(function () {
                    if ($.inArray(parseInt($(this).val(), 10), writableCategories) === -1) {
                        $(this).remove();
                    }
                });
            }

            parent.guests.init();
            parent.attendance.init();
            parent.organizers.init();
            parent.locations.init();
            parent.booking.init();
            parent.schedules.init();
            parent.media.init();
            parent.tinymce.init();
            controller();

            if (glob.options.edit_event) {
                fillEditEventData();
            }

        };

    }

    window.stecBuilderSubmissionForm = StachethemesBuilderSubmissionForm;

})(window.jQuery);