(function ($) {

    'use strict';

    var proto;
    var moment = window.moment;
    moment.locale('en');

    window.StecCalData = function () {
        this.featuredFilter = 0;
        this.calendarFilter = [];
        this.calendarsPool = [];
        this.categoriesPool = [0];
        this.categoriesFilter = [0];
        this.locationsPool = ['none'];
        this.locationsFilter = ['none'];
        this.organizersPool = ['none'];
        this.organizersFilter = ['none'];
        this.helper = '';
        this.glob = '';
        this.eventsPool = [];
        this.preProcessFrom = moment.utc().startOf('day').subtract(2, 'years').toDate();
        this.preProcessTo = moment.utc().endOf('day').add(2, 'years').toDate();
        this.preProcessEvents = true;
    };

    proto = window.StecCalData.prototype;

    proto.set = function (obj, data) {

        if (typeof this[obj] !== 'undefined') {
            this[obj] = data;
        }

        return this;

    };

    /**
     * Returns event by id
     * 
     * @param {int} eventId
     * @returns event object
     * @todo add params for time offset
     */
    proto.getEventById = function (eventId) {

        var event = [];

        $(this.eventsPool).each(function () {
            if (parseInt(this.id, 10) === parseInt(eventId, 10)) {
                event = this;
                return false; // break;
            }
        });

        return event;
    };

    // sort by featured
    // @todo recheck solution...
    proto.sortByFeatured = function (a, b) {

        return 0;

        if (a.featured !== 0 && b.featured !== 0) {
            return a.start_date_timestamp - b.start_date_timestamp;
        } else {
            return a.featured - b.featured;
        }

    };

    // sort by timestamp oldest -> newest
    proto.sortByTimestamp = function (a, b) {
        if (a.start_date_timestamp < b.start_date_timestamp)
            return -1;
        else if (a.start_date_timestamp > b.start_date_timestamp)
            return 1;
        else
            return 0;
    };

    proto.removeFromEventsPool = function (eventId) {

        var newPool = []; // probably the safest way to do it...

        $(this.eventsPool).each(function () {
            if (this.id !== eventId) {
                newPool.push(this);
            }
        });

        this.eventsPool = newPool;
    };

    /**
     * Add events to the events ppol
     * Note since 2.9: Processes rrules
     * @param {type} events
     */
    proto.addToEventsPool = function (events) {

        this.helper.extendBind("stachethemes_ec_extend", "beforeProcessEvents");

        if (true === this.preProcessEvents) {
            this.eventsPool = this.eventsPool.concat(this.processEvents(this.preProcessFrom, this.preProcessTo, events));
        } else {
            $.each(events, function () {
                this.start_date_timestamp = moment.tz(this.start_date, this.timezone).unix();
                this.end_date_timestamp = moment.tz(this.end_date, this.timezone).unix();
            });
            this.eventsPool = this.eventsPool.concat(events);
        }

        this.helper.extendBind("stachethemes_ec_extend", "onAddToEventsPool", this.eventsPool);
    };

    proto.addDataToEvent = function (data) {

        var parent = this;

        $(parent.eventsPool).each(function () {
            if (parseInt(this.id, 10) === parseInt(data.general.id), 10) {
                this.data = data;
            }
        });
    };

    /**
     * Pulls all events from now up to 12 months in the future, including repeated events
     */
    proto.getFutureEvents = function () {

        var events;
        var date = new Date(this.glob.options.year, this.glob.options.month, this.glob.options.day);
        var a = date;
        var b = new Date(a);
        var ahead = parseInt(this.glob.options.future_months_ahead, 10);

        this.glob.options.future_months_ahead = isNaN(ahead) ? 12 : ahead;

        b.setMonth(b.getMonth() + this.glob.options.future_months_ahead);

        events = this.getEvents(a, b);

        return events;
    };

    /**
     * 
     * Return all events for given timespan
     * IF EVENT IS REPEATED IT WILL SEARCH FOR NEAREST 
     * TO DATE RECURRENCE AND WILL RETURN ONLY THIS INSTANCE
     * Nearest requirement: Must be in between range and closest to now
     * 
     * @param {date} startDate
     * @param {date} endDate
     * @param {bool} incAapproval
     * @returns {array} 
     */
    proto.getNearestDateEvents = function (startDate, endDate, incAapproval) {

        var parent = this,
                a,
                b,
                overrides,
                pick = [],
                now = moment().unix();

        if (!incAapproval) {
            incAapproval = false;
        }

        a = this.getRangeUnix(startDate);
        b = this.getRangeUnix(endDate ? endDate : startDate, true);

        // Get overrides in this range and add them to "pick" array
        overrides = this.getOverridesInRange(a, b);
        if (overrides.length > 0) {
            parent.filterGetEvents = [];
            parent.filterGetEvents = overrides;
            overrides = parent.filterGetEvents;
            $.merge(pick, parent.filterGetEvents);
        }

        // Exclude overriden
        parent.filterGetEvents = [];
        parent.filterGetEvents = $.grep(this.eventsPool, function (poolEvent) {
            return !poolEvent.recurrence_id;
        });

        this.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

        $(parent.filterGetEvents).each(function () {

            var eventLoop = this;

            if (incAapproval !== true && parseInt(this.approved, 10) === 0) {
                return true;
            }

            // Process repeater if events were not preprocessed
            if (false === parent.preProcessEvents && this.rrule !== '') {
                eventLoop = parent.processEvents(startDate, endDate, this);
            }

            // Picker
            $(eventLoop).each(function () {

                var start, end;

                if (overrides.length > 0 && parent.isOverriden(this)) {
                    return true; // continue;
                }

                start = this.start_date_timestamp;
                end = this.end_date_timestamp;

                // if param expired_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.expired_only, 10) === 1) {
                    if (end > now) {
                        return true; // continue;
                    }
                } else

                // if param upcoming_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.upcoming_only, 10) === 1) {
                    if (start < now) {
                        return true; // continue;
                    }
                }

                if (b > start && end >= a) {
                    pick.push(this);
                }
            });

        });

        // reorder full events by timestamp
        parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

        // reorder by featured
        if (this.glob.options.view !== 'agenda' && this.glob.options.view !== 'grid' && this.glob.options.view !== 'boxgrid') {
            parent.filterGetEvents.sort(this.sortByFeatured);
        }

        this.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

        return parent.filterGetEvents;
    };

    /**
     * Helper function for getEvents()
     * Gets Year Month and Date from date or string
     * Dates are treated as UTC
     * returns the utc timespan for given date
     * 
     * @param {mixed} JS Date or String
     * @param {bool} endOf adds 24 hours to the date
     * @returns {string} unixstamp
     */
    proto.getRangeUnix = function (dateOrString, endOf) {

        var momentDate;
        var dateArray = null;

        if (!dateOrString) {
            dateArray = [this.glob.options.year, this.glob.options.month, this.glob.options.day];
        } else if (true === dateOrString instanceof Date) {
            dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
        } else if ('string' === typeof dateOrString) {
            dateOrString = moment.utc(dateOrString).toDate();
            dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
        } else {
            console.log('Unsupported dateOrString');
        }

        momentDate = window.moment.tz('UTC');
        momentDate.set({year: dateArray[0], month: dateArray[1], date: dateArray[2], hour: 0, minute: 0, second: 0, millisecond: 0});

        if (endOf) {
            momentDate.add(24, 'hours');
        }

        return momentDate.unix();

    };

    /**
     * Return all events for given timespan
     * @param {Date} startDate
     * @param {Date} endDate
     * @param {bool} incAapproval
     * @returns {Array|Window.filterGetEvents}
     */
    proto.processEvents = function (startDate, endDate, events) {

        var parent = this,
                a,
                b,
                pick = [],
                now = moment().unix();

        a = this.getRangeUnix(startDate);
        b = this.getRangeUnix(endDate ? endDate : startDate, true);

        $(events).each(function () {

            var eventLoop = this;

            // Add unix start/end stamp
            if (!this.start_date_timestamp) {
                this.start_date_timestamp = moment.tz(this.start_date, this.timezone).unix();
                this.end_date_timestamp = moment.tz(this.end_date, this.timezone).unix();
            }

            //  If repeating event get repeater
            if (this.rrule && this.rrule !== '') {

                try {
                    eventLoop = parent.repeaterGet(this, a, b);
                } catch (err) {
                    console.log(err);
                    console.error('Error processing repeater for event ' + this.id, 'with RRULE String: ' + this.rrule);
                }

            }

            // Picker
            $(eventLoop).each(function () {
                
                var start, end;

                start = this.start_date_timestamp;
                end = this.end_date_timestamp;

                // if param expired_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.expired_only, 10) === 1) {
                    if (end > now) {
                        return true; // continue;
                    }
                } else

                // if param upcoming_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.upcoming_only, 10) === 1) {
                    if (start < now) {
                        return true; // continue;
                    }
                }

                if (b > start && end >= a) {
                    pick.push(this);
                }
            });

        });

        return pick;
    };

    /**
     * Return all events for given timespan
     * Note version > 2.9: No longer processes RRULE strings
     * @param {Date} startDate
     * @param {Date} endDate
     * @param {bool} incAapproval
     * @returns {Array|Window.filterGetEvents}
     */
    proto.getEvents = function (startDate, endDate, incAapproval) {
        
        var parent = this,
                a,
                b,
                pick = [],
                overrides,
                now = moment().unix();

        if (!incAapproval) {
            incAapproval = false;
        }

        a = this.getRangeUnix(startDate);
        b = this.getRangeUnix(endDate ? endDate : startDate, true);

        // Get overrides in this range and add them to "pick" array
        overrides = this.getOverridesInRange(a, b);
        if (overrides.length > 0) {
            parent.filterGetEvents = [];
            parent.filterGetEvents = overrides;
            overrides = parent.filterGetEvents;
            $.merge(pick, parent.filterGetEvents);
        }

        // Exclude overriden
        parent.filterGetEvents = [];
        parent.filterGetEvents = $.grep(this.eventsPool, function (poolEvent) {
            return !poolEvent.recurrence_id;
        });

        this.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

        $(parent.filterGetEvents).each(function () {

            var eventLoop = this;

            if (incAapproval !== true && parseInt(this.approved, 10) === 0) {
                return true;
            }
            
            // Process repeater if events were not preprocessed
            if (false === parent.preProcessEvents && this.rrule !== '') {
                eventLoop = parent.processEvents(startDate, endDate, this);
            }

            // Picker
            $(eventLoop).each(function () {
                var start, end;

                if (overrides.length > 0 && parent.isOverriden(this)) {
                    return true; // continue;
                }
                
               /**
                * Update 3.0.1
                * Since time is relative treat dates as UTC
                */
                start = moment.tz(this.start_date, 'UTC').unix();
                end = moment.tz(this.end_date, 'UTC').unix();
                
                // if param expired_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.expired_only, 10) === 1) {
                    if (end > now) {
                        return true; // continue;
                    }
                } else

                // if param upcoming_only=1 in [stachethemes_ec]
                if (parseInt(parent.glob.options.upcoming_only, 10) === 1) {
                    if (start < now) {
                        return true; // continue;
                    }
                }

                if (b > start && end >= a) {
                    pick.push(this);
                }
            });

        });

        // reorder full events by timestamp
        parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

        // reorder by featured
        if (this.glob.options.view !== 'agenda' && this.glob.options.view !== 'grid' && this.glob.options.view !== 'boxgrid') {
            parent.filterGetEvents.sort(this.sortByFeatured);
        }

        this.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

        return parent.filterGetEvents;
    };

    proto.isOverriden = function (event) {

        var recurrenceId = moment.unix(event.start_date_timestamp).format('YMMDDTHmss');
        var overrides = this.getOverrides(event.uid, recurrenceId);
        if (overrides.length > 0) {
            return true;
        }

        return false;

    };

    proto.getOverridesInRange = function (a, b, filteredPool) {

        var pool = filteredPool ? filteredPool : this.eventsPool;

        return  $.grep(pool, function (event) {

            if (!event.recurrence_id) {
                return false;
            }

            if (b > event.start_date_timestamp && event.end_date_timestamp >= a) {
                return true;
            }

            return false;
        });
    };

    /**
     * Looks for recurrence-id override
     * @param {string} uid 
     * @param {string} recurrenceId
     * 
     * The "recurrence_id" value must contain the override time in format YYYYMMDDTHHmmss
     * 
     * @return Array
     */
    proto.getOverrides = function (uid, recurrenceId) {

        return  $.grep(this.eventsPool, function (poolEvent) {

            var m, offset = 0;

            if (poolEvent.recurrence_id && poolEvent.recurrence_id.indexOf('Z') !== -1) {

                m = new moment(poolEvent.recurrence_id);

                if (m.isDST()) {
                    offset = -1 * ((poolEvent.timezone_utc_offset - 3600) / 3600);
                } else {
                    offset = -1 * (poolEvent.timezone_utc_offset / 3600);
                }

                m.utcOffset(offset);

                poolEvent.recurrence_id = m.format('YYYYMMDDTHHmmss');

            }

            return uid === poolEvent.uid && poolEvent.recurrence_id === recurrenceId;
        });
    };

    proto.repeaterGet = function (event, rangeStart, rangeEnd) {

        var parent = this;

        var eventStartDate = moment.utc(event.start_date);
        var rangeStartDate = moment.unix(rangeStart).utc();
        var rangeEndDate = moment.unix(rangeEnd).utc();
        var rfcString = event.rrule + ';DTSTART=' + this.helper.dateToRFC(eventStartDate);

        var rruleSet;
        var rEvents = [];
        var result;

        var exdates;
        var exdatesFixedArray;
        
        if (parent.preProcessEvents === false && eventStartDate.isBefore(rangeStartDate)) {
            rangeStartDate = eventStartDate;
        }

        if (event.exdate) {

            exdates = event.exdate.split(',');
            exdatesFixedArray = [];

            $.each(exdates, function () {

                var dt, st;

                if (this.length === 8) {
                    
                    st = moment.tz(event.start_date, event.timezone);
                    dt = moment.tz(this, event.timezone);
                    
                    exdatesFixedArray.push(dt
                            .hours(st.hours())
                            .minutes(st.minutes())
                            .seconds(0)
                            .format('YYYYMMDD\THHmmss') + 'Z');

                } else {

                    exdatesFixedArray.push(this);

                }
            });
            
            rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');
            
        }

        rruleSet = window.rrule.rrulestr(rfcString, {
            forceset: true
        });

        result = rruleSet.between(rangeStartDate.toDate(), rangeEndDate.toDate(), true);

        $.each(result, function () {
            var rEvent = parent.getREvent(event, this);

            if (rEvent.end_date_timestamp < rangeStart) {
                return true;
            }

            rEvents.push(rEvent);
        });

        return rEvents;

    };

    proto.getREvent = function (event, date) {

        var rEvent;
        var offset = 0;
        var initialStartDate = moment.tz(event.start_date, event.timezone);
        var initialEndDate = moment.tz(event.end_date, event.timezone);
        var occurenceDate = moment.tz(date, event.timezone);
        occurenceDate.subtract(occurenceDate.utcOffset(), 'minutes');

        offset = occurenceDate.diff(initialStartDate, 'seconds');

        // not by reference! need fresh copy
        rEvent = JSON.parse(JSON.stringify(event));

        rEvent.start_date_timestamp = initialStartDate.unix() + offset;
        rEvent.end_date_timestamp = initialEndDate.unix() + offset;

        rEvent.start_date = initialStartDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
        rEvent.end_date = initialEndDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
        rEvent.repeat_time_offset = offset;

        return rEvent;

    };

    proto.repeaterGetClosest = function (event, date) {

        var parent = this;
        var eventStartDate = moment.utc(event.start_date);
        var rfcString = event.rrule + ';DTSTART=' + this.helper.dateToRFC(eventStartDate);
        var exdates;
        var exdatesFixedArray;
        var rruleSet;
        var rEvents = [];
        var result;

        if (!date) {
            date = moment.utc().toDate();
        }

        if (event.exdate) {

            exdates = event.exdate.split(',');
            exdatesFixedArray = [];

            $.each(exdates, function () {

                var dt;

                if (this.length === 8) {

                    dt = moment(this);

                    exdatesFixedArray.push(dt
                            .hours(eventStartDate.toDate().getHours())
                            .minutes(eventStartDate.toDate().getMinutes())
                            .seconds(0)
                            .utc().format('YYYYMMDD\THHmmss') + 'Z');

                } else {

                    exdatesFixedArray.push(this);

                }
            });

            rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');
        }

        rruleSet = window.rrule.rrulestr(rfcString, {
            forceset: true
        });

        result = rruleSet.after(date, true);

        if (!result || result.length <= 0) {
            result = rruleSet.before(date, true);
        }

        if (!result || result.length <= 0) {
            return rEvents;
        }

        $(result).each(function () {

            var rEvent = parent.getREvent(event, this);

            rEvents.push(rEvent);

        });

        return rEvents;
    };

})(window.jQuery);