(function ($) {

    'use strict';

    var proto;
    var moment = window.moment;
    moment.locale('en');

    window.StecHelper = function () {
        this.animate = false;
        this.glob = {};
        this.calData = {};
        this.$instance = '';
        this.instance = '';
        this.layout = '';
        this.eventsHandler = '';

    };

    proto = window.StecHelper.prototype;

    proto.set = function (obj, data) {

        if (typeof this[obj] !== 'undefined') {
            this[obj] = data;
        }

        return this;
    };

    proto.isEmail = function (email) {
        var regex = /^([a-zA-Z0-9_.+-])+\@(([a-zA-Z0-9-])+\.)+([a-zA-Z0-9]{2,4})+$/;
        return regex.test(email);
    };

    /**
     * Date to ICS time string
     * @param {date} date
     * @returns yyyymmddThhmmssZ
     */
    proto.dateToRFC = function (date) {
        var dt = moment(date);
        var rfc = dt.utc().format('YYYYMMDDTHHmmss') + 'Z';
        return rfc;
    };

    proto.eventToGoogleCalImportLink = function (eventId, repeatOffset) {

        var event, d1, d2, startDate, endDate, description, location;

        event = this.calData.getEventById(eventId);

        d1 = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');
        d2 = moment.tz(event.end_date, event.timezone).add(repeatOffset, 'seconds');

        if (event.all_day) {
            startDate = d1.format('YYYYMMDD');
            endDate = moment(d2).add(1, 'day').format('YYYYMMDD');
        } else {
            startDate = this.dateToRFC(d1);
            endDate = this.dateToRFC(d2);
        }


        description = event.description_short;
        location = event.location.full_address ? event.location.full_address : '';

        return "//calendar.google.com/calendar/render?action=TEMPLATE&text=" + event.title + "&dates=" + startDate + "/" + endDate + "&details=" + description + "&location=" + location + "&sf=true&output=xml";
    };


    proto.beautifyScheduleTimespan = function (start) {

        var d1 = new moment(start),
                format = '',
                timeFormat = 'HH:mm',
                timeFormatSetting = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.time_format : this.glob.options.time_format,
                dateFormat = '',
                delimeter = '',
                trLabel = [],
                label;

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.date_format : this.glob.options.date_format;
        delimeter = dateFormat === 'dd.mm.yyyy' ? '. ' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'DD. MMM ';
                break;
            case 'dd-mm-yy' :
                format = 'DD MMM ';
                break;
            case 'mm-dd-yy' :
                format = 'MMM DD ';
                break;
            case 'yy-mm-dd' :
                format = 'MMM DD';
                break;
        }

        format += ' ' + timeFormat;

        label = d1.format(format).split(delimeter);
        trLabel = [];

        $.each(label, function () {

            var part = typeof window.stecLang[$.trim(this.toLowerCase())] !== 'undefined' ? window.stecLang[$.trim(this.toLowerCase())] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);

            trLabel.push(part);
        });

        return trLabel.join(delimeter);
    };

    proto.beautifyDate = function (date) {

        var d1,
                format,
                timeFormat,
                timeFormatSetting,
                dateFormat,
                delimeter,
                timespanLabel,
                trTimespanLabel;

        d1 = moment(date);

        format = '';
        timeFormat = 'HH:mm';

        timeFormatSetting = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.time_format : this.glob.options.time_format;

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.date_format : this.glob.options.date_format;

        delimeter = dateFormat === 'dd.mm.yyyy' ? '.' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'D. MMM. YYYY';
                break;
            case 'dd-mm-yy' :
                format = 'D MMM YYYY';
                break;
            case 'mm-dd-yy' :
                format = 'MMM D YYYY';
                break;
            case 'yy-mm-dd' :
                format = 'YYYY MMM D';
                break;
            default:
                format = dateFormat;
        }

        format += ' ' + timeFormat;

        timespanLabel = d1.format(format);
        timespanLabel = timespanLabel.split(delimeter);
        trTimespanLabel = [];

        $.each(timespanLabel, function () {
            var part = typeof window.stecLang[$.trim(this.toLowerCase())] !== 'undefined' ? window.stecLang[$.trim(this.toLowerCase())] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);
            trTimespanLabel.push(part);
        });

        trTimespanLabel = trTimespanLabel.join(delimeter);

        return trTimespanLabel[0].toUpperCase() + trTimespanLabel.substr(1);
    };

    /**
     * Returns user friendly timespan
     * Accepted params for start and end are String (DbDate) or Date object
     * @param {mixed} start
     * @param {mixed} end 
     * @param {int} allDay 1/0
     * @returns {String}
     * @since 2.0.0 uses moment
     */
    proto.beautifyTimespan = function (start, end, allDay) {

        var d1,
                d2,
                format,
                timeFormat,
                timeFormatSetting,
                dateFormat,
                delimeter,
                timespanLabel,
                formatEnd,
                trTimespanLabel;

        allDay = parseInt(allDay, 10);

        d1 = new moment(start);
        d2 = end ? new moment(end) : null;

        format = '';
        timeFormat = 'HH:mm';

        timeFormatSetting = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.time_format : this.glob.options.time_format;

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = $('.stec-layout-single').length <= 0 ? this.glob.options.general_settings.date_format : this.glob.options.date_format;

        delimeter = dateFormat === 'dd.mm.yyyy' ? '.' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'D. MMM. YYYY';
                break;
            case 'dd-mm-yy' :
                format = 'D MMM YYYY';
                break;
            case 'mm-dd-yy' :
                format = 'MMM D YYYY';
                break;
            case 'yy-mm-dd' :
                format = 'YYYY MMM D';
                break;
            default:
                format = dateFormat;
        }

        // Show time only if not all_day event
        if (parseInt(allDay, 10) !== 1) {
            format += ' ' + timeFormat;
        }

        if (end) {

            // Same Day
            if (d1.isSame(d2, 'day')) {

                timespanLabel = d1.format(format);

                if (parseInt(allDay, 10) !== 1) {
                    timespanLabel += " - " + d2.format(timeFormat);
                }

            } else

            // Same Month & Year
            if (d1.isSame(d2, 'month')) {

                formatEnd = format.replace('YYYY', '');

                timespanLabel = d1.format(format);
                timespanLabel += " - " + d2.format(formatEnd);

            }

            // Default
            else {

                timespanLabel = d1.format(format);
                timespanLabel += " - " + d2.format(format);
            }

        } else {
            timespanLabel = d1.format(format);
        }

        timespanLabel = timespanLabel.split(delimeter);
        trTimespanLabel = [];

        $.each(timespanLabel, function () {

            var part = typeof window.stecLang[$.trim(this.toLowerCase())] !== 'undefined' ? window.stecLang[$.trim(this.toLowerCase())] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);

            trTimespanLabel.push(part);
        });

        trTimespanLabel = trTimespanLabel.join(delimeter);

        return trTimespanLabel[0].toUpperCase() + trTimespanLabel.substr(1);

    };

    proto.diffDays = function (dt1, dt2) {
        return Math.abs(new moment(dt2).diff(dt1, 'days'));
    };

    proto.diffWeeks = function (d1, d2) {
        return Math.abs(new moment(d2).diff(d1, 'weeks'));
    };

    proto.focus = function (el) {

        if (parseInt(this.glob.options.general_settings.event_auto_focus, 10) !== 1) {
            return;
        }

        $('html, body').animate({
            scrollTop: $(el).offset().top - $("#wpadminbar").height() + parseInt(this.glob.options.general_settings.event_auto_focus_offset, 10)
        }, {
            duration: 750,
            easing: "stecExpo"
        });
    };

    proto.nl2br = function (txt) {
        return txt.replace(/(\r\n|\n\r|\r|\n)/g, "<br>");
    };

    proto.capitalize = function (text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    };

    proto.imgLoaded = function ($img, callback, step) {

        var total, loaded;

        if (typeof $.fn.imagesLoaded !== "undefined") {
            // imagesLoaded script is loaded
            $img.imagesLoaded(function () {
                if (typeof callback === "function") {
                    callback.call($img);
                }
            });
            return;
        }

        total = $img.length;
        loaded = 0;

        if (total <= 0) {
            if (typeof callback === "function") {
                callback.call($img);
            }
        }

        $img.each(function () {

            var image = new Image();

            image.onload = function () {

                if (typeof step === "function") {
                    step();
                }

                loaded++;
                if (loaded >= total) {
                    if (typeof callback === "function") {
                        callback.call($img);
                    }
                }
            };

            image.onerror = function () {
                console.warn("Could not load image");
            };

            image.src = $(this).attr("src");
        });
    };

    // @todo change with date.UTC ?
    // used for dateToRFC
    proto.treatAsUTC = function (date) {

        var result = date instanceof Date ? date : new Date(date);
        var adjustedMinutes = result.getMinutes() + result.getTimezoneOffset();
        result.setMinutes(adjustedMinutes);
        return result;
    };

    proto.capitalizeFirstLetter = function (string) {
        return string.charAt(0).toUpperCase() + string.slice(1);
    };

    /**
     * @param {string} dbDate string
     * @param {int} offset seconds
     * @returns {date} date object
     * @since 2.0.0 uses moment.js
     */
    proto.dbDateOffset = function (string, offset) {
        return moment(string).add(offset, 'seconds').toDate();
    };

    /**
     * expects months range 1-12
     * converts to 0-11
     * @param {str} string yy-mm-dd h:i:s
     * @since 2.0.0 uses moment.js
     */
    proto.dbDateTimeToDate = function (string) {
        return new moment(string).toDate();
    };

    /*
     * Converts js date to db date
     * @param {obj} date
     * @returns {String}
     */
    proto.dateToDbDateTime = function (date) {

        var string = '';

        var y = date.getFullYear();
        var m = date.getMonth() + 1;
        var d = date.getDate();
        var h = date.getHours();
        var i = date.getMinutes();
        var s = date.getSeconds();

        string += y;
        string += '-';
        string += m < 10 ? '0' + m : m;
        string += '-';
        string += d < 10 ? '0' + d : d;
        string += ' ';
        string += h < 10 ? '0' + h : h;
        string += ':';
        string += i < 10 ? '0' + i : i;
        string += ':';
        string += s < 10 ? '0' + s : s;

        return string;
    };

    // @since 2.0.0
    proto.dateToUnixStamp = function (date) {
        return new moment(date).unix();
    };

    proto.getColorBrightness = function (hex) {

        var rgb, r, g, b, luma;

        /*jslint bitwise: true */

        hex = hex.substring(1);      // strip #
        rgb = parseInt(hex, 16); // convert rrggbb to decimal
        r = (rgb >> 16) & 0xff;  // extract red
        g = (rgb >> 8) & 0xff;   // extract green
        b = (rgb >> 0) & 0xff;   // extract blue
        luma = 0.2126 * r + 0.7152 * g + 0.0722 * b; // per ITU-R BT.709
        return luma;
    };

    proto.extendBind = function (funcArr, sub, rtrn) {

        var parent = this;

        if (typeof window[funcArr] !== "undefined") {

            if (sub) {

                if (typeof window[funcArr][sub] !== "undefined") {
                    $(window[funcArr][sub]).each(function () {

                        if (typeof this === "function") {

                            this({
                                instance: parent.instance,
                                $instance: parent.$instance,
                                glob: parent.glob,
                                helper: parent,
                                calData: parent.calData,
                                layout: parent.layout,
                                eventsHandler: parent.eventsHandler
                            }, rtrn ? rtrn : null);
                        }
                    });
                }

            } else {

                $(window[funcArr]).each(function () {

                    if (typeof this === "function") {

                        this({
                            instance: parent.instance,
                            $instance: parent.$instance,
                            glob: parent.glob,
                            helper: parent,
                            calData: parent.calData,
                            layout: parent.layout,
                            eventsHandler: parent.eventsHandler
                        }, rtrn ? rtrn : null);

                    }
                });

            }
        }

    };

    proto.onResizeEnd = function (callback, time, keyslug) {
        var id, handle;
        time = time ? time : 10;
        handle = keyslug ? 'resize.' + keyslug : 'resize';

        $(window).on(handle, function () {

            clearTimeout(id);
            id = setTimeout(function () {
                if (typeof callback === "function") {
                    callback.call();
                }
            }, time);
        });
    };

    proto.iso8601Week = function (date) {
        var time;
        var checkDate = new Date(date.getTime());

        // Find Thursday of this week starting on Monday
        checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));

        time = checkDate.getTime();
        checkDate.setMonth(0); // Compare with Jan 1
        checkDate.setDate(1);
        return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
    };

    /**
     * @param {Date} alt new Date()
     */
    proto.getWeekInfo = function (alt) {

        var year, month, day, activeDate, firstDayOfTheWeek, lastDayOfTheWeek;

        if (alt) {
            year = alt.getFullYear();
            month = alt.getMonth();
            day = alt.getDate();
        } else {
            year = this.glob.options.year;
            month = this.glob.options.month;
            day = this.glob.options.day;
        }

        activeDate = new Date(year, month, day);

        switch (this.glob.options.general_settings.first_day_of_the_week) {
            case 'mon' :

                if (activeDate.getDay() !== 1) {
                    while (activeDate.getDay() !== 1) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;

            case 'sat' :

                if (activeDate.getDay() !== 6) {
                    while (activeDate.getDay() !== 6) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;

            case 'sun' :

                if (activeDate.getDay() !== 0) {
                    while (activeDate.getDay() !== 0) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;
        }

        firstDayOfTheWeek = new Date(activeDate);
        lastDayOfTheWeek = new Date(firstDayOfTheWeek);
        lastDayOfTheWeek.setDate(lastDayOfTheWeek.getDate() + 6);

        return {
            start: {
                day: firstDayOfTheWeek.getDate(),
                month: firstDayOfTheWeek.getMonth(),
                year: firstDayOfTheWeek.getFullYear()
            },
            end: {
                day: lastDayOfTheWeek.getDate(),
                month: lastDayOfTheWeek.getMonth(),
                year: lastDayOfTheWeek.getFullYear()
            },
            week: this.iso8601Week(activeDate)
        };
    };

    proto.getMonthInfo = function (month, year) {

        var daysInMonth, firstDay, startingDay, monthLength, monthInfo, dayOffset = '';

        if (isNaN(month)) {
            month = parseInt(this.glob.options.month, 10);
        }

        if (isNaN(year)) {
            year = parseInt(this.glob.options.year, 10);
        }

        daysInMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

        firstDay = new Date(year, month, 1);
        startingDay = firstDay.getDay();

        monthLength = daysInMonth[month];

        if (month === 1) {
            if ((year % 4 === 0 && year % 100 !== 0) || year % 400 === 0) {
                monthLength = 29;
            }
        }

        switch (this.glob.options.general_settings.first_day_of_the_week) {
            case 'mon' :
                dayOffset = startingDay - 1 < 0 ? startingDay + 6 : startingDay - 1;
                break;

            case 'sat' :
                dayOffset = startingDay - 7 < 0 ? startingDay + 1 : startingDay - 7;

                break;

            case 'sun' :
                dayOffset = startingDay;
                break;
        }

        monthInfo = {
            startingDay: startingDay,
            monthLength: monthLength,
            year: year,
            month: month,
            dayOffset: dayOffset,
            monthName: this.glob.options.monthLabels[month],
            monthNameShort: this.glob.options.monthLabelsShort[month]
        };

        return monthInfo;
    };

    /**
     * data-date="yyyy-mm-dd" to Date()
     * @param {string} str yyyy-mm-dd
     * @returns {Date} returns new Date()
     */
    proto.getDateFromData = function (str) {
        var d = str.split('-');
        return new Date(d[0], d[1], d[2]);
    };

    /**
     * Date() to yyy-mm-dd string
     * @param {Date} date
     * @returns {String}
     */
    proto.getDataFromDate = function (date) {
        return date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
    };

    /**
     * Translates labels
     * @param {string} label
     * @param {object} week
     * @returns string
     */
    proto.getWeekLabel = function (label, week) {

        return  label.replace(/sday/g, week.start.day)
                .replace(/smonth/g, this.glob.options.monthLabelsShort[week.start.month])
                .replace(/syear/g, week.start.year)
                .replace(/eday/g, week.end.day)
                .replace(/emonth/g, this.glob.options.monthLabelsShort[week.end.month])
                .replace(/eyear/g, week.end.year);

    };

    proto.isMobile = function () {
        return (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent)) ? true : false;
    };

    proto.clickHandle = function (sub) {

        var tap = sub ? 'vclick.' + sub : 'vclick';
        var click = sub ? 'click.' + sub : 'click';


        return this.isMobile() ? tap : click;

    };

    proto.instaClickHandle = function () {
        return this.isMobile() ? "touchstart" : "mousedown";
    };

    proto.insertAtIndex = function (str, index, insert) {
        return [str.slice(0, index), insert, str.slice(index)].join('');
    };

    proto.getPermalink = function (permalink, offset) {
        var argPos = null;

        if (offset && parseInt(offset, 10) > 0) {
            argPos = permalink.indexOf('?');
            if (-1 !== argPos) {
                permalink = this.insertAtIndex(permalink, argPos, offset.toString() + '/');
            } else {
                permalink = this.insertAtIndex(permalink, permalink.length, offset.toString() + '/');
            }
        }
        return permalink;

    };

    proto.lang = function (textToTranslate) {
        return window.stecLang[textToTranslate] ? window.stecLang[textToTranslate] : textToTranslate;
    };

    proto.attendanceHasFreeSlots = function (event, date) {
        var freeSlots = event.rsvp_limit;

        if (event.rsvp_limit === 0) {
            return 9999;
        }

        if (event.attendance.length > 0) {
            $.each(event.attendance, function () {
                if (1 === parseInt(this.status[date], 10)) {
                    freeSlots = freeSlots - this.people;
                }
            });
        }

        return Math.max(0, freeSlots);
    };

    proto.getEventState = function (event, offset) {

        var nowUnix, startUnix, endUnix;

        if (!offset) {
            offset = 0;
        }

        nowUnix = moment().unix();
        startUnix = event.start_date_timestamp + offset;
        endUnix = event.end_date_timestamp + offset;

        if (nowUnix > endUnix) {
            return 2; // expired
        }

        if (nowUnix > startUnix) {
            return 1; // in progress
        }

        if (nowUnix < startUnix) {
            return 0; // not started
        }


    };

    proto.toast = function (msg, append, delay) {
        var html = '', element;

        if (!delay) {
            delay = 1000;
        }

        if (!append) {
            append = 'body';
        }

        $('.stec-toast').remove();

        html = ('<div class="stec-toast">' + msg + '</div>');

        element = $(html).appendTo(append);

        setTimeout(function () {
            element.fadeTo(250, 0, function () {
                $(this).remove();
            });
        }, delay);
    };

    proto.getMapStyles = function (theme) {

        var aubergine = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#242f3e"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#746855"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#242f3e"
                            }
                        ]
                    }, {
                        "featureType": "administrative.locality",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#263c3f"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#6b9a76"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#38414e"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#212a37"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9ca5b3"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#746855"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#1f2835"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#f3d19c"
                            }
                        ]
                    }, {
                        "featureType": "transit",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#2f3948"
                            }
                        ]
                    }, {
                        "featureType": "transit.station",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#17263c"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#515c6d"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#17263c"
                            }
                        ]
                    }
                ],
                {name: 'Aubergine'}

        );

        var retro = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#ebe3cd"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#523735"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#f5f1e6"
                            }
                        ]
                    }, {
                        "featureType": "administrative",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#c9b2a6"
                            }
                        ]
                    }, {
                        "featureType": "administrative.land_parcel",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#dcd2be"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#ae9e90"
                            }
                        ]
                    }, {
                        "featureType": "landscape.natural",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#93817c"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#a5b076"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#447530"
                            }
                        ]
                    },
                    {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f5f1e6"
                            }
                        ]
                    }, {
                        "featureType": "road.arterial",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#fdfcf8"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f8c967"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#e9bc62"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e98d58"
                            }
                        ]
                    }, {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#db8555"
                            }
                        ]
                    },
                    {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#806b63"
                            }
                        ]
                    }, {
                        "featureType": "transit.line",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.line",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#8f7d77"
                            }
                        ]
                    }, {
                        "featureType": "transit.line",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#ebe3cd"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.station",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#b9d3c2"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#92998d"
                            }
                        ]
                    }
                ],
                {name: 'Retro'}
        );

        var night = new window.google.maps.StyledMapType(
                [
                    {elementType: 'geometry', stylers: [{color: '#242f3e'}]}, {elementType: 'labels.text.stroke', stylers: [{color: '#242f3e'}]},
                    {elementType: 'labels.text.fill', stylers: [{color: '#746855'}]}, {
                        featureType: 'administrative.locality',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    },
                    {
                        featureType: 'poi',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    }, {
                        featureType: 'poi.park',
                        elementType: 'geometry',
                        stylers: [{color: '#263c3f'}]
                    },
                    {
                        featureType: 'poi.park',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#6b9a76'}]
                    }, {
                        featureType: 'road',
                        elementType: 'geometry',
                        stylers: [{color: '#38414e'}]
                    },
                    {
                        featureType: 'road',
                        elementType: 'geometry.stroke',
                        stylers: [{color: '#212a37'}]
                    }, {
                        featureType: 'road',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#9ca5b3'}]
                    },
                    {
                        featureType: 'road.highway',
                        elementType: 'geometry',
                        stylers: [{color: '#746855'}]
                    }, {
                        featureType: 'road.highway',
                        elementType: 'geometry.stroke',
                        stylers: [{color: '#1f2835'}]
                    },
                    {
                        featureType: 'road.highway',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#f3d19c'}]
                    }, {
                        featureType: 'transit',
                        elementType: 'geometry',
                        stylers: [{color: '#2f3948'}]
                    },
                    {
                        featureType: 'transit.station',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    }, {
                        featureType: 'water',
                        elementType: 'geometry',
                        stylers: [{color: '#17263c'}]
                    },
                    {
                        featureType: 'water',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#515c6d'}]
                    }, {
                        featureType: 'water',
                        elementType: 'labels.text.stroke',
                        stylers: [{color: '#17263c'}]
                    }
                ], {name: 'Night'}
        );

        var silver = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f5f5f5"
                            }
                        ]
                    }, {
                        "elementType": "labels.icon",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#f5f5f5"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#bdbdbd"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#eeeeee"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e5e5e5"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#ffffff"
                            }
                        ]
                    },
                    {
                        "featureType": "road.arterial",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dadada"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.line",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e5e5e5"
                            }
                        ]
                    }, {
                        "featureType": "transit.station",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#eeeeee"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#c9c9c9"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    }
                ], {name: 'Silver'}
        );


        var dark = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#212121"
                            }
                        ]
                    }, {
                        "elementType": "labels.icon",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#212121"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "administrative.country",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    }, {
                        "featureType": "administrative.locality",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#bdbdbd"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#181818"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#1b1b1b"
                            }
                        ]
                    },
                    {
                        "featureType": "road",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#2c2c2c"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#8a8a8a"
                            }
                        ]
                    },
                    {
                        "featureType": "road.arterial",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#373737"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#3c3c3c"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#4e4e4e"
                            }
                        ]
                    }, {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    },
                    {
                        "featureType": "transit",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#3d3d3d"
                            }
                        ]
                    }], {name: 'Dark'}
        );

        switch (theme) {

            case 'silver' :
                return silver;

            case 'aubergine' :
                return aubergine;

            case 'retro' :
                return retro;

            case 'night' :
                return night;

            case 'dark' :
                return dark;

            default:
                return false;
        }
    };

})(window.jQuery);