(function ($) {

    'use strict';

    var Helper = new window.StecHelper();

    var proto;
    window.StecScroll = function () {
        this.currentPosition = 0;
        this.speed = 40;
        this.ratio = 1;
        this.originalStyle = null;
        this.container = null;
        this.id = 'stec-scroll-id-' + Math.random();
        this.inner = null;
        this.outer = null;
        this.onScrollCallback = null;
    };

    proto = window.StecScroll.prototype;

    proto.create = function (container) {

        var scrollHeightInPercents;
        var innerHeight;
        var outerHeight;

        this.container = container;
        
        if ($(container).height() >= $(container).children().height()) {
            return;
        }

        this.originalStyle = {
            position: $(container).css('position'),
            height: $(container).css('height'),
            maxHeight: $(container).css('max-height'),
            overflow: $(container).css('overflow')
        };

        this.inner = $(container)
                .wrapAll('<div class="stec-scroll-container"></div>');

        this.outer = this.inner.parent('.stec-scroll-container');

        this.outer.css({
            position: 'relative',
            height: this.inner.height(),
            maxHeight: this.inner.css('max-height'),
            overflow: 'hidden'
        });

        this.inner.css({
            position: 'relative',
            height: 'initial',
            maxHeight: 'initial',
            overflow: 'visible'
        });

        scrollHeightInPercents = 0;
        innerHeight = this.inner.outerHeight();
        outerHeight = this.outer.outerHeight();

        scrollHeightInPercents = 100 - ((Math.abs((outerHeight - innerHeight)) / innerHeight) * 100) + '%';

        this.scrollBar = $('<div class="stec-scroll-bar"></div>')
                .height(scrollHeightInPercents)
                .appendTo(this.outer);

        this.ratio = Math.abs(this.outer.outerHeight() / this.scrollBar.outerHeight());

        if (Helper.isMobile()) {
            this.mobileController();
        } else {
            this.Controller();
        }

    };

    proto.reset = function () {
        this.destroy();
        this.create(this.container);
    };

    proto.destroy = function () {

        if (this.inner) {
            this.inner.css(this.originalStyle);
        }
        if (this.outer) {
            this.outer.find('.stec-scroll-bar').remove();
            this.outer.contents().unwrap();
        }
        $(document).off('keydown.' + this.id);
    };

    proto.scrollUp = function () {

        this.currentPosition += this.speed;

        if (this.currentPosition > 0) {
            this.currentPosition = 0;
        }

        this.inner.css({
            top: this.currentPosition
        });

        this.positionScrollBar();
    };

    proto.scrollDown = function () {

        var diff = Math.abs(this.outer.outerHeight() - this.inner.outerHeight());

        this.currentPosition -= this.speed;

        if (Math.abs(this.currentPosition) > diff) {
            this.currentPosition = -1 * diff;
        }

        this.inner.css({
            top: this.currentPosition
        });

        this.positionScrollBar();

    };

    proto.positionScrollBar = function () {

        this.scrollBar.css({
            top: -1 * this.inner.position().top / this.ratio
        });

    };

    proto.scrollTo = function (value) {

        var diff = Math.abs(this.outer.outerHeight() - this.inner.outerHeight());

        if (Math.abs(value) > diff) {
            value = -1 * diff;
        }

        if (value > 0) {
            value = 0;
        }

        this.inner.css('top', value);
        this.currentPosition = value;
        this.positionScrollBar();

        if (typeof this.onScrollCallback === 'function') {
            this.onScrollCallback(this.currentPosition);
        }
    };

    proto.onScroll = function (callback) {
        this.onScrollCallback = callback;
    };

    proto.mobileController = function () {

        var parent = this;

        this.container.css({
            'overflow-y': 'scroll',
            'max-height': ''
        });

        this.outer.on('touchmove', function (e) {
            parent.currentPosition = parent.container.scrollTop();
            if (typeof parent.onScrollCallback === 'function') {
                parent.onScrollCallback(parent.currentPosition);
            }
        });

        return;
    };

    proto.Controller = function () {

        var parent = this;

        this.outer.on('mousewheel DOMMouseScroll', function (event) {

            if (event.originalEvent.wheelDelta > 0 || event.originalEvent.detail < 0) {

                parent.scrollUp();

            } else {

                parent.scrollDown();

            }

            if (typeof parent.onScrollCallback === 'function') {
                parent.onScrollCallback(parent.currentPosition);
            }

            event.preventDefault();
        });

        this.scrollBar.draggable({
            containment: parent.outer,
            axis: 'y',
            drag: function (e, ui) {

                var offset = parent.scrollBar.position().top * parent.ratio;

                parent.inner.css({
                    top: -1 * offset
                });

                parent.currentPosition = -1 * offset;

                if (typeof parent.onScrollCallback === 'function') {
                    parent.onScrollCallback(parent.currentPosition);
                }

            },
            stop: function () {

            }
        });

        this.outer.hover(function () {
            $(this).addClass('active');
        }, function () {
            if (!parent.scrollBar.hasClass('ui-draggable-dragging')) {
                $(this).removeClass('active');
            }
        });

        $(document).on('keydown.' + this.id, function (e) {

            switch (e.which) {
                case 38:
                case 33:
                    e.preventDefault();
                    parent.scrollUp();
                    break;
                case 40:
                case 34:
                    e.preventDefault();
                    parent.scrollDown();
                    break;
            }

        });

    };

})(window.jQuery);