<?php

namespace Stachethemes\Stec;

class AssetsCompressor {

    private static $cache_js_folder  = 'js/front/';
    private static $cache_css_folder = 'css/front/';
    private static $file_prefix      = 'ac_cache_';

    private static function get_cache_js_folder() {
        $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();
        return $stachethemes_ec_main->get_path('ASSETS') . self::$cache_js_folder;
    }

    private static function get_cache_css_folder() {
        $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();
        return $stachethemes_ec_main->get_path('ASSETS') . self::$cache_css_folder;
    }

    private static function get_file_name() {
        global $post;

        switch ($post->post_type) {
            case 'stec_event' :
                return self::$file_prefix . $post->post_type;

            default:
                return self::$file_prefix . $post->ID;
        }
    }

    private static function get_js_enqueues() {
        $wp_scripts = wp_scripts();
        $exclude    = apply_filters('stec_ac_exclude_js', array('stec-google-captcha-api', 'stec-google-maps'));
        $scripts    = array();

        foreach ($wp_scripts->queue as $script) :

            if (strpos($script, 'stec-') === false) {
                continue;
            }

            if (true === in_array($script, $exclude)) {
                continue;
            }

            $scripts[] = $wp_scripts->registered[$script];
        endforeach;

        return $scripts;
    }

    private static function get_css_enqueues() {
        global $wp_styles;

        $styles  = array();
        $exclude = apply_filters('stec_ac_exclude_css', array('stec-ui', 'stec-font-awesome', 'stec-colorpicker'));

        foreach ($wp_styles->queue as $style) :

            if (strpos($style, 'stec-') === false) {
                continue;
            }

            if (true === in_array($style, $exclude)) {
                continue;
            }

            $styles[] = $wp_styles->registered[$style];
        endforeach;

        return $styles;
    }

    private static function deregister_js($scripts) {
        foreach ($scripts as $script) {
            wp_deregister_script($script->handle);
        }
    }

    private static function deregister_css($styles) {
        foreach ($styles as $style) {
            wp_deregister_style($style->handle);
        }
    }

    private static function minify_js($js) {

        $url  = esc_url('https://javascript-minifier.com/raw');
        $data = array(
            'input' => $js,
        );

        $minified = wp_remote_post($url, array(
            'timeout' => 60,
            'body'    => $data
        ));

        if (is_wp_error($minified)) {
            return $js;
        }

        return isset($minified['body']) ? $minified['body'] : $js;
    }

    private static function minify_css($css) {
        $css = preg_replace('!/\*[^*]*\*+([^/][^*]*\*+)*/!', '', $css);
        $css = str_replace(': ', ':', $css);
        $css = str_replace(array("\r\n", "\r", "\n", "\t", '  ', '    ', '    '), '', $css);
        return $css;
    }

    public static function compress_js() {

        global $post;

        $stachethemes_ec = Stachethemes_Event_Calendar::get_instance();
        $scripts         = self::get_js_enqueues();
        $fs              = $stachethemes_ec->get_file_system();
        $folder          = self::get_cache_js_folder();

        /**
         * Deregister calendar scripts
         */
        self::deregister_js($scripts);

        /**
         * Check cache exists
         */
        if (!$fs->exists($folder . self::get_file_name() . '.js')) {

            /**
             * Load files and put them in cache file
             */
            $cache = array();
            foreach ($scripts as $script) {

                $r = wp_remote_get($script->src);
                if (is_wp_error($r)) {
                    continue;
                }
                $cache[] = $r['body'];
            }

            $content = self::minify_js(implode(PHP_EOL, $cache));

            if (!$fs->is_dir($folder)) {
                $fs->mkdir($folder);
            }

            if (false === $fs->put_contents($folder . self::get_file_name() . '.js', $content, FS_CHMOD_FILE)) {

                if (is_wp_error($fs->errors) && $fs->errors->get_error_code()) {
                    throw new Stec_Exception($fs->errors->get_error_message());
                }

                throw new Stec_Exception(esc_html__('Unexpected error', 'stec'));
            }

            $stachethemes_ec->add_asset_js('stec-js-cache', self::$cache_js_folder . self::get_file_name() . '.js');
        }

        foreach ($scripts as $script) {

            if ($script->deps) {
                foreach ($script->deps as $dep) {
                    wp_enqueue_script($dep);
                }
            }

            if ($script->extra) {
                foreach ($script->extra as $extra) {
                    add_action('wp_head', function() use ($extra) {
                        ?>
                        <script type="text/javascript">/* <![CDATA[ */
                        <?php echo Admin_Helper::esc($extra, 'raw'); ?>
                            /* ]]> */</script>
                        <?php
                    });
                }
            }
        }
        $stachethemes_ec->add_asset_js('stec-js-cache', self::$cache_js_folder . self::get_file_name() . '.js','jquery,jquery-ui-draggable,jquery-ui-datepicker');
    }

    public static function compress_css() {

        global $post;

        $stachethemes_ec = Stachethemes_Event_Calendar::get_instance();
        $styles          = self::get_css_enqueues();
        $fs              = $stachethemes_ec->get_file_system();
        $folder          = self::get_cache_css_folder();

        /**
         * Deregister calendar scripts
         */
        self::deregister_css($styles);

        /**
         * Check cache exists
         */
        if (!$fs->exists($folder . self::get_file_name() . '.css')) {

            /**
             * Load files and put them in cache file
             */
            $cache = array();
            foreach ($styles as $style) {

                $r = wp_remote_get($style->src);
                if (is_wp_error($r)) {
                    continue;
                }
                $cache[] = $r['body'];
            }

            $content = self::minify_css(implode(PHP_EOL, $cache));

            if (!$fs->is_dir($folder)) {
                $fs->mkdir($folder);
            }

            if (false === $fs->put_contents($folder . self::get_file_name() . '.css', $content, FS_CHMOD_FILE)) {

                if (is_wp_error($fs->errors) && $fs->errors->get_error_code()) {
                    throw new Stec_Exception($fs->errors->get_error_message());
                }

                throw new Stec_Exception(esc_html__('Unexpected error', 'stec'));
            }
        }

        $stachethemes_ec->add_asset_css('stec-css-cache', self::$cache_css_folder . self::get_file_name() . '.css');
    }

    public static function purge() {

        $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();
        $fs                   = $stachethemes_ec_main->get_file_system();
        $js_files             = glob(self::get_cache_js_folder() . self::$file_prefix . '*.js');
        $css_files            = glob(self::get_cache_css_folder() . self::$file_prefix . '*.css');
        $files                = array_merge($js_files, $css_files);

        foreach ($files as $file) {
            if ($fs->exists($file)) {
                $fs->delete($file);
            }
        }

        return true;
    }

}
