<?php

namespace Stachethemes\Stec;

class Builder_Elements {

    private static $hours_list         = null;
    private static $minutes_list       = null;
    private static $icons              = null;
    private static $writable_calendars = null;
    private static $locations          = null;
    private static $categories         = null;

    private static function get_writable_calendars() {

        if (null === self::$writable_calendars) {

            $options = array();

            $writable_cals = Calendars::get_writable_calendar_list();

            foreach ($writable_cals as $cal) :
                $options[] = "<option data-color='{$cal->get_color()}' value='{$cal->get_id()}'>{$cal->get_title()}</option>";
            endforeach;

            self::$writable_calendars = implode(PHP_EOL, $options);
        }

        return self::$writable_calendars;
    }

    private static function get_location_form_type() {

        $location_type = (int) Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        switch ($location_type) {

            case 1:
                return self::get_address_form();
            case 2:
                return self::get_location_form();
        }

        return '';
    }

    private static function get_location_form() {
        ob_start();
        ?>
        <div class="stec-builder-submission-new-location">
            <input class="stec-builder-element-content-input-style" type="text" name="location_address" placeholder="<?php esc_attr_e('Address', 'stec'); ?>" >
            <input class="stec-builder-element-content-input-style" type="text" name="location_city" placeholder="<?php esc_attr_e('City', 'stec'); ?>" >
            <input class="stec-builder-element-content-input-style" type="text" name="location_country" placeholder="<?php esc_attr_e('Country', 'stec'); ?>" >
            <input class="stec-builder-element-content-input-style" type="text" name="location_coordinates" placeholder="<?php esc_attr_e('Coordinates', 'stec'); ?>" >
            <textarea class="stec-builder-element-content-textarea-style" name="location_details" placeholder="<?php esc_attr_e('Details', 'stec'); ?>"></textarea>
        </div>
        <?php
        return ob_get_clean();
    }

    private static function get_address_form() {

        $locations = Locations::get_location_country_city();

        ob_start();
        ?>
        <div class="stec-builder-submission-new-address">

            <select class="stec-builder-element-content-select-style" type="text" name="location_country" placeholder="<?php esc_attr_e('Country', 'stec'); ?>" >
                <?php
                printf('<option selected disabled>%s</option>', esc_html__('-- Country --', 'stec'));
                foreach ($locations as $country => $cities) {
                    printf('<option value="%s">%s</option>', esc_attr($country), esc_html($country));
                }
                ?>
            </select>

            <select class="stec-builder-element-content-select-style" type="text" name="location_city" placeholder="<?php esc_attr_e('City', 'stec'); ?>" >
                <?php
                printf('<option selected disabled>%s</option>', esc_html__('-- City --', 'stec'));
                foreach ($locations as $country => $cities) {
                    foreach ($cities as $city) {
                        printf('<option data-country="%s" value="%s">%s</option>', $country, esc_attr($city), esc_html($city));
                    }
                }
                ?>
            </select>
            <input class="stec-builder-element-content-input-style" type="text" name="location_address" placeholder="<?php esc_attr_e('Address', 'stec'); ?>" >
            <input class="stec-builder-element-content-input-style" type="text" name="location_coordinates" placeholder="<?php esc_attr_e('Coordinates', 'stec'); ?>" >
            <textarea class="stec-builder-element-content-textarea-style" name="location_details" placeholder="<?php esc_attr_e('Details', 'stec'); ?>"></textarea>
        </div>
        <?php
        return ob_get_clean();
    }

    private static function get_locations() {

        $location_type = (int) Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        if (null === self::$locations) {

            $options = array();

            $locations = Locations::get_locations_list();

            if ($location_type > 0) {

                $options[] = "<optgroup label='" . esc_attr__('New location', 'stec') . "'>";

                switch ($location_type) :
                    case 1:
                        $options[] = "<option value='custom_address'>" . esc_html__('Add new address', 'stec') . "</option>";
                        break;
                    case 2:
                        $options[] = "<option value='custom'>" . esc_html__('Add new location', 'stec') . "</option>";
                        break;
                endswitch;

                $options[] = "</optgroup>";
            }
            $options[] = "<optgroup label='" . esc_attr__('Locations', 'stec') . "'>";
            $options[] = "<option selected value=''>" . esc_html__('No location', 'stec') . "</option>";
            foreach ($locations as $k => $v) :
                $options[] = "<option value='{$k}'>{$v}</option>";
            endforeach;
            $options[]       = "</optgroup>";
            self::$locations = implode(PHP_EOL, $options);
        }

        return self::$locations;
    }

    private static function get_categories() {

        if (null === self::$categories) {

            $options = array();

            $categories = Categories::get_categories_list();

            foreach ($categories as $k => $v) :
                $options[] = "<option value='{$k}'>{$v}</option>";
            endforeach;

            self::$categories = implode(PHP_EOL, $options);
        }

        return self::$categories;
    }

    private static function get_attachments_content() {
        ob_start();

        if (is_user_logged_in()) {
            ?>
            <button type="button" class="stec-builder-element-content-button-style stec-builder-add-media-attachments">
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Add Attachments', 'stec'); ?></span>
                <input type="hidden" name="has_attachments" value="1">
            </button>
            <?php
        } else {
            ?>
            <button type="button" class="stec-builder-element-content-button-style stec-builder-upload-attachments">
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Upload Attachment', 'stec'); ?></span>
            </button>
            <input class="stec-builder-hide-upload" name="stec_public_file_attachment" type="file" />
            <span class="stec-builder-submission-add-attachment-filename stec-builder-element-content-p-style"></span>
            <?php
        }

        return ob_get_clean();
    }

    private static function get_image_content() {
        if (is_user_logged_in()) {
            return array(
                '<div {{params}}><button type="button" class="stec-builder-element-content-button-style stec-builder-add-media-photo">' . esc_html__('Add Images', 'stec') . '</button></div><input type="hidden" name="stec_has_media" value="1" />'
            );
        } else {

            ob_start();
            ?>
            <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-image'>
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Image', 'stec'); ?></span>
            </button>
            <span class="stec-builder-submission-add-image-src stec-builder-element-content-p-style"></span>
            <?php
            return array(
                '<div {{params}}><input class="stec-builder-hide-upload" name="stec_public_file_image" type="file" accept="image/png, image/jpeg"/>' . ob_get_clean() . '</div>',
            );
        }
    }

    private static function get_schedule_hours_and_minutes() {

        if (self::$hours_list === null) {
            self::$hours_list   = Admin_Helper::get_hours_array();
            self::$minutes_list = Admin_Helper::minutes_array();
        }

        ob_start();
        ?>
        <select class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="schedule[0][schedule_time_hours_from]">
            <?php
            foreach (self::$hours_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <select  class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="schedule[0][schedule_time_minutes_from]">
            <?php
            foreach (self::$minutes_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <?php
        return ob_get_clean();
    }

    private static function get_hours_and_minutes($prefix = 'start') {

        if (self::$hours_list === null) {
            self::$hours_list   = Admin_Helper::get_hours_array();
            self::$minutes_list = Admin_Helper::minutes_array();
        }

        ob_start();
        ?>
        <select {{params}} name="<?php echo esc_attr__($prefix) . '_time_hours'; ?>">
            <?php
            foreach (self::$hours_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <select {{params}} name="<?php echo esc_attr__($prefix) . '_time_minutes'; ?>">
            <?php
            foreach (self::$minutes_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <?php
        return ob_get_clean();
    }

    private static function get_icons() {

        if (null === self::$icons) {
            $icons   = Admin_Helper::get_icon_list();
            $options = array();
            foreach ($icons as $k => $icon) {
                $options[] = "<option value='{$k}'>{$icon}</option>";
            }
            self::$icons = implode(PHP_EOL, $options);
        }

        return self::$icons;
    }

    private static function get_repeat_options() {

        $options = array();

        $options[] = "<option value='0'>" . esc_html__('No repeat', 'stec') . "</option>";
        $options[] = "<option value='1'>" . esc_html__('Daily', 'stec') . "</option>";
        $options[] = "<option value='2'>" . esc_html__('Weekly', 'stec') . "</option>";
        $options[] = "<option value='3'>" . esc_html__('Monthly', 'stec') . "</option>";
        $options[] = "<option value='4'>" . esc_html__('Yearly', 'stec') . "</option>";

        return implode(PHP_EOL, $options);
    }

    private static function get_repeater_content() {
        ob_start();
        ?>
        <div class="stec-repeater-content">

            <div class="stec-repeater-content-weekdays">
                <span><?php esc_html_e('By day:', 'stec'); ?></span>
                <div class="stec-repeater-content-repeat-on">
                    <span>
                        <label title="Sunday">
                            <input name="SU" title="Sunday" type="checkbox">
                            <?php esc_html_e('SU', 'stec'); ?></label>
                    </span>
                    <span>
                        <label title="Monday">
                            <input name="MO" title="Monday" type="checkbox">
                            <?php esc_html_e('MO', 'stec'); ?></label>
                    </span>
                    <span>
                        <label title="Tuesday">
                            <input name="TU" title="Tuesday" type="checkbox">
                            <?php esc_html_e('TU', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Wednesday">
                            <input name="WE" title="Wednesday" type="checkbox">
                            <?php esc_html_e('WE', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Thursday">
                            <input name="TH" title="Thursday" type="checkbox">
                            <?php esc_html_e('TH', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Friday">
                            <input name="FR" title="Friday" type="checkbox">
                            <?php esc_html_e('FR', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Saturday">
                            <input name="SA" title="Saturday" type="checkbox">
                            <?php esc_html_e('SA', 'stec'); ?>
                        </label>
                    </span>
                </div>
            </div>

            <div class="stec-repeater-content-repeat-gap">
                <span><?php esc_html_e('Repeat gap:', 'stec'); ?></span>
                <select class="stec-builder-element-content-select-style" name="repeat_gap">
                    <option value=""><?php esc_html_e('No gap', 'stec'); ?></option>
                    <?php
                    $gap = 2;
                    while ($gap <= 30) {
                        printf("<option value='%s'>%s</option>", $gap, ($gap - 1));
                        $gap++;
                    }
                    ?>
                </select>
            </div>

            <div class="stec-repeater-content-repeat-ends-on">
                <span><?php esc_html_e('Repeat ends on:', 'stec'); ?></span>
                <div class="stec-repeater-content-endson-options">
                    <div>
                        <label>
                            <input name="repeat_endson" value="0" checked="checked" type="radio">
                            <?php esc_html_e('Never', 'stec'); ?>
                        </label>
                    </div>
                    <div>
                        <label>
                            <input name="repeat_endson" value="1" type="radio">
                            <span><?php esc_html_e('After', 'stec'); ?></span>
                            <input class="stec-builder-element-content-input-style" type="number" name="repeat_occurences" max="99" min="1" value="" disabled="disabled">
                            <span><?php esc_html_e('occurrences', 'stec'); ?></span>
                        </label>
                    </div>
                    <div>
                        <label>
                            <input name="repeat_endson" value="2" type="radio">
                            <span><?php esc_html_e('On date', 'stec'); ?></span>
                            <input class="stec-builder-element-content-input-style" name="repeat_ends_on_date" size="10" value="" disabled="disabled" autocomplete="off">
                            <input type="hidden" name="repeat_ends_on_date_alt">
                        </label>
                    </div>
                </div>
            </div>

            <div class="stec-repeater-content-exdate-menu">
                <span><?php esc_html_e('Date exception:', 'stec'); ?></span>
                <div class="stec-repeater-content-exdate-options">
                    <div>
                        <input class="stec-builder-element-content-input-style" name="repeat_exdate_datepicker" value="" type="text">
                        <input type="hidden" name="repeat_exdate_datepicker_alt">
                        <a class="stec-repeater-content-add-exdate-button" href="javascript:void(0);"><?php esc_html_e('Add exdate', 'stec'); ?></a>
                    </div>
                    <ul class="stec-repeater-content-exdate-datelist">
                        <li class="stec-repeater-content-exdate-datelist-template">
                            <span>stec_replace_date</span>
                            <a class="stec-remove-exdate" href="javascript:void(0);"><?php esc_html_e('Remove', 'stec'); ?></a>
                        </li>
                    </ul>
                </div>
            </div>

            <input type="hidden" name="rrule" value="">
            <input type="hidden" name="exdate" value="">

        </div>
        <?php
        return ob_get_clean();
    }

    private static function get_privacy_policy() {
        if (0 === (int) Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'display_privacy_agreement')) {
            return '';
        }

        $text = wpautop(Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'privacy_agreement_content'));

        $html = '<div>'
                . '<div>' . $text . '</div>'
                . ' <div class="stec-builder-element-content-checkbox-style">'
                . '    <label>'
                . '        <input type="checkbox" name="privacy_policy" required />'
                . '        <p class="stec-builder-element-content-p-style">' . esc_html__('I agree', 'stec') . '</p>'
                . '     </label>'
                . ' </div>'
                . '</div>';

        return $html;
    }

    private static function get_organizers_content() {

        ob_start();
        ?>

        <div class="stec-builder-submission-organizer-list">
            <input type="hidden" name="organizers[]" value="" />
            <select class="stec-builder-element-content-select-style" name="organizers[]" multiple>
                <option disabled value=''><?php esc_html_e('Select organizer(s)', 'stec'); ?></option>"
                <?php
                $organizers = Organizers::get_organizers_list();

                foreach ($organizers as $id => $name) {
                    printf("<option value='%s'>%s</option>", esc_attr($id), esc_html($name));
                }
                ?>

            </select>
        </div>

        <div class="stec-builder-submission-organizer stec-builder-submission-organizer-template">

            <div class="stec-builder-submission-organizer-head">
                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Insert new Organizer', 'stec'); ?></p>
                <div>
                    <button type="button" class="stec-builder-element-content-button-style delete"><?php esc_html_e('Delete', 'stec'); ?></button>            
                </div>
            </div>

            <div class="stec-builder-submission-organizer-toggle-wrap">
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Name', 'stec'); ?></p>
                    <input class="stec-builder-element-content-input-style" type="text" name="name" placeholder="<?php esc_html_e('Organizer name', 'stec'); ?>" value="">
                </div>
                <div>
                    <button class="stec-builder-element-content-button-style stec-builder-add-media-photo"><?php esc_html_e('Add Photo', 'stec'); ?></button>       
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('About', 'stec'); ?></p>
                    <textarea class="stec-builder-element-content-textarea-style" name="details" placeholder="<?php esc_html_e('About your organizer', 'stec'); ?>"></textarea>
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Contact e-mail', 'stec'); ?></p>
                    <input class="stec-builder-element-content-input-style" type="email" name="email" placeholder="<?php esc_html_e('E-mail', 'stec'); ?>" value="">
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Social Links', 'stec'); ?></p>            
                    <div class="stec-builder-submission-organizer-social">
                        <select class="stec-builder-element-content-select-style" name="social[0][ico]" autocomplete="off">
                            <?php
                            $social = Admin_Helper::social_array();
                            foreach ($social as $id => $name) {
                                printf("<option value='%s'>%s</option>", esc_attr($id), esc_html($name));
                            }
                            ?>
                        </select>
                        <input class="stec-builder-element-content-input-style" type="text" name="social[0][link]" placeholder="<?php esc_html_e('Link', 'stec'); ?>" value="">
                        <i class="stec-builder-submission-organizer-social-remove fa fa-times"></i>
                    </div>
                </div>
                <button type="button" class="stec-builder-element-content-button-style add-organizers-soclink"><?php esc_html_e('Add social link', 'stec'); ?></button>       
            </div>

            <div class="status" data-default-text="<?php esc_html_e('Adding new organizer. Please wait...', 'stec'); ?>">
                <p class="stec-builder-element-content-p-style">-</p>
            </div>

            <button type='button' class='stec-builder-element-content-button-style' data-action="insert-organizer">
                <i class="fas fa-plus"></i>
                <?php esc_html_e('Add to Organizers', 'stec'); ?>
            </button>

        </div>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-organizer'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('New Organizer', 'stec'); ?>
        </button>

        <?php
        return ob_get_clean();
    }

    private static function get_schedule_content() {
        ob_start();
        ?>

        <div class="stec-builder-submission-schedule stec-builder-submission-schedule-template">

            <div class="stec-builder-submission-schedule-head">
                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Schedule Timespan', 'stec'); ?></p>
                <div>
                    <button type="button" class="stec-builder-element-content-button-style expand"><?php esc_html_e('Expand', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style collapse"><?php esc_html_e('Collapse', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style delete"><?php esc_html_e('Delete', 'stec'); ?></button>            
                </div>
            </div>

            <div class="stec-builder-submission-schedule-toggle-wrap">
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Date and time', 'stec'); ?></p>
                    <div>
                        <input class="stec-builder-element-content-datetime-style stec-builder-element-content-input-style" type="text" name="schedule[0][schedule_date_from]" />
                        <input type="hidden" name="schedule[0][schedule_date_from_alt]" />
                        <?php echo self::get_schedule_hours_and_minutes(); ?>
                    </div>
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Name', 'stec'); ?></p>
                    <input class="stec-builder-element-content-input-style" type="text" name="schedule[0][schedule_title]" placeholder="<?php esc_html_e('Title', 'stec'); ?>" value="">
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Icon', 'stec'); ?></p>
                    <select class="stec-builder-element-content-select-style" name="schedule[0][schedule_icon]">
                        <?php echo self::get_icons(); ?>
                    </select>
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Color', 'stec'); ?></p>
                    <input class="stec-builder-element-content-colorpicker-style" name="schedule[0][schedule_icon_color]" value="#000" />
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Details', 'stec'); ?></p>
                    <textarea class="stec-builder-element-content-textarea-style" name="schedule[0][schedule_details]" placeholder="<?php esc_html_e('Timespan details', 'stec'); ?>"></textarea>
                </div>
            </div>

        </div>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-schedule'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Timespan', 'stec'); ?>
        </button>

        <?php
        return ob_get_clean();
    }

    private static function get_guests_content() {
        ob_start();
        ?>

        <div class="stec-builder-submission-guest stec-builder-submission-guest-template">

            <div class="stec-builder-submission-guest-head">
                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Guest', 'stec'); ?></p>
                <div>
                    <button type="button" class="stec-builder-element-content-button-style expand"><?php esc_html_e('Expand', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style collapse"><?php esc_html_e('Collapse', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style delete"><?php esc_html_e('Delete', 'stec'); ?></button>            
                </div>
            </div>

            <div class="stec-builder-submission-guest-toggle-wrap">
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Name', 'stec'); ?></p>
                    <input class="stec-builder-element-content-input-style" type="text" name="guests[0][name]" placeholder="<?php esc_html_e('Guest name', 'stec'); ?>" value="">
                </div>
                <div>
                    <button class="stec-builder-element-content-button-style stec-builder-add-media-photo"><?php esc_html_e('Add Photo', 'stec'); ?></button>       
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('About', 'stec'); ?></p>
                    <textarea class="stec-builder-element-content-textarea-style" name="guests[0][about]" placeholder="<?php esc_html_e('About your guest', 'stec'); ?>"></textarea>
                </div>
                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Social Links', 'stec'); ?></p>            
                    <div class="stec-builder-submission-guest-social">
                        <select class="stec-builder-element-content-select-style" name="guests[0][social][0][ico]" autocomplete="off">
                            <?php
                            $social = Admin_Helper::social_array();
                            foreach ($social as $k => $v) {
                                printf('<option value="%s">%s</option>', esc_attr($k), esc_html($v));
                            }
                            ?>
                        </select>
                        <input class="stec-builder-element-content-input-style" type="text" name="guests[0][social][0][link]" placeholder="<?php esc_html_e('Link', 'stec'); ?>" value="">
                        <i class="stec-builder-submission-guest-social-remove fa fa-times"></i>
                    </div>
                </div>
                <button type="button" class="stec-builder-element-content-button-style add-guests-soclink"><?php esc_html_e('Add social link', 'stec'); ?></button>       
            </div>

        </div>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-guest'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Guest', 'stec'); ?>
        </button>

        <input type="hidden" name="has_guests_list" value="1" />

        <?php
        return ob_get_clean();
    }

    private static function get_attendance_content() {
        ob_start();
        ?>

        <div class="stec-builder-submission-attendance stec-builder-submission-attendance-template">

            <div class="stec-builder-submission-attendee-head">
                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Attendee', 'stec'); ?></p>
                <div>
                    <button type="button" class="stec-builder-element-content-button-style expand"><?php esc_html_e('Expand', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style collapse"><?php esc_html_e('Collapse', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style delete"><?php esc_html_e('Delete', 'stec'); ?></button>            
                </div>
            </div>

            <div class="stec-builder-submission-attendee-toggle-wrap">
                <div>
                    <input class="stec-builder-element-content-input-style" type="email" name="attendees[0][email]" placeholder="<?php esc_html_e('Attendee e-mail', 'stec'); ?>" value="">
                    <input class="stec-builder-element-content-input-style" type="number" min="1" name="attendees[0][people]" placeholder="<?php esc_html_e('People', 'stec'); ?>" value="">
                    <div class="stec-builder-element-content-checkbox-style">
                        <label>
                            <input type="checkbox" name="attendees[0][send_email]" placeholder="<?php esc_html_e('Send e-mail', 'stec'); ?>" value="">
                            <p class="stec-builder-element-content-p-style"><?php esc_html_e('Send E-mail', 'stec'); ?></p>
                        </label>
                    </div>
                </div>
            </div>

        </div>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-attendee'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Attendee', 'stec'); ?>
        </button>

        <input type="hidden" name="has_attendance_list" value="1" />

        <?php
        return ob_get_clean();
    }

    private static function get_booking_content() {

        if (false === class_exists('WooCommerce') || false === is_user_logged_in()) {
            return '';
        }

        ob_start();
        ?>

        <div class="stec-builder-submission-ticket stec-builder-submission-ticket-template">

            <div class="stec-builder-submission-ticket-head">
                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Ticket', 'stec'); ?></p>
                <div>
                    <button type="button" class="stec-builder-element-content-button-style expand"><?php esc_html_e('Expand', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style collapse"><?php esc_html_e('Collapse', 'stec'); ?></button>
                    <button type="button" class="stec-builder-element-content-button-style delete"><?php esc_html_e('Delete', 'stec'); ?></button>            
                </div>
            </div>

            <div class="stec-builder-submission-ticket-toggle-wrap">
                <div class='stec-builder-submission-ticket-flex'>
                    <div>
                        <p class="stec-builder-element-content-p-style"><?php esc_html_e('Name', 'stec'); ?></p>
                        <input class="stec-builder-element-content-input-style" type="text" name="tickets[0][name]" placeholder="<?php esc_html_e('Ticket name', 'stec'); ?>" value="">
                        <input type="hidden" name="tickets[0][id]" value="0" />
                    </div>
                    <div>
                        <p class="stec-builder-element-content-p-style"><?php printf('%s (%s)', esc_html__('Price', 'stec'), get_woocommerce_currency()); ?></p>
                        <input class="stec-builder-element-content-input-style" step="0.1" type="number" min='0' name="tickets[0][price]" placeholder="<?php esc_html_e('Ticket price', 'stec'); ?>" value="">
                    </div>
                </div>
                <div class='stec-builder-submission-ticket-flex'>
                    <div>
                        <p class="stec-builder-element-content-p-style"><?php esc_html_e('Total Quantity', 'stec'); ?></p>
                        <input class="stec-builder-element-content-input-style" type="number" min='0' name="tickets[0][qty]" placeholder="<?php esc_html_e('Total Quantity', 'stec'); ?>" value="0">
                    </div>
                    <div>
                        <p class="stec-builder-element-content-p-style"><?php esc_html_e('Quantity limit per event', 'stec'); ?></p>
                        <input class="stec-builder-element-content-input-style" type="number" min='0' name="tickets[0][qty_per_event]" placeholder="<?php esc_html_e('Quantity per event', 'stec'); ?>" value="0">
                    </div>
                </div>

                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Stop purchases', 'stec') ?></p>
                <div class="stec-builder-submission-ticket-flex stec-builder-submission-ticket-stop-before">
                    <div>
                        <input class="stec-builder-element-content-input-style" type="number" min='0' name="tickets[0][wc_book_create_ticket_stop_purchase][value]" placeholder="<?php esc_html_e('Total Quantity', 'stec'); ?>" value="0">
                    </div>
                    <div>
                        <select class="stec-builder-element-content-input-style" name="tickets[0][wc_book_create_ticket_stop_purchase][units]">
                            <option value="hours"><?php esc_attr_e('Hour(s)', 'stec') ?></option>
                            <option value="days"><?php esc_attr_e('Day(s)', 'stec') ?></option>
                        </select>
                    </div>
                    <div>
                        <p class="stec-builder-element-content-p-style"><?php echo esc_html_x('before the event', 'Days/Hours before the event', 'stec'); ?></p>
                    </div>
                </div>

                <div>
                    <p class="stec-builder-element-content-p-style"><?php esc_html_e('Short Description', 'stec'); ?></p>
                    <textarea class="stec-builder-element-content-textarea-style" name="tickets[0][about]" placeholder="<?php esc_html_e('About your ticket', 'stec'); ?>"></textarea>
                </div>

                <p class="stec-builder-element-content-p-style"><?php esc_html_e('Attributes', 'stec'); ?></p>            
                <div class="stec-builder-submission-ticket-attributes">
                    <input class="stec-builder-element-content-input-style" type="text" name="tickets[0][attr][0][attr_name]" placeholder="<?php esc_html_e('Name', 'stec'); ?>" value="">
                    <input class="stec-builder-element-content-input-style" type="text" name="tickets[0][attr][0][attr_value]" placeholder="<?php esc_html_e('Value', 'stec'); ?>" value="">
                    <i class="stec-builder-submission-ticket-attrib-remove fa fa-times"></i>
                </div>
                <button type="button" class="stec-builder-element-content-button-style add-ticket-attribute"><?php esc_html_e('Add attribute', 'stec'); ?></button>       
            </div>

        </div>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-ticket'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Ticket', 'stec'); ?>
        </button>

        <input type="hidden" name="has_tickets" value="1" />
        <?php
        return ob_get_clean();
    }

    public static function get_element($id) {

        $elements = array(
            "full-width"        => array(
                "<div {{params}}>{{content}}</div>"
            ),
            "half-width"        => array(
                "<div {{params}}>{{content}}</div>",
                "<div {{params}}>{{content}}</div>",
            ),
            "headers"           => array(
                "<h1 {{params}}>{{content}}</h1>"
            ),
            "paragraph"         => array(
                "<p {{params}}>{{content}}</p>"
            ),
            "title"             => array(
                "<input {{params}} type='text' name='title' value='' placeholder='" . esc_html__('Event title', 'stec') . "' />"
            ),
            "description"       => array(
                "<textarea {{params}} name='description' placeholder='" . esc_html__('Description', 'stec') . "'></textarea>"
            ),
            "short-description" => array(
                "<input {{params}} type='text' name='description_short' placeholder='" . esc_html__('Short description', 'stec') . "' />"
            ),
            "location"          => array(
                "<select {{params}} name='location'>" . self::get_locations() . "</select>",
                self::get_location_form_type(),
            ),
            "start-date"        => array(
                "<input {{params}} type='text' name='start_date' placeholder='" . esc_html__('Starts on', 'stec') . "' />",
                "<input type='hidden' name='start_date_alt' />",
                self::get_hours_and_minutes('start')
            ),
            "end-date"          => array(
                "<input {{params}} type='text' name='end_date' placeholder='" . esc_html__('Ends on', 'stec') . "' />",
                "<input type='hidden' name='end_date_alt' />",
                self::get_hours_and_minutes('end')
            ),
            "repeater"          => array(
                "<select {{params}} name='repeater'><option disabled selected value=''>" . esc_html__('Repeater Scheme', 'stec') . "</option>" . self::get_repeat_options() . "</select>"
                . self::get_repeater_content()
            ),
            "all-day-check"     => array(
                "<div {{params}}><label><input name='all_day' type='checkbox' /><p class='stec-builder-element-content-p-style'>" . esc_html__('All day', 'stec') . "</p></label></div>"
            ),
            "image"             => self::get_image_content(),
            "attachments"       => array(
                '<div {{params}}>' . self::get_attachments_content() . '</div>'
            ),
            "icon"              => array(
                "<select {{params}} name='icon'><option disabled selected value=''>" . esc_html__('Select Icon', 'stec') . "</option>" . self::get_icons() . "</select>",
            ),
            "calendar"          => array(
                "<select {{params}} name='calendar_id'><option disabled selected value=''>" . esc_html__('Select Calendar', 'stec') . "</option>" . self::get_writable_calendars() . "</select>",
            ),
            "category"          => array(
                "<select {{params}} multiple name='event_categories[]'><option disabled value=''>" . esc_html__('Select categories', 'stec') . "</option>" . self::get_categories() . "</select>",
            ),
            "color"             => array(
                "<input {{params}} type='text' name='event_color' value='#f15e6e' placeholder='" . esc_html__('Color', 'stec') . "' />",
            ),
            "counter"           => array(
                "<select {{params}} name='counter'>"
                . "<option disabled selected value=''>" . esc_html__('Counter', 'stec') . "</option>"
                . "<option value='0'>" . esc_html__('Disable', 'stec') . "</option>"
                . "<option value='1'>" . esc_html__('Enable', 'stec') . "</option>"
                . "</select>",
            ),
            "comments"          => array(
                "<select {{params}} name='comments'>"
                . "<option disabled selected value=''>" . esc_html__('Comments', 'stec') . "</option>"
                . "<option value='0'>" . esc_html__('Disable', 'stec') . "</option>"
                . "<option value='1'>" . esc_html__('Enable', 'stec') . "</option>"
                . "</select>",
            ),
            "featured"          => array(
                "<select {{params}} name='featured'>"
                . "<option disabled selected value=''>" . esc_html__('Featured', 'stec') . "</option>"
                . "<option value='0'>" . esc_html__('No', 'stec') . "</option>"
                . "<option value='1'>" . esc_html__('Yes', 'stec') . "</option>"
                . "<option value='2'>" . esc_html__('Yes, with background', 'stec') . "</option>"
                . "</select>",
            ),
            "keywords"          => array(
                "<input {{params}} type='text' name='keywords' placeholder='" . esc_html__('Search keywords', 'stec') . "' />",
            ),
            "email"             => array(
                "<input {{params}} type='email' name='contact_email' value='" . Admin_Helper::get_current_user_email() . "' placeholder='" . esc_html__('Contact e-mail', 'stec') . "' />"
            ),
            "website-url"       => array(
                "<input {{params}} type='url' name='link' placeholder='" . esc_html__('Website URL', 'stec') . "' />"
            ),
            "notes"             => array(
                "<textarea {{params}} name='review_note' placeholder='" . esc_html__('Notes to the reviewer', 'stec') . "'></textarea>"
            ),
            "submit"            => array(
                "<div {{params}}><input type='submit' name='submit' value='" . esc_html__('Submit event', 'stec') . "' /></div>"
            ),
            "privacy"           => array(
                "<div {{params}}>" . self::get_privacy_policy() . "</div>"
            ),
            "captcha"           => array(
                "<div {{params}}><div></div></div>"
            ),
            "guests"            => array(
                "<div {{params}}>" . self::get_guests_content() . "</div>"
            ),
            "organizers"        => array(
                "<div {{params}}>" . self::get_organizers_content() . "</div>"
            ),
            "attendance"        => array(
                "<div {{params}}>" . self::get_attendance_content() . "</div>"
            ),
            "schedule"          => array(
                "<div {{params}}>" . self::get_schedule_content() . "</div>"
            ),
            "booking"           => array(
                "<div {{params}}>" . self::get_booking_content() . "</div>"
            ),
            "rsvp"              => array(
                '<select {{params}} name="rsvp[collect]">
                    <option disabled selected>' . esc_html__('Collect RSVP', 'stec') . '</option>
                    <option value="0">' . esc_html__('Disable', 'stec') . '</option>
                    <option value="1">' . esc_html__('Enable', 'stec') . '</option>
                </select>',
                '<input {{params}} type="number" min="0" name="rsvp[limit]" placeholder="' . esc_attr__('Attendees limit', 'stec') . '" />',
                '<select {{params}} name="rsvp[visibility]">
                    <option disabled selected>' . esc_html__('Attendance visibility', 'stec') . '</option>
                    <option value="public">' . esc_html__('Public', 'stec') . '</option>
                    <option value="private">' . esc_html__('Private', 'stec') . '</option>
                </select>',
            ),
        );

        $elements = apply_filters('stec_builder_front_elements', $elements);

        if (!isset($elements[$id])) {
            throw new Stec_Exception(sprintf(esc_html__('Builder element %s not found', 'stec'), $id));
        }
        return $elements[$id];
    }

}

class Builder_Element {

    private $params              = null;
    private $content             = null;
    private $children            = null;
    private $structure           = null;
    private $content_placeholder = '{{content}}';
    private $params_placeholder  = '{{params}}';

    public function __construct(\stdClass $element) {

        $this->params   = $element->params;
        $this->content  = $element->content;
        $this->children = $element->children;
    }

    private function set_content_params() {

        foreach ($this->content as $g => $content) {
            $params = array();

            if (isset($content->params->classes)) {
                $params[] = "class='{$content->params->classes}'";
            }

            if ($this->params->required) {
                $params[] = 'required';
            }


            if (!isset($this->structure[$g])) {
                continue;
            }

            $this->structure[$g] = str_replace($this->params_placeholder, implode(' ', $params), $this->structure[$g]);
        }
    }

    private function set_children() {

        $children_html_array = $this->get_children();

        foreach ($this->structure as $k => $content) {

            if (isset($children_html_array[$k])) {
                $this->structure[$k] = str_replace($this->content_placeholder, implode(PHP_EOL, $children_html_array[$k]), $content);
            } else {
                $this->structure[$k] = str_replace($this->content_placeholder, '', $content);
            }
        }
    }

    private function set_content() {

        if ($this->children) {

            $this->set_children();
        } else {

            foreach ($this->content as $j => $content) {

                if (!isset($this->structure[$j])) {
                    continue;
                }

                if (isset($content->params->text)) {
                    $this->structure[$j] = str_replace($this->content_placeholder, esc_html($content->params->text), $this->structure[$j]);
                } else {
                    $this->structure[$j] = str_replace($this->content_placeholder, '', $this->structure[$j]);
                }
            }
        }
    }

    private function get_children() {

        $children_html_array = array();

        foreach ($this->children as $k => $child_position) {

            foreach ($child_position as $child) {
                $children_element          = new Builder_Element($child);
                $children_html_array[$k][] = $children_element->get_html();
            }
        }

        return $children_html_array;
    }

    private function get_element_style() {

        $style  = array();
        $search = array(
            'padding'   => 'padding',
            'margin'    => 'margin',
            'textAlign' => 'text-align',
            'height'    => 'height'
        );

        foreach ($search as $k => $prop) {
            if (isset($this->params->$k)) {
                $style[] = "{$prop}:{$this->params->$k}";
            }
        }

        return "style='" . implode("; ", $style) . "'";
    }

    public function get_html() {

        $this->structure = Builder_Elements::get_element($this->params->id);

        $this->set_content_params();

        $this->set_content();

        return '<div ' . $this->get_element_style() . ' class="stec-builder-form-element">' . implode(PHP_EOL, $this->structure) . '</div>';
    }

}
