<?php

namespace Stachethemes\Stec;

class Builder {

    static private function get_next_index($project_type) {
        $indexes                = get_option('stec_builder_index', array());
        $index                  = isset($indexes[$project_type]) ? (int) $indexes[$project_type] : 0;
        $indexes[$project_type] = ++$index;

        update_option('stec_builder_index', $indexes, false);

        return $index;
    }

    static public function get_projects_list_array($project_type) {
        $projects = self::get_projects($project_type);
        $options  = array('0' => esc_html__('Default', 'stec'));
        foreach ($projects as $project) {
            $options[$project['id']] = $project['title'];
        }

        return $options;
    }

    static public function project_exists($project_type, $project_id) {
        $projects = self::get_projects();

        if (isset($projects[$project_type]) && isset($projects[$project_type][$project_id]['elements'])) {
            return true;
        }
        return false;
    }

    static public function get_project($project_type, $project_id) {

        $projects = self::get_projects(null, true);

        if (isset($projects[$project_type]) && isset($projects[$project_type][$project_id]['elements'])) {
            return wp_json_encode(array(
                'plot'     => $projects[$project_type][$project_id]['plot'],
                'elements' => $projects[$project_type][$project_id]['elements'],
                'title'    => $projects[$project_type][$project_id]['title']
            ));
        }

        return wp_json_encode(array(
            'plot'     => null,
            'elements' => null,
            'title'    => esc_html__('Untitled', 'stec')
        ));
    }

    static public function get_project_elements($project_type, $project_id) {
        $projects = self::get_projects(null, true);

        if (isset($projects[$project_type]) && isset($projects[$project_type][$project_id]['elements'])) {
            return $projects[$project_type][$project_id]['elements'];
        }

        return null;
    }

    static public function get_project_plot($project_type, $project_id) {
        $projects = self::get_projects(null, true);

        if (isset($projects[$project_type]) && isset($projects[$project_type][$project_id]['plot'])) {
            return $projects[$project_type][$project_id]['plot'];
        }

        return null;
    }

    static public function get_projects($type = null, $inc_def = false) {

        $projects = get_option('stec_builder_projects', array());

        if (true === $inc_def) {

            $project_types = array('event_submission_form');

            foreach ($project_types as $project_type) {

                try {
                    $json_form = self::get_default($project_type);
                } catch (Stec_Exception $ex) {
                    Admin_Helper::debug_log($ex->getMessage());
                    continue;
                }

                if (!$json_form) {
                    continue;
                }

                $default_form = (array) json_decode($json_form);

                if (!isset($projects[$project_type])) {
                    $projects[$project_type] = array();
                }

                $projects[$project_type]['default'] = array_merge($default_form, array(
                    'id'   => 'default',
                    'type' => $project_type
                ));
            }
        }

        if ($type) {
            return isset($projects[$type]) ? $projects[$type] : array();
        } else {
            return $projects;
        }
    }

    static public function save_project($plot, $elements, $project_type, $project_title, $id = null) {

        $projects = self::get_projects();

        if (null === $id || 'default' === $id) {
            $id = self::get_next_index($project_type);
        }

        $projects[$project_type][$id] = array(
            'plot'         => $plot,
            'elements'     => $elements,
            'project_type' => $project_type,
            'title'        => $project_title,
            'id'           => $id
        );

        update_option('stec_builder_projects', $projects, false);

        return array(
            'id' => $id
        );
    }

    static public function duplicate_project($project_type, $id) {

        $projects = self::get_projects(null, true);

        if (isset($projects[$project_type]) && isset($projects[$project_type][$id])) {


            $project = array(
                'plot'     => self::get_project_plot($project_type, $id),
                'elements' => self::get_project_elements($project_type, $id),
                'title'    => $projects[$project_type][$id]['title'],
                'id'       => self::get_next_index($project_type)
            );

            self::save_project(
                    $project['plot'],
                    $project['elements'],
                    $project_type,
                    $project['title'],
                    $project['id']
            );

            return $project;
        }

        return false;
    }

    static public function delete_project($project, $id) {

        $projects = self::get_projects();

        if (isset($projects[$project]) && isset($projects[$project][$id])) {
            unset($projects[$project][$id]);
            update_option('stec_builder_projects', $projects, false);
        }

        return true;
    }

    static public function bulk_delete_projects($project_type, $ids) {

        $projects = self::get_projects();

        foreach ($ids as $id) {
            if (isset($projects[$project_type]) && isset($projects[$project_type][$id])) {
                unset($projects[$project_type][$id]);
            }
        }

        update_option('stec_builder_projects', $projects, false);

        return true;
    }

    /**
     * Returns default form in json format
     * @param string $project_type
     * @return json
     * @throws Stec_Exception
     */
    static public function get_default($project_type) {
        switch ($project_type) {
            case 'event_submission_form' :
                ob_start();
                ?>{"plot":"{\"id\":\"plot\",\"structure\":false,\"classes\":\"\",\"title\":\"Workflow\",\"iconClass\":\"\",\"limit\":null,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 0 0\",\"required\":false,\"ismust\":false}","elements":"[{\"params\":{\"id\":\"full-width\",\"structure\":true,\"classes\":\"\",\"title\":\"Full Width\",\"iconClass\":\"stec-builder-icon-full-width\",\"limit\":null,\"padding\":\"20px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":1,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-full-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"headers\",\"structure\":false,\"classes\":\"\",\"title\":\"Header\",\"iconClass\":\"stec-builder-icon-header\",\"limit\":null,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":2},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-header-style\",\"droppable\":false,\"text\":\"Event Details\"},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"half-width\",\"structure\":true,\"classes\":\"\",\"title\":\"50\/50\",\"iconClass\":\"stec-builder-icon-half-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 0px 0px\",\"required\":false,\"ismust\":false,\"hid\":3,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"title\",\"structure\":false,\"classes\":\"\",\"title\":\"Title\",\"iconClass\":\"stec-builder-icon-title\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":true,\"ismust\":true,\"hid\":4},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"description\",\"structure\":false,\"classes\":\"\",\"title\":\"Description\",\"iconClass\":\"stec-builder-icon-description\",\"limit\":1,\"padding\":\"0px 0px 0px 0px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"height\":\"343px\",\"hid\":5,\"textAlign\":null},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-textarea-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"short-description\",\"structure\":false,\"classes\":\"\",\"title\":\"Short description\",\"iconClass\":\"stec-builder-icon-description-short\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":6},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}],[{\"params\":{\"id\":\"image\",\"structure\":false,\"classes\":\"\",\"title\":\"Image\",\"iconClass\":\"stec-builder-icon-image\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":8},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-media-images-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"location\",\"structure\":false,\"classes\":\"\",\"title\":\"Location\",\"iconClass\":\"stec-builder-icon-location\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":9},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"website-url\",\"structure\":false,\"classes\":\"\",\"title\":\"Website URL\",\"iconClass\":\"stec-builder-icon-url\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":7},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"organizers\",\"structure\":false,\"classes\":\"\",\"title\":\"Organizers\",\"iconClass\":\"stec-builder-icon-organizers\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":19},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-organizers-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"full-width\",\"structure\":true,\"classes\":\"\",\"title\":\"Full Width\",\"iconClass\":\"stec-builder-icon-full-width\",\"limit\":null,\"padding\":\"10px 20px 0px 20px\",\"margin\":\"0px 0px 0px 0px\",\"required\":false,\"ismust\":false,\"hid\":10,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-full-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"headers\",\"structure\":false,\"classes\":\"\",\"title\":\"Header\",\"iconClass\":\"stec-builder-icon-header\",\"limit\":null,\"padding\":\"0px 0px 0px 0px\",\"margin\":\"0px 0px px 0px\",\"required\":false,\"ismust\":false,\"hid\":11,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-header-style\",\"droppable\":false,\"text\":\"Date & Time\"},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"half-width\",\"structure\":true,\"classes\":\"\",\"title\":\"50\/50\",\"iconClass\":\"stec-builder-icon-half-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":12,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"start-date\",\"structure\":false,\"classes\":\"\",\"title\":\"Starts on\",\"iconClass\":\"stec-builder-icon-start-date\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":true,\"ismust\":true,\"hid\":13},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"all-day-check\",\"structure\":false,\"classes\":\"\",\"title\":\"All day check\",\"iconClass\":\"stec-builder-icon-all-day\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":15},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-checkbox-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}],[{\"params\":{\"id\":\"end-date\",\"structure\":false,\"classes\":\"\",\"title\":\"Ends on\",\"iconClass\":\"stec-builder-icon-end-date\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":true,\"ismust\":true,\"hid\":14},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-datetime-style stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"repeater\",\"structure\":false,\"classes\":\"\",\"title\":\"Repeater\",\"iconClass\":\"stec-builder-icon-repeater\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":16},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"full-width\",\"structure\":true,\"classes\":\"\",\"title\":\"Full Width\",\"iconClass\":\"stec-builder-icon-full-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 0px 0px\",\"required\":false,\"ismust\":false,\"hid\":17,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-full-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"headers\",\"structure\":false,\"classes\":\"\",\"title\":\"Header\",\"iconClass\":\"stec-builder-icon-header\",\"limit\":null,\"padding\":\"0px 0px 0px 0px\",\"margin\":\"0px 0px px 0px\",\"required\":false,\"ismust\":false,\"hid\":18,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-header-style\",\"droppable\":false,\"text\":\"Guests & Schedule\"},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"half-width\",\"structure\":true,\"classes\":\"\",\"title\":\"50\/50\",\"iconClass\":\"stec-builder-icon-half-width\",\"limit\":null,\"padding\":\"10px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":20,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"guests\",\"structure\":false,\"classes\":\"\",\"title\":\"Guests\",\"iconClass\":\"stec-builder-icon-guests\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":21},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-guests-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}],[{\"params\":{\"id\":\"schedule\",\"structure\":false,\"classes\":\"\",\"title\":\"Schedule\",\"iconClass\":\"stec-builder-icon-schedule\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":22},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-schedules-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"full-width\",\"structure\":true,\"classes\":\"\",\"title\":\"Full Width\",\"iconClass\":\"stec-builder-icon-full-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":23,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-full-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"headers\",\"structure\":false,\"classes\":\"\",\"title\":\"Header\",\"iconClass\":\"stec-builder-icon-header\",\"limit\":null,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":24},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-header-style\",\"droppable\":false,\"text\":\"Additional Information\"},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"half-width\",\"structure\":true,\"classes\":\"\",\"title\":\"50\/50\",\"iconClass\":\"stec-builder-icon-half-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":26,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}},{\"params\":{\"classes\":\"stec-builder-element-content-half-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"color\",\"structure\":false,\"classes\":\"\",\"title\":\"Color\",\"iconClass\":\"stec-builder-icon-color\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":27},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-colorpicker-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"calendar\",\"structure\":false,\"classes\":\"\",\"title\":\"Calendar\",\"iconClass\":\"stec-builder-icon-calendar\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":true,\"ismust\":true,\"hid\":29},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"featured\",\"structure\":false,\"classes\":\"\",\"title\":\"Featured\",\"iconClass\":\"stec-builder-icon-featured\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":33},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"keywords\",\"structure\":false,\"classes\":\"\",\"title\":\"Keywords\",\"iconClass\":\"stec-builder-icon-search\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":31},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"counter\",\"structure\":false,\"classes\":\"\",\"title\":\"Counter\",\"iconClass\":\"stec-builder-icon-counter\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":32},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"comments\",\"structure\":false,\"classes\":\"\",\"title\":\"Comments\",\"iconClass\":\"stec-builder-icon-comments\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":34},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}],[{\"params\":{\"id\":\"icon\",\"structure\":false,\"classes\":\"\",\"title\":\"Icon\",\"iconClass\":\"stec-builder-icon-icon\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":28},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"category\",\"structure\":false,\"classes\":\"\",\"title\":\"Category\",\"iconClass\":\"stec-builder-icon-category\",\"limit\":1,\"padding\":\"0px 0px 0px 0px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"height\":\"260px\",\"hid\":30,\"textAlign\":null},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-select-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}]]},{\"params\":{\"id\":\"full-width\",\"structure\":true,\"classes\":\"\",\"title\":\"Full Width\",\"iconClass\":\"stec-builder-icon-full-width\",\"limit\":null,\"padding\":\"0px 20px 0px 20px\",\"margin\":\"0px 0px 10px 0px\",\"required\":false,\"ismust\":false,\"hid\":35,\"textAlign\":null,\"height\":\"\"},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-full-width-style\",\"droppable\":true},\"css\":{}}],\"children\":[[{\"params\":{\"id\":\"headers\",\"structure\":false,\"classes\":\"\",\"title\":\"Header\",\"iconClass\":\"stec-builder-icon-header\",\"limit\":null,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":36},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-header-style\",\"droppable\":false,\"text\":\"Author Information\"},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"notes\",\"structure\":false,\"classes\":\"\",\"title\":\"Reviewer notes\",\"iconClass\":\"stec-builder-icon-notes\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"height\":200,\"hid\":37},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-textarea-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"email\",\"structure\":false,\"classes\":\"\",\"title\":\"Contact e-mail\",\"iconClass\":\"stec-builder-icon-email\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":38},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-input-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"privacy\",\"structure\":false,\"classes\":\"\",\"title\":\"Privacy Policy\",\"iconClass\":\"stec-builder-icon-privacy-policy\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":39},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-privacy-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"captcha\",\"structure\":false,\"classes\":\"\",\"title\":\"Captcha\",\"iconClass\":\"stec-builder-icon-captcha\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":false,\"ismust\":false,\"hid\":40},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-captcha-style\",\"droppable\":false},\"css\":{}}],\"children\":[]},{\"params\":{\"id\":\"submit\",\"structure\":false,\"classes\":\"\",\"title\":\"Submit button\",\"iconClass\":\"stec-builder-icon-submit\",\"limit\":1,\"padding\":\"0 0 0 0\",\"margin\":\"0 0 10px 0\",\"required\":true,\"ismust\":true,\"hid\":41},\"content\":[{\"params\":{\"classes\":\"stec-builder-element-content-submit-style\",\"droppable\":false},\"css\":{}}],\"children\":[]}]]}]","title":"Default"}<?php
                return ob_get_clean();

            default:
                throw new Stec_Exception(esc_html__('Not found', 'stec'), 404);
        }
    }

}
