<?php

namespace Stachethemes\Stec;

class Categories {

    /**
     * Get array list of categories
     * @param string $taxomony the taxonomy name
     * @return array
     */
    public static function get_categories() {

        $categories = array();

        $terms = get_terms('stec_event_category', array(
            'hide_empty' => false,
            'fields'     => 'ids',
        ));

        foreach ($terms as $id) {
            $category = new Category($id);

            if (false === is_super_admin()) {
                if ('private' === $category->get_visibility()) {
                    if ($category->get_author() !== get_current_user_id()) {
                        continue;
                    }
                }
            }

            $categories[] = $category;
        }

        return $categories;
    }

    public static function get_categories_list() {
        $categories_list = array();
        
        $categories = self::get_categories();
        
        foreach ($categories as $category) {
            $categories_list[$category->get_id()] = $category->get_title();
        }

        return $categories_list;
    }

    public static function bulk_delete($ids = array()) {
        $filter_cat_ids = filter_var_array($ids, FILTER_VALIDATE_INT);
        $unique_cat_ids = array_filter($filter_cat_ids);

        foreach ($unique_cat_ids as $cat_id) {
            $category = new Category($cat_id);
            if (!$category->get_id()) {
                continue;
            }

            $category->delete();
        }
        unset($cat_id);

        return true;
    }

    public static function get_category_by_id($id) {
        $cats = self::get_categories();

        foreach ($cats as $cat) {
            if ($cat->get_id() === $id) {
                return $cat;
            }
        }

        return null;
    }

    public static function get_category_by_name($name) {
        $cats = self::get_categories();

        foreach ($cats as $cat) {
            if ($cat->get_title() === $name) {
                return $cat;
            }
        }

        return null;
    }

}
