<?php

namespace Stachethemes\Stec;

class Csv_Settings {

    private static $assoc_fields = null;

    public static function reset() {
        delete_option('stec_csv_event_fields_assoc');
        delete_option('stec_csv_datetime_format');
        self::$assoc_fields = null;
    }

    public static function get_event_assoc_fields() {

        $assoc_fields = get_option('stec_csv_event_fields_assoc', array(
            array(
                'name'  => esc_html__('Title', 'stec'),
                'field' => 'Title',
                'assoc' => 'title',
            ),
            array(
                'name'  => esc_html__('Description', 'stec'),
                'field' => 'Description',
                'assoc' => 'description',
            ),
            array(
                'name'  => esc_html__('Short Description', 'stec'),
                'field' => 'Short Description',
                'assoc' => 'description_short',
            ),
            array(
                'name'  => esc_html__('Images IDs', 'stec'),
                'field' => 'Images IDs',
                'assoc' => 'images',
            ),
            array(
                'name'  => esc_html__('Color', 'stec'),
                'field' => 'Event Color',
                'assoc' => 'color',
            ),
            array(
                'name'  => esc_html__('Icon', 'stec'),
                'field' => 'Event Icon',
                'assoc' => 'icon',
            ),
            array(
                'name'  => esc_html__('Location Address', 'stec'),
                'field' => 'Address',
                'assoc' => 'location_address',
            ),
            array(
                'name'  => esc_html__('Location City', 'stec'),
                'field' => 'City',
                'assoc' => 'location_city',
            ),
            array(
                'name'  => esc_html__('Location Country', 'stec'),
                'field' => 'Country',
                'assoc' => 'location_country',
            ),
            array(
                'name'  => esc_html__('Location Details', 'stec'),
                'field' => 'Location Details',
                'assoc' => 'location_details',
            ),
            array(
                'name'  => esc_html__('Categories ID', 'stec'),
                'field' => 'Categories ID',
                'assoc' => 'categories_id',
            ),
            array(
                'name'  => esc_html__('Start Date', 'stec'),
                'field' => 'Start Date',
                'assoc' => 'start_date',
            ),
            array(
                'name'  => esc_html__('End Date', 'stec'),
                'field' => 'End Date',
                'assoc' => 'end_date',
            ),
            array(
                'name'  => esc_html__('All Day', 'stec'),
                'field' => 'All Day',
                'assoc' => 'all_day',
            ),
            array(
                'name'  => esc_html__('RRULE', 'stec'),
                'field' => 'RRULE',
                'assoc' => 'rrule',
            ),
            array(
                'name'  => esc_html__('Exdate', 'stec'),
                'field' => 'EXDATE',
                'assoc' => 'exdate',
            ),
            array(
                'name'  => esc_html__('Recurrence ID', 'stec'),
                'field' => 'Recurrence ID',
                'assoc' => 'recurrence_id',
            ),
            array(
                'name'  => esc_html__('Featured', 'stec'),
                'field' => 'Featured',
                'assoc' => 'featured',
            ),
            array(
                'name'      => esc_html__('External URL', 'stec'),
                'field'     => 'External Link',
                'assoc'     => 'link',
                'call_func' => 'set_link'
            ),
            array(
                'name'  => esc_html__('RSVP', 'stec'),
                'field' => 'RSVP',
                'assoc' => 'rsvp'
            ),
            array(
                'name'  => esc_html__('Attendees Limit', 'stec'),
                'field' => 'Attendees Limit',
                'assoc' => 'rsvp_limit'
            ),
            array(
                'name'  => esc_html__('Attendees Visibility', 'stec'),
                'field' => 'Attendees Visibility',
                'assoc' => 'attendance_visibility'
            ),
            array(
                'name'  => esc_html__('Keywords', 'stec'),
                'field' => 'Keywords',
                'assoc' => 'keywords',
            ),
            array(
                'name'  => esc_html__('Author ID', 'stec'),
                'field' => 'Author ID',
                'assoc' => 'author',
            ),
            array(
                'name'  => esc_html__('UID', 'stec'),
                'field' => 'UID', // CSV Field
                'assoc' => 'uid', // Event Field
            ),
        ));

        self::$assoc_fields = apply_filters('stec_csv_event_fields_assoc', $assoc_fields);

        return self::$assoc_fields;
    }

    public static function get_event_assoc_field($field = null) {

        if (!$field) {
            return null;
        }
        
        $fields = self::get_event_assoc_fields();
        
        foreach ($fields as $f) {
            if ($f['field'] === $field) {
                return $f['assoc'];
            }
        }

        return null;
    }

    public static function update_event_assoc_settings(Array $fields) {

        $settings = self::get_event_assoc_fields();

        if (!$fields) {
            return true;
        }

        foreach ($settings as &$setting) {

            foreach ($fields as $key => $value) {
                if ($key === $setting['assoc']) {
                    $setting['field'] = $value;
                }
            }
        }

        return update_option('stec_csv_event_fields_assoc', $settings, false);
    }

    public static function update_datetime_format($format) {
        update_option('stec_csv_datetime_format', sanitize_text_field($format), false);
    }

    public static function get_datetime_format() {
        return get_option('stec_csv_datetime_format', 'Y-m-d H:i:s');
    }

}
