<?php

namespace Stachethemes\Stec;

use Stachethemes\Stec\Admin_Helper as Help;

class DTools {

    public static function repair_broken_tickets() {
        $count   = 0;
        $tickets = Booking::get_bookable_products();

        foreach ($tickets as $t) {
            if ('1' !== get_post_meta($t->get_id(), 'stec_bookable', true)) {
                update_post_meta($t->get_id(), 'stec_bookable', '1');
                $count++;
            }
        }

        return $count;
    }

    public static function has_broken_tickets() {
        $tickets = Booking::get_bookable_products();

        foreach ($tickets as $t) {
            if ('1' !== get_post_meta($t->get_id(), 'stec_bookable', true)) {
                return true;
            }
        }

        return false;
    }

    public static function remove_tickets() {

        $count = 0;

        $events_with_tickets = get_posts(array(
            'post_type'  => 'stec_event',
            'fields'     => 'ids',
            'meta_query' => array(
                array(
                    'key'     => 'bookable_products',
                    'value'   => '',
                    'compare' => '!='
                ))
        ));

        if (!$events_with_tickets) {
            return $count;
        }


        foreach ($events_with_tickets as $event_id) {

            $event_post        = new Event_Post($event_id);
            $bookable_products = get_post_meta($event_id, 'bookable_products', true);

            if (is_array($bookable_products)) {
                foreach ($bookable_products as $k => $bookable) {

                    if (false === $bookable instanceof Event_Meta_Bookable_Product) {
                        continue;
                    }

                    if ('1' !== get_post_meta($bookable->get_id(), 'stec_bookable', true)) {
                        $event_booking_tickets_sold_meta = Booking::get_tickets_sold($event_post);

                        if (isset($event_booking_tickets_sold_meta[$bookable->get_id()])) {
                            unset($event_booking_tickets_sold_meta[$bookable->get_id()]);
                            update_post_meta($event_id, 'stec-bookable-tickets-sold', $event_booking_tickets_sold_meta);
                        }

                        unset($bookable_products[$k]);
                        $count++;
                    }
                }

                update_post_meta($event_id, 'bookable_products', $bookable_products);
            }
        }

        return $count;
    }

    public static function remove_floating_events() {

        ini_set('max_execution_time', 0);

        Help::debug_log('Removing obsolete events');

        $cids = get_posts(array(
            'posts_per_page' => -1,
            'post_type'      => 'stec_calendar',
            'fields'         => 'ids',
        ));

        $floating_events = get_posts(array(
            'posts_per_page' => -1,
            'post_type'      => 'stec_event',
            'fields'         => 'ids',
            'meta_query'     => array(
                array(
                    'key'     => 'calid',
                    'value'   => $cids,
                    'compare' => 'NOT IN'
                )
            ),
        ));

        Help::debug_log('Total floating events found: ' . count($floating_events));

        if ($floating_events) {
            foreach ($floating_events as $id) {
                wp_delete_post($id, true);
                Help::debug_log('Deleted event: ' . $id);
            }
        }

        return count($floating_events);
    }

    public static function delete_reminders() {
        Reminder::delete_reminders();
        Help::debug_log('Reminders deleted');
    }

    public static function delete_gdpr_emails() {
        GDPR::delete_emails();
        Help::debug_log('GDPR emails deleted');
    }

    public static function delete_fs_version() {
        delete_option('stec_filesystem');
        Help::debug_log('stec_filesystem deleted');
    }

    public static function reset_permalinks() {
        delete_option('stec_permalinks');

        do_action('stec_add_rewrite_rules');

        flush_rewrite_rules();

        Help::debug_log('Permalink values restored to their defaults. Rules flushed');
    }

    public static function delete_builder_projects() {
        delete_option('stec_builder_projects');
        delete_option('stec_builder_index');
        Help::debug_log('Builder projects deleted');
    }

}
