<?php

namespace Stachethemes\Stec;

class Events {

    public static function get_admin_events_count(Array $params = array()) {

        $params['per_page'] = -1;
        $events             = self::get_admin_events($params);

        foreach ($events as $k => $event) {
            if (false === self::user_can_edit_event($event)) {
                unset($events[$k]);
            }
        }

        return apply_filters('stec_get_admin_events_count', count($events), $params, $events);
    }

    public static function get_admin_events(Array $params = array()) {

        $defaults = array(
            'page'       => 0,
            'per_page'   => 20,
            'meta_query' => array(),
            'tax_query'  => array(),
            'query'      => array(
                's' => isset($params['search']) ? trim($params['search']) : null
            )
        );

        foreach ($defaults as $k => $v) {
            if (!isset($params[$k])) {
                $params[$k] = $v;
            }
        }

        if (isset($params['calid'])) {
            $params['meta_query'] = array_merge($params['meta_query'], array(
                array(
                    'key'     => 'calid',
                    'value'   => $params['calid'],
                    'compare' => is_array($params['calid']) ? 'IN' : '=',
                )
            ));
        }

        $params = apply_filters('stec_get_admin_events_params', $params);

        $event_ids = get_posts(array_merge(array(
            'posts_per_page'   => $params['per_page'],
            'offset'           => max(0, $params['page'] * $params['per_page']),
            'post_type'        => 'stec_event',
            'fields'           => 'ids',
            'tax_query'        => $params['tax_query'],
            'meta_query'       => $params['meta_query'],
            'meta_key'         => 'start_date',
            'orderby'          => 'meta_value',
            'order'            => 'DESC',
            'suppress_filters' => 0), $params['query'])
        );

        $events = array();

        if (!$event_ids) {
            return array();
        }

        foreach ($event_ids as $id) {
            $event = new Event_Post($id);
            if (Events::user_can_edit_event($event)) {
                $events[] = $event;
            }
        }

        return apply_filters('stec_get_admin_events', $events, $params);
    }

    /**
     * Get events visible on the front-end by the current visitor
     */
    public static function get_front_events(Array $params = array()) {
        $events = self::get_events($params);

        foreach ($events as $k => $event) {
            if (false === self::user_can_view_event($event)) {
                unset($events[$k]);
            }
        }

        return apply_filters('stec_front_get_events', array_values($events), $params);
    }

    public static function get_events(Array $params = array()) {

        $defaults = array(
            'page'       => 0,
            'per_page'   => -1,
            'meta_query' => array(),
            'tax_query'  => array(),
            'query'      => array(
                's' => isset($params['search']) ? trim($params['search']) : null
            )
        );

        foreach ($defaults as $k => $v) {
            if (!isset($params[$k])) {
                $params[$k] = $v;
            }
        }

        if (isset($params['calid'])) {
            $params['meta_query'] = array_merge($params['meta_query'], array(
                array(
                    'key'     => 'calid',
                    'value'   => $params['calid'],
                    'compare' => is_array($params['calid']) ? 'IN' : '=',
                )
            ));
        }

        $params = apply_filters('stec_get_events_params', $params);

        $event_ids = get_posts(array_merge($params['query'], array(
            'posts_per_page'   => $params['per_page'],
            'offset'           => max(0, $params['page'] * $params['per_page']),
            'post_type'        => 'stec_event',
            'fields'           => 'ids',
            'tax_query'        => $params['tax_query'],
            'meta_query'       => $params['meta_query'],
            'meta_key'         => 'start_date',
            'orderby'          => 'meta_value',
            'order'            => 'DESC',
            'suppress_filters' => 0))
        );

        $events = array();

        if (!$event_ids) {
            return array();
        }

        foreach ($event_ids as $id) {
            $event    = new Event_Post($id);
            $events[] = $event;
        }

        return apply_filters('stec_get_events', $events, $params);
    }

    public static function get_aaproval_count($calendar_id = null) {

        $meta_query = array(
            array(
                'key'     => 'approved',
                'value'   => '0',
                'compare' => '=',
                'type'    => ''
            )
        );

        if ($calendar_id) {
            $meta_query[] = array(
                'key'     => 'calid',
                'value'   => $calendar_id,
                'compare' => '=',
                'type'    => ''
            );
        }


        $event_ids = get_posts(array(
            'posts_per_page' => -1,
            'post_type'      => 'stec_event',
            'fields'         => 'ids',
            'meta_query'     => $meta_query
        ));

        return count($event_ids);
    }

    public static function bulk_delete($event_ids = array()) {

        if (empty($event_ids)) {
            return true;
        }

        global $wpdb;
        $filter_event_ids = filter_var_array($event_ids, FILTER_VALIDATE_INT);
        $unique_event_ids = array_filter($filter_event_ids);

        $placeholders = array_fill(0, count($unique_event_ids), '%d');
        $format       = implode(', ', $placeholders);

        $sql = $wpdb->prepare("DELETE `POSTS`, `META`, `TERM`  
                FROM {$wpdb->prefix}posts as POSTS
                LEFT JOIN {$wpdb->prefix}postmeta AS META 
                ON META.post_id = POSTS.ID
                LEFT JOIN {$wpdb->prefix}term_relationships AS TERM 
                ON TERM.object_id = POSTS.ID
                WHERE POSTS.post_type = 'stec_event' AND POSTS.ID IN ($format)", $unique_event_ids);

        $result = $wpdb->query($sql);

        if (is_wp_error($result)) {
            throw new Stec_Exception($result->get_error_message());
        }

        $taxonomies = array(
            'stec_event_category',
            'stec_event_location',
            'stec_event_organizer'
        );

        foreach ($taxonomies as $taxonomy) {

            $update_terms = get_terms(array(
                'taxonomy'   => $taxonomy,
                'fields'     => 'ids',
                'hide_empty' => false,
            ));

            if ($update_terms) {
                wp_update_term_count_now($update_terms, $taxonomy);
            }
        }

        return true;
    }

    /**
     * Inserts or Updates post
     */
    public static function insert_post(Event_Post $event) {


        $post_data = apply_filters('stec_insert_post', $event->get_post_data());

        if ($event->get_id()) {
            $result = wp_update_post($post_data, true);
        } else {
            $result = wp_insert_post($post_data, true);
        }


        if (is_wp_error($result)) {
            throw new Stec_Exception($result->get_error_message());
        }

        if (!$result) {
            throw new Stec_Exception(esc_html__('Error processing post', 'stec'));
        }

        return $result;
    }

    public static function duplicate_event($event_id, $args = array()) {

        $event_post = new Event_Post($event_id);

        if ($event_post->get_id()) {

            $event_post->set_id(null);
            $event_post->set_uid(null);
            $event_post->set_title($event_post->get_title() . ' (' . esc_html__('DUPLICATE', 'stec') . ')');

            $result = $event_post->insert_post();

            if ($result) {
                if (isset($args['lang'])) {
                    global $wpdb;
                    $wpdb->update($wpdb->prefix . 'icl_translations', array('language_code' => $args['lang']), array('element_id' => $result));
                }
            }

            return $result;
        } else {
            throw new Stec_Exception(sprintf(esc_html__('Event with id %s not found', 'stec'), $event_id));
        }
    }

    public static function approve_event($event_id) {

        $event = new Event_Post($event_id);

        if (!$event->get_id()) {
            return false;
        }

        $event->set_approved(1);
        $event->insert_post();

        Mail::send_event_approved_mail($event);

        return true;
    }

    // Checks if user can view the event on the front-end
    public static function user_can_view_event(Event_Post $event_post) {

        $filter = apply_filters('stec_user_can_view_event', null, $event_post);

        if (null !== $filter) {
            return (bool) $filter;
        }

        if (is_super_admin()) {
            return true;
        }

        $visibility = $event_post->get_visibility();

        if (in_array('stec_cal_default', $visibility)) {
            $calendar_post = new Calendar_Post($event_post->get_calid());
            if (true === Calendars::user_can_view_calendar($calendar_post)) {
                return true;
            }
        }

        if (in_array('stec_private', $visibility)) {
            $created_by_cookie = $event_post->get_custom_meta('created_by_cookie');
            $user_cookie       = Submit_Event::get_user_cookie(false);

            if (null !== $user_cookie) {
                if ($created_by_cookie === $user_cookie) {
                    return true;
                }
            }

            if (!$created_by_cookie) {
                if ($event_post->get_author() === get_current_user_id()) {
                    return true;
                }
            }
        }

        $user_roles = Admin_Helper::get_user_roles();

        return (bool) array_intersect($visibility, $user_roles);
    }

    // Checks if user can edit the event
    public static function user_can_edit_event(Event_Post $event_post) {

        $filter = apply_filters('stec_user_can_edit_event', null, $event_post);

        if (null !== $filter) {
            return (bool) $filter;
        }

        if (is_super_admin()) {
            return true;
        }

        $back_visibility = $event_post->get_back_visibility();

        if (in_array('stec_cal_default', $back_visibility)) {
            $calendar_post = new Calendar_Post($event_post->get_calid());
            if (true === Calendars::user_can_edit_calendar($calendar_post)) {
                return true;
            }
        }

        if (in_array('stec_private', $back_visibility)) {
            $created_by_cookie = $event_post->get_custom_meta('created_by_cookie');
            $user_cookie       = Submit_Event::get_user_cookie(false);

            if (null !== $user_cookie) {
                if ($created_by_cookie === $user_cookie) {
                    return true;
                }
            }

            if (!$created_by_cookie) {
                if ($event_post->get_author() === get_current_user_id()) {
                    return true;
                }
            }
        }

        $user_roles = Admin_Helper::get_user_roles();

        return (bool) array_intersect($back_visibility, $user_roles);
    }

    public static function get_events_between($start, $end, $tz = 'UTC', $params = array(), $include_all = false) {

        $range_start            = new \DateTime($start, new \DateTimeZone($tz));
        $range_end              = new \DateTime($end, new \DateTimeZone($tz));
        $events                 = true === $include_all ? self::get_events($params) : self::get_front_events($params);
        $processed_events_array = array();

        foreach ($events as $event) {
            if ($event->get_rrule() !== '') {
                $occurencies = $event->get_occurrences($range_start->format('Y-m-d'), $range_end->format('Y-m-d'));
                foreach ($occurencies as $k => $occurence) {
                    if ($occurence['repeat_offset'] === 0 && $k === 0) {
                        continue;
                    }
                    $r_event = clone $event;
                    $r_event->set_start_date($r_event->get_start_date('Y-m-d H:i:s', $occurence['repeat_offset']));
                    $r_event->set_end_date($r_event->get_end_date('Y-m-d H:i:s', $occurence['repeat_offset']));
                    if ($r_event->get_start_date('U') < $range_start->format('U') || $r_event->get_end_date('U') > $range_end->format('U')) {
                        continue;
                    }
                    $processed_events_array[] = $r_event;
                }
            } else {
                if ($event->get_start_date('U') < $range_start->format('U') || $event->get_end_date('U') > $range_end->format('U')) {
                    continue;
                }
                $processed_events_array[] = $event;
            }
        }
        usort($processed_events_array, function($a, $b) {
            return $a->get_start_date('U') > $b->get_start_date('U') ? 1 : -1;
        });
        return $processed_events_array;
    }

}
