<?php

namespace Stachethemes\Stec;

class Csv_Export {

    private $calendar;
    private $content;
    private $fields;
    private $schedule_fields;
    private $schedule_content;
    private $guests_fields;
    private $guests_content;
    private $organizers_fields;
    private $organizers_content;
    private $attendance_fields;
    private $attendance_content;
    private $booking_fields;
    private $booking_content;
    private $shop_fields;
    private $shop_content;

    public function __construct(Calendar_Post $calendar) {
        $this->calendar = $calendar;

        $this->fields = apply_filters('stec_csv_export_fields', array(
            'id'                    => esc_html__('Event ID', 'stec'),
            'title'                 => esc_html__('Title', 'stec'),
            'start_date'            => esc_html__('Start Date', 'stec'),
            'end_date'              => esc_html__('End Date', 'stec'),
            'all_day'               => esc_html__('All Day', 'stec'),
            'rrule'                 => esc_html__('RRULE', 'stec'),
            'exdate'                => esc_html__('EXDATE', 'stec'),
            'recurrence_id'         => esc_html__('Recurrence ID', 'stec'),
            'description'           => esc_html__('Description', 'stec'),
            'description_short'     => esc_html__('Short Description', 'stec'),
            'location_address'      => esc_html__('Address', 'stec'),
            'location_city'         => esc_html__('City', 'stec'),
            'location_country'      => esc_html__('Country', 'stec'),
            'location_details'      => esc_html__('Location Details', 'stec'),
            'color'                 => esc_html__('Event Color', 'stec'),
            'icon'                  => esc_html__('Event Icon', 'stec'),
            'images'                => esc_html__('Images IDs', 'stec'),
            'featured'              => esc_html__('Featured', 'stec'),
            'calendar'              => esc_html__('Calendar Name', 'stec'),
            'calendar_id'           => esc_html__('Calendar ID', 'stec'),
            'categories'            => esc_html__('Categories', 'stec'),
            'categories_id'         => esc_html__('Categories ID', 'stec'),
            'link'                  => esc_html__('External Link', 'stec'),
            'permalink'             => esc_html__('Permalink', 'stec'),
            'rsvp'                  => esc_html__('RSVP', 'stec'),
            'rsvp_limit'            => esc_html__('Attendees Limit', 'stec'),
            'attendance_visibility' => esc_html__('Attendees Visibility', 'stec'),
            'uid'                   => esc_html__('UID', 'stec'),
            'keywords'              => esc_html__('Keywords', 'stec'),
            'author'                => esc_html__('Author Name', 'stec'),
        ));

        $this->schedule_fields = apply_filters('stec_csv_export_schedule_fields', array(
            'id'          => esc_html__('Event ID', 'stec'),
            'title'       => esc_html__('Title', 'stec'),
            'start_date'  => esc_html__('Start Date', 'stec'),
            'description' => esc_html__('Description', 'stec'),
            'color'       => esc_html__('Color', 'stec'),
            'icon'        => esc_html__('Icon', 'stec')
        ));

        $this->guests_fields = apply_filters('stec_csv_export_guests_fields', array(
            'id'          => esc_html__('Event ID', 'stec'),
            'name'        => esc_html__('Name', 'stec'),
            'photo'       => esc_html__('Photo ID', 'stec'),
            'photo_src'   => esc_html__('Photo URL', 'stec'),
            'about'       => esc_html__('About', 'stec'),
            'social_link' => esc_html__('Social Link', 'stec')
        ));

        $this->organizers_fields = apply_filters('stec_csv_export_organizers_fields', array(
            'id'          => esc_html__('Event ID', 'stec'),
            'name'        => esc_html__('Name', 'stec'),
            'email'       => esc_html__('E-mail', 'stec'),
            'photo'       => esc_html__('Photo ID', 'stec'),
            'photo_src'   => esc_html__('Photo URL', 'stec'),
            'details'     => esc_html__('Details', 'stec'),
            'social_link' => esc_html__('Social Link', 'stec')
        ));

        $this->attendance_fields = apply_filters('stec_csv_export_attendance_fields', array(
            'id'        => esc_html__('Event ID', 'stec'),
            'userid'    => esc_html__('User ID', 'stec'),
            'username'  => esc_html__('User Name', 'stec'),
            'email'     => esc_html__('Email', 'stec'),
            'mail_sent' => esc_html__('Invitation Sent', 'stec'),
            'status'    => esc_html__('Status', 'stec'),
        ));

        $this->booking_fields = apply_filters('stec_csv_export_booking_fields', array(
            'id'                 => esc_html__('Event ID', 'stec'),
            'product_id'         => esc_html__('Product ID', 'stec'),
            'title'              => esc_html__('Title', 'stec'),
            'sku'                => esc_html__('SKU', 'stec'),
            'url'                => esc_html__('URL', 'stec'),
            'purchesable'        => esc_html__('Purchesable', 'stec'),
            'is_in_stock'        => esc_html__('In Stock', 'stec'),
            'stock_quantity'     => esc_html__('Quantity', 'stec'),
            'quantity_per_event' => esc_html__('Quantity limit per Event', 'stec'),
            'price'              => esc_html__('Price', 'stec'),
            'currency'           => esc_html__('Currency', 'stec'),
            'sales'              => esc_html__('Total Sales', 'stec'),
        ));

        $this->shop_fields = apply_filters('stec_csv_export_shop_fields', array(
            'id'             => esc_html__('Event ID', 'stec'),
            'product_id'     => esc_html__('Product ID', 'stec'),
            'title'          => esc_html__('Title', 'stec'),
            'sku'            => esc_html__('SKU', 'stec'),
            'url'            => esc_html__('URL', 'stec'),
            'purchesable'    => esc_html__('Purchesable', 'stec'),
            'is_in_stock'    => esc_html__('In Stock', 'stec'),
            'stock_quantity' => esc_html__('Quantity', 'stec'),
            'price'          => esc_html__('Price', 'stec'),
            'currency'       => esc_html__('Currency', 'stec'),
        ));

        $this->content            = implode(',', $this->fields) . PHP_EOL;
        $this->schedule_content   = implode(',', $this->schedule_fields) . PHP_EOL;
        $this->guests_content     = implode(',', $this->guests_fields) . PHP_EOL;
        $this->organizers_content = implode(',', $this->organizers_fields) . PHP_EOL;
        $this->attendance_content = implode(',', $this->attendance_fields) . PHP_EOL;
        $this->booking_content    = implode(',', $this->booking_fields) . PHP_EOL;
        $this->shop_content       = implode(',', $this->shop_fields) . PHP_EOL;
    }

    private function array2csv($fields) {
        ob_start();
        $buffer = fopen('php://output', 'w+');
        fputcsv($buffer, $fields);
        fclose($buffer);
        return ob_get_clean();
    }

    public function get_content() {
        return rtrim($this->content);
    }

    public function get_schedule_content() {
        return rtrim($this->schedule_content);
    }

    public function get_guests_content() {
        return rtrim($this->guests_content);
    }

    public function get_organizers_content() {
        return rtrim($this->organizers_content);
    }

    public function get_attendance_content() {
        return rtrim($this->attendance_content);
    }

    public function get_booking_content() {
        return rtrim($this->booking_content);
    }

    public function get_shop_content() {
        return rtrim($this->shop_content);
    }

    public function proccess_event(Event_Post $event) {
        $data     = array();
        $location = $event->get_parsed_location();
        foreach ($this->fields as $k => $v) {
            switch ($k) :
                case 'id' :
                    $data[]           = $event->get_id();
                    break;
                case 'title' :
                    $data[]           = $event->get_title();
                    break;
                case 'start_date' :
                    $data[]           = $event->get_start_date();
                    break;
                case 'end_date' :
                    $data[]           = $event->get_end_date();
                    break;
                case 'all_day' :
                    $data[]           = $event->get_all_day();
                    break;
                case 'rrule' :
                    $data[]           = $event->get_rrule();
                    break;
                case 'exdate' :
                    $data[]           = $event->get_exdate();
                    break;
                case 'description' :
                    $data[]           = $event->get_description(true);
                    break;
                case 'description_short' :
                    $data[]           = $event->get_description_short();
                    break;
                case 'location_address' :
                    $data[]           = $location ? $location->address : '';
                    break;
                case 'location_details' :
                    $data[]           = $location ? $location->details : '';
                    break;
                case 'location_city' :
                    $data[]           = $location ? $location->city : '';
                    break;
                case 'location_country' :
                    $data[]           = $location ? $location->country : '';
                    break;
                case 'author' :
                    $data[]           = get_the_author_meta('display_name', $event->get_author());
                    break;
                case 'categories' :
                    $cats             = array();
                    $event_categories = $event->get_parsed_categories();

                    if ($event_categories) {
                        foreach ($event_categories as $cat) {
                            $cats[] = $cat['title'];
                        }
                    }
                    $data[]           = implode(', ', $cats);
                    break;
                case 'categories_id' :
                    $cats             = array();
                    $event_categories = $event->get_parsed_categories();

                    if ($event_categories) {
                        foreach ($event_categories as $cat) {
                            $cats[] = $cat['id'];
                        }
                    }
                    $data[] = implode(', ', $cats);
                    break;
                case 'calendar' :
                    $data[] = $this->calendar->get_title();
                    break;
                case 'calendar_id' :
                    $data[] = $this->calendar->get_id();
                    break;
                case 'color' :
                    $data[] = $event->get_color();
                    break;
                case 'rsvp' :
                    $data[] = $event->get_rsvp();
                    break;
                case 'rsvp_limit' :
                    $data[] = $event->get_rsvp_limit();
                    break;
                case 'attendance_visibility' :
                    $data[] = $event->get_attendance_visibility();
                    break;
                case 'icon' :
                    $data[] = $event->get_icon();
                    break;
                case 'featured' :
                    $data[] = $event->get_featured();
                    break;
                case 'link' :
                    $data[] = $event->get_link();
                    break;
                case 'permalink' :
                    $data[] = $event->get_permalink();
                    break;
                case 'uid' :
                    $data[] = $event->get_uid();
                    break;
                case 'keywords' :
                    $data[] = $event->get_keywords();
                    break;
                case 'recurrence_id' :
                    $data[] = $event->get_recurrence_id();
                    break;
                case 'images' :
                    $data[] = implode(',', $event->get_images());
                    break;

                default:
                    $filter = apply_filters('stec_csv_export_proccess_event_custom_fields', array(
                        'case'       => $k,
                        'event_post' => $event,
                        'content'    => ''
                    ));
                    $data[] = $filter['content'];
            endswitch;
        }

        $this->content .= $this->array2csv($data);
    }

    public function proccess_event_schedule(Event_Post $event) {

        $schedule = $event->get_parsed_schedule();

        if ($schedule) {
            foreach ($schedule as $s) {

                $data = array();

                foreach ($this->schedule_fields as $k => $v) {
                    switch ($k) :
                        case 'id' :
                            $data[] = $event->get_id();
                            break;
                        case 'title' :
                            $data[] = $s->title;
                            break;
                        case 'start_date' :
                            $data[] = $s->start_date;
                            break;
                        case 'color' :
                            $data[] = $s->icon_color;
                            break;
                        case 'icon' :
                            $data[] = $s->icon;
                            break;
                        case 'description' :
                            $data[] = $s->details;
                            break;
                        default:
                            $filter = apply_filters('stec_csv_export_proccess_schedule_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->schedule_content .= $this->array2csv($data);
            }
        }
    }

    public function proccess_event_guests(Event_Post $event) {

        $guests = $event->get_parsed_guests();

        if ($guests) {
            foreach ($guests as $guest) {

                $data = array();

                foreach ($this->guests_fields as $k => $v) {
                    switch ($k) :
                        case 'id' :
                            $data[]    = $event->get_id();
                            break;
                        case 'name' :
                            $data[]    = $guest->name;
                            break;
                        case 'photo' :
                            $data[]    = $guest->photo;
                            break;
                        case 'photo_src' :
                            $data[]    = $guest->photo_full;
                            break;
                        case 'about' :
                            $data[]    = $guest->about;
                            break;
                        case 'social_link' :
                            $links     = $guest->links;
                            $links_csv = array();
                            foreach ($links as $link) {
                                if ($link['link']) {
                                    $links_csv[] = $link['ico'] . ' - ' . $link['link'];
                                }
                            }
                            $data[] = $this->array2csv($links_csv);
                            break;
                        default:
                            $filter = apply_filters('stec_csv_export_proccess_guests_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->guests_content .= $this->array2csv($data);
            }
        }
    }

    public function proccess_event_organizers(Event_Post $event) {

        $organizers = $event->get_parsed_organizers(true);

        if ($organizers) {
            foreach ($organizers as $organizer) {

                $data = array();

                foreach ($this->organizers_fields as $k => $v) {
                    switch ($k) :
                        case 'id' :
                            $data[]      = $event->get_id();
                            break;
                        case 'name' :
                            $data[]      = $organizer->name;
                            break;
                        case 'photo' :
                            $data[]      = $organizer->photo;
                            break;
                        case 'photo_src' :
                            $data[]      = $organizer->photo_src;
                            break;
                        case 'details' :
                            $data[]      = $organizer->details;
                            break;
                        case 'email' :
                            $data[]      = $organizer->email;
                            break;
                        case 'social_link' :
                            $social      = $organizer->social;
                            $socials_csv = array();
                            foreach ($social as $link) {
                                if ($link['link']) {
                                    $socials_csv[] = $link['ico'] . ' - ' . $link['link'];
                                }
                            }
                            $data[] = $this->array2csv($socials_csv);
                            break;
                        default:
                            $filter = apply_filters('stec_csv_export_proccess_organizers_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->organizers_content .= $this->array2csv($data);
            }
        }
    }

    public function proccess_event_attendance(Event_Post $event) {

        $attendance = $event->get_parsed_attendees(true, true);

        if ($attendance) {
            foreach ($attendance as $attendee) {

                $data      = array();
                $user_info = null;
                if ($attendee->userid) {
                    $user_info = get_userdata($attendee->userid);
                }

                foreach ($this->attendance_fields as $k => $v) {
                    switch ($k) :
                        case 'id' :
                            $data[] = $event->get_id();
                            break;
                        case 'userid' :
                            $data[] = $attendee->userid;
                            break;
                        case 'username' :
                            $data[] = isset($user_info->user_nicename) ? $user_info->user_nicename : '';
                            break;
                        case 'email' :
                            if (!$attendee->email && isset($user_info->user_email)) {
                                $data[] = $user_info->user_email;
                            } else {
                                $data[] = $attendee->email;
                            }
                            break;
                        case 'mail_sent' :
                            $data[]   = $attendee->mail_sent;
                            break;
                        case 'status' :
                            $statuses = array();

                            foreach ($attendee->status as $date => $status) {
                                switch ((int) $status):
                                    case 1:
                                        $status = esc_html__('Accepted', 'stec');
                                        break;
                                    case 2:
                                        $status = esc_html__('Declined', 'stec');
                                        break;
                                    default:
                                        $status = esc_html__('No Response', 'stec');
                                endswitch;
                                $statuses[] = $date . ' - ' . $status;
                            }

                            $data[] = implode(PHP_EOL, $statuses);
                            break;
                        default:
                            $filter = apply_filters('stec_csv_export_proccess_attendance_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->attendance_content .= $this->array2csv($data);
            }
        }
    }

    public function proccess_event_booking(Event_Post $event) {

        $bookable_products = $event->get_parsed_bookable_products();

        if ($bookable_products) {
            foreach ($bookable_products as $product) {

                foreach ($this->booking_fields as $k => $v) {

                    switch ($k) :

                        case 'id' :
                            $data[] = $event->get_id();
                            break;

                        case 'product_id' :
                            $data[] = $product->id;
                            break;

                        case 'title' :
                            $data[] = $product->title;
                            break;

                        case 'sku' :
                            $data[] = $product->sku;
                            break;

                        case 'url' :
                            $data[] = $product->url;
                            break;

                        case 'purchesable' :
                            $data[] = $product->is_purchesable;
                            break;

                        case 'is_in_stock' :
                            $data[] = $product->is_in_stock;
                            break;

                        case 'stock_quantity' :
                            $data[] = $product->stock_quantity;
                            break;

                        case 'quantity_per_event' :
                            $data[] = $product->qty_limit_per_event;
                            break;

                        case 'price' :
                            $data[] = $product->price;
                            break;

                        case 'currency' :
                            $data[] = html_entity_decode(get_woocommerce_currency_symbol());
                            break;

                        case 'sales' :
                            $data[] = Booking::get_total_sales($product->id);
                            break;

                        default:
                            $filter = apply_filters('stec_csv_export_proccess_booking_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->booking_content .= $this->array2csv($data);
            }
        }
    }

    public function proccess_event_shop(Event_Post $event) {

        $shop_products = $event->get_parsed_products();

        if ($shop_products) {
            foreach ($shop_products as $product) {

                foreach ($this->shop_fields as $k => $v) {

                    switch ($k) :

                        case 'id' :
                            $data[] = $event->get_id();
                            break;

                        case 'product_id' :
                            $data[] = $product->id;
                            break;

                        case 'title' :
                            $data[] = $product->title;
                            break;

                        case 'sku' :
                            $data[] = $product->sku;
                            break;

                        case 'url' :
                            $data[] = $product->url;
                            break;

                        case 'purchesable' :
                            $data[] = $product->purchesable;
                            break;

                        case 'is_in_stock' :
                            $data[] = $product->is_in_stock;
                            break;

                        case 'stock_quantity' :
                            $data[] = $product->stock_quantity;
                            break;

                        case 'price' :
                            $data[] = $product->price;
                            break;

                        case 'currency' :
                            $data[] = html_entity_decode(get_woocommerce_currency_symbol());
                            break;

                        default:
                            $filter = apply_filters('stec_csv_export_proccess_shop_custom_fields', array(
                                'case'       => $k,
                                'event_post' => $event,
                                'content'    => ''
                            ));
                            $data[] = $filter['content'];
                    endswitch;
                }

                $this->shop_content .= $this->array2csv($data);
            }
        }
    }

    public function download_content($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ".csv\"");

        echo Admin_Helper::esc($this->get_content(), 'noscript');

        exit();
    }

    public function download_schedule($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Schedule' . ".csv\"");

        echo Admin_Helper::esc($this->get_schedule_content(), 'noscript');

        exit();
    }

    public function download_guests($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Guests' . ".csv\"");

        echo Admin_Helper::esc($this->get_guests_content(), 'noscript');

        exit();
    }

    public function download_organizers($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Organizers' . ".csv\"");

        echo Admin_Helper::esc($this->get_organizers_content(), 'noscript');

        exit();
    }

    public function download_attendance($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Attendance' . ".csv\"");

        echo Admin_Helper::esc($this->get_attendance_content(), 'noscript');

        exit();
    }

    public function download_booking($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Booking' . ".csv\"");

        echo Admin_Helper::esc($this->get_booking_content(), 'noscript');

        exit();
    }

    public function download_shop($filename = false) {

        $title = $filename ? $filename : $this->calendar->get_title() . ' ' . date('y-m-d His');

        header('Content-type: text/calendar; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $title . ' Shop' . ".csv\"");

        echo Admin_Helper::esc($this->get_shop_content(), 'noscript');

        exit();
    }

    public function download_all_zip($filename) {

        $stachethemes_ec = Stachethemes_Event_Calendar::get_instance();

        // Check FS Credentials
        if (!$stachethemes_ec->get_file_system_credentials()) {
            throw new Stec_Exception(esc_html__('Unable to connect to the file system. Check your credentials.', 'stec'));
        }

        $stec = Stachethemes_Event_Calendar::get_instance();
        $fs   = $stachethemes_ec->get_file_system();

        $filepath = $stec->get_path('ROOT') . 'temp' . time();

        if (!$fs->is_dir($filepath)) {
            $fs->mkdir($filepath);
        }

        $zip = new \ZipArchive();

        if ($zip->open($filepath . '/' . $filename . '.zip', \ZipArchive::CREATE) !== TRUE) {
            throw new Stec_Exception(sprintf(esc_html__('Cannot open %s zip file', 'stec'), $filename));
        }

        $zip->addFromString($filename . ".csv", $this->get_content());
        $zip->addFromString($filename . " Schedule.csv", $this->get_schedule_content());
        $zip->addFromString($filename . " Organizers.csv", $this->get_organizers_content());
        $zip->addFromString($filename . " Guests.csv", $this->get_guests_content());
        $zip->addFromString($filename . " Attendance.csv", $this->get_attendance_content());
        if (class_exists('WooCommerce')) {
            $zip->addFromString($filename . " Booking.csv", $this->get_booking_content());
            $zip->addFromString($filename . " Shop.csv", $this->get_shop_content());
        }
        $zip->close();

        ob_start();
        echo Admin_Helper::esc($fs->get_contents($filepath . '/' . $filename . '.zip'), 'noscript');
        $fs->delete($filepath, true);

        header('Content-type: application/zip; charset=utf-8');
        header("Content-Disposition: attachment; filename=\"" . $filename . ".zip\"");

        ob_get_flush();
        exit();
    }

}
