<?php

namespace Stachethemes\Stec;

class Export {

    private $calendar_id = null;
    private $event_id    = null;
    private $idlist      = null;
    private $category_id = null;

    public function __construct($calendar_id, $event_id = null, $idlist = null) {

        $this->calendar_id = (int) $calendar_id;

        if (!$this->calendar_id) {
            throw new Stec_Exception(esc_html__('No calendar id provided', 'stec'));
        }

        if ($event_id) {
            $this->event_id = (int) $event_id;
        }

        if (is_array($idlist)) {
            $this->idlist = $idlist;
        }
    }

    public function set_category($category_id) {
        $this->category_id = (int) $category_id;
    }

    public function export_ics() {

        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $ical_export = new Ical_Export($calendar);
        $events      = array();
        $filename    = false;

        // include event by single id
        if ($this->event_id) :

            $event = new Event_Post($this->event_id);

            if (!$event->get_id()) {
                throw new Stec_Exception(esc_html__('Event does not exists', 'stec'));
            }

            if ($this->category_id) {
                $event_categories = $event->get_categories();
                if (!in_array($this->category_id, $event_categories)) {
                    throw new Stec_Exception(esc_html__('Event category does not exists', 'stec'));
                }
            }

            if (!Events::user_can_view_event($event)) {
                throw new Stec_Exception(esc_html__('You have no permission to view this event', 'stec'));
            }

            $events[] = $event;
            $filename = $event->get_title();
        endif;

        // include selected events from idlist array
        if ($this->idlist) :
            foreach ($this->idlist as $event_id) {
                $event = new Event_Post($event_id);
                if ($this->category_id) {
                    $event_categories = $event->get_categories();
                    if (!in_array($this->category_id, $event_categories)) {
                        continue;
                    }
                }
                $events[] = $event;
            }
        endif;
        unset($event_id);

        if ($this->event_id === null && $this->idlist === null) {
            // Get all front-visible events from calendar id
            $events = Events::get_front_events(array(
                'calid' => $this->calendar_id
            ));
        }

        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            if ($this->category_id) {
                $event_categories = $event->get_categories();
                if (!in_array($this->category_id, $event_categories)) {
                    continue;
                }
            }

            $ical_export->proccess_event($event);
        endforeach;

        $ical_export->download($filename);
    }

    private function get_events() {

        $events = array();

        // include event by single id
        if ($this->event_id) :

            $event = new Event_Post($this->event_id);

            if (!$event->get_id()) {
                throw new Stec_Exception(esc_html__('Event does not exists', 'stec'));
            }

            if ($this->category_id) {
                $event_categories = $event->get_categories();
                if (!in_array($this->category_id, $event_categories)) {
                    throw new Stec_Exception(esc_html__('Event category does not exists', 'stec'));
                }
            }

            if (!Events::user_can_view_event($event)) {
                throw new Stec_Exception(esc_html__('You have no permission to view this event', 'stec'));
            }

            $events[] = $event;
        endif;

        // include selected events from idlist array
        if ($this->idlist) :
            foreach ($this->idlist as $event_id) {
                $event = new Event_Post($event_id);
                if ($this->category_id) {
                    $event_categories = $event->get_categories();
                    if (!in_array($this->category_id, $event_categories)) {
                        continue;
                    }
                }
                $events[] = $event;
            }
        endif;
        unset($event_id);

        if ($this->event_id === null && $this->idlist === null) {
            // Get all front-visible events from calendar id
            $events = Events::get_front_events(array(
                'calid' => $this->calendar_id
            ));
        }

        return $events;
    }

    public function export_csv() {

        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $events   = $this->get_events();
        $filename = $calendar->get_title();

        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        $csv_export = new Csv_Export($calendar);

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            if ($this->category_id) {
                $event_categories = $event->get_categories();
                if (!in_array($this->category_id, $event_categories)) {
                    continue;
                }
            }

            $csv_export->proccess_event($event);
            $csv_export->proccess_event_schedule($event);
            $csv_export->proccess_event_organizers($event);
            $csv_export->proccess_event_guests($event);
            $csv_export->proccess_event_attendance($event);
            if (class_exists('WooCommerce')) {
                $csv_export->proccess_event_booking($event);
                $csv_export->proccess_event_shop($event);
            }
        endforeach;

        $csv_export->download_all_zip($filename);
    }

    public function export_csv_content() {

        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $events   = $this->get_events();
        $filename = $calendar->get_title();

        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        $csv_export = new Csv_Export($calendar);

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            if ($this->category_id) {
                $event_categories = $event->get_categories();
                if (!in_array($this->category_id, $event_categories)) {
                    continue;
                }
            }

            $csv_export->proccess_event($event);
        endforeach;

        $csv_export->download_content($filename);
    }

    public function export_csv_schedule() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }

        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_schedule($event);
        endforeach;

        $csv_export->download_schedule($filename);
    }

    public function export_csv_guests() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }

        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_guests($event);
        endforeach;

        $csv_export->download_guests($filename);
    }

    public function export_csv_organizers() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }
        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_organizers($event);
        endforeach;

        $csv_export->download_organizers($filename);
    }

    public function export_csv_attendance() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }
        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_attendance($event);
        endforeach;

        $csv_export->download_attendance($filename);
    }

    public function export_csv_booking() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }
        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_booking($event);
        endforeach;

        $csv_export->download_booking($filename);
    }

    public function export_csv_shop() {
        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            throw new Stec_Exception(esc_html__('Calendar does not exists', 'stec'));
        }

        $csv_export = new Csv_Export($calendar);
        $events     = $this->get_events();
        $filename   = $calendar->get_title();

        if (count($events) === 1) {
            $filename = $events[0]->get_title();
        }
        if (empty($events)) {
            Admin_Helper::set_message(esc_html__('No events to export', 'stec'), 'error');
            return true;
        }

        foreach ($events as $event) :

            if (false === Events::user_can_view_event($event)) {
                continue;
            }

            $csv_export->proccess_event_shop($event);
        endforeach;

        $csv_export->download_shop($filename);
    }

}
