<?php

namespace Stachethemes\Stec;

use DateTime;
use DateTimeZone;

class Admin_Helper {

    private static $messages;
    private static $default_locale;

    private static function session($session = null) {

        if (null !== $session) {
            return update_option('stec-session', $session);
        }

        return get_option('stec-session', array());
    }

    public static function debug_log($msg) {

        if (defined('WP_DEBUG_LOG') && true === WP_DEBUG_LOG) {
            $pretext = 'STEC LOG :: ';
            error_log($pretext . $msg);
        }
    }

    public static function get($var, $default = false, $filter = FILTER_DEFAULT) {

        $value = filter_input(INPUT_GET, $var, $filter);

        return $value ? $value : $default;
    }

    public static function isset_post($var) {
        return $_SERVER['REQUEST_METHOD'] == 'POST' && array_key_exists($var, $_POST);
    }

    public static function isset_get($var) {
        return $_SERVER['REQUEST_METHOD'] == 'GET' && array_key_exists($var, $_GET);
    }

    public static function post($var, $default = false, $filter = FILTER_DEFAULT, $opt = FILTER_REQUIRE_SCALAR) {

        $value = filter_input(INPUT_POST, $var, $filter, $opt);

        return $value || $value === 0 || $value === '0' ? $value : $default;
    }

    public static function set_message($text, $type = '') {

        self::session_set('stec-admin-message', array(
            'text' => $text,
            'type' => $type
        ));
    }

    public static function session_set($var, $value = '') {

        $userid  = 'user-' . get_current_user_id();
        $session = self::session();

        if (!isset($session[$userid])) {
            $session[$userid] = array();
        }

        $session[$userid][$var] = $value;

        return self::session($session);
    }

    public static function session_get($var, $default = false) {
        $session = self::session();
        $userid  = 'user-' . get_current_user_id();

        if (!isset($session[$userid])) {
            return $default;
        }

        return isset($session[$userid][$var]) ? $session[$userid][$var] : $default;
    }

    public static function display_message() {

        self::$messages = self::session_get('stec-admin-message');

        if (self::$messages !== false) {

            switch (self::$messages['type']) :

                case 'notice' :
                    echo "<p class='stachethemes-admin-msg-attention'>" . Admin_Helper::esc(self::$messages['text'], 'noscript') . "</p>";
                    break;

                case 'error' :
                    echo "<p class='stachethemes-admin-msg-error'>" . Admin_Helper::esc(self::$messages['text'], 'noscript') . "</p>";
                    break;

                default:
                    echo "<p class='stachethemes-admin-msg-notice'>" . Admin_Helper::esc(self::$messages['text'], 'noscript') . "</p>";

            endswitch;
        }

        self::session_set('stec-admin-message', false);
        self::$messages = false;
    }

    public static function verify_nonce($action, $method = "POST") {

        switch (strtolower($method)) :

            case 'ajax' :

                if (check_ajax_referer('stec-nonce-string', 'security', false) === false) {
                    wp_die();
                }

                break;

            case 'get' :

                $nonce = self::get('_wpnonce');

                if (!wp_verify_nonce($nonce)) {
                    wp_die('Nonce did not verify');
                }

                break;


            default:

                $nonce = self::post('stec_admin_form_nonce');

                if (!wp_verify_nonce($nonce, $action)) {
                    wp_die('Nonce did not verify');
                }
        endswitch;
    }

    public static function get_hours_array() {

        $arr = array();

        $time_format = Settings::get_admin_setting_value('stec_menu__general', 'time_format');

        if ($time_format === '24') {
            // 24 hours

            $hour = 0;

            for ($i = 0; $i < 24; $i++) {
                $hour       = sprintf("%02d", $hour);
                $arr[$hour] = $hour;
                $hour++;
            }
        } else {
            // 12/12 hours
            $h24  = 0;
            $hour = 12;
            $ampm = "am";

            for ($i = 0; $i < 24; $i++) {


                if ($i === 12) {
                    $ampm = "pm";
                }

                if ($hour > 12) {
                    $hour = 1;
                }

                $hour = sprintf("%02d", $hour);
                $val  = $hour . "" . $ampm;

                $arr[sprintf("%02d", $h24)] = $val;

                $h24++;
                $hour++;
            }
        }

        return $arr;
    }

    public static function minutes_array() {
        $arr = array();
        for ($i = 0; $i < 60; $i += 5) {
            $val       = sprintf("%02d", $i);
            $arr[$val] = $val;
        }
        return $arr;
    }

    public static function social_array() {

        $arr = array();

        $arr["fab fa-behance"]     = "Behance";
        $arr["fab fa-deviantart"]  = "Devian Art";
        $arr["fab fa-dribbble"]    = "Dribbble";
        $arr["fab fa-facebook"]    = "Facebook";
        $arr["fab fa-flickr"]      = "Flickr";
        $arr["fab fa-github"]      = "Github";
        $arr["fab fa-google-plus"] = "Google Plus";
        $arr["fab fa-lastfm"]      = "LastFM";
        $arr["fab fa-linkedin"]    = "LinkedIn";
        $arr["fab fa-reddit"]      = "Reddit";
        $arr["fab fa-soundcloud"]  = "Soundcloud";
        $arr["fab fa-tumblr"]      = "Tumblr";
        $arr["fab fa-twitch"]      = "Twitch";
        $arr["fab fa-twitter"]     = "Twitter";
        $arr["fab fa-vimeo"]       = "Vimeo";
        $arr["fab fa-youtube"]     = "Youtube";
        $arr["fab fa-instagram"]   = "Instagram";
        $arr["fab fa-pinterest"]   = "Pinterest";
        $arr["fab fa-skype"]       = "Skype";
        $arr["fab fa-steam"]       = "Steam";
        $arr["fas fa-envelope"]    = "E-Mail";
        $arr["fas fa-globe"]       = "Website";

        return apply_filters('stec_social', $arr);
    }

    /**
     * Event icons
     * @return arr
     */
    public static function get_icon_list() {


        $arr = array();

        $arr['fa']                                        = esc_html__('No Icon', 'stec');
        $arr['fa fa-paw']                                 = esc_html__('Animals', 'stec');
        $arr['fa fa-plane']                               = esc_html__('Airport', 'stec');
        $arr['fas fa-glass-martini']                      = esc_html__('Bar', 'stec');
        $arr['fa fa-birthday-cake']                       = esc_html__('Birthday Party', 'stec');
        $arr['fa fa-briefcase']                           = esc_html__('Business', 'stec');
        $arr['fa fa-bus']                                 = esc_html__('Bus tour', 'stec');
        $arr['fa fa-motorcycle']                          = esc_html__('Bike', 'stec');
        $arr['fa fa-bomb']                                = esc_html__('Bomb', 'stec');
        $arr['fa fa-tree']                                = esc_html__('Camping ', 'stec');
        $arr['fa fa-ship']                                = esc_html__('Cruise', 'stec');
        $arr['fa fa-coffee']                              = esc_html__('Coffee', 'stec');
        $arr['fa fa-bicycle']                             = esc_html__('Cycling', 'stec');
        $arr['fa fa-balance-scale']                       = esc_html__('Court Hall', 'stec');
        $arr['fas fa-futbol']                             = esc_html__('Football', 'stec');
        $arr['fa fa-gamepad']                             = esc_html__('Gaming', 'stec');
        $arr['fa fa-graduation-cap']                      = esc_html__('Graduation', 'stec');
        $arr['fa fa-suitcase']                            = esc_html__('Hotel', 'stec');
        $arr['fa fa-microphone']                          = esc_html__('Karaoke', 'stec');
        $arr['fa fa-book']                                = esc_html__('Library ', 'stec');
        $arr['fa fa-film']                                = esc_html__('Movie', 'stec');
        $arr['fa fa-university']                          = esc_html__('Museum', 'stec');
        $arr['fa fa-music']                               = esc_html__('Music', 'stec');
        $arr['fa fa-camera']                              = esc_html__('Photo', 'stec');
        $arr['far fa-image']                              = esc_html__('Picture/Painting', 'stec');
        $arr['fa fa-utensils']                            = esc_html__('Restaurant', 'stec');
        $arr['fa fa-flag-checkered']                      = esc_html__('Race', 'stec');
        $arr['fa fa-shopping-basket']                     = esc_html__('Shopping', 'stec');
        $arr['fa fa-magic']                               = esc_html__('Show', 'stec');
        $arr['fa fa-beer']                                = esc_html__('Tavern', 'stec');
        $arr['fa fa-comments']                            = esc_html__('Talks', 'stec');
        $arr['fas fa-ticket-alt']                         = esc_html__('Ticket', 'stec');
        $arr['fa fa-train']                               = esc_html__('Train', 'stec');
        $arr['fa fa-wheelchair']                          = esc_html__('Wheelchair', 'stec');
        $arr['fa fa-binoculars']                          = esc_html__('Binocular', 'stec');
        $arr['fa fa-bed']                                 = esc_html__('Bed', 'stec');
        $arr['fa fa-bolt']                                = esc_html__('Bolt', 'stec');
        $arr['fa fa-check']                               = esc_html__('Check Sign', 'stec');
        $arr['fas fa-comment']                            = esc_html__('Conversation', 'stec');
        $arr['fa fa-credit-card']                         = esc_html__('Credit Card', 'stec');
        $arr['fas fa-gem']                                = esc_html__('Diamond', 'stec');
        $arr['fas fa-envelope']                           = esc_html__('Envelope', 'stec');
        $arr['fa fa-female']                              = esc_html__('Female Silhouette', 'stec');
        $arr['fa fa-male']                                = esc_html__('Male Silhouette', 'stec');
        $arr['fa fa-child']                               = esc_html__('Child Silhouette', 'stec');
        $arr['fa fa-fire-extinguisher']                   = esc_html__('Fire Extinguisher', 'stec');
        $arr['fas fa-hand-peace']                         = esc_html__('Hand Peace Sign', 'stec');
        $arr['fas fa-hand-point-up']                      = esc_html__('Hand Point Up', 'stec');
        $arr['fas fa-heart']                              = esc_html__('Heart', 'stec');
        $arr['fa fa-key']                                 = esc_html__('Key', 'stec');
        $arr['fa fa-gavel']                               = esc_html__('Gravel', 'stec');
        $arr['fa fa-life-ring']                           = esc_html__('Life Ring', 'stec');
        $arr['fas fa-lightbulb']                          = esc_html__('Lightbulb', 'stec');
        $arr['fas fa-money-bill-alt']                     = esc_html__('Money', 'stec');
        $arr['fas fa-moon']                               = esc_html__('Moon', 'stec');
        $arr['fa fa-shopping-bag']                        = esc_html__('Shopping Bag', 'stec');
        $arr['fas fa-video']                              = esc_html__('Video Camera', 'stec');
        $arr['fa fa-ambulance']                           = esc_html__('Ambulance', 'stec');
        $arr['fab fa-paypal']                             = esc_html__('PayPal', 'stec');
        $arr['fab fa-google-wallet']                      = esc_html__('Google Wallet', 'stec');
        $arr['fas fa-dollar-sign']                        = esc_html__('USD Sign', 'stec');
        $arr['fab fa-btc']                                = esc_html__('Bitcoin Sign', 'stec');
        $arr['fas fa-pound-sign']                         = esc_html__('British Pound Sign', 'stec');
        $arr['fas fa-euro-sign']                          = esc_html__('Euro Sign', 'stec');
        $arr['fab fa-apple']                              = esc_html__('Apple Logo', 'stec');
        $arr['fab fa-amazon']                             = esc_html__('Amazon Logo', 'stec');
        $arr['fab fa-android']                            = esc_html__('Android Logo', 'stec');
        $arr['fab fa-instagram']                          = esc_html__('Instagram Logo', 'stec');
        $arr['fab fa-facebook']                           = esc_html__('Facebook Logo', 'stec');
        $arr['fab fa-google-plus']                        = esc_html__('Google Plus Logo', 'stec');
        $arr['fab fa-steam']                              = esc_html__('Steam Logo', 'stec');
        $arr['fab fa-twitch']                             = esc_html__('Twitch Logo', 'stec');
        $arr['fab fa-yelp']                               = esc_html__('Yelp Logo', 'stec');
        $arr['fab fa-twitter']                            = esc_html__('Twitter Logo', 'stec');
        $arr['fab fa-youtube']                            = esc_html__('YouTube Logo', 'stec');
        $arr['fa fa-stethoscope']                         = esc_html__('Stethoscope', 'stec');
        $arr['fa fa-heartbeat']                           = esc_html__('Heartbeat', 'stec');
        $arr['fa fa-user-md']                             = esc_html__('Doctor', 'stec');
        $arr['fas fa-hospital-symbol']                    = esc_html__('Hospital', 'stec');
        $arr['fa fa-medkit']                              = esc_html__('Medkit', 'stec');
        $arr['fas fa-cut']                                = esc_html__('Scissors', 'stec');
        $arr['fa fa-chart-pie']                           = esc_html__('Pie Chart', 'stec');
        $arr['fa fa-transgender']                         = esc_html__('Transgender Symbol', 'stec');
        $arr['fa fa-mars']                                = esc_html__('Male Sex Symbol', 'stec');
        $arr['fa fa-venus']                               = esc_html__('Female Sex Symbol', 'stec');
        $arr['fa fa-blind']                               = esc_html__('Blind Men Symbol', 'stec');
        $arr['fa fa-american-sign-language-interpreting'] = esc_html__('Sign Language Symbol', 'stec');
        $arr['fas fa-newspaper']                          = esc_html__('Newspaper', 'stec');
        $arr['fa fa-flag']                                = esc_html__('Flag', 'stec');
        $arr['fa fa-hashtag']                             = esc_html__('Hashtag', 'stec');
        $arr['fab fa-snapchat-ghost']                     = esc_html__('Snapchat Logo', 'stec');
        $arr['fa fa-subway']                              = esc_html__('Subway', 'stec');
        $arr['fa fa-share-alt']                           = esc_html__('Share Icon', 'stec');
        $arr['fa fa-sign-language']                       = esc_html__('Sign Language Icon', 'stec');
        $arr['fab fa-odnoklassniki']                      = esc_html__('Odnoklassniki Logo', 'stec');
        $arr['fa fa-language']                            = esc_html__('Multi-language Icon', 'stec');
        $arr['fa fa-book']                                = esc_html__('Book Icon', 'stec');
        $arr['fa fa-puzzle-piece']                        = esc_html__('Puzzle Piece', 'stec');
        $arr['fas fa-tv']                                 = esc_html__('Television', 'stec');
        $arr['fa fa-desktop']                             = esc_html__('Desktop', 'stec');
        $arr['fa fa-calculator']                          = esc_html__('Calculator', 'stec');
        $arr['fa fa-cubes']                               = esc_html__('Cubes', 'stec');

        $user_icons = get_option('stec_user_fa_icons', array());

        return apply_filters('stec_icons', array_merge($arr, $user_icons));
    }

    public static function timezones_list() {

        $zones_array = array();
        $timestamp   = time();

        foreach (timezone_identifiers_list() as $zone) {
            date_default_timezone_set($zone);
            $zones_array[$zone] = $zone . ' (' . 'UTC/GMT ' . date('P', $timestamp) . ')';
        }

        return $zones_array;
    }

    public static function activate_license($purchase_code, $server_name) {

        $result = wp_remote_get("//api.stachethemes.com/calendar/activate/{$purchase_code}/{$server_name}");
        $json   = json_decode($result['body']);

        if (isset($json->success) && $json->success === 1) {
            $nfo = array(
                'purchase_code' => $purchase_code,
                'server_name'   => $server_name
            );
            update_option('stec_activated', $nfo, false);
            return true;
        }

        return false;
    }

    public static function deactivate_license($purchase_code, $server_name) {

        $result = wp_remote_get("//api.stachethemes.com/calendar/deactivate/{$purchase_code}/{$server_name}");
        $json   = json_decode($result["body"]);
        if (isset($json->success) && $json->success === 1) {
            delete_option('stec_activated');
            return true;
        }

        return false;
    }

    public static function calendar_writable_list() {
        $arr                   = self::get_user_roles_list();
        $arr['stec_private']   = 'Just Me';
        $arr['stec_logged_in'] = 'Logged in users';
        $arr['stec_public']    = 'Anyone';
        return apply_filters('stec_calendar_writable_list', array_reverse($arr));
    }

    public static function calendar_visibility_list() {
        $arr                   = self::get_user_roles_list();
        $arr['stec_private']   = 'Just Me';
        $arr['stec_logged_in'] = 'Logged in users';
        $arr['stec_public']    = 'Public';
        return apply_filters('stec_calendar_visibility_list', array_reverse($arr));
    }

    public static function event_visibility_list() {
        $arr                     = self::get_user_roles_list();
        $arr['stec_private']     = 'Just Me';
        $arr['stec_logged_in']   = 'Logged in users';
        $arr['stec_public']      = 'Public';
        $arr['stec_cal_default'] = 'Calendar Default';
        return apply_filters('stec_event_visibility_list', array_reverse($arr));
    }

    public static function get_wp_users_list() {
        $users = get_users(array(
            'fields' => array('ID', 'display_name')
        ));

        $users_array = array();

        foreach ($users as $user) {
            $users_array[$user->ID] = $user->display_name;
        }

        return $users_array;
    }

    public static function get_wp_roles_list() {
        global $wp_roles;
        if (!isset($wp_roles) || !($wp_roles instanceof \WP_Roles)) {
            $wp_roles = new \WP_Roles();
        }
        return $wp_roles->get_names();
    }

    public static function get_user_roles_list() {

        require_once( ABSPATH . '/wp-admin/includes/user.php' );

        $roles = get_editable_roles();
        $roles = array_reverse($roles);
        $arr   = array();

        foreach ($roles as $role => $details) {
            $name       = translate_user_role($details['name']);
            $arr[$role] = $name;
        }

        return apply_filters('stec_get_user_roles_list', $arr);
    }

    public static function get_user_roles($userid = null) {

        if (null === $userid) {
            $userid = get_current_user_id();
        }

        $userid = (int) $userid;

        $user_info    = get_userdata($userid);
        $user_roles   = isset($user_info->roles) ? $user_info->roles : array();
        $user_roles[] = 'stec_public';

        if (is_user_logged_in()) {
            $user_roles[] = 'stec_logged_in';
        }

        return $user_roles;
    }

    public static function get_plugin_version() {
        return STEC_PLUGIN_VERSION;
    }

    /**
     * Returns human readable format of the timespan
     * @param object $event the event object
     * @return string the timespan
     */
    public static function get_the_timespan(Event_Post $event, $repeat_offset = 0, $plaintext = false) {

        $tz = new DateTimeZone($event->get_timezone());

        $dateformat      = Settings::get_admin_setting_value('stec_menu__general', 'date_format');
        $timeformat      = Settings::get_admin_setting_value('stec_menu__general', 'time_format');
        $show_utc_offset = Settings::get_admin_setting_value('stec_menu__general', 'date_label_gmtutc');

        $start_datetime = new \DateTime($event->get_start_date('Y-M-d H:i:s'), $tz);
        $end_datetime   = new \DateTime($event->get_end_date('Y-M-d H:i:s'), $tz);

        if ($repeat_offset) {
            $start_datetime->add(new \DateInterval('P' . floor($repeat_offset / 86400) . 'D'));
            $end_datetime->add(new \DateInterval('P' . floor($repeat_offset / 86400) . 'D'));
        }

        $start_stamp = $start_datetime->format('U') + $tz->getOffset($start_datetime);
        $end_stamp   = $end_datetime->format('U') + $tz->getOffset($end_datetime);

        if (true === $plaintext) {

            // Translate months
            $sm = ucfirst(date_i18n('F', $start_stamp));
            $em = ucfirst(date_i18n('F', $end_stamp));

            switch ($dateformat) :

                case 'dd.mm.yyyy' :
                    $start_date = $start_datetime->format('d. ') . $sm . $start_datetime->format('. Y');
                    $end_date   = $end_datetime->format('d. ') . $em . $end_datetime->format('. Y');

                    break;

                case 'dd-mm-yy' :
                    $start_date = $start_datetime->format('d ') . $sm . $start_datetime->format(' Y');
                    $end_date   = $end_datetime->format('d ') . $em . $end_datetime->format(' Y');

                    break;

                case 'mm-dd-yy' :
                    $start_date = $sm . $start_datetime->format('-d-') . $start_datetime->format('Y');
                    $end_date   = $em . $end_datetime->format('-d-') . $end_datetime->format('Y');

                    break;

                case 'yy-mm-dd' :

                default:
                    $start_date = $start_datetime->format('Y-') . $sm . $start_datetime->format('-d');
                    $end_date   = $end_datetime->format('Y- ') . $em . $end_datetime->format('-d');

            endswitch;

            switch ($timeformat) :
                case '12':
                    $timeformat = 'h:ia';
                    break;

                case '24':
                    $timeformat = 'H:i';
                default:

            endswitch;

            $start_time = $start_datetime->format($timeformat);
            $end_time   = $end_datetime->format($timeformat);

            if ($event->get_hide_end()) {

                $the_date = "$start_date $start_time";
            } elseif ($start_date === $end_date) {

                if ((int) $event->get_all_day() === 1) {
                    $the_date = $start_date;
                } else {
                    $the_date = "$start_date $start_time - $end_time";
                }
            } else {

                if ((int) $event->get_all_day() === 1) {
                    $the_date = "$start_date - $end_date";
                } else {
                    $the_date = "$start_date $start_time - $end_date $end_time";
                }
            }

            if ((int) $show_utc_offset === 1) {
                $the_date .= ' ' . $event->get_timezone();
            }

            return $the_date;
        }

        // Translate months
        $sm = '<span class="stec-layout-single-month-full">' . ucfirst(date_i18n('F', $start_stamp)) . '</span>';
        $em = '<span class="stec-layout-single-month-full">' . ucfirst(date_i18n('F', $end_stamp)) . '</span>';

        $sm .= '<span class="stec-layout-single-month-short">' . ucfirst(date_i18n('M', $start_stamp)) . '</span>';
        $em .= '<span class="stec-layout-single-month-short">' . ucfirst(date_i18n('M', $end_stamp)) . '</span>';

        switch ($dateformat) :

            case 'dd.mm.yyyy' :

                $start_date = '<span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span>. ' .
                        $sm .
                        '. <span class="stec-layout-single-year">' . $start_datetime->format('Y') . '</span>';

                $end_date = '<span class="stec-layout-single-day">' . $end_datetime->format('d') . '</span>. ' .
                        $em .
                        '. <span class="stec-layout-single-year">' . $end_datetime->format('Y') . '</span>';

                break;

            case 'dd-mm-yy' :

                $start_date = '<span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ' .
                        $sm .
                        ' <span class="stec-layout-single-year">' . $start_datetime->format('Y') . '</span>';

                $end_date = '<span class="stec-layout-single-day">' . $end_datetime->format('d') . '</span> ' .
                        $em .
                        ' <span class="stec-layout-single-year">' . $end_datetime->format('Y') . '</span>';
                break;

            case 'mm-dd-yy' :

                $start_date = $sm .
                        ' <span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ' .
                        ' <span class="stec-layout-single-year">' . $start_datetime->format('Y') . '</span>';

                $end_date = $em .
                        ' <span class="stec-layout-single-day">' . $end_datetime->format('d') . '</span> ' .
                        ' <span class="stec-layout-single-year">' . $end_datetime->format('Y') . '</span>';

                break;

            case 'yy-mm-dd' :

            default:
                $start_date = ' <span class="stec-layout-single-year">' . $start_datetime->format('Y') . '</span>' .
                        $sm .
                        ' <span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ';

                $end_date = ' <span class="stec-layout-single-year">' . $end_datetime->format('y') . '</span>' .
                        $em .
                        ' <span class="stec-layout-single-day">' . $end_datetime->format('d') . '</span> ';

        endswitch;

        switch ($timeformat) :
            case '12':
                $timeformat = 'h:ia';
                break;

            case '24':
                $timeformat = 'H:i';
            default:

        endswitch;

        $start_time = '<span class="stec-layout-single-start-time">' . $start_datetime->format($timeformat) . '</span>';
        $end_time   = '<span class="stec-layout-single-end-time">' . $end_datetime->format($timeformat) . '</span>';

        if ($event->get_hide_end()) {

            $the_date = "$start_date $start_time";
        } elseif ($start_date === $end_date) {

            if ((int) $event->get_all_day() === 1) {
                $the_date = $start_date;
            } else {
                $the_date = "$start_date $start_time - $end_time";
            }
        } else {

            if ((int) $event->get_all_day() === 1) {
                $the_date = "$start_date - $end_date";
            } else {
                $the_date = "$start_date $start_time - $end_date $end_time";
            }
        }

        if ((int) $show_utc_offset === 1) {
            $the_date .= ' ' . '<span class="stec-layout-single-timezone">' . $event->get_timezone() . '</span>';
        }

        return $the_date;
    }

    public static function meta_schema_datetime_iso8601(Event_Post $event, $repeat_offset = 0) {
        ?>
        <meta itemprop="startDate" content="<?php esc_attr_e($event->get_start_date('Y-m-d\TH:i', $repeat_offset)); ?>"/>
        <meta itemprop="endDate" content="<?php esc_attr_e($event->get_end_date('Y-m-d\TH:i', $repeat_offset)); ?>"/>
        <?php
    }

    /**
     * Whether or not event has valid reminder
     * @param type $event
     * @return boolean
     */
    public static function reminder_expired(Event_Post $event, $repeat_offset = 0) {


        if (Settings::get_admin_setting_value('stec_menu__general', 'reminder') === '0') {
            return true;
        }

        $start = strtotime($event->get_start_date()) + $repeat_offset;


        $utc_start = self::to_utc_time(date('Y-m-d H:i:s', $start), $event->get_timezone(), 'U');

        if (gmdate('U') >= $utc_start) {
            return true;
        }

        return false;
    }

    /**
     * 
     * @param DateTime $date Date object
     * @param type $timezone event timezone string
     * @param type $format time format
     * @return type UTC time string
     */
    public static function to_utc_time($date, $timezone, $format = 'Ymd\THis') {

        $UTC    = new DateTimeZone("UTC");
        $caltTZ = new DateTimeZone($timezone);

        $date = new DateTime($date, $caltTZ);
        $date->setTimezone($UTC);

        return $date->format($format);
    }

    public static function user_is_invited(Event_Post $event) {

        if (is_user_logged_in() === false) {
            return false;
        }

        $userid = get_current_user_id();

        foreach ($event->get_attendance(true) as $attendee) {

            if ($attendee->get_userid() === $userid) {
                return true;
            }
        }

        return false;
    }

    public static function get_user_attendance_status(Event_Post $event, $userid_or_email = false, $repeat_offset = 0) {

        if (!$userid_or_email) {
            $userid_or_email = get_current_user_id();
        }

        $date = $event->get_start_date('Y-m-d', $repeat_offset);

        foreach ($event->get_attendance(true) as $attendee) {

            switch ((bool) filter_var($userid_or_email, FILTER_VALIDATE_EMAIL)) :
                case false:
                    if ($attendee->get_userid() === $userid_or_email && $attendee->get_status($date)) {
                        return $attendee->get_status($date);
                    }
                    break;

                case true:
                    if ($attendee->get_email() === $userid_or_email && $attendee->get_status($date)) {
                        return $attendee->get_status($date);
                    }
                    break;
            endswitch;
        }

        return 0;
    }

    public static function event_has_tabs(Event_Post $event) {

        $has_weather_api_key = Settings::get_admin_setting_value('stec_menu__general', 'weather_api_key');
        $location_coords     = '';

        if ($event->get_location()) {
            $location = $event->get_parsed_location();
            if ($location->coordinates) {
                $location_coords = $location->coordinates;
            }
        }

        if (
                !($event->get_schedule()) ||
                !($event->get_guests()) ||
                !($event->get_attendance()) ||
                !($event->get_products()) ||
                ($location_coords !== '' && $has_weather_api_key !== '') ||
                (int) $event->get_comments() !== 0
        ) {
            return true;
        }

        return false;
    }

    /**
     * Returns human readable format of the timespan
     */
    public static function get_the_schedule_timespan($date_string, Event_Post $event, $repeat_offset = 0) {

        $tz = new DateTimeZone($event->get_timezone());

        $dateformat = Settings::get_admin_setting_value('stec_menu__general', 'date_format');
        $timeformat = Settings::get_admin_setting_value('stec_menu__general', 'time_format');

        $start_datetime = new \DateTime($date_string, new DateTimeZone($event->get_timezone()));
        $start_stamp    = $start_datetime->format('U') + $tz->getOffset($start_datetime);

        if ($repeat_offset) {
            $start_datetime->add(new \DateInterval('P' . floor($repeat_offset / 86400) . 'D'));
        }

        // Translate months

        $sm = '<span class="stec-layout-single-month-full">' . ucfirst(date_i18n('M', $start_stamp)) . '</span>';

        switch ($dateformat) :

            case 'dd-mm-yy' :

                $start_date = '<span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ' .
                        $sm;

                break;

            case 'mm-dd-yy' :
                $start_date = $sm .
                        ' <span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ';
                break;

            case 'yy-mm-dd' :
            default:
                $start_date = $sm . ' <span class="stec-layout-single-day">' . $start_datetime->format('d') . '</span> ';
                break;
        endswitch;

        switch ($timeformat) :
            case '12':
                $timeformat = 'h:ia';
                break;

            case '24':
                $timeformat = 'H:i';
            default:

        endswitch;

        $start_time = $start_datetime->format($timeformat);
        $the_date   = "$start_date <span class='stec-layout-single-time'>$start_time</span>";

        return $the_date;
    }

    public static function get_product_price($product) {

        $price = $product->html_price;

        if ($product->has_child) {
            $price .= '<a href="' . esc_url($product->url) . '">' . esc_html__('(Select Options)', 'stec') . '</a>';
        }

        return $price;
    }

    public static function update_attendance_by_url($event_id, $access_token, $status) {

        $event = new Event_Post($event_id);

        if (!$event->get_id()) {
            return false;
        }

        $attendees = $event->get_attendance(true);

        foreach ($attendees as &$attendee) {

            if ($attendee->get_access_token() !== $access_token) {
                continue;
            }
        }

        $email  = filter_var($attendee->get_email(), FILTER_VALIDATE_EMAIL);
        $params = array('date' => $event->get_start_date('Y-m-d'));

        if ($email) {
            $params['email'] = $email;
        } elseif ($attendee->get_userid()) {
            $params['userid'] = $attendee->get_userid();
        } else {
            return false;
        }

        return $event->set_attendee_status((int) $status, $params, true);
    }

    /**
     * @return json
     */
    public static function get_weather_data($location) {

        $apikey = Settings::get_admin_setting_value('stec_menu__general', 'weather_api_key');
        $url    = esc_url("https://api.darksky.net/forecast/{$apikey}/" . urlencode($location));
        $data   = '';

        $cache = self::get_cache(array($apikey, $url), true);

        if ($cache) {
            return $cache;
        }

        $response = wp_remote_get($url);

        if (is_array($response)) {
            $data = ($response['body']);
            self::set_cache(array($apikey, $url), $data, 3600, true);
        }

        return $data;
    }

    public static function get_user_created_by_cookie() {

        $cookie_name = "stachethemes_ec_anon_user";

        return isset($_COOKIE[$cookie_name]) && strlen($_COOKIE[$cookie_name]) > 13 ? $_COOKIE[$cookie_name] : '';
    }

    public static function get_current_user() {

        $the_user = false;

        if (is_user_logged_in()) {
            $the_user = wp_get_current_user();
        }

        return $the_user;
    }

    public static function get_current_user_email() {
        $the_user = self::get_current_user();

        return $the_user && isset($the_user->user_email) ? $the_user->user_email : false;
    }

    private static function get_cache_string($request_data) {
        if ($request_data instanceof WP_REST_Request) {
            $cache_string = md5(serialize($request_data->get_params()));
        } else {
            $cache_string = md5(serialize($request_data));
        }

        return 'stec_transient_' . $cache_string;
    }

    // Is event cache enabled check
    private static function do_cache() {

        return Settings::get_admin_setting_value('stec_menu__cache', 'cache');
    }

    public static function get_cache($request_data, $force_get = false) {

        if (false === $force_get && !self::do_cache()) {
            return false;
        }

        $cache_string = self::get_cache_string($request_data);
        $cache        = get_transient($cache_string);

        return $cache;
    }

    public static function set_cache($request_data, $content, $time = 86400, $force_set = false) {

        if (false === $force_set && !self::do_cache()) {
            return false;
        }

        $cache_string = self::get_cache_string($request_data);

        return set_transient($cache_string, $content, $time);
    }

    public static function delete_cache() {
        global $wpdb;

        $wpdb->query("DELETE FROM `$wpdb->options` WHERE `option_name` LIKE ('_transient%stec_transient_%')");
        $wpdb->query("DELETE FROM `$wpdb->options` WHERE `option_name` LIKE ('_transient_timeout%stec_transient_%')");
        wp_cache_flush();

        return true;
    }

    public static function get_permalinks($page = null, $default = null) {

        $permalinks = get_option('stec_permalinks', array(
            'calendar'  => 'stec_calendar',
            'event'     => 'stec_event',
            'cron'      => 'stec_cron',
            'edit'      => 'stec_edit',
            'organizer' => 'stec_organizer'
        ));

        if ($page) {
            return isset($permalinks[$page]) ? $permalinks[$page] : $default;
        }

        return $permalinks;
    }

    public static function get_share_links($permalink = '#stec_replace_event_single_url') {

        $messenger_app_id = Settings::get_admin_setting_value('stec_menu__general', 'messenger_app_id');
        $share_links      = array();

        $share_links['facebook'] = '<a target="_BLANK" href="' . sprintf("//www.facebook.com/share.php?u=%s", esc_url($permalink)) . '"><i class="fab fa-facebook"></i></a>';
        $share_links['twitter']  = '<a target="_BLANK" href="' . sprintf("//twitter.com/intent/tweet?text=%s", esc_url($permalink)) . '"><i class="fab fa-twitter"></i></a>';
        $share_links['whatsapp'] = '<a target="_BLANK" href="' . sprintf("//api.whatsapp.com/send?text=%s", esc_url($permalink)) . '"><i class="fab fa-whatsapp"></i></a>';
        if ($messenger_app_id) {
            $share_links['messenger'] = '<a target="_BLANK" href="' . esc_url(sprintf('//www.facebook.com/dialog/send?display=popup&app_id=%1$s&link=%2$s&redirect_uri=%2$s', $messenger_app_id, $permalink)) . '"><i class="fab fa-facebook-messenger"></i></a>';
        }
        $share_links['viber'] = '<a target="_BLANK" href="' . sprintf("viber://forward?text=%s", esc_url($permalink)) . '"><i class="fab fa-viber"></i></a>';
        $share_links['share'] = '<a target="_BLANK" class="stec-embed-button" href="' . esc_url($permalink) . '"><i class="fa fa-link"></i></a>';
        $share_links['email'] = '<a class="stec-email-button" href="' . esc_url($permalink) . '"><i class="fa fa-envelope"></i></a>';
        $shares               = apply_filters('stec-share-links', $share_links, $permalink);

        ob_start();
        foreach ($shares as $share) {
            echo Admin_Helper::esc($share, 'noscript');
        }
        return ob_get_clean();
    }

    public static function is_admin_menu() {
        global $plugin_page;

        if ($plugin_page) {
            if (strpos($plugin_page, 'stec_menu__') === 0) {
                return true;
            }
        }

        return false;
    }

    public static function image_exists($slug) {
        $args        = array(
            'post_type'      => 'attachment',
            'name'           => sanitize_title($slug),
            'posts_per_page' => 1
        );
        $attachments = get_posts($args);

        $attachment = $attachments ? array_pop($attachments) : null;

        if (isset($attachment->ID)) {
            if (wp_attachment_is_image($attachment->ID)) {
                return (int) $attachment->ID;
            }
        }

        return false;
    }

    public static function attachment_exists($slug) {

        $args        = array(
            'post_type'      => 'attachment',
            'name'           => sanitize_title($slug),
            'posts_per_page' => 1
        );
        $attachments = get_posts($args);

        $attachment = $attachments ? array_pop($attachments) : null;

        if (isset($attachment->ID)) {
            return (int) $attachment->ID;
        }

        return false;
    }

    public static function file_to_media_upload($filepath, $name = null, $check_exists = true) {
        $file     = $filepath;
        $filename = $name ? basename($name) : basename($filepath);

        if (true === $check_exists) {
            $attachment_exists = self::attachment_exists($filename);
            if (false !== filter_var($attachment_exists, FILTER_VALIDATE_INT)) {
                return (int) $attachment_exists;
            }
        }

        $upload_file = wp_upload_bits($filename, null, file_get_contents($file));

        if (!$upload_file['error']) {

            $wp_filetype = wp_check_filetype($filename, null);

            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title'     => sanitize_title($filename),
                'post_slug'      => sanitize_title($filename),
                'post_content'   => '',
                'post_status'    => 'inherit'
            );

            $attachment_id = wp_insert_attachment($attachment, $upload_file['file'], 0, true);

            if (!is_wp_error($attachment_id)) {
                require_once(ABSPATH . "wp-admin" . '/includes/image.php');
                $attachment_data = wp_generate_attachment_metadata($attachment_id, $upload_file['file']);
                wp_update_attachment_metadata($attachment_id, $attachment_data);
                return $attachment_id;
            }
        }

        return false;
    }

    public static function file_image_to_media_upload($filepath, $name = null, $check_exists = true) {
        $file     = $filepath;
        $filename = $name ? basename($name) : basename($file);

        if (true === $check_exists) {
            $image_exists = self::image_exists($filename);
        }

        if (false !== filter_var($image_exists, FILTER_VALIDATE_INT)) {
            return (int) $image_exists;
        }

        $upload_file = wp_upload_bits($filename, null, file_get_contents($file));

        if (!$upload_file['error']) {

            $wp_filetype = wp_check_filetype($filename, null);

            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title'     => sanitize_title($filename),
                'post_slug'      => sanitize_title($filename),
                'post_content'   => '',
                'post_status'    => 'inherit'
            );

            $attachment_id = wp_insert_attachment($attachment, $upload_file['file'], 0, true);

            if (!is_wp_error($attachment_id)) {
                require_once(ABSPATH . "wp-admin" . '/includes/image.php');
                $attachment_data = wp_generate_attachment_metadata($attachment_id, $upload_file['file']);
                wp_update_attachment_metadata($attachment_id, $attachment_data);
                return $attachment_id;
            }
        }

        return false;
    }

    public static function url_image_to_media_upload($image_url) {

        if (!function_exists('media_handle_upload')) {
            require_once(ABSPATH . "wp-admin" . '/includes/image.php');
            require_once(ABSPATH . "wp-admin" . '/includes/file.php');
            require_once(ABSPATH . "wp-admin" . '/includes/media.php');
        }

        $tmp = download_url($image_url);

        if (is_wp_error($tmp)) {
            throw new Stec_Exception(esc_html__('Error downloading image', 'stec'));
        }

        if (false === @getimagesize($tmp)) {
            @unlink($tmp);
            throw new Stec_Exception(esc_html__('File is not an image', 'stec'));
        }

        $image_exists = self::image_exists(basename($image_url));

        if (false !== filter_var($image_exists, FILTER_VALIDATE_INT)) {
            return (int) $image_exists;
        }

        $file_array = array(
            'name'     => basename($image_url),
            'type'     => mime_content_type($tmp),
            'tmp_name' => $tmp,
            'error'    => 0,
            'size'     => filesize($tmp),
        );

        $sideload = wp_handle_sideload($file_array, array(
            'test_form' => false,
            'test_size' => true,
        ));

        if (isset($sideload['error'])) {
            throw new Stec_Exception($sideload['error']);
        }

        $id = wp_insert_attachment(array(
            'post_mime_type' => $sideload['type'],
            'guid'           => $sideload['url'],
            'post_title'     => $file_array['name'],
            'post_content'   => '',
                ), $sideload['file'], 0, true);

        if (is_wp_error($id)) {
            throw new Stec_Exception(esc_html__('Unable to insert attachment image', 'stec'));
        }

        $attach_data = wp_generate_attachment_metadata($id, $sideload['file']);
        wp_update_attachment_metadata($id, $attach_data);

        return $id;
    }

    public static function convert_date_string($date_string, $format = 'Y-m-d H:i:s') {
        $date = new \DateTime($date_string, new \DateTimeZone('UTC'));
        return $date->format($format);
    }

    public static function get_event_og(Event_Post $event, $offset = 0) {

        $image = '';

        if ($event->get_images()) {
            $images = $event->get_parsed_images();
            $image  = $images[0]->src;
        }

        $og = apply_filters('stec_get_event_og', array(
            'og:title'         => $event->get_title(),
            'og:type'          => 'event',
            'event:start_time' => $event->get_start_date(DATE_ISO8601, $offset),
            'og:description '  => $event->get_description_short(),
            'og:image'         => $image,
            'og:url'           => $event->get_permalink() . ($offset ? '/' . $offset : '')
                ), $event, $offset);

        if (!$og || !is_array($og)) {
            return '';
        }

        $meta_html = '<!-- OpenGraph generated by St. Event Calendar -->' . PHP_EOL;
        foreach ($og as $key => $value) {
            $meta_html .= "<meta property='{$key}' content='{$value}' />" . PHP_EOL;
        }
        return $meta_html;
    }

    public static function encrypt($text) {

        if (true !== function_exists('openssl_encrypt')) {
            throw new Stec_Exception('openssl_encrypt not found');
        }

        if (is_array($text)) {
            $text = serialize($text);
        }

        $encrypt_method = "AES-256-CBC";
        $secret_key     = wp_salt('secure_auth');
        $secret_iv      = wp_salt('nonce');
        $key            = hash('sha256', $secret_key);
        $iv             = substr(hash('sha256', $secret_iv), 0, 16);
        $output         = openssl_encrypt($text, $encrypt_method, $key, 0, $iv);

        return $output;
    }

    public static function decrypt($secret) {

        if (true !== function_exists('openssl_decrypt')) {
            throw new Stec_Exception('openssl_decrypt not found');
        }

        if (!$secret) {
            return $secret;
        }

        $encrypt_method = "AES-256-CBC";
        $secret_key     = wp_salt('secure_auth');
        $secret_iv      = wp_salt('nonce');
        $key            = hash('sha256', $secret_key);
        $iv             = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt($secret, $encrypt_method, $key, 0, $iv);

        if (false === $output) {
            throw new Stec_Exception('Decryption failed');
        }

        return $output;
    }

    public static function check_migration_required() {

        $requires_migration = false;
        $current_version    = get_option('stec_db_ver', false);

        if (version_compare($current_version, '2.6.1', '<')) {
            $has_meta_organizers = get_posts(array(
                'posts_per_page'   => 1,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'organizers',
                'suppress_filters' => true
            ));

            $has_meta_locations = get_posts(array(
                'posts_per_page'   => 1,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'location',
                'suppress_filters' => true
            ));

            if (count($has_meta_locations) > 0 || count($has_meta_organizers) > 0) {
                $requires_migration = true;
            } else {
                update_option('stec_db_ver', '2.6.1', false);
            }
        }

        return $requires_migration;
    }

    public static function get_default_locale() {

        if (!self::$default_locale) {

            self::$default_locale = get_locale();

            if (defined('ICL_LANGUAGE_CODE')) {
                global $sitepress;
                if ($sitepress instanceof \SitePress) {
                    self::$default_locale = $sitepress->get_default_language();
                }
            }
        }

        return self::$default_locale;
    }

    public static function get_locale() {
        if (defined('ICL_LANGUAGE_CODE')) {
            $lang = ICL_LANGUAGE_CODE;
        } else {
            $lang = get_locale();
        }

        return $lang;
    }

    public static function get_stec_page($name, $lang = null) {

        $pages = get_option('stec-pages', array());

        if (!$lang) {
            $lang = self::get_locale();
        }

        switch (strtolower($name)) {
            case 'organizer' :

                if (isset($pages[$lang]) && isset($pages[$lang][$name])) {
                    $id = $pages[$lang][$name];
                    return filter_var($id, FILTER_VALIDATE_INT);
                }
        }

        return false;
    }

    public static function set_stec_page($name, $id, $lang = null) {

        $pages = get_option('stec-pages', array());

        if (!$lang) {
            $lang = self::get_locale();
        }

        switch (strtolower($name)) {
            case 'organizer' :

                if (!isset($pages[$lang])) {
                    $pages[$lang] = array();
                }

                if (!isset($pages[$lang][$name])) {
                    $pages[$lang][$name] = '';
                }

                $pages[$lang][$name] = (int) $id;

                return update_option('stec-pages', $pages, false);
        }

        return false;
    }

    public static function delete_stec_page($name, $lang = null) {

        $pages = get_option('stec-pages', array());

        if (!$lang) {
            $lang = self::get_locale();
        }

        switch (strtolower($name)) {
            case 'organizer' :

                if (isset($pages[$lang]) && isset($pages[$lang][$name])) {
                    unset($pages[$lang][$name]);

                    return update_option('stec-pages', $pages, false);
                }
        }

        return false;
    }

    public static function esc($data, $context) {

        switch ($context) {

            case 'json':
            case 'raw' :
                return apply_filters('stec_esc_raw_before_return', $data);

            case 'href' :
                $allowed = array('#', 'javascript:void(0);');

                if (in_array($data, $allowed)) {
                    return $data;
                }

                return esc_url($data);

            case 'noscript':
                $data = preg_replace('/<\/?script(.*?)>/i', '', $data);
                return apply_filters('stec_esc_noscript_before_return', $data);
        }

        return esc_html($data);
    }

    public static function get_qrcode($text) {
        $qr_directory   = 'stec_qrcode';
        $upload_dir_nfo = wp_get_upload_dir();
        $uploads_dir    = trailingslashit($upload_dir_nfo['basedir']) . $qr_directory;
        $filename       = md5($text) . '.png';

        if (file_exists($uploads_dir . '/' . $filename)) {
            return $upload_dir_nfo['baseurl'] . '/stec_qrcode/' . $filename;
        }

        wp_mkdir_p($uploads_dir);

        if (!file_exists($uploads_dir)) {
            return false;
        }

        $stec = Stachethemes_Event_Calendar::get_instance();
        require_once($stec->get_path('ASSETS') . 'php/libs/phpqrcode/qrlib.php');

        try {
            \QRcode::png($text, $uploads_dir . '/' . $filename);
        } catch (\Exception $ex) {
            Admin_Helper::debug_log('Admin_Helper::get_qrcode: ' . $ex->getMessage());
            return false;
        }

        if (file_exists($uploads_dir . '/' . $filename)) {
            return $upload_dir_nfo['baseurl'] . '/stec_qrcode/' . $filename;
        }

        return false;
    }

    public static function is_placeholder() {

        if (self::is_divi_builder() || self::is_elementor()) {
            return true;
        }

        return false;
    }

    public static function is_elementor() {
        if (class_exists('\Elementor\Plugin')) {

            if (\Elementor\Plugin::$instance->editor->is_edit_mode() || \Elementor\Plugin::$instance->preview->is_preview_mode()) {
                return true;
            }
        }

        return false;
    }

    public static function is_divi_builder() {

        if (function_exists('et_core_is_fb_enabled') && function_exists('is_et_pb_preview')) {
            if (true === et_core_is_fb_enabled() || true === is_et_pb_preview()) {
                return true;
            }
        }

        return false;
    }

}
