<?php

namespace Stachethemes\Stec;

class Admin_Html {

    public static function html_color($name, $value, $default = '', $desc = '') {

        $val = $value ? $value : $default;

        printf("<input class='stachethemes-admin-colorpicker' name='%s' value='%s' autocomplete='off' title='%s' />", esc_attr($name), esc_attr($val), esc_attr($desc));
    }

    public static function html_info($text) {
        if ($text === '') {
            return;
        }
        echo "<p class='stachethemes-admin-info'>" . Admin_Helper::esc($text, 'noscript') . "</p>";
    }

    public static function html_desc($text) {
        if ($text === '') {
            return;
        }
        echo "<p class='desc'>" . esc_html($text) . "</p>";
    }

    public static function html_input($name, $value, $default = "", $placeholder = "", $required = false, $type = "text", $custom = '') {

        $val = $value ? $value : $default;
        $req = (bool) $required ? 'required="required"' : '';

        printf("<input class='stachethemes-admin-input' type='%s' name='%s' placeholder='%s' value='%s' %s %s />",
                esc_attr($type), esc_attr($name), esc_attr($placeholder), esc_attr($val), $req, esc_attr($custom)
        );
    }

    public static function html_checkbox($name, $checked, $default = '', $label = false, $required = false, $custom = '') {

        $id = 'id-' . uniqid();

        $checked = (bool) $checked ? 'checked' : '';
        $req     = (bool) $required ? 'required="required"' : '';

        printf("<input id='%s' class='stachethemes-admin-checkbox' type='checkbox' name='%s' %s %s %s />",
                esc_attr($id), esc_attr($name), $checked, $req, esc_attr($custom));

        if ($label) {
            printf("<label for='%s' class='stachethemes-admin-checkbox-label'>%s</label>", esc_attr($id), esc_html($label));
        }
    }

    public static function html_radio($name, $checked, $default = '', $label = false, $required = false) {

        $id = 'id-' . uniqid();

        $checked = (bool) $checked ? 'checked' : '';
        $req     = (bool) $required ? 'required="required"' : '';

        printf("<input id='%s' class='stachethemes-admin-checkbox' type='radio' name='%s' %s %s />",
                esc_attr($id), esc_attr($name), $checked, $req
        );

        if ($label) {
            printf("<label for='%s' class='stachethemes-admin-checkbox-label'>%s</label>", esc_attr($id), esc_html($label));
        }
    }

    public static function html_date($name, $value, $default = "", $placeholder = "", $required = false, $type = "text") {

        $val = $value ? $value : $default;
        $req = (bool) $required ? 'required="required"' : '';

        printf("<input class='stachethemes-admin-input input-date' type='%s' name='%s' placeholder='%s' value='%s' %s />",
                esc_attr($type), esc_attr($name), $placeholder, esc_attr($val), $req
        );
    }

    public static function html_hidden($name, $value) {
        printf("<input type='hidden' name='%s' value='%s' />", esc_attr($name), esc_attr($value));
    }

    public static function html_textarea($name, $value, $default = "", $placeholder = "", $required = false, $custom = '') {

        $val = $value ? $value : $default;
        $req = (bool) $required ? 'required="required"' : '';

        printf("<textarea class='stachethemes-admin-textarea' name='%s' placeholder='%s' %s %s>%s</textarea>",
                esc_attr($name), esc_attr($placeholder), $req, esc_attr($custom), esc_textarea($val)
        );
    }

    /**
     * @todo Graphic implementation
     */
    public static function html_icon($name, $list, $default = "", $required = false) {
        return self::html_select($name, $list, $default, $required);
    }

    public static function html_select($name, $options, $default = "", $required = false, $multiple = '', $custom = '') {

        if ($multiple) {
            $name     .= '[]';
            $multiple = 'multiple';
        }

        $html = array();

        $req    = (bool) $required ? 'required="required"' : '';
        $html[] = sprintf("<select class='stachethemes-admin-select' name='%s' %s autocomplete='off' %s %s>",
                esc_attr($name), $req, esc_attr($multiple), esc_attr($custom)
        );

        foreach ($options as $val => $name) :

            if (is_array($default)) {
                $sel = in_array($val, $default) ? 'selected="selected"' : "";
            } else {
                $sel = (string) $default === (string) $val ? 'selected="selected"' : "";
            }
            $html[] = sprintf("<option value='%s' %s>%s</option>", esc_attr($val), $sel, esc_html($name));
        endforeach;

        $html[] = "</select>";

        echo Admin_Helper::esc(implode('', $html), 'noscript');
    }

    public static function html_button($text, $href = false, $noclear = false, $customclass = "", $custom = '') {

        $noclr = (bool) $noclear === true ? 'stachethemes-admin-button-no-clear' : '';

        if ($href) {

            printf("<a class='stachethemes-admin-button-a %s %s' %s href='%s' >%s</a>",
                    $noclr, esc_attr($customclass), esc_attr($custom), Admin_Helper::esc($href, 'href'), esc_html($text));
        } else {
            printf("<button class='stachethemes-admin-button %s %s' %s>%s</button>",
                    $noclr,
                    esc_attr($customclass),
                    esc_attr($custom),
                    esc_html($text));
        }
    }

    public static function html_form_start($action = "/", $method = "POST", $formdata = false) {

        $enctype = (bool) $formdata ? 'enctype="multipart/form-data"' : '';

        if (strtolower($method) === 'get') {
            $action = wp_nonce_url($action);
        }

        printf("<form class='stachethemes-admin-form' action='%s' method='%s' %s>", esc_url($action), esc_attr($method), $enctype);

        if ($method === 'POST') {
            wp_nonce_field($action, 'stec_admin_form_nonce');
        }
    }

    public static function html_form_end() {

        echo "</form>";
    }

    public static function html_add_image($name, $default = false, $title = "Add Image", $required = false, $single = false, $size = 'medium') {

        $req = (bool) $required ? 'required="required"' : '';

        printf("<ul class='stachethemes-admin-add-image-list' data-single='%s' data-name='%s' %s>", (bool) $single, esc_attr($name), $req);

        if ($default) :

            if (!is_array($default)) {
                $default = explode(',', $default);
            }

            foreach ($default as $image_id) :
                $image_source = wp_get_attachment_image_src($image_id, $size);
                ?>
                <li class="stachethemes-admin-image-container"><i class="fa fa-times"></i>
                    <img src="<?php echo esc_url($image_source[0]); ?>" data-id="<?php echo (int) $image_id; ?>"><input type="hidden" value="<?php echo (int) $image_id; ?>" name="<?php esc_attr_e($name); ?>">
                </li>
                <?php
            endforeach;

        endif;

        echo "</ul>";

        printf("<button data-single='%s' data-name='%s' class='stachethemes-admin-add-image'>%s</button>", (bool) $single, esc_attr($name), esc_html($title));
    }

    public static function setting_to_html($setting) {

        $setting = (object) $setting;

        switch ($setting->type) :

            case 'checkbox' :

                self::html_checkbox($setting->name, $setting->value, $setting->default, $setting->title, $setting->req);

                break;

            case 'input' :

                self::html_info($setting->title);
                self::html_input($setting->name, $setting->value, $setting->default, "", $setting->req);

                break;

            case 'textarea' :

                self::html_info($setting->title);
                self::html_textarea($setting->name, $setting->value, $setting->default, $placeholder = "", $setting->req);

                break;

            case 'select' :

                if (isset($setting->select_call_func) && $setting->select_call_func) {
                    $func = explode('::', $setting->select_call_func);
                    if (method_exists($func[0], $func[1])) {
                        $setting->select = call_user_func($setting->select_call_func, $setting->select_call_param);
                    } else {
                        $setting->select = array();
                    }
                }


                self::html_info($setting->title);
                self::html_select($setting->name, $setting->select, $setting->value, $setting->req, isset($setting->multiple) ? 'multiple' : '');

                break;

            case 'color' :

                self::html_info($setting->title);
                self::html_color($setting->name, $setting->value, $setting->default, $setting->desc);

                break;

            case 'font' :

                $fontsizes = array();
                for ($i = 9; $i <= 100; $i++) {
                    $fontsizes["{$i}px"] = "{$i}px";
                }

                $fontweights = array();
                for ($i = 1; $i <= 9; $i++) {
                    $fontweights[$i * 100] = $i * 100;
                }

                $fontlineheights = array();
                $lh              = 0.9;
                while ($lh < 3) {
                    $lh                     = $lh + 0.1;
                    $fontlineheights["$lh"] = $lh;
                }

                self::html_info($setting->title);
                echo '<div class="stachethemes-admin-section-flex">';
                self::html_input($setting->name . '_font_family', $setting->value[0], $setting->default[0], "Font-Family", false);
                self::html_select($setting->name . '_font_weight', $fontweights, $setting->value[1], $setting->req);
                self::html_select($setting->name . '_font_size', $fontsizes, $setting->value[2], $setting->req);
                if (isset($setting->value[3]) && $setting->value[3]) {
                    self::html_select($setting->name . '_font_lineheight', $fontlineheights, $setting->value[3], $setting->req);
                }
                echo '</div>';
                break;

        endswitch;
    }

    /**
     * Build html settings from given $page and $name settings
     */
    public static function build_settings_html($page, $name = false) {

        $setting = Settings::get_admin_setting($page, $name);

        if ($setting === null || empty($setting)) {
            return;
        }

        if ($name === false) {
            foreach ($setting as $single) {
                self::setting_to_html($single);
            }
        } else {
            self::setting_to_html($setting);
        }
    }

}
