<?php

namespace Stachethemes\Stec;

class Ical_Import extends \ICal\ICal {

    protected $_recurrence = false;
    private $tz            = 'UTC';

    public function __construct($filename = false, $recurrence = false) {

        $this->_recurrence = $recurrence;

        parent::__construct($filename);
    }

    /**
     * Now checks if $recurrence is true/false
     * This creates duplicate for every recurrence
     */
    public function processRecurrences() {

        if ($this->_recurrence === true) {
            parent::processRecurrences();
        }
    }

    public function setTheDateTz($tz = 'UTC') {
        $this->tz = $tz;
    }

    /**
     * Reads an entire file or URL into an array
     * Fixes Facebook problem with blocked content
     *
     * @param  string $filename
     * @return array
     * @throws Exception
     */
    protected function fileOrUrl($filename) {

        // Use the default method for files
        if (file_exists($filename)) {
            return parent::fileOrUrl($filename);
        }

        $user = '';
        $pass = '';

        $result = wp_remote_get($filename, array(
            'headers' => array(
                'Authorization' => 'Basic ' . base64_encode($user . ':' . $pass)
            )
        ));

        if ($result['response']['code'] != 200) {
            throw new \Exception('Error code: ' . $result['response']['code']);
        }

        $body  = explode("\n", $result['body']);
        $lines = array();

        foreach ($body as $line) {
            if (!trim($line)) {
                continue;
            }

            $lines[] = $line;
        }

        return $lines;
    }

    public function iCalDateWithTimeZone(array $event, $key, $format = 'U') {

        return parent::iCalDateWithTimeZone($event, $key, $format);
    }
    
    /**
     * Retrieves dtstart / dtend with correct timezone
     * @param type $datetime YYYYMMDDTHis(Z)
     * @return \DateTime
     */
    public function getDT($datetime) {
        $dt = new \DateTime();

        if (strpos($datetime, 'Z') === false) {
            $dt->setTimezone(new \DateTimeZone($this->tz));
        } else {
            $dt->setTimezone(new \DateTimeZone('UTC'));
        }

        $datetime_arr = explode('T', $datetime);

        $y = substr($datetime_arr[0], 0, 4);
        $m = substr($datetime_arr[0], 4, 2);
        $d = substr($datetime_arr[0], 6, 2);
        
        $dt->setDate($y, $m, $d);

        $h = 0;
        $i = 0;
        $s = 0;

        if (isset($datetime_arr[1])) {
            $h = substr($datetime_arr[1], 0, 2);
            $i = substr($datetime_arr[1], 2, 2);
            $s = substr($datetime_arr[1], 4, 2);
        }

        $dt->setTime($h, $i, $s);
        
        if (strpos($datetime, 'Z') !== false) {
            $dt->setTimezone(new \DateTimeZone($this->tz));
        }

        return $dt;
    }

}
