<?php

namespace Stachethemes\Stec;

use DateTime;
use DateTimeZone;

class Import {

    private $csv_url           = false;
    private $ics_url           = false;
    private $calendar_id       = false;
    private $recurrence        = false;
    private $icon              = '';
    private $ignore_expired    = false;
    private $overwrite_events  = false;
    private $delete_removed    = false;
    private $download_images   = false;
    private $create_categories = false;
    private $create_locations  = false;

    public function get_ics_url() {
        return $this->ics_url;
    }

    public function get_calendar_id() {
        return $this->calendar_id;
    }

    public function get_recurrence() {
        return $this->recurrence;
    }

    public function get_icon() {
        return $this->icon;
    }

    public function get_ignore_expired() {
        return $this->ignore_expired;
    }

    public function get_overwrite_events() {
        return $this->overwrite_events;
    }

    public function get_delete_removed() {
        return $this->delete_removed;
    }

    public function get_create_locations() {
        return $this->create_locations;
    }

    public function get_download_images() {
        return $this->download_images;
    }

    public function get_create_categories() {
        return $this->create_categories;
    }

    public function set_ics_url($ics_url) {
        $this->ics_url = $ics_url;
    }

    public function set_csv_url($csv_url) {
        $this->csv_url = $csv_url;
    }

    public function set_calendar_id($calendar_id) {
        $this->calendar_id = $calendar_id;
    }

    public function set_recurrence($recurrence) {
        $this->recurrence = $recurrence;
    }

    public function set_icon($icon) {
        $this->icon = $icon;
    }

    public function set_ignore_expired($ignore_expired) {
        $this->ignore_expired = $ignore_expired;
    }

    public function set_overwrite_events($overwrite_events) {
        $this->overwrite_events = $overwrite_events;
    }

    public function set_download_images($truefalse) {
        $this->download_images = $truefalse;
    }

    public function set_create_locations($truefalse) {
        $this->create_locations = $truefalse;
    }

    public function set_create_categories($truefalse) {
        $this->create_categories = $truefalse;
    }

    public function set_delete_removed($delete_removed) {
        $this->delete_removed = $delete_removed;
    }

    public function import_ics() {

        if ($this->ics_url === false || is_nan($this->calendar_id)) {
            Admin_Helper::debug_log('Import job no valid source or calendar destination');
            return false;
        }

        Admin_Helper::debug_log('Import job importing ' . $this->ics_url . ' to calendar ' . $this->calendar_id);

        set_time_limit(STEC_EXEC_TIME_LIMIT);

        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            Admin_Helper::debug_log("Calendar does not exists");
            return false;
        }

        if (false === Calendars::user_can_edit_calendar($calendar)) {
            Admin_Helper::debug_log(sprintf(esc_html__("User can't edit calendar %d", 'stec'), $calendar->get_id()));
            return false;
        }

        $ical_import       = new Ical_Import($this->ics_url, $this->recurrence);
        $events            = $ical_import->events();
        $imported_count    = 0;
        $overwritten_count = 0;
        $ics_uids          = array();

        $ical_import->setTheDateTz($calendar->get_timezone());

        foreach ($events as $event) {

            $ics_uids[] = $event->uid;

            if (!isset($event->summary) || $event->summary === '') {
                /**
                 * Don't import unnamed events
                 */
                continue;
            }

            $all_day  = 0;
            $start_dt = $ical_import->getDT($event->dtstart);
            $end_dt   = null;

            if (isset($event->dtend)) {
                $end_dt = $ical_import->getDT($event->dtend);
            } else {
                $end_dt = $start_dt;
                $end_dt->add(new \DateInterval('PT1H'));
            }

            $ical_import->setTheDateTz($calendar->get_timezone());

            if ($end_dt->format('U') - $start_dt->format('U') === 0) {
                $all_day  = 1;
                $start_dt = $end_dt;
            } elseif (($end_dt->format('U') - $start_dt->format('U')) % 86400 === 0) {
                // Assume multiple day all day event
                $all_day = 1;
                if ($end_dt->format('Hi') === '0000') {
                    $end_dt->sub(new \DateInterval('P1D'));
                }
            }

            // check expired filter
            if ($this->ignore_expired === true) {

                if (gmdate('U', time()) > $end_dt->format('U') && (!isset($event->rrule) || $event->rrule === '')) {
                    continue;
                }
            }

            if (!isset($event->recurrence_id)) {
                $event->recurrence_id = '';
            }

            // check if uid exists in THIS ID calendar
            // allow recurrence-id override
            $exists = Events::get_events(array(
                        'calid'      => $this->calendar_id,
                        'meta_query' => array(
                            array(
                                'key'     => 'uid',
                                'value'   => $event->uid,
                                'compare' => '=',
                                'type'    => ''
                            ),
                            array(
                                'key'     => 'recurrence_id',
                                'value'   => $event->recurrence_id,
                                'compare' => '=',
                                'type'    => ''
                            )
                        )
            ));

            if ($exists) {
                if (true === $this->overwrite_events) {
                    foreach ($exists as $exist) {
                        if (false !== $exist->delete_post()) {
                            $overwritten_count++;
                        }
                    }

                    unset($exist);
                } else {
                    // Event uid already exists; Don't add again
                    continue;
                }
            }

            $visibility = 'stec_cal_default';

            if (isset($event->class)) :

                if ($event->class === 'PUBLIC') {
                    $visibility = 'stec_public';
                }

                if ($event->class === 'PRIVATE') {
                    $visibility = 'stec_private';
                }

            endif;

            $event_post = new Event_Post();

            $event_post->set_calid($calendar->get_id());
            $event_post->set_title($event->summary);
            $event_post->set_color($calendar->get_color());
            $event_post->set_icon($this->icon);
            $event_post->set_visibility($visibility);
            $event_post->set_start_date($start_dt->format('Y-m-d H:i:s'));
            $event_post->set_end_date($end_dt->format('Y-m-d H:i:s'));
            $event_post->set_all_day($all_day);
            $event_post->set_rrule(isset($event->rrule) ? $event->rrule : '');
            $event_post->set_exdate(isset($event->exdate) ? $event->exdate : '');
            $event_post->set_is_advanced_rrule(isset($event->is_advanced_rrule) ? $event->is_advanced_rrule : 0);
            $event_post->set_approved(1);

            if (isset($event->location) && $event->location !== '') {
                $found_location = Locations::get_location_by_name($event->location);
                
                if ($found_location) {
                    $event_post->set_location($found_location->get_id());
                } else {
                    if (true === $this->create_locations) {
                        try {
                            $new_location    = new Location();
                            $new_location->set_address($event->location);
                            $new_location_id = $new_location->insert_location();

                            if ($new_location_id) {
                                $event_post->set_location($new_location_id);
                            }
                        } catch (Stec_Exception $ex) {
                            Admin_Helper::debug_log($ex->getMessage());
                        }
                    }
                }
            }

            $description = '';

            if (isset($event->description) && trim($event->description) !== '') {
                $description = html_entity_decode(str_replace("\\n", "\n", $event->description));
            }

            $event_post->set_description($description);
            $event_post->set_uid($event->uid);
            $event_post->set_recurrence_id($event->recurrence_id);

            $filter_data = apply_filters('stec_before_import_event', array(
                'calendar'   => $calendar,
                'event_post' => $event_post,
                'ics_event'  => $event
            ));

            try {
                $filter_data['event_post']->insert_post();
                $imported_count++;
            } catch (Stec_Exception $ex) {
                Admin_Helper::debug_log($ex->getMessage());
            }
        }

        // Remove events with uid NOT in the ics and in this calendar 
        $deleted_count = 0;
        if ($this->delete_removed === true) :

            /**
             * @todo get with meta_query id not in...?
             */
            $calendar_events = Events::get_events(array(
                        'calid' => $this->calendar_id
            ));

            foreach ($calendar_events as $event) {

                if (in_array($event->get_uid(), $ics_uids) !== true) {
                    if (false !== $event->delete_post()) {
                        $deleted_count++;
                    }
                }
            }
        endif;

        Admin_Helper::debug_log('Import job ' . $this->ics_url . ' imported');

        return (object) array(
                    'deleted_count'     => $deleted_count,
                    'overwritten_count' => $overwritten_count,
                    'imported_count'    => $imported_count
        );
    }

    public function import_csv() {

        if ($this->csv_url === false || is_nan($this->calendar_id)) {
            Admin_Helper::debug_log('Import job no valid source or calendar destination');
            return false;
        }

        Admin_Helper::debug_log('Import job importing ' . $this->csv_url . ' to calendar ' . $this->calendar_id);

        set_time_limit(STEC_EXEC_TIME_LIMIT);

        $calendar = new Calendar_Post($this->calendar_id);

        if (!$calendar->get_id()) {
            Admin_Helper::debug_log("Calendar does not exists");
            return false;
        }

        if (false === Calendars::user_can_edit_calendar($calendar)) {
            Admin_Helper::debug_log("You don't have permission to edit this calendar");
            return false;
        }

        $csv_import        = new Csv_Import($this->csv_url);
        $events            = $csv_import->events();
        $imported_count    = 0;
        $overwritten_count = 0;
        $csv_uids          = array();
        $datetime_format   = Csv_Settings::get_datetime_format();

        foreach ($events as $event) {

            if (empty($event)) {
                continue;
            }

            if (isset($event->uid) && $event->uid) {
                $csv_uids[] = $event->uid;
            }

            if (!isset($event->title) || $event->title === '') {
                /**
                 * Don't import unnamed events
                 */
                continue;
            }

            $start_dt = DateTime::createFromFormat($datetime_format, $event->start_date, new \DateTimeZone($calendar->get_timezone()));
            $end_dt   = DateTime::createFromFormat($datetime_format, $event->end_date, new \DateTimeZone($calendar->get_timezone()));

            /**
             * Invalid dt object
             */
            if (false === $start_dt) {
                Admin_Helper::debug_log('CSV Import invalid start date ' . $event->start_date, 'stec');
                continue;
            }

            // check expired filter
            if ($this->ignore_expired === true) {
                if (gmdate('U', time()) > $end_dt->format('U') && (!isset($event->rrule) || $event->rrule === '')) {
                    continue;
                }
            }

            if (!isset($event->recurrence_id)) {
                $event->recurrence_id = '';
            }

            // check if uid exists in THIS ID calendar
            // allow recurrence-id override
            $exists = Events::get_events(array(
                        'calid'      => $this->calendar_id,
                        'meta_query' => array(
                            'key'     => 'uid',
                            'value'   => $event->uid,
                            'compare' => '=',
                            'type'    => ''
                        ),
                        array(
                            'key'     => 'recurrence_id',
                            'value'   => $event->recurrence_id,
                            'compare' => '=',
                            'type'    => ''
                        )
            ));

            if ($exists) {
                if (true === $this->overwrite_events) {
                    foreach ($exists as $exist) {

                        if (false === $exist->delete_post()) {
                            Admin_Helper::debug_log(sprintf(esc_html__('Unable to overwrite event %s. Access denied.', 'stec'), $exist->get_id()));
                            continue;
                        }

                        $overwritten_count++;
                    }

                    unset($exist);
                } else {
                    // Event uid already exists; Don't add again
                    continue;
                }
            }

            $event_post = new Event_Post();
            $event_post->set_calid($calendar->get_id());
            $event_post->set_title($event->title);
            $event_post->set_color(isset($event->color) ? $event->color : $calendar->get_color());
            $event_post->set_icon(isset($event->icon) ? $event->icon : $calendar->get_icon());

            if (isset($event->images) && $event->images) {

                $images = explode(',', $event->images);

                foreach ($images as &$image) {
                    try {
                        switch (is_numeric($images)) {
                            case true:
                                // Convert to integer type
                                $image = (int) $image;
                                if (false === wp_attachment_is_image($image)) {
                                    throw new Stec_Exception(sprintf(esc_html__('Invalid media image: %s', 'stec'), $image));
                                }
                                break;

                            case false:
                                if (false === filter_var($image, FILTER_VALIDATE_URL)) {
                                    throw new Stec_Exception(sprintf(esc_html__('Invalid CSV image: %s', 'stec'), $image));
                                }

                                if (true === $this->download_images) {
                                    $image = Admin_Helper::url_image_to_media_upload($image);
                                }
                                break;
                        }
                    } catch (Stec_Exception $ex) {
                        Admin_Helper::debug_log($image);
                        Admin_Helper::debug_log($ex->getMessage());
                        $image = null;
                        continue;
                    }
                }
                unset($image);

                $event_post->set_images($images);
            }
            
            $event_post->set_visibility('stec_cal_default');
            $event_post->set_start_date($start_dt->format('Y-m-d H:i:s'));
            $event_post->set_end_date($end_dt->format('Y-m-d H:i:s'));
            $event_post->set_all_day(isset($event->all_day) ? $event->all_day : 0);
            $event_post->set_rrule(isset($event->rrule) ? $event->rrule : '');
            $event_post->set_exdate(isset($event->exdate) ? $event->exdate : '');
            $event_post->set_is_advanced_rrule(isset($event->is_advanced_rrule) ? $event->is_advanced_rrule : 0);
            $event_post->set_rsvp(isset($event->rsvp) ? $event->rsvp : 0);
            $event_post->set_rsvp_limit(isset($event->rsvp_limit) ? (int) $event->rsvp_limit : 0);
            $event_post->set_attendance_visibility(isset($event->attendance_visibility) ? $event->attendance_visibility : 'public');
            $event_post->set_approved(1);

            $location_full_address = array();

            if (isset($event->location_address) && trim($event->location_address !== '')) {
                $location_full_address[] = trim($event->location_address);
            }

            if (isset($event->location_city) && trim($event->location_city) !== '') {
                $location_full_address[] = trim($event->location_city);
            }

            if (isset($event->location_country) && trim($event->location_country !== '')) {
                $location_full_address[] = trim($event->location_country);
            }

            $location_full_address = implode(', ', $location_full_address);

            if ($location_full_address) {
                $found_location = Locations::get_location_by_name($location_full_address);

                if ($found_location) {
                    $event_post->set_location($found_location->get_id());
                } else {
                    if (true === $this->create_locations) {
                        try {
                            $new_location = new Location();

                            if (isset($event->location_address) && $event->location_address) {
                                $new_location->set_address($event->location_address);
                            }

                            if (isset($event->location_city) && $event->location_city) {
                                $new_location->set_city($event->location_city);
                            }

                            if (isset($event->location_country) && $event->location_country) {
                                $new_location->set_country($event->location_country);
                            }

                            if (isset($event->location_details) && $event->location_details) {
                                $new_location->set_details($event->location_details);
                            }

                            $new_location_id = $new_location->insert_location();

                            if ($new_location_id) {
                                $event_post->set_location($new_location_id);
                            }
                        } catch (Stec_Exception $ex) {
                            Admin_Helper::debug_log($ex->getMessage());
                        }
                    }
                }
            }

            $event_post->set_description(isset($event->description) ? $event->description : '');
            $event_post->set_description_short(isset($event->description_short) ? $event->description_short : '');
            $event_post->set_uid(isset($event->uid) ? $event->uid : null);
            $event_post->set_recurrence_id($event->recurrence_id);
            $event_post->set_featured(isset($event->featured) ? $event->featured : 0);
            $event_post->set_author(isset($event->author) ? $event->author : get_current_user_id());

            if (isset($event->categories_id) && $event->categories_id) {
                $categories_array = explode(',', $event->categories_id);

                foreach ($categories_array as &$category_id) {

                    $category_id = trim($category_id);

                    try {
                        if ($category_id === '') {
                            throw new Stec_Exception(esc_html__('Empty category_id', 'stec'));
                        }

                        switch (is_numeric($category_id)) :

                            case true:

                                // Convert to integer type
                                $category_id = (int) $category_id;

                                if (false === (bool) term_exists($category_id, 'stec_event_category')) {
                                    throw new Stec_Exception(esc_html__('Category id does not exists', 'stec'));
                                }
                                break;

                            case false:
                                if (true === $this->create_categories) {
                                    $category = Categories::get_category_by_name($category_id);

                                    if ($category) {
                                        $category_id = $category->get_id();
                                    } else {
                                        $new_category = new Category();
                                        $new_category->set_color($calendar->get_color());
                                        $new_category->set_title($category_id);
                                        $new_category->insert_category();
                                        $category_id  = $new_category->get_id();
                                    }
                                }
                                break;

                        endswitch;
                    } catch (Stec_Exception $ex) {
                        Admin_Helper::debug_log($category_id);
                        Admin_Helper::debug_log($ex->getMessage());
                        $category_id = null;
                        continue;
                    }
                }

                unset($category_id);

                if ($categories_array) {
                    $event_post->set_categories(array_map('intval', $categories_array));
                }
            }

            $event_post->set_link(isset($event->link) ? $event->link : '');
            $event_post->set_keywords(isset($event->keywords) ? $event->keywords : '');

            $filter_data = apply_filters('stec_before_import_event', array(
                'calendar'   => $calendar,
                'event_post' => $event_post,
                'csv_event'  => $event
            ));

            try {
                $filter_data['event_post']->insert_post();
                $imported_count++;
            } catch (Stec_Exception $ex) {
                Admin_Helper::debug_log($ex->getMessage());
            }
        }

        // Remove events with uid NOT in the ics and in this calendar 
        $deleted_count = 0;
        if ($this->delete_removed === true) :

            /**
             * @todo get with meta_query id not in...?
             */
            $calendar_events = Events::get_events(array(
                        'calid' => $this->calendar_id
            ));

            foreach ($calendar_events as $event) {

                if (in_array($event->get_uid(), $csv_uids) !== true) {
                    if (false !== $event->delete_post()) {
                        $deleted_count++;
                    }
                }
            }
        endif;

        Admin_Helper::debug_log('Import job ' . $this->ics_url . ' imported');

        return (object) array(
                    'deleted_count'     => $deleted_count,
                    'overwritten_count' => $overwritten_count,
                    'imported_count'    => $imported_count
        );
    }

}
