<?php

namespace Stachethemes\Stec;

class Locations {

    public static function get_locations() {

        $locations = array();

        $terms = get_terms('stec_event_location', array(
            'hide_empty' => false,
            'fields'     => 'ids'
        ));

        foreach ($terms as $id) {
            $location = new Location($id);

            if (false === is_super_admin()) {
                if ('private' === $location->get_visibility()) {
                    if ($location->get_author() !== get_current_user_id()) {
                        continue;
                    }
                }
            }

            $locations[] = $location;
        }

        return $locations;
    }

    public static function get_locations_list() {
        $locations_list = array();

        $locations = self::get_locations();

        foreach ($locations as $location) {
            $location_name = array($location->get_full_address());
            if ($location->get_title()) {
                array_unshift($location_name, $location->get_title());
            }
            $locations_list[$location->get_id()] = implode(', ', $location_name);
        }

        return $locations_list;
    }

    public static function bulk_delete($ids = array()) {
        $filter_ids = filter_var_array($ids, FILTER_VALIDATE_INT);
        $unique_ids = array_filter($filter_ids);

        foreach ($unique_ids as $id) {
            $location = new Location($id);
            if (!$location->get_id()) {
                continue;
            }

            $location->delete();
        }
        unset($id);

        return true;
    }

    public static function get_location_by_id($id) {
        $cats = self::get_locations();

        foreach ($cats as $cat) {
            if ($cat->get_id() === $id) {
                return $cat;
            }
        }

        return null;
    }

    public static function get_location_by_name($name) {
        $locations = self::get_locations();

        foreach ($locations as $location) {
            if ($location->get_full_address() === $name) {
                return $location;
            }
        }

        return null;
    }

    public static function get_location_country_city() {

        $array     = array();
        $locations = self::get_locations();

        foreach ($locations as $location) {
            $country = $location->get_country();
            $city    = $location->get_city();

            if (!$country || !$city) {
                continue;
            }

            if (!isset($array[$country])) {
                $array[$country] = array();
            }

            if (!isset($array[$country][$city])) {
                $array[$country][$city] = $city;
            }
        }

        return $array;
    }

    public static function has_country($country) {
        $locations = self::get_location_country_city();

        return isset($locations[$country]) ? true : false;
    }

    public static function has_city($country, $city) {
        $locations = self::get_location_country_city();

        return isset($locations[$country]) && isset($locations[$country][$city]) ? true : false;
    }

}
