<?php

namespace Stachethemes\Stec;

class Organizers {

    /**
     * Get array list of organizers
     * @param int $userid user id
     * @param string $taxomony the taxonomy name
     * @return array
     */
    public static function get_organizers() {

        $organizers = array();

        $terms = get_terms('stec_event_organizer', array(
            'hide_empty' => false,
            'fields'     => 'ids'
        ));

        foreach ($terms as $id) {
            $organizer = new Organizer($id);

            if (false === is_super_admin()) {
                if ('private' === $organizer->get_visibility()) {
                    if ($organizer->get_author() !== get_current_user_id()) {
                        continue;
                    }
                }
            }

            $organizers[] = $organizer;
        }
        
        /**
         * Note the organizers are not sorted during get_terms method because the data in mysql is encrypted
         */
        usort($organizers, function($a, $b) {
            return strcasecmp($a->get_name(), $b->get_name());
        });

        return $organizers;
    }

    public static function get_organizers_list() {
        $organizers_list = array();

        $organizers = self::get_organizers();

        foreach ($organizers as $organizer) {
            $organizers_list[$organizer->get_id()] = $organizer->get_name();
        }

        return $organizers_list;
    }

    public static function bulk_delete($ids = array()) {
        $filter_ids = filter_var_array($ids, FILTER_VALIDATE_INT);
        $unique_ids = array_filter($filter_ids);

        foreach ($unique_ids as $id) {
            $organizer = new Organizer($id);
            if (!$organizer->get_id()) {
                continue;
            }

            $organizer->delete();
        }
        unset($id);

        return true;
    }

    public static function get_organizer_by_id($id) {
        $organizers = self::get_organizers();

        foreach ($organizers as $organizer) {
            if ($organizer->get_id() === $id) {
                return $organizer;
            }
        }

        return null;
    }

    public static function get_organizer_by_name($name) {
        $organizers = self::get_organizers();

        foreach ($organizers as $organizer) {
            if ($organizer->get_name() === $name) {
                return $organizer;
            }
        }

        return null;
    }

    public static function get_organizer_by_email($email) {
        $organizers = self::get_organizers();

        foreach ($organizers as $organizer) {
            if ($organizer->get_email() === $email) {
                return $organizer;
            }
        }

        return null;
    }

}
