<?php

namespace Stachethemes\Stec;

class Reminder {

    public static function get_reminders() {
        $reminders = get_option('stec_reminder', array());

        if (is_string($reminders)) {
            try {
                $reminders = Admin_Helper::decrypt($reminders);
                $reminders = maybe_unserialize($reminders);
            } catch (Stec_Exception $ex) {
                Admin_Helper::debug_log('Reminder get_reminders:: ' . $ex->getMessage());
            }
        }

        if (is_array($reminders)) {
            return $reminders;
        }

        return array();
    }

    public static function set_reminders(Array $reminders) {
        return update_option('stec_reminder', Admin_Helper::encrypt($reminders), false);
    }

    public static function set($event_id, $repeat_offset, $email, $remind_date) {

        if (is_nan($event_id)) {
            return array(
                'msg'   => esc_html__('Invalid event id', 'stec'),
                'error' => 1
            );
        }

        $event = new Event_Post($event_id);
        if (!$event->get_id()) {
            return array(
                'msg'   => esc_html__('Event does not exist', 'stec'),
                'error' => 1
            );
        }

        if (is_nan($repeat_offset)) {
            return array(
                'msg'   => esc_html__('Invalid event repeat offset', 'stec'),
                'error' => 1
            );
        }

        if (false === $event->is_valid_offset($repeat_offset)) {
            return array(
                'msg'   => esc_html__('Invalid event date', 'stec'),
                'error' => 1
            );
        }

        if (!$email || filter_var($email, FILTER_VALIDATE_EMAIL) === false) {
            return array(
                'msg'   => esc_html__('Invalid email address', 'stec'),
                'error' => 1
            );
        }

        if (!$remind_date) {
            return array(
                'msg'   => esc_html__('Invalid remind date', 'stec'),
                'error' => 1
            );
        }

        // Check if confirmation is required before processing
        if (false === GDPR::is_confirmed($email)) {

            $data_query = http_build_query(array(
                'task'          => 'gdpr_set_reminder',
                'event_id'      => $event_id,
                'repeat_offset' => $repeat_offset,
                'date'          => $remind_date,
                'email'         => $email
            ));


            GDPR::request_confirmation($email, $data_query);

            return array(
                'error'       => 0,
                'msg'         => esc_html__('Check your email to confirm identity', 'stec'),
                'remind_date' => $remind_date
            );
        }

        $new_remind = new Remind_Object($event_id, $repeat_offset, $email, $remind_date);
        $reminders  = self::get_reminders();

        $exists = false;
        foreach ($reminders as $reminder) {

            if (!$reminder instanceof Remind_Object) {
                continue;
            }

            if ($reminder->get_uid() === $new_remind->get_uid()) {
                $exists = true;
                break;
            }
        }

        if (false === $exists) {
            array_push($reminders, $new_remind);
            self::set_reminders($reminders);
        }

        return array(
            'error'       => 0,
            'msg'         => esc_html__('Reminder has been set', 'stec'),
            'remind_date' => $remind_date
        );
    }

    public static function delete_reminders() {
        delete_option('stec_reminder');
    }

}
