<?php

namespace Stachethemes\Stec;

class Rsvp {
    
    /**
     * Check if there are free attendee slots
     * @param \Stachethemes\Stec\Event_Post $event
     * @param string $date ex: 2019-09-20
     * @return mixed | boolean
     */
    public static function has_free_slots(Event_Post $event, $date, $people = 1) {
        
        $limit = $event->get_rsvp_limit();
        
        if (0 === $limit) {
            return 9999;
        }
        
        $attendees = $event->get_attendance(true);
        
        foreach($attendees as $attendee) {
            if (false === $attendee instanceof Event_Meta_Attendee) {
                continue;
            }
            
            if (1 === $attendee->get_status($date)) {
                $limit = $limit - $attendee->get_people();
            }
        }
        
        return $limit >= $people ? $limit : false;
    }

    private static function validate(Event_Post $event, Event_Meta_Attendee $attendee) {

        if (!$event->get_id()) {
            throw new Stec_Exception(esc_html__('Event no longer exists', 'stec'));
        }

        if (1 !== (int) $event->get_rsvp()) {
            throw new Stec_Exception(esc_html__('RSVP is not enabled for this event', 'stec'));
        }

        if (false === filter_var($attendee->get_email(), FILTER_VALIDATE_EMAIL)) {
            throw new Stec_Exception(esc_html__('Invalid e-mail', 'stec'));
        }

        if (true === $event->attendee_exists($attendee)) {
            throw new Stec_Exception(esc_html__('You are already attending this event', 'stec'));
        }
    }

    public static function add(Event_Post $event, $date, Event_Meta_Attendee $attendee) {

        self::validate($event, $attendee);

        $attendee_email = $attendee->get_email();

        // Check if confirmation is required before processing
        if (false === GDPR::is_confirmed($attendee_email)) {

            $data_query = http_build_query(array(
                'task'     => 'gdpr_rsvp_to_event',
                'email'    => $attendee_email,
                'event_id' => $event->get_id(),
                'date'     => $date,
                'attendee' => serialize($attendee)
            ));

            GDPR::request_confirmation($attendee_email, $data_query);

            throw new Stec_Exception(esc_html__('Check your email to confirm identity', 'stec'));
        }

        Mail::send_event_rsvp_accepted($attendee_email, $event);
        Mail::send_event_author_new_attendee($attendee, $event, $date);

        $attendee->set_mail_sent(1);
        $event->set_attendee($attendee);
        $attendees = $event->get_attendance(true);
        update_post_meta($event->get_id(), 'attendance', Admin_Helper::encrypt($attendees));

        return true;
    }

}
