<?php

namespace Stachethemes\Stec;

class Updater {

    /**
     * The plugin current version
     * @var string
     */
    public $current_version;

    /**
     * The plugin remote update path
     * @var string
     */
    public $update_path;

    /**
     * Plugin Slug (plugin_directory/plugin_file.php)
     * @var string
     */
    public $plugin_slug;

    /**
     * Plugin name (plugin_file)
     * @var string
     */
    public $slug;

    /**
     * Initialize a new instance of the WordPress Auto-Update class
     */
    function __construct() {
        // Set the class public variables

        $this->current_version = STEC_PLUGIN_VERSION;
        $this->update_path     = 'https://api.stachethemes.com/stec/updater/v2/';
        $this->plugin_slug     = plugin_basename(STACHETHEMES_EC_FILE__);
        list ($t1, $t2) = explode('/', $this->plugin_slug);
        $this->slug            = str_replace('.php', '', $t2);

        // define the alternative API for updating checking
        add_filter('pre_set_site_transient_update_plugins', array(&$this, 'check_update'));

        // Define the alternative response for information checking
        add_filter('plugins_api', array(&$this, 'check_info'), 10, 3);
    }

    private function get_lcns() {
        $nfo  = get_option('stec_activated', false);
        $lcns = '';
        if (isset($nfo['purchase_code'])) {
            $lcns = $nfo['purchase_code'];
        }
        return $lcns;
    }

    /**
     * Add our self-hosted autoupdate plugin to the filter transient
     *
     * @param $transient
     * @return object $ transient
     */
    public function check_update($transient) {

        if (empty($transient->checked)) {
            return $transient;
        }

        // Get the remote version
        $remote_version = $this->get_remote_version();

        $nfo  = get_option('stec_activated', false);
        $lcns = '';
        if (isset($nfo['purchase_code'])) {
            $lcns = $nfo['purchase_code'];
        }

        // If a newer version is available, add the update
        if (version_compare($this->current_version, $remote_version, '<')) {
            $obj                                     = new \stdClass();
            $obj->slug                               = $this->slug;
            $obj->new_version                        = $remote_version;
            $obj->url                                = $this->update_path;
            $obj->package                            = $this->update_path . 'downloads/' . $this->get_lcns();
            $transient->response[$this->plugin_slug] = $obj;
        }

        return $transient;
    }

    /**
     * Add our self-hosted description to the filter
     *
     * @param boolean $false
     * @param array $action
     * @param object $arg
     * @return bool|object
     */
    public function check_info($false, $action, $arg) {
        if ($arg->slug === $this->slug) {
            $information = $this->get_remote_info();
            return $information;
        }
        return false;
    }

    /**
     * Return the remote version
     * @return string $remote_version
     */
    public function get_remote_version() {

        $request = wp_remote_get($this->update_path . 'version');

        if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
            $response =  json_decode($request['body']);
            return $response->data->version;
        }
        return false;
    }

    /**
     * Get information about the remote version
     * @return bool|object
     */
    public function get_remote_info() {
        $request = wp_remote_get($this->update_path . 'info');
        if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
            $response = json_decode($request['body']);
            $content                = unserialize($response->data->nfo);
            $content->download_link .= '/' . $this->get_lcns();
            return $content;
        }
        return false;
    }

    /**
     * Return the status of the plugin licensing
     * @return boolean $remote_license
     */
    public function get_remove_license() {
        $request = wp_remote_get($this->update_path . 'license');
        if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
            return $request['body'];
        }
        return false;
    }

}
