<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

header('Content-Type: application/json');


if (false === check_ajax_referer('stecApiVerifyNonce', 'security', false)) {
    echo wp_json_encode(array(
        'error'     => 1,
        'error_msg' => esc_html__('Invalid nonce', 'stec')
    ));
    exit;
}

$task = Admin_Helper::post('task');

switch ($task) :

    case 'front_delete_event' :

        $event_id = Admin_Helper::post('id', false, FILTER_VALIDATE_INT);

        try {

            if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
                $secret           = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'secret_key');
                $response         = Admin_Helper::post('g-recaptcha-response');
                $ip               = $_SERVER['REMOTE_ADDR'];
                $is_captcha_valid = Submit_Event::validate_captcha($secret, $response, $ip);
                if ($is_captcha_valid !== true) {
                    throw new Stec_Exception(esc_html__('Invalid captcha', 'stec'));
                }
            }

            if ($event_id === false) {
                throw new Stec_Exception(esc_html__('Invalid event id', 'stec'));
            }

            if (true === Submit_Event::delete_event($event_id)) {
                echo wp_json_encode(array(
                    'error' => 0,
                    'msg'   => esc_html__('Event deleted', 'stec')
                ));
            } else {
                throw new Stec_Exception(esc_html__('Unexpected error', 'stec'));
            }
        } catch (Stec_Exception $ex) {
            echo wp_json_encode(array(
                'error' => 1,
                'msg'   => $ex->getMessage()
            ));
        }

        break;

    case 'front_create_event' :

        if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {

            $secret   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'secret_key');
            $response = Admin_Helper::post('g-recaptcha-response');
            $ip       = $_SERVER['REMOTE_ADDR'];

            $is_captcha_valid = Submit_Event::validate_captcha($secret, $response, $ip);

            if ($is_captcha_valid !== true) {

                echo wp_json_encode(array(
                    'error' => 1,
                    'msg'   => esc_html__('Invalid captcha', 'stec')
                ));

                die();
            }
        }

        try {

            $event = Submit_Event::create_event();

            if ((int) $event->get_approved() === 0) {
                // Send email to administrator
                Mail::send_event_submitted_admin_mail($event);

                // Send email to event owner
                Mail::send_event_submitted_owner_mail($event);
            }

            echo wp_json_encode(array(
                'error' => 0,
                'msg'   => esc_html__('Event created', 'stec'),
                'event' => (array) $event->get_front_data()
            ));
        } catch (Stec_Exception $ex) {

            echo wp_json_encode(array(
                'error' => 1,
                'msg'   => $ex->getMessage()
            ));
        }

        break;

    case 'get_events' :

        $events       = array();
        $cat          = Admin_Helper::post('cat', null, FILTER_SANITIZE_STRING);
        $organizer_id = Admin_Helper::post('organizer_id', null, FILTER_SANITIZE_STRING);
        $cal          = Admin_Helper::post('cal', null, FILTER_SANITIZE_STRING);
        $min_date     = Admin_Helper::post('min_date', null, FILTER_SANITIZE_STRING);
        $max_date     = Admin_Helper::post('max_date', null, FILTER_SANITIZE_STRING);

        if ($cal) {
            $cal = explode(',', $cal);
        }

        $meta_query = array();
        $tax_query  = array();

        if ($min_date || $max_date) {

            $meta_query[] = array(
                'relation' => 'OR',
                array(
                    'key'     => 'start_date',
                    'value'   => array(($min_date ? $min_date : '1970-01-01'), ($max_date ? $max_date : '2038-01-19')),
                    'compare' => 'BETWEEN',
                    'type'    => 'DATE'
                ),
                array(
                    'key'     => 'rrule',
                    'value'   => '',
                    'compare' => '!=',
                )
            );
        }

        if (!is_user_logged_in()) {
            $meta_query[] = array(
                'relation' => 'OR',
                array(
                    'key'     => 'approved',
                    'value'   => '1',
                    'compare' => '='
                ),
                array(
                    array(
                        'key'     => 'approved',
                        'value'   => '0',
                        'compare' => '='
                    ),
                    array(
                        'key'     => 'created_by_cookie',
                        'value'   => Admin_Helper::get_user_created_by_cookie(),
                        'compare' => '='
                    )
                )
            );
        }

        if ($cat) {
            $cat         = explode(',', $cat);
            $tax_query[] = array(
                'taxonomy' => 'stec_event_category',
                'field'    => 'term_id',
                'terms'    => $cat,
            );
        }

        if ($organizer_id) {
            $organizer_id_array = explode(',', $organizer_id);
            $tax_query[]        = array(
                'taxonomy' => 'stec_event_organizer',
                'field'    => 'term_id',
                'terms'    => $organizer_id_array,
            );
        }

        $cache = Admin_Helper::get_cache(array($cal, $meta_query, $tax_query, get_current_user_id()));

        if ($cache) {
            echo wp_json_encode($cache);
            exit;
        }

        $events = Events::get_front_events(array(
                    'calid'      => $cal,
                    'meta_query' => $meta_query,
                    'tax_query'  => $tax_query
        ));

        foreach ($events as $key => $event) {
            $events[$key] = $event->get_front_data();
        }

        Admin_Helper::set_cache(array($cal, $meta_query, $tax_query, get_current_user_id()), $events);
        echo wp_json_encode($events);

        break;

    case 'set_user_event_attendance' :

        $event_id = Admin_Helper::post('event_id', false, FILTER_VALIDATE_INT);
        $status   = Admin_Helper::post('status', 0, FILTER_VALIDATE_INT);
        $people   = Admin_Helper::post('people', 1, FILTER_VALIDATE_INT);
        $date     = Admin_Helper::post('date', 0, FILTER_DEFAULT);

        try {

            if ($event_id === false) {
                throw new Stec_Exception(esc_html__('Invalid event id', 'stec'));
            }

            if (!is_user_logged_in()) {
                throw new Stec_Exception(esc_html__('User not logged in', 'stec'));
            }

            $event_post = new Event_Post($event_id);

            if (!$event_post->get_id()) {
                throw new Stec_Exception(esc_html__('Invalid event id', 'stec'));
            }

            if (1 === $status && false === Rsvp::has_free_slots($event_post, $date, $people)) {
                throw new Stec_Exception(esc_html__('Not enough places', 'stec'));
            }

            $user   = wp_get_current_user();
            $result = $event_post->set_attendee_status($status, array(
                'people' => $people,
                'userid' => $user->ID,
                'date'   => $date
            ));

            echo wp_json_encode(array(
                'error'      => 0,
                'slots_left' => (int) Rsvp::has_free_slots($event_post, $date),
                'status'     => $status,
                'msg'        => esc_html__('Status has been updated', 'stec')
            ));
        } catch (Stec_Exception $ex) {

            echo wp_json_encode(array(
                'status'     => 0,
                'error'      => 1,
                'slots_left' => (int) Rsvp::has_free_slots($event_post, $date),
                'msg'        => $ex->getMessage()
            ));
        }

        break;

    case 'get_weather_data' :

        $location = Admin_Helper::post('location', '');

        $weather_data = Admin_Helper::get_weather_data($location);

        echo Admin_Helper::esc($weather_data, 'json');

        break;

    case 'add_reminder' :

        $event_id      = Admin_Helper::post('event_id', false, FILTER_VALIDATE_INT);
        $repeat_offset = Admin_Helper::post('repeat_offset', 0, FILTER_VALIDATE_INT);
        $before        = Admin_Helper::post('before', 3600, FILTER_VALIDATE_INT);
        $email         = Admin_Helper::post('email', false, FILTER_VALIDATE_EMAIL);
        $event         = new Event_Post($event_id);

        $date_time = new \DateTime();
        $date_time->setTimezone(new \DateTimeZone($event->get_timezone()));
        $date_time->setTimestamp($event->get_start_date('U'));
        $date_time->add(new \DateInterval("P" . floor($repeat_offset / 86400) . "D"));
        $date_time->sub(new \DateInterval("PT" . $before . "S"));
        $date      = $date_time->format('Y-m-d H:i:s');

        $reminder = Reminder::set($event_id, $repeat_offset, $email, $date);

        echo wp_json_encode($reminder);

        break;

    case 'rsvp_to_event' :

        try {

            if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
                $secret   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'secret_key');
                $response = Admin_Helper::post('g-recaptcha-response');
                $ip       = $_SERVER['REMOTE_ADDR'];

                $is_captcha_valid = Submit_Event::validate_captcha($secret, $response, $ip);

                if ($is_captcha_valid !== true) {
                    throw new Stec_Exception(esc_html__('Invalid captcha', 'stec'));
                }
            }

            $event_id  = Admin_Helper::post('event_id', false, FILTER_VALIDATE_INT);
            $name      = Admin_Helper::post('name', false, FILTER_DEFAULT);
            $email     = Admin_Helper::post('email', false, FILTER_VALIDATE_EMAIL);
            $people    = Admin_Helper::post('people', 1, FILTER_VALIDATE_INT);
            $date      = Admin_Helper::post('date', 0, FILTER_DEFAULT);
            $is_update = Admin_Helper::post('update', 0, FILTER_VALIDATE_INT);
            $event     = new Event_Post($event_id);

            $attendee = new Event_Meta_Attendee();
            $attendee->set_email($email);
            $attendee->set_name($name);
            $attendee->set_status(1, $date);
            $attendee->set_people($people);

            $user = get_user_by('email', $email);

            if ($user) {
                $attendee->set_userid($user->ID);
                $attendee->set_name($user->display_name);
                $attendee->set_avatar(get_avatar_url($attendee->get_userid()));
            }

            if (true === $event->attendee_exists($attendee)) {
                throw new Stec_Exception(esc_html__('You are already invited to this event', 'stec'));
            }

            if (false === Rsvp::has_free_slots($event, $date, $people)) {
                throw new Stec_Exception(esc_html__('Not enough places', 'stec'));
            }
            
            $attendee = apply_filters('stec_rsvp_to_event_before_add', $attendee);

            Rsvp::add($event, $date, $attendee);

            echo wp_json_encode(array(
                'msg'   => esc_html__('RSVP has been set', 'stec'),
                'data'  => array(
                    'attendee' => $attendee->get_data()
                ),
                'error' => 0
            ));
        } catch (Stec_Exception $ex) {
            echo wp_json_encode(array(
                'msg'   => $ex->getMessage(),
                'error' => 1
            ));
        }

        break;

    case 'booking_add_to_cart' :

        try {

            $data = Admin_Helper::post('data', null, FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if (!$data || !isset($data['items']) || !is_array($data['items'])) {
                throw new Stec_Exception(esc_html__('Invalid purchase request', 'stec'));
            }

            if (isset($data['coupon']) && '' !== trim($data['coupon'])) {
                Booking::add_coupon($data['coupon']);
            }

            $items_ids = array_map(function($item) {
                return $item['id'];
            }, $data['items']);

            foreach ($data['items'] as $item) {

                $requires_ticket = get_post_meta($item['id'], 'stec_requires_ticket', true);
                $requires_role   = get_post_meta($item['id'], 'stec_requires_role', true);

                if (is_array($requires_ticket) && $requires_ticket) {

                    foreach ($requires_ticket as $required_ticket_id) {
                        if (false === in_array($required_ticket_id, $items_ids)) {
                            throw new Stec_Exception(sprintf(esc_html__("You can't purchase %s without purchasing %s as well", 'stec'),
                                            get_the_title($item['id']), get_the_title($required_ticket_id)));
                        }
                    }
                }

                if (is_array($requires_role) && $requires_role) {

                    $current_user_roles = Admin_Helper::get_user_roles();

                    if (false === (bool) array_intersect($current_user_roles, $requires_role)) {
                        throw new Stec_Exception(sprintf(esc_html__("You don't have the required user role to purchase %s", 'stec'),
                                        get_the_title($item['id'])));
                    }
                }

                Booking::add_to_cart($item);
            }

            echo wp_json_encode(array(
                'msg'   => esc_html__('Success', 'stec'),
                'cart'  => wc_get_cart_url(),
                'error' => 0
            ));
        } catch (Stec_Exception $ex) {

            echo wp_json_encode(array(
                'msg'   => $ex->getMessage(),
                'error' => 1
            ));
        }

        break;

    case 'apply_discount_coupon' :

        $result = 0;

        \WC()->cart->remove_coupons();

        $coupon = Admin_Helper::post('coupon_code', false, FILTER_DEFAULT);

        if ($coupon) {
            $result = \WC()->cart->add_discount($coupon);
        }

        echo wp_json_encode(array(
            'coupon' => $coupon,
            'result' => (int) $result
        ));

        break;

    case 'insert_new_organizer' :

        $r = new response_object();

        try {

            if (false === Admin_Helper::isset_post('name')) {
                throw new Stec_Exception(esc_html__('Invalid organizer name', 'stec'));
            }

            $organizer = new Organizer();
            $organizer->set_name(Admin_Helper::post('name', '', FILTER_SANITIZE_STRING));
            $organizer->set_email(Admin_Helper::post('email', '', FILTER_VALIDATE_EMAIL));
            $organizer->set_social(Admin_Helper::post('social', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY));
            $organizer->set_details(Admin_Helper::post('details', '', FILTER_DEFAULT));
            $organizer->set_notes(Admin_Helper::post('notes', '', FILTER_DEFAULT));
            $organizer->set_photo(Admin_Helper::post('photo', '', FILTER_VALIDATE_INT));
            $organizer->set_visibility('private');

            $id = $organizer->insert_organizer();

            $r->set_data(array(
                'success'   => 1,
                'organizer' => $organizer->get_data()
            ));
        } catch (Stec_Exception $ex) {
            $r->set_error_msg($ex->getMessage());
            $r->set_error(1);
        }

        $r->get();

        break;

endswitch;

exit;
