<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

$stachethemes_ec_main = $this;

add_action('stec_add_admin_menu', function() use ($stachethemes_ec_main) {

    if (true === Admin_Helper::check_migration_required()) :

        Admin_Helper::set_message(sprintf(esc_html__('Stachethemes Event Calendar requires database update. Go to %s to start the update.', 'stec'), '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__migrate')) . '">' . esc_html__('migrate menu', 'stec') . '</a>'), 'notice');

        $stachethemes_ec_main->add_menu("St. Event Calendar", "stec_menu__general", "dashicons-calendar-alt", 26);
        $stachethemes_ec_main->add_submenu(esc_html__('General', 'stec'), "stec_menu__general");
        $stachethemes_ec_main->add_submenu(esc_html__('Migrate', 'stec'), "stec_menu__migrate");
        $stachethemes_ec_main->add_submenu(esc_html__('Activator', 'stec'), "stec_menu__license");

    else :

        // Admin Menu
        $has_calendars = count(Calendars::get_admin_calendars()) > 0 ? true : false;
        $stachethemes_ec_main->add_menu("St. Event Calendar", "stec_menu__general", "dashicons-calendar-alt", 26);
        $stachethemes_ec_main->add_submenu(esc_html__('General', 'stec'), "stec_menu__general");
        $stachethemes_ec_main->add_submenu(esc_html__('Fonts & Colors', 'stec'), "stec_menu__fontsandcolors");
        $stachethemes_ec_main->add_submenu(esc_html__('Themes', 'stec'), "stec_menu__themes");
        $stachethemes_ec_main->add_submenu(esc_html__('Builder', 'stec'), "stec_menu__builder");
        $stachethemes_ec_main->add_submenu(esc_html__('Calendars', 'stec'), "stec_menu__calendars");
        $stachethemes_ec_main->add_submenu(esc_html__('Categories', 'stec'), "stec_menu__categories");
        $stachethemes_ec_main->add_submenu(esc_html__('Locations', 'stec'), "stec_menu__locations");
        $stachethemes_ec_main->add_submenu(esc_html__('Organizers', 'stec'), "stec_menu__organizers");

        if ($has_calendars) {
            $stachethemes_ec_main->add_submenu(esc_html__('Events', 'stec'), "stec_menu__events");
            $stachethemes_ec_main->add_submenu(esc_html__('Import Events', 'stec'), "stec_menu__import");
            $stachethemes_ec_main->add_submenu(esc_html__('Export Events', 'stec'), "stec_menu__export");
        }

        if (class_exists('WooCommerce')) {
            $stachethemes_ec_main->add_submenu(esc_html__('Tickets', 'stec'), "stec_menu__tickets");
        }

        $stachethemes_ec_main->add_submenu(esc_html__('Cache', 'stec'), "stec_menu__cache");
        $stachethemes_ec_main->add_submenu(esc_html__('Backup Settings', 'stec'), "stec_menu__iesettings");
        $stachethemes_ec_main->add_submenu(esc_html__('User Guide', 'stec'), "stec_menu__manual");
        $stachethemes_ec_main->add_submenu(esc_html__('Activator', 'stec'), "stec_menu__license");

    endif;
});

add_action('stec_add_admin_menu_assets', function() use ($stachethemes_ec_main) {

    add_action('admin_enqueue_scripts', function($hook) use ($stachethemes_ec_main) {
        if (false !== strpos($hook, 'stec_menu')) {

            if (false !== strpos($hook, 'stec_menu__builder')) {
                // Builder css and js
                $stachethemes_ec_main->add_asset_css('stec-uicss', "//ajax.googleapis.com/ajax/libs/jqueryui/1.12.1/themes/smoothness/jquery-ui.css");
                $stachethemes_ec_main->add_asset_css('stec-font-awesome', "css/fonts/css/all.min.css");
                $stachethemes_ec_main->add_asset_css('stec-builder', 'css/admin/builder.css');
                $stachethemes_ec_main->add_asset_js('stec-builder-main', 'js/admin/builder/main.js', 'jquery,jquery-ui-droppable,jquery-ui-draggable,jquery-ui-sortable,jquery-ui-resizable');
                $stachethemes_ec_main->add_asset_js('stec-builder-elements', 'js/admin/builder/elements.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-elements-list', 'js/admin/builder/elements-list.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-settings', 'js/admin/builder/settings.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-history', 'js/admin/builder/history.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-top', 'js/admin/builder/top.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-sections', 'js/admin/builder/sections.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-panel', 'js/admin/builder/panel.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js('stec-builder-plot', 'js/admin/builder/plot.js', 'stec-builder-main');
                $stachethemes_ec_main->add_asset_js("stec-db-js", "js/db.js", "stec-builder-main");
            } else {
                // All other menu pages
                $stachethemes_ec_main->add_asset_js("stec-admin-google-maps", "//maps.googleapis.com/maps/api/js?key=" . Settings::get_admin_setting_value('stec_menu__general', 'gmaps_api_key'));
                $stachethemes_ec_main->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");
                $stachethemes_ec_main->add_asset_css("stec-ui", "//ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css");
                $stachethemes_ec_main->add_asset_css("stec-css", "css/admin/style.css");
                $stachethemes_ec_main->add_asset_css("stec-colorpicker", "css/colorpicker/css/colorpicker.css");
                $stachethemes_ec_main->add_asset_js("stec-colorpicker-js", "js/libs/colorpicker/colorpicker.js", 'jquery');
                $stachethemes_ec_main->add_asset_js("stec-rrule-js", "js/libs/rrule.js");
                $stachethemes_ec_main->add_asset_js("stec-nlp-js", "js/libs/nlp.js", "stec-rrule-js");
                $stachethemes_ec_main->add_asset_js("stec-js", "js/admin/main.js", 'jquery-ui-datepicker,jquery-ui-sortable');
                $stachethemes_ec_main->add_asset_js("stec-moment-js", "js/libs/moment.js", 'stec-js');
                $stachethemes_ec_main->add_asset_js("stec-moment-tz-js", "js/libs/moment-timezone.js", 'stec-moment-js');
                $stachethemes_ec_main->add_asset_js("stec-db-js", "js/db.js", "stec-js");
            }

            // Add DB nonce
            $stachethemes_ec_main->localize('stec-db-js', 'stecApiNonce', array(
                'nonce' => wp_create_nonce('stecApiVerifyNonce')
            ));

            add_action('admin_head', function() {
                ?>
                <script type="text/javascript">
                    var stec_ajax_nonce = "<?php echo wp_create_nonce("stec-nonce-string"); ?>";
                </script>
                <?php
            });

            $stachethemes_ec_main->load_admin_assets();
        }
    });
});

do_action('stec_add_admin_menu');
do_action('stec_add_admin_menu_assets');

$this->register_menu();
