<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

$stachethemes_ec_main = $this;

add_filter('single_template', function($single) use ($stachethemes_ec_main) {

    global $post;

    // If is not part of plugin single pages return normal
    if (strpos($post->post_type, 'stec_') === false) {
        return $single;
    }

    // Check if is edit page
    if ($post->post_type === 'stec_event' && null !== (get_query_var('stec_edit_event', null))) {

        $event = new Event_Post(get_the_ID());

        add_action('wp_enqueue_scripts', function() {
            do_action('stec_load_builder_scripts');
        });

        add_action('wp_head', function() use ($event) {
            do_action('stec_single_edit_wp_head', $event);
        });

        $atts                 = array();
        $atts['id']           = uniqid('stec-submission-form-');
        $atts['project_type'] = 'event_submission_form';

        if (!isset($atts['project_id']) || !$atts['project_id']) {
            $atts['project_id'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');
        }

        $atts['captcha']['enabled']                    = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled');
        $atts['captcha']['site_key']                   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'site_key');
        $atts['create_event_form_allow_expired']       = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_expired');
        $atts['create_event_form_allow_location_type'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        add_filter('stec_event_get_front_data', function($data) use ($event) {
            $data['contact_email'] = $event->get_custom_meta('contact_email');
            $data['review_note']   = $event->get_custom_meta('review_note');
            return $data;
        });

        $atts['edit_event']  = $event->get_front_data();
        $atts['date_format'] = Settings::get_admin_setting_value('stec_menu__general', 'date_format');

        $json = wp_json_encode($atts);

        add_filter('the_content', function() use($stachethemes_ec_main, $event, $atts) {
            ob_start();
            include $stachethemes_ec_main->get_path('VIEW') . 'front/edit.php';
            return ob_get_clean();
        });

        add_action('wp_footer', function() use ($event, $json) {
            ?>
            <script type="text/javascript">
                if (typeof stec_builder_form_instance === "undefined") {
                    var stec_builder_form_instance = [];
                }
                stec_builder_form_instance.push(<?php echo Admin_Helper::esc($json, 'json'); ?>);
            </script>
            <?php
            do_action('stec_single_edit_wp_footer', $event);
        });

        return $single;
    }

    // Check if single stec file exists in the theme and return it
    if (strpos($single, 'stec_') !== false) {
        if (file_exists($single)) {
            return $single;
        }
    }

    switch ($post->post_type) {
        case 'stec_event' :

            $event         = new Event_Post(get_the_ID());
            $repeat_offset = get_query_var('stec_repeat_offset', 0);

            if (!Events::user_can_view_event($event) || false === $event->is_valid_offset($repeat_offset)) {
                global $wp_query;
                $wp_query->set_404();
                status_header(404);
                get_template_part(404);
                exit();
            }

            add_filter('has_post_thumbnail', function($has_thumbnail, $post) {
                if ('stec_event' === get_post_type($post)) {
                    return false;
                }
                return $has_thumbnail;
            },10 ,2);

            add_action('wp_head', function() use ($event, $repeat_offset) {
                do_action('stec_single_wp_head', $event);

                echo Admin_Helper::get_event_og($event, $repeat_offset);
            });

            add_action('wp_enqueue_scripts', function() use ($stachethemes_ec_main) {

                // calendar default fonts
                $stachethemes_ec_main->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");

                // main stylesheet
                $stachethemes_ec_main->add_asset_css("stec-single", "css/front/style.single.css");

                if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
                    $stachethemes_ec_main->add_asset_js("stec-jquery-mobile-js", "js/libs/jquery.mobile.js", "jquery");
                }
                if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_gmaps') === '1') {
                    $stachethemes_ec_main->add_asset_js("stec-google-maps", "//maps.googleapis.com/maps/api/js?key=" . Settings::get_admin_setting_value('stec_menu__general', 'gmaps_api_key'));
                }
                if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_chart_js') === '1') {
                    $stachethemes_ec_main->add_asset_js("stec-chart", "js/libs/chart-2.8.0.min.js", "jquery");
                }
                if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
                    $stachethemes_ec_main->add_asset_js("stec-moment", "js/libs/moment.js", "jquery");
                    $stachethemes_ec_main->add_asset_js("stec-moment-tz", "js/libs/moment-timezone.js", "jquery");
                }
                if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
                    $stachethemes_ec_main->add_asset_js("stec-google-captcha-api", "https://www.google.com/recaptcha/api.js?render=explicit async defer");
                }

                $stachethemes_ec_main->add_asset_js("stec-helper-js", "js/front/helper.js");
                $stachethemes_ec_main->add_asset_js("stec-single-js", "js/front/stec-single.js", "jquery,stec-helper-js");
                $stachethemes_ec_main->add_asset_js("stec-db-js", "js/db.js", "stec-single-js");
                $stachethemes_ec_main->add_asset_js("stec-reminder-js", "js/front/reminder.js", "stec-single-js");
                $stachethemes_ec_main->add_asset_js("stec-rsvp-js", "js/front/rsvp.js", "stec-single-js");

                if (class_exists('WooCommerce')) {
                    $stachethemes_ec_main->add_asset_js("stec-booking-js", "js/front/booking.js", "stec-single-js");
                }

                $stachethemes_ec_main->add_asset_js("stec-scroll-js", "js/libs/scroll.js", "stec-helper-js,jquery-ui-draggable");

                // Share and embeding
                $stachethemes_ec_main->add_asset_js("stec-sharer-js", "js/front/share.js");

                // Add DB nonce
                $stachethemes_ec_main->localize('stec-db-js', 'stecApiNonce', array(
                    'nonce' => wp_create_nonce('stecApiVerifyNonce')
                ));

                $stachethemes_ec_main->load_head();
                $stachethemes_ec_main->load_js_locales();

                if (Settings::get_admin_setting_value('stec_menu__cache', 'ac') === '1') {
                    AssetsCompressor::compress_js();
                    AssetsCompressor::compress_css();
                }
            });

            add_filter('the_content', function() use ($stachethemes_ec_main, $event, $repeat_offset) {
                ob_start();
                $the_permalink   = $event->get_permalink($repeat_offset);
                $rsvp_expired    = false;
                $booking_expired = false;
                $has_tickets     = false;
                
                if ($event->get_start_date('U', $repeat_offset) < gmdate('U')) {
                    $rsvp_expired    = true;
                    $booking_expired = true;
                }

                if (class_exists('WooCommerce') && $event->get_bookable_products() && false === $booking_expired) {
                    $has_tickets = true;
                }
                include $stachethemes_ec_main->get_path('VIEW') . 'front/pages/single.php';
                return ob_get_clean();
            });

            add_action('wp_footer', function() use ($event, $repeat_offset) {
                $general_settings            = Settings::get_admin_settings_kv('stec_menu__general');
                $general_settings['captcha'] = Settings::get_admin_settings_kv('stec_menu__general_google_captcha');
                if (is_user_logged_in()) {
                    $general_settings['userid']    = get_current_user_id();
                    $userdata                      = get_userdata($general_settings['userid']);
                    $general_settings['useremail'] = $userdata->user_email;
                    $general_settings['username']  = $userdata->display_name;
                }
                $general_settings['site_url']      = get_site_url();
                $general_settings['repeat_offset'] = $repeat_offset;

                if (class_exists('WooCommerce')) {
                    $general_settings['woocommerce'] = array(
                        'price_format'        => get_woocommerce_price_format(),
                        'symbol'              => get_woocommerce_currency_symbol(),
                        'decimals'            => wc_get_price_decimals(),
                        'decimals_separator'  => wc_get_price_decimal_separator(),
                        'thousands_separator' => wc_get_price_thousand_separator(),
                    );
                }
                ?>
                <script type="text/javascript">
                    var stecSingleOptions = <?php echo wp_json_encode($general_settings); ?>;
                    var stecSingleEvent = <?php echo wp_json_encode($event->get_front_data()); ?>;
                </script>
                <?php
                do_action('stec_single_wp_footer', $event);
            });

            return $single;

        default: return $single;
    }
});
