<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

function add_event_tab($slug, $title, $icon, $content = "", $file = false) {

    add_action('stec_add_event_tab', function() use($slug, $title, $icon) {
        $slug  = esc_html($slug);
        $title = esc_html($title);
        $icon  = esc_html($icon);

        echo "<li data-tab='stec-layout-event-inner-{$slug}'><i class='{$icon}'></i><p>{$title}</p></li>";
    });

    add_action('stec_add_event_tab_content', function() use($slug, $content, $file) {

        $slug = esc_html($slug);

        echo "<div class='stec-layout-event-inner-{$slug}'>";

        if ($file !== false && 0 === validate_file($file)) :
            $stec     = Stachethemes_Event_Calendar::get_instance();
            $path     = $stec->get_path('VIEW') . 'front/tabs/';
            $filepath = $path . '/' . $file;
            if (file_exists($filepath)) {
                include($filepath);
            }
        endif;

        if (trim($content) !== '') :
            echo Admin_Helper::esc($content, 'noscript');
        endif;

        echo "</div>";
    });
}

function add_event_single_tab($slug, $title, $icon, $content = "", $file = false) {

    // Skip intro and location tabs since they are outside tabs
    if (in_array($slug, apply_filters('stec_add_event_single_tab_exclude', array('intro', 'location')))) {
        return;
    }

    add_action('stec_add_event_single_tab', function() use($slug, $title, $icon) {
        ?>
        <li class="stec-layout-event-title2-fontandcolor" data-tab="stec-layout-single-<?php esc_html_e($slug); ?>">
            <i class="<?php esc_html_e($icon) ?>"></i>
            <p><?php esc_html_e($title); ?></p>
        </li>
        <?php
    });

    add_action('stec_add_event_single_tab_content', function() use($slug, $content, $file) {

        echo "<div class='stec-layout-single-{$slug}'>";

        if ($file !== false && 0 === validate_file($file)) :
            $stec     = Stachethemes_Event_Calendar::get_instance();
            $path     = $stec->get_path('VIEW') . 'front/tabs/';
            $filepath = $path . '/' . $file;
            if (file_exists($filepath)) {
                include($filepath);
            }
        endif;

        if (trim($content) !== '') :
            echo Admin_Helper::esc($content, 'noscript');
        endif;

        echo "</div>";
    });
}

/**
 * Add event inner tabs
 * You can comment out tab to disable it or change tabs order
 */
$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();
$event_tabs           = array(
    array("intro", esc_html__('Event Info', 'stec'), "fa fa-info", "", "intro.php"),
    array("location", esc_html__('Location', 'stec'), "fa fa-map-marker", "", "location.php"),
    array("schedule", esc_html__('Schedule', 'stec'), "fa fa-th-list", "", "schedule.php"),
    array("guests", esc_html__('Guests', 'stec'), "fas fa-star", "", "guests.php"),
    array("attendance", esc_html__('Attendance', 'stec'), "fa fa-user", "", "attendance.php")
);

// if woocommerce is running
if (class_exists('WooCommerce')) {
    $event_tabs[] = array("shop", esc_html__('Shop', 'stec'), "fa fa-shopping-cart", "", "shop.php");
}

$event_tabs[] = array("forecast", esc_html__('Forecast', 'stec'), "fas fa-sun", "", "forecast.php");
$event_tabs[] = array("comments", esc_html__('Comments', 'stec'), "fas fa-comments", "", false);
$event_tabs   = apply_filters('stec_event_tabs', $event_tabs);

foreach ($event_tabs as $event_tab) {
    add_event_tab($event_tab[0], $event_tab[1], $event_tab[2], $event_tab[3], $event_tab[4]);
    add_event_single_tab($event_tab[0], $event_tab[1], $event_tab[2], $event_tab[3], $event_tab[4]);
}