<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

// Attendance by URL Listener
$event_id     = filter_input(INPUT_GET, "stec_attendance", FILTER_VALIDATE_INT);
$access_token = filter_input(INPUT_GET, "access_token", FILTER_SANITIZE_STRING);
$status       = filter_input(INPUT_GET, "status", FILTER_VALIDATE_INT);

if ($event_id && $access_token && $status) {

    $result = Admin_Helper::update_attendance_by_url($event_id, $access_token, $status);

    if ($result === true) {

        switch ($status) :

            case 1:
                $this->set_fixed_message(esc_html__('You have accepted event invitation', 'stec'), 'success');
                break;

            case 2:
                $this->set_fixed_message(esc_html__('You have declined event invitation', 'stec'), 'error');
                break;

        endswitch;
    }

    wp_redirect(site_url('/'));

    exit;
}

// GDPR url listener
GDPR::task_handler();

// $_GET
switch (Admin_Helper::get("task")) {

    case 'stec_duplicate_event' :

        Admin_Helper::verify_nonce('', 'GET');

        $event_id = Admin_Helper::get('event_id', 0, FILTER_VALIDATE_INT);
        $lang     = Admin_Helper::get('stec_change_lang', null, FILTER_DEFAULT);

        $args = array(
            'lang' => $lang
        );

        $result = Events::duplicate_event($event_id, $args);

        if ($result) {
            Admin_Helper::set_message(esc_html__('Event duplicated', 'stec'));
        } else {
            Admin_Helper::set_message(esc_html__('Unable to duplicate event', 'stec'), 'error');
        }

        wp_redirect(esc_url(admin_url('admin.php?page=stec_menu__events')));

        exit;

        break;

    case 'stec_export_to_ics' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_ics();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_public_export_to_ics' :

        $calendar_id = Admin_Helper::get('calendar_id', false, FILTER_VALIDATE_INT);
        $category_id = Admin_Helper::get('category_id', false, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', false, FILTER_VALIDATE_INT);

        $exporter = new Export($calendar_id, $event_id);

        if ($category_id) {
            $exporter->set_category($category_id);
        }

        try {
            $exporter->export_ics();
        } catch (Stec_Exception $ex) {
            
        }

        break;

    case 'stec_export_to_csv_content' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_content();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_schedule' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_schedule();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_guests' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_guests();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_organizers' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_organizers();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_attendance' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_attendance();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_booking' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_booking();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv_shop' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv_shop();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_export_to_csv' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = Admin_Helper::get('calendar_id', null, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::get('event_id', null, FILTER_VALIDATE_INT);
        $exporter    = new Export($calendar_id, $event_id);

        try {
            $exporter->export_csv();
        } catch (Stec_Exception $ex) {
            if (is_admin()) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_delete_calendar' :

        Admin_Helper::verify_nonce('', 'GET');

        $calendar_id = (int) Admin_Helper::get("calendar_id", false, FILTER_VALIDATE_INT);

        try {

            if (!$calendar_id) {
                throw new Stec_Exception(esc_html__('Calendar has no id', 'stec'));
            }

            $calendar_post = new Calendar_Post($calendar_id);

            if (!$calendar_post->get_id()) {
                throw new Stec_Exception(esc_html__("Calendar does not exist", 'stec'));
            }

            if (false === Calendars::user_can_edit_calendar($calendar_post)) {
                throw new Stec_Exception(esc_html__("Unable to delete calendar. Check permissions.", 'stec'));
            }

            $calendar_post->delete_post();
            Admin_Helper::set_message(esc_html__('Calendar deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;


    case 'stec_check_order_key' :

        try {
            $key = Admin_Helper::get('key', null, FILTER_SANITIZE_STRING);

            if (!$key) {
                throw new Stec_Exception(esc_html__('Invalid ticket code', 'stec'));
            }

            $order_id = wc_get_order_id_by_order_key($key);

            if (!$order_id) {
                throw new Stec_Exception(esc_html__('Order not found', 'stec'));
            }

            $order = wc_get_order($order_id);

            if (!$order) {
                throw new Stec_Exception(esc_html__('Order not found', 'stec'));
            }

            if (false === Booking::order_has_tickets($order)) {
                throw new Stec_Exception(esc_html__('No tickets were found in this order', 'stec'));
            }
            
            // Check if template file exists in the theme and return it
            // stec-order-details
            $template_filepath = get_template_directory() . '/stec-order-details.php';
            
            if (file_exists($template_filepath)) {
                include $template_filepath;
                exit;
            }

            add_action('wp_enqueue_scripts', function() {
                $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();
                $stachethemes_ec_main->add_asset_css("stec-order-details", "css/front/style.order-details-page.css");
            });
                        
            include($this->get_path('VIEW') . 'front/pages/order-details.php');
            
            exit;
        } catch (Stec_Exception $ex) {
            $this->set_fixed_message(strtoupper($ex->getMessage()), 'error');
        }

        break;
}

// $_POST
switch (Admin_Helper::post("task")) {

    case 'stec_export_to_ics_bulk' :

        $calendar_id = Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT);
        $idlist      = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        Admin_Helper::verify_nonce("?page=stec_menu__export&view=list&calendar_id=" . $calendar_id);

        $exporter = new Export($calendar_id, null, $idlist);

        try {
            $exporter->export_ics();
        } catch (Stec_Exception $ex) {
            
        }

        break;

    case 'stec_export_to_csv_bulk' :

        $calendar_id = Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT);
        $idlist      = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        Admin_Helper::verify_nonce("?page=stec_menu__export&view=list&calendar_id=" . $calendar_id);

        $exporter = new Export($calendar_id, null, $idlist);

        try {
            $exporter->export_csv();
        } catch (Stec_Exception $ex) {
            
        }

        break;

    case 'save_import_csv_settings' :

        Admin_Helper::verify_nonce("?page=stec_menu__import&view=csv_settings");

        $fields          = Admin_Helper::post('event_field', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
        $datetime_format = Admin_Helper::post('csv_datetime_format', 'Y-m-d H:i:s', FILTER_SANITIZE_STRING);

        try {
            Csv_Settings::update_event_assoc_settings($fields);
            Csv_Settings::update_datetime_format($datetime_format);
            Admin_Helper::set_message(esc_html__("CSV Settings Updated", 'stec'));
        } catch (Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'stec_public_export_to_ics' :

        $calendar_id = Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT);
        $event_id    = Admin_Helper::post('event_id', false, FILTER_VALIDATE_INT);

        $exporter = new Export($calendar_id, $event_id);

        try {
            $exporter->export_ics();
        } catch (Stec_Exception $ex) {
            Admin_Helper::debug_log($ex->getMessage());
        }

        break;

    case "stec_export_settings" :

        Admin_Helper::verify_nonce("?page=stec_menu__iesettings");

        Settings::export_settings();

        exit;
        break;

    case "stec_import_settings" :

        Admin_Helper::verify_nonce("?page=stec_menu__iesettings");

        if (strtolower(pathinfo($_FILES["settings_filename"]["name"], PATHINFO_EXTENSION)) !== 'stec') {
            Admin_Helper::set_message(esc_html__("File extension is not .stec", 'stec'), 'error');
            break;
        }

        $file   = $_FILES["settings_filename"]["tmp_name"];
        $result = Settings::import_settings($file);

        if (true === $result) {
            Admin_Helper::set_message(esc_html__("Settings Imported", 'stec'));
        } else {
            Admin_Helper::set_message(esc_html__("Error importing settings", 'stec'), 'error');
        }

        break;

    case "stec_export_color_scheme" :

        Admin_Helper::verify_nonce("?page=stec_menu__themes");

        Settings::export_color_scheme();

        exit;
        break;

    case "stec_import_color_scheme" :

        Admin_Helper::verify_nonce("?page=stec_menu__themes");

        if (strtolower(pathinfo($_FILES["stec_cs_filename"]["name"], PATHINFO_EXTENSION)) !== 'stec-cs') {
            Admin_Helper::set_message(esc_html__("File extension is not .stec-cs", 'stec'), 'error');
            break;
        }

        $file   = $_FILES["stec_cs_filename"]["tmp_name"];
        $result = Settings::import_color_scheme($file);

        if (true === $result) {
            Admin_Helper::set_message(esc_html__("Color scheme imported", 'stec'));
        } else {
            Admin_Helper::set_message(esc_html__("Error importing color scheme", 'stec'), 'error');
        }

        break;

    case 'stec_update_calendar' :

        Admin_Helper::verify_nonce("?page=stec_menu__calendars");

        $calendar_id     = Admin_Helper::post('calendar_id', false, FILTER_VALIDATE_INT);
        $title           = Admin_Helper::post('calendar_name', false, FILTER_SANITIZE_STRING);
        $color           = Admin_Helper::post('calendar_color', "#f15e6e", FILTER_SANITIZE_STRING);
        $icon            = Admin_Helper::post('calendar_icon', 'fa', FILTER_SANITIZE_STRING);
        $timezone        = Admin_Helper::post('calendar_timezone', "UTC", FILTER_SANITIZE_STRING);
        $back_visibility = Admin_Helper::post('calendar_back_visibility', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $visibility      = Admin_Helper::post('calendar_visibility', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $writable        = Admin_Helper::post('calendar_writable', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $req_approval    = Admin_Helper::post('calendar_req_approval', false) ? 1 : 0;

        try {

            if (!$calendar_id) {
                throw new Stec_Exception(esc_html__('Calendar has no id', 'stec'));
            }

            if (!$title) {
                throw new Stec_Exception(esc_html__('Calendar has no title', 'stec'));
            }

            $calendar_post = new Calendar_Post($calendar_id);

            if (!$calendar_post->get_id()) {
                throw new Stec_Exception(esc_html__('Calendar does not exist', 'stec'));
            }

            if (false === Calendars::user_can_edit_calendar($calendar_post)) {
                throw new Stec_Exception(esc_html__("Unable to update calendar. Check permissions.", 'stec'));
            }

            $calendar_post->set_title($title);
            $calendar_post->set_icon($icon);
            $calendar_post->set_color($color);
            $calendar_post->set_timezone($timezone);
            $calendar_post->set_visibility($visibility);
            $calendar_post->set_back_visibility($back_visibility);
            $calendar_post->set_writable($writable);
            $calendar_post->set_req_approval($req_approval);

            $result = $calendar_post->insert_post();

            if ($result) {
                Admin_Helper::set_message(esc_html__('Calendar updated', 'stec'));
            } else {
                Admin_Helper::set_message(esc_html__('Unable to update calendar', 'stec'), 'error');
            }
        } catch (Stec_Exception $ex) {

            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'stec_create_calendar' :

        Admin_Helper::verify_nonce("?page=stec_menu__calendars");

        $title           = Admin_Helper::post('calendar_name', false, FILTER_SANITIZE_STRING);
        $color           = Admin_Helper::post('calendar_color', "#f15e6e", FILTER_SANITIZE_STRING);
        $icon            = Admin_Helper::post('calendar_icon', 'fa', FILTER_SANITIZE_STRING);
        $timezone        = Admin_Helper::post('calendar_timezone', "UTC", FILTER_SANITIZE_STRING);
        $visibility      = Admin_Helper::post('calendar_visibility', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $back_visibility = Admin_Helper::post('calendar_back_visibility', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $writable        = Admin_Helper::post('calendar_writable', array(), FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
        $req_approval    = Admin_Helper::post('calendar_req_approval', false) ? 1 : 0;

        try {

            if (!$title) {
                throw new Stec_Exception(esc_html__('Calendar has no title', 'stec'));
            }

            $calendar_post = new Calendar_Post();
            $calendar_post->set_title($title);
            $calendar_post->set_icon($icon);
            $calendar_post->set_color($color);
            $calendar_post->set_timezone($timezone);
            $calendar_post->set_visibility($visibility);
            $calendar_post->set_back_visibility($back_visibility);
            $calendar_post->set_writable($writable);
            $calendar_post->set_req_approval($req_approval);

            $result = $calendar_post->insert_post();

            if ($result) {
                Admin_Helper::set_message(esc_html__('Calendar created', 'stec'));
            } else {
                Admin_Helper::set_message(esc_html__('Unable to create new calendar', 'stec'), 'error');
            }
        } catch (Stec_Exception $ex) {

            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'stec_delete_bulk_calendars' :

        Admin_Helper::verify_nonce("?page=stec_menu__calendars");

        $calendars = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        if ($calendars) {

            try {
                Calendars::bulk_delete($calendars);
                Admin_Helper::set_message(esc_html__('Calendars deleted', 'stec'));
            } catch (Stec_Exception $ex) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;

    case 'stec_update_event' :

        $send_approved_mail = (bool) Admin_Helper::post('send_approved_mail', false, FILTER_VALIDATE_INT);

    case 'stec_create_event' :

        try {

            $event_id    = Admin_Helper::post('event_id', null);
            $calendar_id = Admin_Helper::post('calendar_id');
            $old_event   = null;

            Admin_Helper::verify_nonce("?page=stec_menu__events&calendar_id={$calendar_id}");

            $event_post = new Event_Post($event_id);

            if ($event_post->get_id()) {
                $old_event = clone $event_post;
            }

            $calendar           = new Calendar_Post($calendar_id);
            $calid              = Admin_Helper::post('calid', false, FILTER_VALIDATE_INT);
            $event_post->set_calid($calid ? $calid : $calendar_id);
            $event_post->set_title(Admin_Helper::post('summary', '', FILTER_SANITIZE_STRING));
            $event_post->set_slug(Admin_Helper::post('alias', '', FILTER_SANITIZE_STRING));
            $event_post->set_color(Admin_Helper::post('event_color', '', FILTER_SANITIZE_STRING));
            $event_post->set_icon(Admin_Helper::post('icon', '', FILTER_SANITIZE_STRING));
            $event_post->set_icon_type(Admin_Helper::post('icon_type', '', FILTER_SANITIZE_STRING));
            $event_post->set_visibility(Admin_Helper::post('visibility', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY));
            $event_post->set_back_visibility(Admin_Helper::post('back_visibility', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY));
            $event_post->set_featured(Admin_Helper::post('featured'));
            $event_post->set_categories(Admin_Helper::post('event_categories', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));
            $start_date         = Admin_Helper::post('start_date');
            $start_time_hours   = Admin_Helper::post('start_time_hours');
            $start_time_minutes = Admin_Helper::post('start_time_minutes');
            $end_date           = Admin_Helper::post('end_date');
            $end_time_hours     = Admin_Helper::post('end_time_hours');
            $end_time_minutes   = Admin_Helper::post('end_time_minutes');
            $start_date_time    = $start_date . ' ' . $start_time_hours . ':' . $start_time_minutes . ':00';
            $end_date_time      = $end_date . ' ' . $end_time_hours . ':' . $end_time_minutes . ':00';
            $event_post->set_start_date($start_date_time);
            $event_post->set_end_date($end_date_time);
            $event_post->set_all_day(Admin_Helper::post('all_day', false) !== false ? 1 : 0);
            $event_post->set_hide_end(Admin_Helper::post('hide_end', false) !== false ? 1 : 0);
            $event_post->set_keywords(Admin_Helper::post('keywords'));
            $event_post->set_rrule(Admin_Helper::post('rrule', ''));
            $event_post->set_exdate(Admin_Helper::post('exdate', ''));
            $event_post->set_is_advanced_rrule(Admin_Helper::post('is_advanced_rrule', 0));
            $event_post->set_counter(Admin_Helper::post('counter'));
            $event_post->set_comments(Admin_Helper::post('comments'));
            $event_post->set_rsvp(Admin_Helper::post('rsvp', 0, FILTER_VALIDATE_INT));
            $event_post->set_rsvp_limit(Admin_Helper::post('rsvp_limit', 0, FILTER_VALIDATE_INT));
            $event_post->set_link(Admin_Helper::post('link', '', FILTER_VALIDATE_URL));
            $event_post->set_approved(Admin_Helper::post('approved', 0, FILTER_VALIDATE_INT));

            $location = Admin_Helper::post('location', '', FILTER_DEFAULT);

            switch ($location) {
                case 'custom' :
                    $location_address    = Admin_Helper::post('location_address', '', FILTER_SANITIZE_STRING);
                    $location_city       = Admin_Helper::post('location_city', '', FILTER_SANITIZE_STRING);
                    $location_country    = Admin_Helper::post('location_country', '', FILTER_SANITIZE_STRING);
                    $location_coords     = Admin_Helper::post('location_coordinates', '', FILTER_DEFAULT);
                    $location_details    = Admin_Helper::post('location_details', '', FILTER_DEFAULT);
                    $location_visibility = Admin_Helper::post('location_visibility', 'public', FILTER_DEFAULT);

                    $new_location = new Location();
                    $new_location->set_address($location_address);
                    $new_location->set_city($location_city);
                    $new_location->set_country($location_country);
                    $new_location->set_coordinates($location_coords);
                    $new_location->set_details($location_details);
                    $new_location->set_visibility($location_visibility);

                    try {
                        $location = $new_location->insert_location();
                    } catch (Stec_Exception $ex) {
                        Admin_Helper::debug_log($ex->getMessage());
                        $location = '';
                    }

                    break;

                default:
                    $location = (int) $location;
            }

            $event_post->set_location($location);

            $event_post->set_images(Admin_Helper::post('images', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));
            $event_post->set_icon_images(Admin_Helper::post('icon_images', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));
            $event_post->set_description(Admin_Helper::post('description', '', FILTER_DEFAULT));
            $event_post->set_description_short(Admin_Helper::post('description_short', '', FILTER_DEFAULT));

            if ($event_post->get_id()) {
                // Empties old data
                $event_post->set_schedules(array());
                $event_post->set_guests(array());
                $event_post->set_organizers(array());
                $event_post->set_attachments(array());
                $event_post->set_products(array());
                $event_post->set_bookable_products(array());
            }

            $schedules = Admin_Helper::post('schedule', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if ($schedules) {
                foreach ($schedules as $schedule) {

                    $schedule_object = new Event_Meta_Schedule();
                    $schedule_object->set_start_date($schedule['schedule_date_from'] . ' ' . $schedule['schedule_time_hours_from'] . ':' . $schedule['schedule_time_minutes_from'] . ':00');
                    $schedule_object->set_title($schedule['schedule_title']);
                    $schedule_object->set_icon($schedule['schedule_icon']);
                    $schedule_object->set_icon_color($schedule['schedule_icon_color']);
                    $schedule_object->set_details($schedule['schedule_details']);

                    $event_post->set_schedule($schedule_object);
                }
            }

            $guests = Admin_Helper::post('guests', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            if ($guests) {
                foreach ($guests as $guest) {

                    if (!isset($guest['name']) || sanitize_text_field($guest['name']) === '') {
                        continue;
                    }

                    $guest_object = new Event_Meta_Guest();
                    $guest_object->set_name($guest['name']);
                    if (isset($guest['photo'])) {
                        $guest_object->set_photo($guest['photo']);
                    }
                    $guest_object->set_links($guest['social']);
                    $guest_object->set_about($guest['about']);
                    $event_post->set_guest($guest_object);
                }
            }

            $event_post->set_organizers(Admin_Helper::post('organizers', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY));

            $attendees = Admin_Helper::post('attendee', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if ($attendees) {

                $attendees_rsvp_date = Admin_Helper::post('attendees_rsvp_date', $start_date, FILTER_DEFAULT);

                foreach ($attendees as &$attendee) {

                    $attendee_object = new Event_Meta_Attendee();

                    if (isset($attendee['email']) && false !== filter_var($attendee['email'], FILTER_VALIDATE_EMAIL)) {
                        $attendee_object->set_email($attendee['email']);
                    }

                    if (isset($attendee['userid'])) {
                        $attendee_object->set_userid($attendee['userid']);
                    }

                    if (isset($attendee['people'])) {
                        $attendee_object->set_people($attendee['people']);
                    }

                    if (isset($attendee['access_token'])) {
                        $attendee_object->set_access_token($attendee['access_token']);
                    }

                    $attendee_object->set_mail_sent(isset($attendee['send']) ? 0 : 1);

                    if (isset($attendee['status'])) {
                        $attendee_object->set_status($attendee['status'], $attendees_rsvp_date);
                    }

                    $attendee = $attendee_object;
                }

                $event_post->update_attendees($attendees);
            } else {

                $event_post->set_attendees(array());
            }

            if (Admin_Helper::isset_post('attendance_visibility')) {
                $event_post->set_attendance_visibility(Admin_Helper::post('attendance_visibility', 'public', FILTER_SANITIZE_STRING));
            }

            $attachments = Admin_Helper::post('attachment', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            if ($attachments) {
                foreach ($attachments as $attachment) {
                    $attachment_object = new Event_Meta_Attachment();
                    $attachment_object->set_id($attachment['id']);
                    $event_post->set_attachment($attachment_object);
                }
            }

            $products = Admin_Helper::post('wc_product', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            if ($products) {
                foreach ($products as $id) {

                    $product_object = new Event_Meta_Product();
                    $product_object->set_id($id);
                    $event_post->set_product($product_object);
                }
            }

            $bookable_products = Admin_Helper::post('wc_bookable_product', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

            if ($bookable_products) {
                foreach ($bookable_products as $id) {
                    $bookable_product_object = new Event_Meta_Bookable_Product();
                    $bookable_product_object->set_id($id);
                    $event_post->set_bookable_product($bookable_product_object);
                }
            }

            $result = $event_post->insert_post();

            if ($result) {

                if ($old_event) {

                    $notify_changes = false;

                    if ($old_event->get_start_date() !== $event_post->get_start_date()) {
                        $notify_changes = true;
                    }

                    if ($old_event->get_location() !== $event_post->get_location()) {
                        $notify_changes = true;
                    }

                    if (true === $notify_changes) {
                        Mail::send_event_changed_mail($old_event, $event_post);
                    }
                }

                if (isset($send_approved_mail) && true === $send_approved_mail) {
                    Mail::send_event_approved_mail($event_post);
                }

                Admin_Helper::set_message(sprintf(esc_html_x('Event %s', 'created/updated', 'stec'), $old_event ? esc_html__('updated', 'stec') : esc_html__('created', 'stec')));
            } else {

                Admin_Helper::set_message(sprintf(esc_html_x('Error %s event', 'creating/updating', 'stec'), $old_event ? esc_html__('updating', 'stec') : esc_html__('creating', 'stec')), 'error');
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'stec_delete_bulk_events' :
        $calendar_id = Admin_Helper::post('calendar_id');

        Admin_Helper::verify_nonce("?page=stec_menu__events&calendar_id={$calendar_id}");

        $events = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        if ($events) {

            try {
                Events::bulk_delete($events);
                Admin_Helper::set_message(esc_html__('Events deleted', 'stec'));
            } catch (Stec_Exception $ex) {
                Admin_Helper::set_message($ex->getMessage(), 'error');
            }
        }

        break;
}