<?php

namespace Stachethemes\Stec;

/**
 * Activation Hook 
 */
register_activation_hook(STACHETHEMES_EC_FILE__, '\Stachethemes\Stec\stec_on_activate');

function stec_on_activate($networkwide) {

    global $wpdb;

    if (function_exists('is_multisite') && is_multisite()) {

        // check if it is a network activation - if so, run the activation function for each blog id

        if ($networkwide) {

            $old_blog = $wpdb->blogid;

            // Get all blog ids
            $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");

            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                stec_on_activate_tables_and_settings();
            }

            switch_to_blog($old_blog);
            return;
        }
    }

    stec_on_activate_tables_and_settings();
}

function stec_on_activate_tables_and_settings() {

    // register plugin settings
    require(dirname(__FILE__) . '/settings/register-settings.php');

    // register cron hooks
    wp_schedule_event(time(), 'hourly', 'stec_cronjobs_hourly');
    wp_schedule_event(time(), 'twicedaily', 'stec_cronjobs_twice_daily');
    wp_schedule_event(time(), 'daily', 'stec_cronjobs_daily');
    wp_schedule_event(time(), 'weekly', 'stec_cronjobs_weekly');


    /**
     * Version 2.5+ Uses new folder structure
     * This code deletes old obsolete files and folders
     * 3.0.0 removes updater folder, users class instead
     */
    $fs_ver = get_option('stec_filesystem', false);
    switch ($fs_ver) {

        case false:

            $stec = Stachethemes_Event_Calendar::get_instance();

            if (!$stec->get_file_system_credentials()) {
                return;
            }

            $fs = $stec->get_file_system();

            $main_path = $stec->get_path('ROOT');
            $paths     = array(
                $main_path . 'admin/',
                $main_path . 'front/',
                $main_path . 'shortcodes/stachethemes_ec_create_form.php',
                $main_path . 'api/',
                $main_path . 'inc/',
                $main_path . 'fonts/',
                $main_path . 'install/',
                $main_path . 'updater/',
                $main_path . 'vc-settings.php',
            );

            foreach ($paths as $path) {
                $fs->delete($path, true);
            }

            update_option('stec_filesystem', 2.5, false);

            break;

        default:
    }

    
}