<?php

namespace Stachethemes\Stec;

/**
 * Un-install hook
 */
register_uninstall_hook(STACHETHEMES_EC_FILE__, '\Stachethemes\Stec\stec_on_uninstall');

function stec_on_uninstall() {

    global $wpdb;

    if (function_exists('is_multisite') && is_multisite()) {

        $old_blog = $wpdb->blogid;

        // Get all blog ids
        $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");

        foreach ($blogids as $blog_id) {
            switch_to_blog($blog_id);
            stec_on_uninstall_tables_and_settings();
        }

        switch_to_blog($old_blog);
        return;
    }

    stec_on_uninstall_tables_and_settings();
}

function stec_on_uninstall_tables_and_settings() {

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'keep_database') === '0') {

        global $wpdb;

        // Delete organizer page
        if (false !== current_user_can('edit_posts')) {

            $pages = get_option('stec-pages', array());
            if ($pages) {
                foreach ($pages as $lang) {
                    foreach ($lang as $page) {
                        $id = filter_var($page, FILTER_VALIDATE_INT);
                        if (false === $id) {
                            continue;
                        }

                        if ('page' === get_post_type((int) $id)) {
                            wp_delete_post((int) $id, true);
                        }
                        unset($id);
                    }
                    unset($page);
                }
                unset($lang);
            }
            unset($pages);
        }

        // Delete custom post types
        $delete_cpt_array = array('stec_event', 'stec_calendar', 'stec_cron');

        foreach ($delete_cpt_array as $post_type) {

            // Delete events
            $sql = $wpdb->prepare("DELETE `POSTS`, `META` 
                FROM {$wpdb->prefix}posts as POSTS
                LEFT JOIN {$wpdb->prefix}postmeta AS META 
                ON META.post_id = POSTS.ID
                WHERE POSTS.post_type = %s ", $post_type);

            $wpdb->query($sql);
        }

        // Delete taxonomies
        $taxonomy_arr = array('stec_event_category', 'stec_event_location', 'stec_event_organizer');
        foreach ($taxonomy_arr as $taxonomy) {

            $sql = $wpdb->prepare("SELECT t.*, tt.* FROM $wpdb->terms AS t "
                    . "INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id "
                    . "WHERE tt.taxonomy IN ('%s') "
                    . "ORDER BY t.name ASC", $taxonomy);

            $terms = $wpdb->get_results($sql);

            // Delete Terms
            if ($terms) {
                foreach ($terms as $term) {
                    $wpdb->delete($wpdb->term_taxonomy, array('term_taxonomy_id' => $term->term_taxonomy_id));
                    $wpdb->delete($wpdb->terms, array('term_id' => $term->term_id));
                    delete_option('prefix_' . $taxonomy->slug . '_option_name');
                }
            }

            // Delete Taxonomy
            $wpdb->delete($wpdb->term_taxonomy, array('taxonomy' => $taxonomy), array('%s'));
        }

        // Delete reminders and builder projects and gdpr emails, and miscs
        delete_option('stec_reminder');
        delete_option('stec_builder_projects');
        delete_option('stec_builder_index');
        delete_option('stec_gdpr_emails');

        delete_option('stec_user_fa_icons'); // delete user custom fa icons
        delete_option('stec_filesystem'); // delete file system version (files structure)
        delete_option("stec_db_ver"); // delete db version
        // Delete settings
        delete_option("stec_menu__license");
        delete_option("stec_menu__general");
        delete_option("stec_menu__general_google_captcha");
        delete_option("stec_menu__general_email_notifications");
        delete_option("stec_menu__general_email_templates");
        delete_option("stec_menu__general_other");
        delete_option("stec_menu__fontsandcolors_top");
        delete_option("stec_menu__fontsandcolors_agenda");
        delete_option("stec_menu__fontsandcolors_monthweek");
        delete_option("stec_menu__fontsandcolors_day");
        delete_option("stec_menu__fontsandcolors_grid");
        delete_option("stec_menu__fontsandcolors_boxgrid");
        delete_option("stec_menu__fontsandcolors_event");
        delete_option("stec_menu__fontsandcolors_preview");
        delete_option("stec_menu__fontsandcolors_tooltip");
        delete_option("stec_menu__fontsandcolors_custom");
        delete_option("stec_menu__cache");
        delete_option("stec-pages");
    }

    delete_option('stec-session'); // remove session msgs if any
    delete_option("stec_activated");
}
