<?php

namespace Stachethemes\Stec;

class Category {

    private $id;
    private $title;
    private $color      = '#000';
    private $notes;
    private $taxonomy   = 'stec_event_category';
    private $author;
    private $count;
    private $visibility = 'public';

    public function __construct($id = null) {

        if ((int) $id) {
            $this->id         = (int) $id;
            $term             = get_term($this->id);
            $this->count      = $term->count;
            $this->title      = $term->name;
            $this->color      = get_term_meta($this->id, 'color', true);
            $this->notes      = get_term_meta($this->id, 'notes', true);
            $this->author     = get_term_meta($this->id, 'author', true);
            $this->visibility = get_term_meta($this->id, 'visibility', true);
        }
    }

    public function get_id() {
        return filter_var($this->id, FILTER_VALIDATE_INT);
    }

    public function get_title() {
        return $this->title;
    }

    public function get_color() {
        return $this->color;
    }

    public function get_notes() {
        return $this->notes;
    }

    public function set_title($title) {
        $this->title = sanitize_text_field($title);
    }

    public function set_color($color) {
        $this->color = sanitize_hex_color($color);
    }

    public function set_notes($notes) {
        $this->notes = sanitize_textarea_field($notes);
    }

    public function get_events_count() {
        if ($this->id) {
            return (int) $this->count;
        }

        return 0;
    }

    public function insert_category() {

        if ($this->id) {
            $term = wp_update_term($this->id, $this->taxonomy, array(
                'name' => $this->title
            ));
        } else {
            $term = wp_insert_term($this->title, $this->taxonomy);
        }

        if (is_wp_error($term)) {
            throw new Stec_Exception($term->get_error_message());
        }

        if (!$this->id) {
            if (!isset($term['term_id'])) {
                throw new Stec_Exception('term_id not found');
            }
            $this->id = $term['term_id'];
        }

        if ($this->id) {
            update_term_meta($this->id, 'notes', $this->notes);
            update_term_meta($this->id, 'color', $this->color);
            update_term_meta($this->id, 'visibility', $this->visibility);
        } else {
            add_term_meta($this->id, 'notes', $this->notes);
            add_term_meta($this->id, 'color', $this->color);
            add_term_meta($this->id, 'visibility', $this->visibility);
        }

        return true;
    }

    public function delete() {
        if ($this->author && $this->author === get_current_user_id() || is_super_admin()) {
            return wp_delete_term($this->id, $this->taxonomy);
        }
    }

    public function get_visibility() {
        return $this->visibility;
    }

    public function set_visibility($visibility) {
        $this->visibility = sanitize_text_field($visibility);
    }

    public function get_author() {
        return (int) $this->author;
    }

}
