<?php

namespace Stachethemes\Stec;

class Event_Meta_Attendee implements Event_Meta_Object {

    private $userid;
    private $email;
    private $name;
    private $people       = 1;
    private $status       = array();
    private $mail_sent    = 0;
    private $access_token = null;
    private $avatar       = null;
    private $custom_data  = array();

    /*
      $status = array(

      'Occurence Date YYYY-MM-DD => (int) 0/1/2',

      )
     */

    public function __construct() {
        $this->access_token = md5(microtime());
    }

    public function get_data() {
        return (object) array(
                    'userid'       => $this->get_userid(),
                    'avatar'       => $this->get_avatar(),
                    'email'        => $this->get_email(),
                    'name'         => $this->get_name(),
                    'people'       => $this->get_people(),
                    'mail_sent'    => $this->get_mail_sent(),
                    'status'       => $this->get_status(),
                    'access_token' => $this->get_access_token(),
        );
    }

    public function get_userid() {
        return (int) $this->userid;
    }

    public function get_email() {
        return filter_var($this->email, FILTER_VALIDATE_EMAIL);
    }

    public function get_people() {
        return (int) $this->people;
    }

    public function get_name() {

        if (!$this->name) {
            if ($this->get_userid()) {
                $userdata = get_userdata($this->get_userid());
                $this->set_name($userdata->display_name);
            } elseif (false !== $this->get_email()) {
                $mailname = explode('@', $this->get_email());
                $this->set_name($mailname[0]);
            }
        }

        return $this->name;
    }

    public function get_status($date = null) {
        if ($date) {
            if (isset($this->status[$date])) {
                return $this->status[$date];
            } else {
                return null;
            }
        }

        return $this->status;
    }

    public function get_mail_sent() {
        return (int) $this->mail_sent;
    }

    public function set_userid($userid) {
        $this->userid = (int) $userid;
    }

    public function set_email($email) {
        $this->email = filter_var($email, FILTER_VALIDATE_EMAIL);
    }

    public function set_name($name) {
        $this->name = sanitize_text_field($name);
    }

    public function set_people($number) {
        $count = filter_var($number, FILTER_VALIDATE_INT, array(
            'default'   => 1,
            'min_range' => 1
        ));

        if ($count > 100) {
            $count = 100;
        }

        $this->people = $count;
    }

    public function set_status($status, $date = null) {

        if ($date) {
            $sanitized_date = sanitize_text_field($date);

            if ($sanitized_date) {
                $this->status[$sanitized_date] = (int) $status;
            }
        } else {
            if (is_array($status)) {
                foreach ($status as &$s) {
                    $s = (int) $s;
                }

                $this->status = $status;
            }
        }
    }

    public function set_mail_sent($mail_sent) {
        $this->mail_sent = (int) $mail_sent;
    }

    public function set_access_token($access_token) {
        $this->access_token = sanitize_text_field($access_token);
    }

    public function get_access_token() {
        return $this->access_token;
    }

    public function get_avatar() {
        if (!$this->avatar) {
            if ($this->userid) {
                $this->set_avatar(get_avatar_url($this->userid));
            } elseif (false !== filter_var($this->email, FILTER_VALIDATE_EMAIL)) {
                $this->set_avatar(get_avatar_url($this->email));
            }
        }
        return apply_filters('stec_attendee_get_avatar', esc_url($this->avatar), $this);
    }

    public function set_avatar($avatar) {
        $this->avatar = filter_var($avatar, FILTER_VALIDATE_URL);
    }

    public function set_custom_data($name, $value) {
        $this->custom_data[$name] = $value;
    }

    public function get_custom_data($name, $default = null) {
        return isset($this->custom_data[$name]) ? $this->custom_data[$name] : $default;
    }

}
