<?php

namespace Stachethemes\Stec;

class Event_Meta_Guest implements Event_Meta_Object {

    private $name;
    private $photo;
    private $links;
    private $about;
    private $photo_full;

    public function get_data() {
        return (object) array(
                    'name'       => $this->get_name(),
                    'photo'      => $this->get_photo(),
                    'photo_full' => $this->get_photo_full(),
                    'links'      => $this->get_links(),
                    'about'      => $this->get_about()
        );
    }

    public function get_name() {
        return $this->name;
    }

    public function get_photo() {
        return filter_var($this->photo, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
    }

    public function get_links() {

        $filtered = array();

        foreach ($this->links as $link) {
            if ($link['link']) {

                $link['ico'] = $link['ico'];

                if (false !== filter_var($link['link'], FILTER_VALIDATE_EMAIL)) {
                    // do nothing
                } elseif (false !== filter_var($link['link'], FILTER_VALIDATE_URL)) {
                    $link['link'] = $link['link'];
                } else {
                    $link['link'] = $link['link'];
                }

                $filtered[] = $link;
            }
        }

        return $filtered;
    }

    public function get_about() {
        return $this->about;
    }

    public function get_photo_full() {

        if (!$this->get_photo()) {
            return null;
        }

        if (!$this->photo_full) {

            if ($this->photo) {
                $photo = wp_get_attachment_image_src($this->photo, 'full');
                $this->set_photo_full($photo[0]);
            }
        }

        return $this->photo_full;
    }

    public function set_photo_full($src) {
        if (false !== filter_var($src, FILTER_VALIDATE_URL)) {
            $this->photo_full = esc_url_raw($src);
        }
    }

    public function set_name($name) {
        $this->name = sanitize_text_field($name);
    }

    public function set_photo($photo) {
        $this->photo = filter_var($photo, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
    }

    public function set_links(array $social_links) {
        $filtered_social_links = array();

        foreach ($social_links as $social) {

            if (!isset($social['link']) || !isset($social['ico'])) {
                continue;
            }

            $value = sanitize_text_field($social['link']);

            if ($value === '') {
                continue;
            }

            $filtered_social_links[] = array(
                'ico'  => sanitize_text_field($social['ico']),
                'link' => $value
            );
        }

        $this->links = $filtered_social_links;
    }

    public function set_about($about) {
        $this->about = sanitize_text_field($about);
    }

}
