<?php

namespace Stachethemes\Stec;

use DateTime;
use DateTimeZone;
use Exception;

class Event_Post extends Stec_Post {

    private $post_type             = 'stec_event';
    private $post_status           = 'publish';
    private $author                = null;
    private $uid                   = null;
    private $recurrence_id         = '';
    private $calid                 = null;
    private $slug                  = null;
    private $title                 = null;
    private $color                 = '#f15e6e';
    private $icon                  = null;
    private $icon_type             = 'icon';
    private $visibility            = 'stec_cal_default';
    private $back_visibility       = 'stec_cal_default';
    private $featured              = 0;
    private $categories            = null;
    private $start_date            = null;
    private $end_date              = null;
    private $all_day               = 0;
    private $hide_end              = 0;
    private $keywords              = '';
    private $counter               = 0;
    private $comments              = 0;
    private $rsvp                  = 0;
    private $rsvp_limit            = 0;
    private $link                  = '';
    private $approved              = 0;
    private $exdate                = null;
    private $rrule                 = '';
    private $is_advanced_rrule     = 0;
    private $location              = null;
    private $description           = '';
    private $description_short     = '';
    private $review_note           = '';
    private $images                = array();
    private $icon_images           = array();
    private $schedule              = array();
    private $guests                = array();
    private $organizers            = array();
    private $attendance            = array();
    private $attendance_visibility = 'public';
    private $attachments           = array();
    private $products              = array();
    private $bookable_products     = array();

    public function __construct($post_id = null) {

        if ((int) $post_id && $this->post_type === get_post_type((int) $post_id)) {
            $this->id = (int) $post_id;
            $this->set_metadata();
        }
    }

    private function set_metadata() {
        $this->post_status       = get_post_status($this->get_id());
        $this->author            = get_post_field('post_author', $this->id);
        $this->title             = get_post_field('post_title', $this->id);
        $this->slug              = get_post_field('post_name', $this->id);
        $this->description       = get_post_field('post_content', $this->id);
        $this->description_short = get_post_field('post_excerpt', $this->id);
        $this->categories        = wp_get_post_terms($this->id, 'stec_event_category', array(
            'fields' => 'ids'
        ));

        $location = wp_get_post_terms($this->id, 'stec_event_location', array(
            'fields' => 'ids'
        ));

        $this->location = $location ? $location[0] : null;

        $organizers = wp_get_post_terms($this->id, 'stec_event_organizer', array(
            'fields' => 'ids'
        ));

        $this->organizers = $organizers ? $organizers : array();

        $metadata = get_metadata('post', $this->id);

        if (is_array($metadata)) {
            foreach ($metadata as $key => $array_value) {
                if (property_exists($this, $key)) {

                    switch ($key) :
                        case 'attendance' :
                        case 'guests' :

                            if (false === is_serialized($array_value[0])) {
                                // Try decrypt
                                try {
                                    $this->$key = maybe_unserialize(Admin_Helper::decrypt($array_value[0]));
                                } catch (Stec_Exception $ex) {
                                    Admin_Helper::debug_log("o.event:: {$key} " . $ex->getMessage());
                                    $this->$key = array();
                                }
                            } else {
                                $this->$key = maybe_unserialize($array_value[0]);
                            }

                            break;
                        default:
                            $this->$key = maybe_unserialize($array_value[0]);

                    endswitch;
                }
            }
        }
    }

    public function get_hide_end() {
        return (int) $this->hide_end;
    }

    public function set_hide_end($hide_end) {
        $this->hide_end = (int) $hide_end;
    }

    public function get_uid() {
        return $this->uid;
    }

    public function get_recurrence_id() {
        return $this->recurrence_id;
    }

    public function get_calid() {
        return filter_var($this->calid, FILTER_VALIDATE_INT, FILTER_NULL_ON_FAILURE);
    }

    public function get_slug() {
        if (!$this->slug && $this->id) {
            return get_post_field('post_name', $this->id);
        }
        return $this->slug;
    }

    public function get_title() {
        return $this->title;
    }

    public function get_color() {
        return $this->color;
    }

    public function get_icon() {
        return $this->icon;
    }

    public function get_icon_type() {
        return $this->icon_type;
    }

    public function get_visibility() {
        return explode(',', $this->visibility);
    }

    public function get_featured() {
        return (int) $this->featured;
    }

    public function get_categories() {
        return array_filter($this->categories, 'is_int');
    }

    public function get_start_date($format = false, $offset = null) {

        if ($format || $offset) {
            return $this->get_date_in_format($this->start_date, $format, $offset);
        }

        return ($this->start_date);
    }

    public function get_end_date($format = false, $offset = null) {

        if ($format || $offset) {
            return $this->get_date_in_format($this->end_date, $format, $offset);
        }

        return ($this->end_date);
    }

    public function get_all_day() {
        return (int) $this->all_day;
    }

    public function get_keywords() {
        return $this->keywords;
    }

    public function get_counter() {
        return (int) $this->counter;
    }

    public function get_comments() {
        return (int) $this->comments;
    }

    public function get_rsvp() {
        return (int) $this->rsvp;
    }

    public function get_rsvp_limit() {
        return (int) $this->rsvp_limit;
    }

    public function set_rsvp_limit($value) {
        $this->rsvp_limit = filter_var($value, FILTER_VALIDATE_INT) ?: 0;
    }

    public function get_link() {
        return $this->link;
    }

    public function get_approved() {
        return (int) $this->approved;
    }

    public function get_exdate() {
        return ($this->exdate);
    }

    public function get_rrule() {
        return ($this->rrule);
    }

    public function get_is_advanced_rrule() {
        return (int) $this->is_advanced_rrule;
    }

    public function get_location() {
        return (int) $this->location;
    }

    public function get_description($no_filters = false) {
        if (true === $no_filters) {
            return Admin_Helper::esc($this->description, 'noscript');
        }
        return wpautop(apply_filters('stec_get_description', Admin_Helper::esc($this->description, 'noscript')));
    }

    public function get_description_short() {
        return $this->description_short;
    }

    public function get_images() {
        if (is_array($this->images)) {
            return array_filter($this->images, 'is_int');
        }

        return array();
    }

    public function get_icon_images() {
        if (is_array($this->icon_images)) {
            return array_filter($this->icon_images, 'is_int');
        }

        return array();
    }

    public function get_parsed_icon_images() {

        $parsed_images = array();

        if (!$this->get_icon_images()) {
            return $parsed_images;
        }

        foreach ($this->get_icon_images() as $id) {
            $attachment = get_post($id);
            $src        = wp_get_attachment_image_src($id, 'thumbnail');
            $thumb      = wp_get_attachment_image_src($id, 'thumbnail');

            $parsed_images[] = (object) array(
                        'alt'         => get_post_meta($attachment->ID, '_wp_attachment_image_alt', true),
                        'caption'     => $attachment->post_excerpt,
                        'description' => $attachment->post_content,
                        'src'         => $src[0],
                        'thumb'       => $thumb[0],
                        'title'       => $attachment->post_title
            );
        }

        return $parsed_images;
    }

    public function get_parsed_images() {

        $parsed_images = array();

        if (!$this->get_images()) {
            return $parsed_images;
        }

        foreach ($this->get_images() as $id) {
            $attachment = get_post($id);
            $src        = wp_get_attachment_image_src($id, 'full');
            $thumb      = wp_get_attachment_image_src($id, 'medium');

            $parsed_images[] = (object) array(
                        'alt'         => get_post_meta($attachment->ID, '_wp_attachment_image_alt', true),
                        'caption'     => $attachment->post_excerpt,
                        'description' => $attachment->post_content,
                        'src'         => $src[0],
                        'thumb'       => $thumb[0],
                        'title'       => $attachment->post_title,
                        'id'          => $id
            );
        }

        return $parsed_images;
    }

    public function get_schedule() {
        return $this->schedule;
    }

    public function get_guests() {

        usort($this->guests, function($a, $b) {
            return strcasecmp($a->get_name(), $b->get_name());
        });

        return $this->guests;
    }

    public function get_organizers() {
        return $this->organizers;
    }

    public function get_attendance($ignore_visibility = false) {

        if (is_super_admin()) {
            $ignore_visibility = true;
        }

        if (true === $ignore_visibility) {
            return $this->attendance;
        }

        switch ($this->attendance_visibility) :
            case 'public' :
                return $this->attendance;

            case 'private' :

                $list   = array();
                $userid = get_current_user_id();

                if ($userid && $this->attendance) {
                    foreach ($this->attendance as $attendee) {
                        if (false === $attendee instanceof Event_Meta_Attendee) {
                            continue;
                        }

                        if ($attendee->get_userid() === $userid) {
                            $list[] = $attendee;
                        } else {
                            $attendee->set_name(false);
                            $attendee->set_email(false);
                            $attendee->set_avatar(false);
                            $attendee->set_access_token(false);
                            $list[] = $attendee;
                        }
                    }
                }

                return $list;

            default:
                return array();
        endswitch;

        return array();
    }

    public function get_attachments() {
        return $this->attachments;
    }

    public function get_parsed_attachments() {

        if (!$this->attachments) {
            return array();
        }

        $parsed_attachments = array();

        foreach ($this->attachments as $attachment) {

            $id = $attachment->get_id();

            $parsed_attachments[] = (object) array(
                        'id'          => $id,
                        'filename'    => basename(get_attached_file($id, true)),
                        'size'        => size_format(filesize(get_attached_file($id))),
                        'link'        => esc_url(wp_get_attachment_url($id)),
                        'description' => Admin_Helper::esc(get_post_field('post_content', $id), 'noscript'),
            );
        }

        return $parsed_attachments;
    }

    public function get_products() {
        return $this->products;
    }

    public function get_bookable_products() {
        return $this->bookable_products;
    }

    public function set_author($author) {
        $this->author = (int) $author;
    }

    public function set_uid($uid = null) {
        if (null === $uid) {
            $uid = uniqid('e-') . '_' . md5(microtime()) . '@stachethemes_ec.com';
        }
        $this->uid = sanitize_text_field($uid);
    }

    public function set_recurrence_id($recurrence_id) {
        if (false !== filter_var($recurrence_id, FILTER_VALIDATE_INT)) {
            $this->recurrence_id = (int) $recurrence_id;
        }
    }

    public function set_calid($calid) {
        if (false === filter_var($calid, FILTER_VALIDATE_INT)) {
            throw new Stec_Exception(esc_html__('Invalid calendar id', 'stec'));
        }
        $this->calid = (int) $calid;
    }

    public function set_slug($slug) {
        $this->slug = sanitize_text_field($slug);
    }

    public function set_title($title) {

        $sanitized_title = sanitize_text_field($title);

        if ($sanitized_title === '') {
            throw new Stec_Exception(esc_html__('Event must have valid title', 'stec'));
        }

        $this->title = $sanitized_title;
    }

    public function set_color($color) {
        $this->color = sanitize_hex_color($color);
    }

    public function set_icon($icon) {
        $this->icon = sanitize_text_field($icon);
    }

    public function set_icon_type($icon_type) {
        $this->icon_type = sanitize_text_field($icon_type);
    }

    public function set_visibility($visibility) {
        if (is_array($visibility)) {
            $visibility = implode(',', $visibility);
        }

        $this->visibility = sanitize_text_field($visibility);
    }

    public function set_featured($featured) {

        if (false === filter_var($featured, FILTER_VALIDATE_INT)) {
            $featured = 0;
        }

        $this->featured = (int) $featured;
    }

    public function set_categories(Array $categories) {
        $this->categories = array_filter($categories, 'is_int');
    }

    public function set_start_date($start_date) {
        try {
            $date             = new \DateTime($start_date, new \DateTimeZone('UTC'));
            $this->start_date = $date->format('Y-m-d H:i:s');
        } catch (\Exception $ex) {
            throw new Stec_Exception(sprintf(esc_html__('Invalid date %s', 'stec'), $start_date));
        }
    }

    public function set_end_date($end_date) {
        try {
            $date           = new \DateTime($end_date, new \DateTimeZone('UTC'));
            $this->end_date = $date->format('Y-m-d H:i:s');
        } catch (\Exception $ex) {
            throw new Stec_Exception(sprintf(esc_html__('Invalid date %s', 'stec'), $end_date));
        }
    }

    public function set_all_day($all_day) {
        $this->all_day = (int) $all_day;
    }

    public function set_keywords($keywords) {
        $this->keywords = sanitize_text_field($keywords);
    }

    public function set_counter($counter) {
        $this->counter = (int) $counter;
    }

    public function get_back_visibility() {
        return explode(',', ($this->back_visibility));
    }

    public function set_back_visibility($back_visibility) {

        if (is_array($back_visibility)) {
            $back_visibility = implode(',', $back_visibility);
        }

        $this->back_visibility = sanitize_text_field($back_visibility);
    }

    public function set_comments($comments) {
        $this->comments = (int) $comments;
    }

    public function set_rsvp($rsvp) {
        $this->rsvp = (int) $rsvp;
    }

    public function set_link($link) {
        if (false !== filter_var($link, FILTER_VALIDATE_URL)) {
            $this->link = esc_url_raw($link);
        } else {
            $this->link = '';
        }
    }

    public function set_approved($approved) {
        $this->approved = (int) $approved;
    }

    public function set_exdate($exdate) {
        $this->exdate = sanitize_text_field($exdate);
    }

    private function get_repaired_rrule($rrule) {

        preg_match('/UNTIL=([0-9]+T?[0-9]+Z?);?/', $rrule, $until);


        if (isset($until) && isset($until[1])) {

            if (strlen($until[1]) > 8) {
                if (strpos($until[1], 'Z') !== false) {
                    // Convert to calendar tz
                    $until_date = new \DateTime($until[1], new \DateTimeZone('UTC'));
                    $until_date->setTimezone(new \DateTimeZone($this->get_timezone()));

                    $rrule = preg_replace('/UNTIL=([0-9]+T?[0-9]+Z?);?/', 'UNTIL=' . $until_date->format('Ymd\THis') . ';', $rrule);
                } else {
                    $until[1] .= 'T000000';
                    $until[1] = substr($until[1], 0, 7);
                    $rrule    = preg_replace('/UNTIL=([0-9]+T?[0-9]+Z?);?/', 'UNTIL=' . $until[1] . ';', $rrule);
                }
            } else {
                $until[1] .= 'T000000';
                $rrule    = preg_replace('/UNTIL=([0-9]+T?[0-9]+Z?);?/', 'UNTIL=' . $until[1] . ';', $rrule);
            }
        }


        return rtrim($rrule, ';');
    }

    public function set_rrule($rrule) {
        $this->rrule = sanitize_text_field($rrule);
    }

    public function set_is_advanced_rrule($is_advanced_rrule) {
        $this->is_advanced_rrule = (int) $is_advanced_rrule;
    }

    public function set_location($location) {
        $this->location = (int) $location;
    }

    public function set_description($description) {
        $this->description = Admin_Helper::esc($description, 'noscript');
    }

    public function set_description_short($description_short) {
        $this->description_short = sanitize_text_field($description_short);
    }

    public function set_icon_images(Array $images) {
        $this->icon_images = array_filter($images, 'is_int');
    }

    public function set_images(Array $images) {
        $this->images = array_filter($images, 'is_int');
    }

    public function set_schedules(Array $schedules) {
        $this->schedule = array();

        foreach ($schedules as $schedule) {
            if ($schedule instanceof Event_Meta_Schedule) {
                $this->set_schedule($schedule);
            }
        }
    }

    public function set_schedule(Event_Meta_Schedule $schedule) {
        $this->schedule[] = $schedule;
    }

    public function set_organizers(Array $organizers) {

        $this->organizers    = array();
        $filtered_organizers = array_filter($organizers, 'is_int');

        foreach ($filtered_organizers as $organizer) {
            $this->set_organizer($organizer);
        }
    }

    public function set_organizer($organizer) {
        if (false !== filter_var($organizer, FILTER_VALIDATE_INT)) {
            $this->organizers[] = (int) $organizer;
        }
    }

    public function set_guests(Array $guests) {
        $this->guests = array();

        foreach ($guests as $guest) {
            if ($guest instanceof Event_Meta_Guest) {
                $this->set_guest($guest);
            }
        }
    }

    public function set_guest(Event_Meta_Guest $guest) {
        $this->guests[] = $guest;
    }

    public function attendee_exists(Event_Meta_Attendee $search_attendee) {
        foreach ($this->attendance as $event_attendee) {
            if (($search_attendee->get_userid() && $search_attendee->get_userid() === $event_attendee->get_userid()) ||
                    $event_attendee->get_email() === $search_attendee->get_email()) {
                return true;
            }
        }

        return false;
    }

    public function set_attendees(Array $attendees) {
        $this->attendance = array();

        foreach ($attendees as $attendee) {
            if ($attendee instanceof Event_Meta_Attendee) {
                $this->set_attendee($attendee);
            }
        }
    }

    public function set_attendee(Event_Meta_Attendee $attendee) {
        $this->attendance[] = $attendee;
    }

    public function set_attachments(Array $attachments) {
        $this->attachments = array();

        foreach ($attachments as $attachment) {
            if ($attachment instanceof Event_Meta_Attachment) {
                $this->set_attachment($attachment);
            }
        }
    }

    public function set_attachment(Event_Meta_Attachment $attachment) {
        $this->attachments[] = $attachment;
    }

    public function set_products(Array $products) {
        $this->products = array();

        foreach ($products as $product) {
            if ($product instanceof Event_Meta_Product) {
                $this->set_product($product);
            }
        }
    }

    public function set_product(Event_Meta_Product $product) {
        $this->products[] = $product;
    }

    public function set_bookable_products(Array $bookable_products) {
        $this->bookable_products = array();

        foreach ($bookable_products as $bookable_product) {
            if ($bookable_product instanceof Event_Meta_Bookable_Product) {
                $this->set_bookable_product($bookable_product);
            }
        }
    }

    public function set_bookable_product(Event_Meta_Bookable_Product $bookable_product) {
        $this->bookable_products[] = $bookable_product;
    }

    public function get_author() {
        return (int) ($this->author ? $this->author : -1);
    }

    public function get_review_note() {
        return esc_html($this->review_note);
    }

    public function set_review_note($review_note) {
        $this->review_note = sanitize_text_field($review_note);
    }

    public function get_post_data() {

        return array(
            'ID'           => $this->id,
            'post_author'  => $this->author ? $this->author : get_current_user_id(),
            'post_content' => $this->description,
            'post_excerpt' => $this->description_short,
            'post_title'   => $this->title,
            'post_name'    => $this->get_slug(),
            'post_type'    => $this->post_type,
            'post_status'  => $this->post_status,
            'meta_input'   => array_merge(array(
                'review_note'           => $this->review_note,
                'uid'                   => $this->uid,
                'recurrence_id'         => $this->recurrence_id,
                'calid'                 => $this->calid,
                'color'                 => $this->color,
                'icon'                  => $this->icon,
                'icon_type'             => $this->icon_type,
                'visibility'            => $this->visibility,
                'back_visibility'       => $this->back_visibility,
                'featured'              => $this->featured,
                'start_date'            => $this->start_date,
                'end_date'              => $this->end_date,
                'all_day'               => $this->all_day,
                'hide_end'              => $this->hide_end,
                'keywords'              => $this->keywords,
                'counter'               => $this->counter,
                'comments'              => $this->comments,
                'rsvp'                  => $this->rsvp,
                'rsvp_limit'            => $this->rsvp_limit,
                'link'                  => $this->link,
                'approved'              => $this->approved,
                'exdate'                => $this->exdate,
                'rrule'                 => $this->rrule,
                'is_advanced_rrule'     => $this->is_advanced_rrule,
                'images'                => $this->images,
                'icon_images'           => $this->icon_images,
                'schedule'              => $this->schedule,
                'guests'                => $this->guests ? Admin_Helper::encrypt($this->guests) : array(),
                'attendance'            => $this->attendance ? Admin_Helper::encrypt($this->attendance) : array(),
                'attendance_visibility' => $this->attendance_visibility ?: 'public',
                'attachments'           => $this->attachments,
                'products'              => $this->products,
                'bookable_products'     => $this->bookable_products,
                    ), $this->custom_meta)
        );
    }

    public function get_timezone_offset() {

        if (!$this->calid) {
            return 0;
        }

        $calendar = new Calendar_Post($this->calid);
        return $calendar->get_timezone_offset();
    }

    public function get_timezone() {

        if (!$this->calid) {
            return 0;
        }

        $calendar = new Calendar_Post($this->calid);
        return $calendar->get_timezone();
    }

    public function get_front_data() {

        $calendar = new Calendar_Post($this->calid);

        return apply_filters('stec_event_get_front_data', array(
            'post_status'           => $this->post_status,
            'id'                    => $this->get_id(),
            'author'                => get_the_author_meta('display_name', $this->get_author()),
            'title'                 => $this->get_title(),
            'slug'                  => $this->get_slug(),
            'description'           => $this->get_description(),
            'description_short'     => $this->get_description_short(),
            'uid'                   => $this->get_uid(),
            'recurrence_id'         => $this->get_recurrence_id(),
            'calid'                 => $this->get_calid(),
            'color'                 => $this->get_color(),
            'icon'                  => $this->get_icon(),
            'icon_type'             => $this->get_icon_type(),
            'visibility'            => $this->get_visibility(),
            'back_visibility'       => $this->get_back_visibility(),
            'featured'              => $this->get_featured(),
            'start_date'            => $this->get_start_date(),
            'end_date'              => $this->get_end_date(),
            'all_day'               => $this->get_all_day(),
            'hide_end'              => $this->get_hide_end(),
            'keywords'              => $this->get_keywords(),
            'counter'               => $this->get_counter(),
            'comments'              => $this->get_comments(),
            'rsvp'                  => $this->get_rsvp(),
            'link'                  => $this->get_link(),
            'approved'              => $this->get_approved(),
            'exdate'                => $this->get_exdate(),
            'rrule'                 => $this->get_rrule(),
            'is_advanced_rrule'     => $this->get_is_advanced_rrule(),
            'location'              => $this->get_parsed_location(),
            'images'                => $this->get_images(),
            'images_meta'           => $this->get_parsed_images(),
            'icon_images'           => $this->get_icon_images(),
            'icon_images_meta'      => $this->get_parsed_icon_images(),
            'schedule'              => $this->get_parsed_schedule(),
            'guests'                => $this->get_parsed_guests(),
            'organizers'            => $this->get_parsed_organizers(),
            'attendance'            => $this->get_parsed_attendees(),
            'attendance_visibility' => $this->get_attendance_visibility(),
            'rsvp_limit'            => $this->get_rsvp_limit(),
            'attachments'           => $this->get_parsed_attachments(),
            'products'              => $this->get_parsed_products(),
            'bookable_products'     => $this->get_parsed_bookable_products(),
            'timezone_utc_offset'   => $this->get_timezone_offset(),
            'timezone'              => $this->get_timezone(),
            'permalink'             => $this->get_permalink(),
            'calendar'              => $calendar->get_front_data(),
            'category'              => $this->get_parsed_categories(),
            'edit_link'             => $this->get_edit_link()
        ));
    }

    public function get_parsed_schedule() {
        $parsed_schedules = array();

        if (is_array($this->schedule)) {
            foreach ($this->schedule as $schedule) {
                $parsed_schedules[] = $schedule->get_data();
            }
        }

        return $parsed_schedules;
    }

    public function get_parsed_categories() {

        if (!$this->categories) {
            return array();
        }

        $parsed_categories = array();

        foreach ($this->categories as $id) {
            $cat                 = new Category($id);
            $parsed_categories[] = array(
                'id'    => $cat->get_id(),
                'title' => $cat->get_title(),
                'color' => $cat->get_color()
            );
        }

        $sort = array();

        foreach ($parsed_categories as $k => $category) {
            $sort[$k] = $category['title'];
        }

        array_multisort($sort, SORT_DESC, $parsed_categories);

        return $parsed_categories;
    }

    public function get_parsed_location() {

        if (!$this->location || is_nan($this->location)) {
            return array();
        }

        $location = new Location($this->location);

        return $location->get_data();
    }

    /**
     * Returns guests with processed images
     * @return object array
     */
    public function get_parsed_guests() {

        $parsed_guests = array();

        if (is_array($this->guests)) {
            foreach ($this->get_guests() as $guest) {
                $parsed_guests[] = $guest->get_data();
            }
        }

        return $parsed_guests;
    }

    /**
     * Returns organizers with processed images
     * strips email
     * @return array
     */
    public function get_parsed_organizers($include_secret = false) {
        if (!$this->organizers) {
            return array();
        }

        $parsed = array();

        foreach ($this->organizers as $id) {
            $organizer = new Organizer($id);
            $data      = $organizer->get_data();
            if (true !== $include_secret) {
                unset($data->email);
            }
            $parsed[] = $data;
        }

        usort($parsed, function($a, $b) {
            return strcasecmp($a->name, $b->name);
        });

        return $parsed;
    }

    /**
     * Returns attendees with processed data
     * @return object array
     */
    public function get_parsed_attendees($include_secret = false, $ignore_visibility = false) {

        $attendees = $this->get_attendance($ignore_visibility);

        if (!$attendees) {
            return array();
        }

        $parsed = array();

        foreach ($attendees as $attendee) {

            if (false === ($attendee instanceof Event_Meta_Attendee)) {
                continue;
            }

            $data = $attendee->get_data();

            if (false === $include_secret) {

                unset($data->access_token);

                if (get_current_user_id() != $data->userid) {
                    unset($data->email);
                    unset($data->id);
                }
            }

            $parsed[] = $data;
        }

        return $parsed;
    }

    public function set_attendee_status($status = 0, $params = array(), $instant = true) {

        foreach ($this->attendance as &$attendee) {

            if (isset($params['userid']) && $attendee->get_userid() === $params['userid']) {

                $attendee->set_status($status, $params['date']);

                if (isset($params['people'])) {
                    $attendee->set_people($params['people']);
                }

                if (true === $instant) {
                    update_post_meta($this->id, 'attendance', '');
                    update_post_meta($this->id, 'attendance', Admin_Helper::encrypt($this->attendance));
                }

                return true;
            } elseif (isset($params['email']) && $attendee->get_email() === $params['email']) {

                $attendee->set_status($status, $params['date']);

                if (isset($params['people'])) {
                    $attendee->set_people($params['people']);
                }

                if (true === $instant) {
                    update_post_meta($this->id, 'attendance', '');
                    update_post_meta($this->id, 'attendance', Admin_Helper::encrypt($this->attendance));
                }

                return true;
            }
        }

        return false;
    }

    /**
     * Updates or Inserts attendees
     * @param Array $attendees 
     */
    public function update_attendees($attendees) {

        if ($this->attendance) :

            // Retrieve old attendees
            $old_attendees = $this->attendance;

            // Clear event attendees array 
            $this->attendance = array();

            // Loop new attendees and search if it was in the old database
            // Note: Loop by reference!
            foreach ($attendees as &$new_attendee) :
                $found = false;

                // Search by email or userid
                foreach ($old_attendees as $old_attendee) :

                    if (false === ($old_attendee instanceof Event_Meta_Attendee) || false === ($new_attendee instanceof Event_Meta_Attendee)) {
                        continue;
                    }

                    if ($old_attendee->get_email() === $new_attendee->get_email() ||
                            ($old_attendee->get_userid() && $old_attendee->get_userid() === $new_attendee->get_userid())) :

                        $found = $old_attendee;

                        break;
                    endif;
                endforeach;

                // If found merge statuses
                if ($found) :
                    $new_attendee->set_status(array_merge($old_attendee->get_status(), $new_attendee->get_status()));
                endif;

            endforeach;

        endif;

        $this->set_attendees($attendees);
    }

    /**
     * Get event edit link
     * @return string | false
     */
    public function get_edit_link() {

        if (!$this->id) {
            return null;
        }

        if (false === Events::user_can_edit_event($this)) {
            return null;
        }

        $edit_link = Admin_Helper::get_permalinks('edit', 'stec_edit');
        $slug      = $this->get_slug();

        return esc_url(home_url($edit_link . '/' . $slug));
    }

    /**
     * Get event permalink with repeat offset if any
     * @param int $repeat_offset
     * @return false|string
     */
    public function get_permalink($repeat_offset = 0) {

        if (!$this->id) {
            return false;
        }

        $permalink = get_the_permalink($this->id);

        if ($repeat_offset && $repeat_offset > 0) {
            $arg_pos = strpos($permalink, '?');

            if (false !== $arg_pos) {
                $permalink = substr_replace($permalink, $repeat_offset . '/', $arg_pos, 0);
            } else {
                $permalink = $permalink . $repeat_offset;
            }
        }

        return esc_url($permalink);
    }

    /**
     * Create google calendar add event link with repeat offset if any
     * @param int $repeat_offset
     * @return string google cal link
     */
    public function get_gcal_link($repeat_offset = 0) {

        // Get real start date with repeat offset
        $start_date = date('Ymd\THis', strtotime($this->start_date) + $repeat_offset);
        $start      = null;

        if ((int) $this->get_all_day() === 1) {
            // Since it's date specific don't UTC the date
            $start = date('Ymd', strtotime($start_date));
        } else {
            $start = Admin_Helper::to_utc_time($start_date, $this->get_timezone()) . 'Z';
        }

        // Get real end date with repeat offset
        $end_date = date('Ymd\THis', strtotime($this->end_date) + $repeat_offset);
        $end      = null;

        if ((int) $this->get_all_day() === 1) {
            // Since it's date specific don't UTC the date
            $end = date('Ymd', strtotime($end_date) + 24 * 3600);
        } else {
            $end = Admin_Helper::to_utc_time($end_date, $this->get_timezone()) . 'Z';
        }

        $location = '';
        if ($this->location) {
            $location = $this->get_parsed_location();
            $location = $location->full_address;
        }

        return esc_url("//calendar.google.com/calendar/render?action=TEMPLATE&amp;text={$this->title}&amp;dates=" . $start . "/" . $end . "&amp;details={$this->description_short}&amp;location={$location}&amp;sf=true&amp;output=xml");
    }

    public function insert_post() {

        if ($this->id) {
            if (!Events::user_can_edit_event($this)) {
                throw new Stec_Exception(esc_html__("You don't have permission to edit this event", 'stec'));
            }
        }

        if (!$this->uid) {
            $this->set_uid();
        }

        $result = parent::insert_post();

        if (is_wp_error($result)) {
            throw new Stec_Exception($result->get_error_message());
        } else {
            $this->id = $result;
        }

        // Set featured image if any
        $images = $this->get_images();

        if ($images) {
            set_post_thumbnail($this->id, $images[0]);
        } else {
            delete_post_thumbnail($this->id);
        }

        if ($this->categories) {

            $add_to_category = wp_set_object_terms($this->id, $this->categories, 'stec_event_category');

            if (is_wp_error($add_to_category)) {
                throw new Stec_Exception($result->get_error_message());
            }
        } else {
            wp_delete_object_term_relationships($this->id, 'stec_event_category');
        }

        if ($this->location) {

            $add_location = wp_set_object_terms($this->id, $this->location, 'stec_event_location');

            if (is_wp_error($add_location)) {
                throw new Stec_Exception($result->get_error_message());
            }
        } else {
            wp_delete_object_term_relationships($this->id, 'stec_event_location');
        }

        if ($this->organizers) {
            $add_organizers = wp_set_object_terms($this->id, array_map('intval', $this->organizers), 'stec_event_organizer');

            if (is_wp_error($add_organizers)) {
                throw new Stec_Exception($result->get_error_message());
            }
        } else {
            wp_delete_object_term_relationships($this->id, 'stec_event_organizer');
        }

        Mail::send_event_invitations_mail($this);

        return $result;
    }

    public function delete_post() {

        if (true !== Events::user_can_edit_event($this)) {
            throw new Stec_Exception("You don't have permission to delete this event", "stec");
        }

        if (0 === (int) $this->approved) {
            Mail::send_event_rejected_mail($this);
        }

        if (1 === (int) $this->approved) {
            Mail::send_event_cancelled_mail($this);
        }

        parent::delete_post();

        return true;
    }

    public function get_parsed_bookable_products() {
        $bookable_products = array();

        if (!class_exists('WooCommerce') || !$this->bookable_products) {
            return $bookable_products;
        }

        $current_user_roles = Admin_Helper::get_user_roles();

        foreach ($this->bookable_products as $k => $bookable_product) {

            try {
                $wc_product = new \WC_Product_Stec_Bookable($bookable_product->get_id());

                $requires_role = $wc_product->get_meta('stec_requires_role');
                if (is_array($requires_role) && $requires_role) {
                    if (false === (bool) array_intersect($current_user_roles, $requires_role)) {
                        continue;
                    }
                }
            } catch (Exception $ex) {
                Admin_Helper::debug_log($ex->getMessage());
                continue;
            }

            $attributes = $wc_product->get_attributes();

            $attrib_array = array();

            foreach ($attributes as $attrib) {
                if (!$attrib instanceof \WC_Product_Attribute) {
                    continue;
                }

                if (false === $attrib->get_visible()) {
                    continue;
                }

                $attrib_array[] = array(
                    'name'  => $attrib->get_name(),
                    'value' => implode('|', $attrib->get_options())
                );
            }

            $bookable_products[$k]                      = new \stdClass();
            $bookable_products[$k]->id                  = $wc_product->get_id();
            $bookable_products[$k]->sku                 = $wc_product->get_sku();
            $bookable_products[$k]->url                 = $wc_product->get_permalink();
            $bookable_products[$k]->is_purchesable      = $wc_product->is_purchasable();
            $bookable_products[$k]->is_in_stock         = $wc_product->is_in_stock();
            $bookable_products[$k]->stock_quantity      = $wc_product->get_stock_quantity();
            $bookable_products[$k]->qty_limit_per_event = $wc_product->get_quantity_limit_per_event();
            $bookable_products[$k]->html_price          = $wc_product->get_price_html();
            $bookable_products[$k]->price               = $wc_product->get_price();
            $bookable_products[$k]->title               = $wc_product->get_title();
            $bookable_products[$k]->short_description   = $wc_product->get_short_description();
            $bookable_products[$k]->stop_before         = $wc_product->get_stop_before();
            $bookable_products[$k]->attributes          = $attrib_array;
            $bookable_products[$k]->tickets_sold        = Booking::get_tickets_sold($this, $wc_product->get_id(), -1);
        }


        return $bookable_products;
    }

    public function get_parsed_products() {
        $products = array();

        if (!class_exists('WooCommerce') || !$this->products) {
            return $products;
        }

        foreach ($this->products as $k => $product) {

            $wc_product = WC()->product_factory->get_product($product->get_id());

            if (!$wc_product) {
                continue;
            }

            $wc_product_post_date = get_post($wc_product->get_id());

            $products[$k]                 = new \stdClass();
            $products[$k]->id             = $wc_product->get_id();
            $products[$k]->sku            = $wc_product->get_sku();
            $products[$k]->url            = $wc_product->get_permalink();
            $products[$k]->purchesable    = $wc_product->is_purchasable();
            $products[$k]->is_in_stock    = $wc_product->is_in_stock();
            $products[$k]->stock_quantity = $wc_product->get_stock_quantity();
            $products[$k]->is_featured    = $wc_product->is_featured();
            $products[$k]->is_on_sale     = $wc_product->is_on_sale();
            $products[$k]->has_child      = $wc_product->has_child();
            $products[$k]->image          = $wc_product->get_image();
            $products[$k]->html_price     = $wc_product->get_price_html();
            $products[$k]->price          = $wc_product->get_price();
            $products[$k]->title          = $wc_product->get_title();
            $products[$k]->post_data      = array(
                'excerpt' => $wc_product_post_date->post_excerpt
            );
        }


        return $products;
    }

    /**
     * Get event occurrences between two dates 
     * 
     * Dates returned are in calendar timezone
     * The returned array includes the event initial start date if in scope
     * Skips exdates
     * 
     * @param string $range_start Y-m-d
     * @param string $range_end Y-m-d
     * @return array 
     */
    public function get_occurrences($range_start, $range_end) {

        if (!$this->rrule) {
            return array();
        }

        $dtrange_start = new DateTime($range_start, new DateTimeZone('UTC'));
        $dtrange_end   = new DateTime($range_end, new DateTimeZone('UTC'));
        $dtstart       = new DateTime($this->start_date, new DateTimeZone('UTC'));
        $rrule_string  = $this->rrule;

        try {
            $rrule = new \RRule\RRule($rrule_string, $dtstart->format('Y-m-d H:i:s'));
        } catch (Exception $ex) {
            $rrule_string = $this->get_repaired_rrule($rrule_string);
            $rrule        = new \RRule\RRule($rrule_string, $dtstart->format('Y-m-d H:i:s'));
        }

        $dates   = $rrule->getOccurrencesBetween($range_start, $range_end);
        $exdates = explode(',', $this->exdate);

        $processed_dates = array();

        if ($dtstart >= $dtrange_start && $dtstart <= $dtrange_end) {

            if (!in_array($dtstart->format('Ymd'), $exdates)) {
                array_push($processed_dates, array(
                    'start_date'    => $this->start_date,
                    'repeat_offset' => 0,
                ));
            }
        }

        foreach ($dates as $date) {

            if (in_array($date->format('Ymd'), $exdates)) {
                continue;
            }

            array_push($processed_dates, array(
                'start_date'    => $date->format('Y-m-d H:i:s'),
                'repeat_offset' => absint($dtstart->format('U') - $date->format('U'))
            ));
        }

        return $processed_dates;
    }

    private function get_date_in_format($the_date = 'now', $format = 'Y-m-d H:i:s', $offset = null) {

        $date = new DateTime($the_date, new DateTimeZone($this->get_timezone()));

        if ($offset) {
            $date->add(new \DateInterval('PT' . $offset . 'S'));
        }

        return $date->format($format);
    }

    /**
     * Checks if the repeat_offset parameter is valid repeat offset
     * @param int $offset 
     * @return boolean true|false on invalid
     */
    public function is_valid_offset($offset) {

        if (false === filter_var($offset, FILTER_VALIDATE_INT)) {
            return false;
        }

        $offset = (int) $offset;

        if ($offset < 0) {
            return false;
        }

        if ($offset === 0) {
            return true;
        }

        if ($this->rrule === '') {
            return false;
        }

        try {
            $range_start = $this->get_start_date('Y-m-d', $offset - 3600 * 24);
            $range_end   = $this->get_start_date('Y-m-d', $offset + 3600 * 24);
        } catch (Exception $ex) {
            return false;
        }

        $occurences = $this->get_occurrences($range_start, $range_end);

        if ($occurences && is_array($occurences)) {

            $event_start_date = $this->get_start_date('Y-m-d H:i:s', $offset);

            foreach ($occurences as $occur) {
                if ($occur['start_date'] === $event_start_date) {
                    return true;
                }
            }
        }

        return false;
    }

    public function get_attendance_visibility() {
        return $this->attendance_visibility;
    }

    public function set_attendance_visibility($attendance_visibility) {
        $accept                      = array('public', 'private');
        $this->attendance_visibility = in_array($attendance_visibility, $accept) ? $attendance_visibility : 'public';
    }

}
