<?php

namespace Stachethemes\Stec;

add_shortcode('stachethemes_ec', function($atts = array()) {

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    ob_start();

    if (Admin_Helper::is_placeholder()) {
        include ($stachethemes_ec_main->get_path('VIEW') . 'front/placeholder/stachethemes_ec.php');
        return ob_get_clean();
    }

    if (true === Admin_Helper::check_migration_required()) {
        printf(esc_html__('<p>Stachethemes Event Calendar requires database update. To update go to %s</p>', 'stec'), '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__migrate')) . '">Dashboard -> Stachethemes Event Calendar -> Migrate</a>');
        return;
    }

    $atts               = apply_filters('stachethemes_ec_atts', $atts);
    $calendar           = new Calendar_Instance($atts);
    $writable_cal_array = isset($atts['create_form_cal']) ? explode(',', $atts['create_form_cal']) : array();

    // Before html hook
    do_action('stec_before_html');

    if ($stachethemes_ec_main->lcns() !== false) {
        include ($stachethemes_ec_main->get_path('VIEW') . 'front/default.php');
    } else {
        printf('<p>' . esc_html__('Stachethemes Event Calendar is not activated. To activate it go to %s', 'stec') . '</p>',
                '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__license')) . '">Dashboard -> Stachethemes Event Calendar -> Activator</a>');
    }

    // After html hook
    do_action('stec_after_html');

    // If ajax events is disabled add events to js var
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'ajax_events') === '0') {
        add_action('wp_footer', function() use ($calendar, $atts) {

            $events       = array();
            $cat          = isset($atts['cat']) ? $atts['cat'] : null;
            $organizer_id = isset($atts['organizer_id']) ? $atts['organizer_id'] : null;
            $cal          = isset($atts['cal']) ? $atts['cal'] : null;
            $min_date     = isset($atts['min_date']) ? $atts['min_date'] : null;
            $max_date     = isset($atts['max_date']) ? $atts['max_date'] : null;

            if ($cal) {
                $cal = explode(',', $cal);
            }

            $meta_query = array();
            $tax_query  = array();

            if ($min_date || $max_date) {

                $meta_query[] = array(
                    'relation' => 'OR',
                    array(
                        'key'     => 'start_date',
                        'value'   => array(($min_date ? $min_date : '1970-01-01'), ($max_date ? $max_date : '2038-01-19')),
                        'compare' => 'BETWEEN',
                        'type'    => 'DATE'
                    ),
                    array(
                        'key'     => 'rrule',
                        'value'   => '',
                        'compare' => '!=',
                    )
                );
            }

            if (!is_user_logged_in()) {
                $meta_query[] = array(
                    'relation' => 'OR',
                    array(
                        'key'     => 'approved',
                        'value'   => '1',
                        'compare' => '='
                    ),
                    array(
                        array(
                            'key'     => 'approved',
                            'value'   => '0',
                            'compare' => '='
                        ),
                        array(
                            'key'     => 'created_by_cookie',
                            'value'   => Admin_Helper::get_user_created_by_cookie(),
                            'compare' => '='
                        )
                    )
                );
            }

            if ($cat) {
                $cat         = explode(',', $cat);
                $tax_query[] = array(
                    'taxonomy' => 'stec_event_category',
                    'field'    => 'term_id',
                    'terms'    => $cat,
                );
            }

            if ($organizer_id) {
                $organizer_id_array = explode(',', $organizer_id);
                $tax_query[]        = array(
                    'taxonomy' => 'stec_event_organizer',
                    'field'    => 'term_id',
                    'terms'    => $organizer_id_array,
                );
            }

            $cache = Admin_Helper::get_cache(array($cal, $meta_query, $tax_query, get_current_user_id()));

            if ($cache) {

                $events = $cache;
            } else {

                $events = Events::get_front_events(array(
                            'calid'      => $cal,
                            'meta_query' => $meta_query,
                            'tax_query'  => $tax_query
                ));
                foreach ($events as $key => $event) {
                    $events[$key] = $event->get_front_data();
                }

                Admin_Helper::set_cache(array($cal, $meta_query, $tax_query, get_current_user_id()), $events);
            }
            ?>
            <script type="text/javascript">
                if (typeof stecJsonEvents === 'undefined') {
                    var stecJsonEvents = [];
                }
                stecJsonEvents['<?php esc_attr_e($calendar); ?>'] = <?php echo wp_json_encode($events); ?>;
            </script>
            <?php
        }, 9999);
    }

    return ob_get_clean();
});

add_action('wp_enqueue_scripts', function() {

    global $post;

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    if (Admin_Helper::is_placeholder()) {
        $stachethemes_ec_main->add_asset_css("stec-shortcode-placeholder", "css/front/placeholder.css");
        return;
    }

    if ($stachethemes_ec_main->scripts_are_forced() === true || (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'stachethemes_ec'))) {

        do_action('stec_load_calendar_scripts');

        if (Settings::get_admin_setting_value('stec_menu__cache', 'ac') === '1') {
            AssetsCompressor::compress_js();
            AssetsCompressor::compress_css();
        }
    }
});

add_action('stec_load_calendar_scripts', function() {

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        $stachethemes_ec_main->add_asset_css("stec-ui", "https://ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css");
    }

    $stachethemes_ec_main->add_asset_css("stec-css", "css/front/style.css");
    $stachethemes_ec_main->add_asset_css("stec-builder-elements", "css/front/builder/main.css");
    $stachethemes_ec_main->add_asset_css("stec-colorpicker", "css/colorpicker/css/colorpicker.css");

    // calendar default fonts
    $stachethemes_ec_main->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");

    if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-google-captcha-api", "//www.google.com/recaptcha/api.js?render=explicit async defer");
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_gmaps') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-google-maps", "//maps.googleapis.com/maps/api/js?key=" . Settings::get_admin_setting_value('stec_menu__general', 'gmaps_api_key'));
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-jquery-mobile-js", "js/libs/jquery.mobile.js", "jquery");
    }

    $stachethemes_ec_main->add_asset_js("stec-touchpunch", "js/libs/touch-punch.js", "jquery,jquery-ui-draggable");

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_chart_js') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-chart", "js/libs/chart-2.8.0.min.js", "jquery");
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-moment-js", "js/libs/moment.js");
        $stachethemes_ec_main->add_asset_js("stec-moment-tz-js", "js/libs/moment-timezone.js");
    }

    $stachethemes_ec_main->add_asset_js("stec-rrule-js", "js/libs/rrule.js");
    $stachethemes_ec_main->add_asset_js("stec-nlp-js", "js/libs/nlp.js", "stec-rrule-js");
    $stachethemes_ec_main->add_asset_js("stec-animate-js", "js/front/animate.js", "jquery");
    $stachethemes_ec_main->add_asset_js("stec-js", "js/front/stec.js", "jquery,jquery-ui-draggable,stec-rrule-js");
    $stachethemes_ec_main->add_asset_js("stec-helper-js", "js/front/helper.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-calData-js", "js/front/caldata.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-db-js", "js/db.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-reminder-js", "js/front/reminder.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-rsvp-js", "js/front/rsvp.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-scroll-js", "js/libs/scroll.js", "stec-helper-js,jquery-ui-draggable");

    /**
     * Add Inner tabs javascript
     */
    $stachethemes_ec_main->add_asset_js("stec-extend", "js/front/stec-extend.js", "stec-js");
    $stachethemes_ec_main->add_asset_js("stec-search-js", "js/front/adds/search.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-top-calfilter-js", "js/front/adds/top.calfilter.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-tooltip-js", "js/front/adds/tooltip.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-media-js", "js/front/adds/media.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-intro-js", "js/front/adds/intro.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-location-js", "js/front/adds/location.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-schedule-js", "js/front/adds/schedule.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-guests-js", "js/front/adds/guests.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-attendance-js", "js/front/adds/attendance.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-comments-js", "js/front/adds/comments.js", "stec-extend");
    $stachethemes_ec_main->add_asset_js("stec-forecast-js", "js/front/adds/forecast.js", "stec-extend");

    // if woocommerce is running
    if (class_exists('WooCommerce')) {
        $stachethemes_ec_main->add_asset_js("stec-shop-js", "js/front/adds/shop.js", "stec-extend");
        $stachethemes_ec_main->add_asset_js("stec-booking-js", "js/front/booking.js", "stec-extend");
    }

    // Front Event Creation
    $stachethemes_ec_main->add_asset_js("stec-colorpicker-js", "js/libs/colorpicker/colorpicker.js", 'jquery');
    $stachethemes_ec_main->add_asset_js("stec-event-create-js", "js/front/adds/event.create.js", "stec-extend,jquery-ui-datepicker");
    $stachethemes_ec_main->add_asset_js("stec-builder-submission-js", "js/front/builder/submission.js", "stec-extend");
    // Add front-end wp media
    wp_enqueue_media();
    wp_enqueue_editor();

    // Share and embeding
    $stachethemes_ec_main->add_asset_js("stec-sharer-js", "js/front/share.js");

    // Add DB nonce
    $stachethemes_ec_main->localize('stec-db-js', 'stecApiNonce', array(
        'nonce' => wp_create_nonce('stecApiVerifyNonce')
    ));

    $stachethemes_ec_main->load_js_locales();

    add_action('wp_head', function() use ($stachethemes_ec_main) {
        $stachethemes_ec_main->load_head();
    }, 9999);
});
