<?php

namespace Stachethemes\Stec;

add_shortcode('stec_builder', function($atts) {

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    ob_start();

    if (Admin_Helper::is_placeholder()) {
        include ($stachethemes_ec_main->get_path('VIEW') . 'front/placeholder/stec_builder.php');
        return ob_get_clean();
    }

    if (true === Admin_Helper::check_migration_required()) {
        printf(esc_html__('<p>Stachethemes Event Calendar requires database update. To update go to %s</p>', 'stec'), '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__migrate')) . '">Dashboard -> Stachethemes Event Calendar -> Migrate</a>');
        return;
    }

    if (!$atts) {
        $atts = array();
    }

    $id         = uniqid('stec-builder-form-');
    $atts['id'] = $id;

    if (isset($atts['project_type'])) {
        switch ($atts['project_type']) {
            case 'event_submission_form' :
                if (!isset($atts['project_id']) || !$atts['project_id']) {
                    $atts['project_id'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');
                }
                break;
        }
    }

    $atts['captcha']['enabled']                    = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled');
    $atts['captcha']['site_key']                   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'site_key');
    $atts['create_event_form_allow_expired']       = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_expired');
    $atts['create_event_form_allow_location_type'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');
    $atts['date_format']                           = Settings::get_admin_setting_value('stec_menu__general', 'date_format');
    $json                                          = wp_json_encode($atts);
    ?>

    <script type="text/javascript">
        if (typeof stec_builder_form_instance === "undefined") {
            var stec_builder_form_instance = [];
        }
        stec_builder_form_instance.push(<?php echo Admin_Helper::esc($json, 'json'); ?>);
    </script>

    <?php
    // Before html hook
    do_action('stec_builder_form_before_html');

    if ($stachethemes_ec_main->lcns() !== false) {
        include ($stachethemes_ec_main->get_path('VIEW') . 'front/builder/index.php');
    } else {
        printf('<p>' . esc_html__('Stachethemes Event Calendar is not activated. To activate it go to %s', 'stec') . '</p>',
                '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__license')) . '">Dashboard -> Stachethemes Event Calendar -> Activator</a>');
    }

    // After html hook
    do_action('stec_builder_form_after_html');

    return ob_get_clean();
});

add_action('wp_enqueue_scripts', function() {

    global $post;

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    if (Admin_Helper::is_placeholder()) {
        return;
    }

    if ($stachethemes_ec_main->scripts_are_forced() === true ||
            (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'stec_builder'))) {

        do_action('stec_load_builder_scripts');

        if (Settings::get_admin_setting_value('stec_menu__cache', 'ac') === '1') {
            AssetsCompressor::compress_js();
            AssetsCompressor::compress_css();
        }
    }
});


add_action('stec_load_builder_scripts', function() {

    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        $stachethemes_ec_main->add_asset_css("stec-ui", "//ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css");
    }

    $stachethemes_ec_main->add_asset_css("stec-builder-elements", "css/front/builder/main.css");
    $stachethemes_ec_main->add_asset_css("stec-colorpicker", "css/colorpicker/css/colorpicker.css");

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        $stachethemes_ec_main->add_asset_css("stec-ui", "//ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css");
    }

    // calendar default fonts
    $stachethemes_ec_main->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-moment-js", "js/libs/moment.js");
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-jquery-mobile-js", "js/libs/jquery.mobile.js", "jquery");
    }

    $stachethemes_ec_main->add_asset_js("stec-colorpicker-js", "js/libs/colorpicker/colorpicker.js", 'jquery');
    $stachethemes_ec_main->add_asset_js("stec-builder-js", "js/front/builder/main.js", "jquery");
    $stachethemes_ec_main->add_asset_js("stec-helper-js", "js/front/helper.js", "stec-builder-js");
    $stachethemes_ec_main->add_asset_js("stec-rrule-js", "js/libs/rrule.js");
    $stachethemes_ec_main->add_asset_js("stec-nlp-js", "js/libs/nlp.js", "stec-rrule-js");
    $stachethemes_ec_main->add_asset_js("stec-db-js", "js/db.js");

    if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
        $stachethemes_ec_main->add_asset_js("stec-google-captcha-api", "https://www.google.com/recaptcha/api.js?render=explicit async defer");
    }

    $stachethemes_ec_main->add_asset_js("stec-builder-submission-js", "js/front/builder/submission.js", "jquery,jquery-ui-datepicker");

    wp_enqueue_editor();

    // Add DB nonce
    $stachethemes_ec_main->localize('stec-db-js', 'stecApiNonce', array(
        'nonce' => wp_create_nonce('stecApiVerifyNonce')
    ));

    // Add front-end wp media
    wp_enqueue_media();

    $stachethemes_ec_main->load_js_locales();

    add_action('wp_head', function() use ($stachethemes_ec_main) {
        $stachethemes_ec_main->load_head();
    }, 9999);
});
