<?php
/*
  Plugin Name: Stachethemes Event Calendar
  Version: 3.0.1
  Description: Stachethemes Event Calendar
  Author: Stachethemes
  Author URI: //www.stachethemes.com/
  License: GNU General Public License 2.0 (GPL) //www.gnu.org/licenses/old-licenses/gpl-2.0.html
  Text Domain: stec
  Domain Path: /languages/
 */

namespace Stachethemes\Stec;

// Do not load directly
if (!defined('ABSPATH')) {
    wp_die('-1');
}

// Define constants
const STACHETHEMES_EC_FILE__ = __FILE__;
const STEC_EXEC_TIME_LIMIT   = 0;
const STEC_PLUGIN_VERSION    = '3.0.1';

require_once(dirname(__FILE__) . "/stachethemes/load.php");

class Stachethemes_Event_Calendar extends Stachethemes_Plugin {

    private static $instance;

    public static function get_instance() {

        if (null === static::$instance) {
            static::$instance = new static();
        }

        return static::$instance;
    }

    protected function __construct() {

        parent::__construct();

        $this->load_asset_classes(array(
            "php/class.settings.php",
        ));

        $this->auto_load_files($this->get_path('ASSETS') . 'php/install/');

        add_action('plugins_loaded', array($this, 'plugins_loaded'));

        add_action('init', array($this, 'init'));

        add_action('wp_loaded', array($this, 'wp_loaded'));
    }

    public function plugins_loaded() {

        parent::load_textdomain('stec');

        include($this->get_path('LANG') . 'inc.js-locale.php');

        // Add meta links in dashboard/plugins/
        add_filter('plugin_row_meta', function($links, $file) {

            if ($file === "stachethemes_event_calendar/stachethemes_event_calendar.php") {

                $row_meta = array(
                    'cc'      => '<a href="' . esc_url('//codecanyon.net/item/stachethemes-event-calendar/16168229?ref=Stachethemes') . '">' . esc_html__('Codecanyon', 'stec') . '</a>',
                    'doc'     => '<a href="' . esc_url('//stachethemes.com/wp-demos/the-calendar/documentation/') . '">' . esc_html__('Documentation', 'stec') . '</a>',
                    'videos'  => '<a href="' . esc_url('//www.youtube.com/channel/UCBcGlm2E4JQsiRlOFejpc4Q/videos') . '">' . esc_html__('Videos', 'stec') . '</a>',
                    'support' => '<a href="' . esc_url('//codecanyon.net/item/stachethemes-event-calendar/16168229/support/contact?ref=Stachethemes') . '">' . esc_html__('Support', 'stec') . '</a>',
                    'license' => '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__license')) . '">' . esc_html__('Activator', 'stec') . '</a>',
                    'news'    => '<a href="' . esc_url('//eepurl.com/dB8ohT') . '">' . esc_html__('Newsletter', 'stec') . '</a>',
                );
                return array_merge($links, $row_meta);
            }
            return (array) $links;
        }, 10, 2);

        /**
         *  Visual Composer Integration
         */
        if (function_exists("vc_map")) {
            $this->load_asset_classes(array('php/inc/inc.vc-settings.php'));
        }
    }

    public function wp_loaded() {

        // Add admin menus
        if (is_admin()) {

            $this->set_permission(apply_filters('stec_permission', $this->permission));
            $this->load_asset_classes(array('php/inc/inc.admin-menu.php'));
        }
    }

    public function init() {

        /**
         * Load classes 
         */
        $this->load_asset_classes(array(
            "php/libs/ics/vendor/autoload.php",
            "php/libs/rrule/vendor/autoload.php",
            "php/exception.php",
            "php/class.helper.php",
            "php/class.cron.php",
            "php/class.html.php",
            "php/class.settings.php",
            "php/abstract.post.php",
            "php/o.remind.php",
            "php/class.reminder.php",
            "php/class.rsvp.php",
            "php/i.event-meta-object.php",
            "php/o.event-meta-attachment.php",
            "php/o.event-meta-attendee.php",
            "php/o.event-meta-guest.php",
            "php/o.event-meta-product.php",
            "php/o.event-meta-product-bookable.php",
            "php/o.event-meta-schedule.php",
            "php/o.event-meta-organizer.php",
            "php/o.cron.php",
            "php/class.calendars.php",
            "php/class.categories.php",
            "php/class.locations.php",
            "php/class.organizers.php",
            "php/class.events.php",
            "php/o.calendar.php",
            "php/o.event.php",
            "php/o.category.php",
            "php/o.location.php",
            "php/o.organizer.php",
            "php/class.export.php",
            "php/class.export-ical.php",
            "php/class.export-csv.php",
            "php/class.import.php",
            "php/class.import-ical.php",
            "php/class.import-csv.php",
            "php/class.csv-settings.php",
            "php/class.wc.shop.php",
            "php/class.wc.booking.php",
            "php/o.wc-bookable.php",
            "php/class.debug-tools.php",
            "php/class.gdpr.php",
            "php/o.ajax.response.php",
            "php/class.builder.php",
            "php/class.builder-form.php",
            "php/class.builder-elements.php",
            "php/class.calendar-instance.php",
            "php/class.submit-event.php",
            "php/class.ac.php",
            "php/class.mail.php",
            "php/class.updater.php"
        ));

        $this->load_asset_classes(array(
            'php/inc/inc.taxonomies.php',
            'php/inc/inc.custom-states.php',
            'php/inc/inc.custom-posts.php',
            'php/inc/inc.posts-templates.php',
            'php/inc/inc.page-templates.php',
            'php/inc/inc.task-handler.php',
            'php/inc/inc.tabs.php',
            'php/inc/inc.admin-notices.php',
            'php/inc/inc.cron.php',
            'php/inc/inc.gutengerb-block.php'
        ));

        if (class_exists('WooCommerce')) {
            Shop::add_filters();
            Booking::add_book_filters();
        }

        //  auto load shortcode files
        $this->auto_load_files($this->get_path("SHORTCODES"));

        // rewrite rules for event posts
        $this->add_rewrite_rules();

        // ajax admin task listener
        $this->ajax_private_action('stec_ajax_action', array($this, 'admin_ajax_task_handler'));

        // ajax public task listener
        $this->ajax_public_action("stec_public_ajax_action", array($this, "admin_ajax_public_task_handler"));

        /**
         * Display fixed message if any in session
         */
        if (Admin_Helper::session_get('stec-fixed-message')) {

            // force load scripts
            $this->force_load_scripts(true);
            $this->display_fixed_message();
        }

        /**
         * Check force load scripts
         */
        if (Settings::get_admin_setting_value('stec_menu__general_other', 'force_load_scripts') === '1') {
            $this->force_load_scripts(true);
        }

        //  auto load api files
        $this->auto_load_files($this->get_path("ASSETS") . 'php/api/');

        // updater
        if (current_user_can('update_plugins')) {
            new Updater();
        }

        // init rest api
        new Api();
    }

    private function add_rewrite_rules() {

        add_action('stec_add_rewrite_rules', function() {
            add_rewrite_tag('%stec_repeat_offset%', '([0-9]+)');
            add_rewrite_tag('%stec_edit_event%', '([0-1])');
            add_rewrite_tag('%stec_organizer_name%', '([^/]*)');

            add_rewrite_rule('^' . Admin_Helper::get_permalinks('event', 'stec_event') . '/(.*)/([0-9]+)/?', 'index.php?stec_event=$matches[1]&stec_repeat_offset=$matches[2]', 'top');
            add_rewrite_rule('^' . Admin_Helper::get_permalinks('event', 'stec_event') . '/(.*)/?', 'index.php?stec_event=$matches[1]', 'top');
            add_rewrite_rule('^' . Admin_Helper::get_permalinks('edit', 'stec_edit') . '/(.*)/?', 'index.php?stec_event=$matches[1]&stec_edit_event=1', 'top');

            if (defined('ICL_SITEPRESS_VERSION')) {
                $languages = icl_get_languages();

                foreach ($languages as $lang) {
                    $lang_code = $lang['code'];
                    $major     = (int) $lang['major'];

                    if (1 === $major) {
                        add_rewrite_rule('^' . Admin_Helper::get_permalinks('organizer', 'stec_organizer') . '/?([^/]*)/?', 'index.php?page_id=' . Admin_Helper::get_stec_page('organizer', $lang_code) . '&stec_organizer_name=$matches[1]', 'top');
                    } else {
                        add_rewrite_rule('^' . $lang_code . '-' . Admin_Helper::get_permalinks('organizer', 'stec_organizer') . '/?([^/]*)/?', 'index.php?page_id=' . Admin_Helper::get_stec_page('organizer', $lang_code) . '&stec_organizer_name=$matches[1]', 'top');
                    }
                }
            } else {
                add_rewrite_rule('^' . Admin_Helper::get_permalinks('organizer', 'stec_organizer') . '/?([^/]*)/?', 'index.php?page_id=' . Admin_Helper::get_stec_page('organizer') . '&stec_organizer_name=$matches[1]', 'top');
            }
        });

        do_action('stec_add_rewrite_rules');

        // Stec edit event fix fox WPML
        if (defined('ICL_SITEPRESS_VERSION')) {
            add_filter('wpml_is_redirected', function ( $redirect, $post_id, $q ) {
                if (isset($q->query_vars['stec_edit_event']) && '1' === $q->query_vars['stec_edit_event']) {
                    return false;
                }
                return $redirect;
            }, 10, 3);
        }
    }

    /**
     * Handles admin ajax tasks
     */
    public function admin_ajax_task_handler() {
        $this->load_asset_classes(array('php/inc/inc.admin-ajax-task-handler.php'));
    }

    /**
     * Handles admin ajax public tasks
     */
    public function admin_ajax_public_task_handler() {
        $this->load_asset_classes(array('php/inc/inc.admin-ajax-public-task-handler.php'));
    }

    private function theme_list() {
        return array();
    }

    public function lcns() {
        $theme = wp_get_theme();
        $nfo   = get_option('stec_activated', false);

        if (isset($nfo['purchase_code'])) {
            return $nfo;
        } else if (in_array($theme->name, $this->theme_list())) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Custom head attributes
     */
    public function load_head() {

        if ($this->head_loaded === true) {
            return;
        }

        /**
         * Meta viewport tag for mobile devices
         * @todo make optional
         */
        ?>
        <meta name="viewport" content="width=device-width, initial-scale=1.0">

        <?php
        if (Settings::get_admin_setting_value('stec_menu__general', 'comments_form') === 'facebook') :
            ?>
            <script>(function (d, s, id) {
                    var js, fjs = d.getElementsByTagName(s)[0];
                    if (d.getElementById(id))
                        return;
                    js = d.createElement(s);
                    js.id = id;
                    js.src = '//connect.facebook.net/en_US/sdk.js#xfbml=1&version=v2.12';
                    fjs.parentNode.insertBefore(js, fjs);
                }(document, 'script', 'facebook-jssdk'));</script>
            <?php
        endif;

        $this->generate_css();
        $this->head_loaded = true;
    }

    public function set_fixed_message($msg, $type = '') {

        Admin_Helper::session_set('stec-fixed-message', array(
            'msg'  => $msg,
            'type' => $type
        ));
    }

    private function display_fixed_message() {

        $session_msg = Admin_Helper::session_get('stec-fixed-message');

        $msg  = $session_msg['msg'];
        $type = $session_msg['type'];

        switch ($type) :

            case 'success' :

                add_action('wp_footer', function() use($msg) {

                    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

                    ob_start();
                    include $stachethemes_ec_main->get_path('VIEW') . '/front/msg/success.php';
                    $html = preg_filter('/stec_replace_msg/', $msg, ob_get_clean());
                    echo Admin_Helper::esc($html, 'noscript');
                });

                break;

            case 'error' :

                add_action('wp_footer', function() use($msg) {

                    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

                    ob_start();
                    include $stachethemes_ec_main->get_path('VIEW') . '/front/msg/error.php';
                    $html = preg_filter('/stec_replace_msg/', $msg, ob_get_clean());
                    echo Admin_Helper::esc($html, 'noscript');
                });

                break;

            default :

                add_action('wp_footer', function() use($msg) {

                    $stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

                    ob_start();
                    include $stachethemes_ec_main->get_path('VIEW') . '/front/msg/msg.php';
                    $html = preg_filter('/stec_replace_msg/', $msg, ob_get_clean());
                    echo Admin_Helper::esc($html, 'noscript');
                });

                break;

        endswitch;

        Admin_Helper::session_set('stec-fixed-message', null);
    }

    private function generate_css() {
        $this->load_asset_classes(array('php/inc/inc.generate-css.php'));
    }

}

$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();