<?php

namespace Stachethemes\Stec;
?>

<div class="stachethemes-admin-wrapper">

    <?php echo Admin_Helper::display_message(); ?>

    <h1><?php esc_html_e('Calendars Management', 'stec'); ?></h1>

    <div class="stachethemes-admin-section">

        <h2><?php esc_html_e('Create new calendar', 'stec'); ?></h2>

        <?php
        Admin_Html::html_form_start("?page=$plugin_page", "POST");

        Admin_Html::html_info(esc_html__('Desired name of the calendar:', 'stec'));
        Admin_Html::html_input('calendar_name', '', '', esc_html__('Calendar Name', 'stec'), true);

        Admin_Html::html_info(esc_html__('Desired color. This will be the default color of the events. Can be changed later.', 'stec'));
        Admin_Html::html_color('calendar_color', '', '#f15e6e');

        Admin_Html::html_info(esc_html__('Default Icon', 'stec'));
        Admin_Html::html_hidden('calendar_icon', 'fa');
        ?>
        <div class="stec-icons-list calendar">
            <ul>
                <?php
                $icons = Admin_Helper::get_icon_list();
                foreach ($icons as $key => $value) {
                    ?><li data-icon="<?php esc_attr_e($key) ?>"><i class="<?php esc_attr_e($key) ?>"></i></li><?php
                }
                ?>
            </ul>

            <?php
            Admin_Html::html_info(esc_html__('Add icon from Font-Awesome', 'stec'));
            Admin_Html::html_input('add_fa_icon', '', '', 'Icon class name');
            Admin_Html::html_button(esc_html__('Add to the list', 'stec'), '', true, 'blue-button add-fa-icon');
            ?>
        </div>
        <?php
        Admin_Html::html_info(esc_html__('Timezone', 'stec'));
        Admin_Html::html_select('calendar_timezone', Admin_Helper::timezones_list(), "UTC", esc_html__('Calendar Timezone', 'stec'));

        Admin_Html::html_info(esc_html__('Front-End Visibility', 'stec'));
        Admin_Html::html_select('calendar_visibility', Admin_Helper::calendar_visibility_list(), 'stec_public', true, true);

        Admin_Html::html_info(esc_html__('Back-End Visibility', 'stec'));
        Admin_Html::html_select('calendar_back_visibility', Admin_Helper::calendar_visibility_list(), 'stec_logged_in', true, true);

        Admin_Html::html_info(esc_html__('Who can add events from the front-end', 'stec'));
        Admin_Html::html_select('calendar_writable', Admin_Helper::calendar_writable_list(), false, false, true);

        Admin_Html::html_checkbox('calendar_req_approval', true, true, esc_html__('Require approval by admin for events added from font-end', 'stec'), false);

        Admin_Html::html_hidden('task', 'stec_create_calendar');
        Admin_Html::html_button(esc_html__('Create Calendar', 'stec'));

        Admin_Html::html_form_end();
        ?>

    </div>

    <h1><?php esc_html_e('Manage Calendars', 'stec'); ?></h1>

    <div class="stachethemes-admin-section">

        <h2><?php esc_html_e('List with your created calendars', 'stec'); ?></h2>

        <?php
        if (empty($calendars)) :

            Admin_Html::html_info(esc_html__('(No calendars to display)', 'stec'));

        else :
            ?>
            <div class="stec-list-bulk">
                <div>
                    <input type="checkbox" name="all_calendars" value="1" /> <?php esc_html__('Select all calendars', 'stec'); ?>
                    <p><?php esc_html_e('Selected all calendars', 'stec'); ?></p>

                    <?php Admin_Html::html_form_start("?page=$plugin_page"); ?>

                    <button class='delete-all-items' data-confirm='<?php esc_html_e('Delete all items?', 'stec'); ?>'><p><i class="fas fa-trash"></i><?php esc_html_e('Delete selected calendars', 'stec'); ?></p></button>

                    <?php
                    Admin_Html::html_hidden('task', 'stec_delete_bulk_calendars');
                    ?>

                    <?php Admin_Html::html_form_end(); ?>
                </div>   
            </div>
            <?php
            echo '<ul class="stec-list">';

            foreach ($calendars as $calendar) :

                if (!$calendar instanceof Calendar_Post) {
                    continue;
                }

                $events_count   = (int) Events::get_admin_events_count(array('calid' => $calendar->get_id()));
                $add_text       = '';
                $aaproval_count = Events::get_aaproval_count($calendar->get_id());
                if ($aaproval_count > 0) {
                    $add_text = '-- ' . $aaproval_count . ' ' . ((int) $aaproval_count === 1 ? esc_html__('event awaiting approval', 'stec') : esc_html__('events awaiting approval', 'stec')) . ' --';
                }
                ?>
                <li>
                    <div class='calinfo'>
                        <input type="checkbox" name="calid" value="<?php echo (int) $calendar->get_id(); ?>" />
                        <p class='color' style='background: <?php esc_attr_e($calendar->get_color()); ?>'>
                            <i class="<?php esc_attr_e($calendar->get_icon()); ?>"></i>
                        </p>
                        <p class='title'>
                            <a href='<?php echo "?page=stec_menu__events&calendar_id={$calendar->get_id()}"; ?>'><?php esc_html_e($calendar->get_title()); ?> (id#<?php echo (int) $calendar->get_id(); ?>)</a>
                            <span class="stec-text-red"><?php esc_html_e($add_text); ?></span>
                            <span class="stec-list-sub-title"><?php
                                printf(esc_html__('%d %s', 'stec'), $events_count, _n('event', 'events', $events_count, 'stec'));
                                ?></span>
                        </p>
                    </div>
                    <div class='ctrl'>
                        <?php if ($events_count) : ?>
                            <a href='<?php echo "?page=stec_menu__events&calendar_id={$calendar->get_id()}"; ?>'><?php esc_html_e('View Events', 'stec'); ?></a>
                        <?php endif; ?>
                        <a href='<?php echo esc_url("?page={$plugin_page}&view=edit&calendar_id={$calendar->get_id()}"); ?>'><?php esc_html_e('Edit', 'stec'); ?></a>
                        <a class='delete-item' data-confirm='<?php esc_html_e('Delete item?', 'stec'); ?>' href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&task=stec_delete_calendar&calendar_id={$calendar->get_id()}")); ?>'><?php esc_html_e('Delete', 'stec'); ?></a>
                    </div>
                </li>
                <?php
            endforeach;

            echo '</ul>';
        endif;
        ?>

    </div>

</div>
