<?php

namespace Stachethemes\Stec;

global $plugin_page;
$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

// GET
switch (Admin_Helper::get("task")) {

    case 'delete' :
        Admin_Helper::verify_nonce('', 'GET');

        $category_id = Admin_Helper::get('category_id', 0, FILTER_VALIDATE_INT);
        $category = new Category($category_id);
        $result = $category->delete();

        if (!is_wp_error($result)) {
            Admin_Helper::set_message(esc_html__('Category deleted', 'stec'));
        } else {
            Admin_Helper::set_message($result->get_error_message(), 'error');
        }

        break;
}

// POST
switch (Admin_Helper::post("task")) {

    case 'update' :
    case 'create' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $category_id = Admin_Helper::post('category_id', null);
        $category = new Category($category_id);

        $category->set_title(Admin_Helper::post('title'));
        $category->set_color(Admin_Helper::post('color'));
        $category->set_notes(Admin_Helper::post('notes'));
        $category->set_visibility(Admin_Helper::post('visibility'));

        try {
            $result = $category->insert_category();

            if ($result) {
                Admin_Helper::set_message(sprintf(esc_html_x('Category %s', 'created/updated', 'stec'), $category->get_id() ? esc_html__('updated', 'stec') : esc_html__('created', 'stec')));
            } else {
                Admin_Helper::set_message(sprintf(esc_html_x('Error %s category', 'creating/updating', 'stec'), $category->get_id() ? esc_html__('updating', 'stec') : esc_html__('creating', 'stec')), 'error');
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_bulk' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $categories = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        if ($categories) {

            $result = Categories::bulk_delete($categories);

            if ($result) {
                Admin_Helper::set_message(esc_html__('Categories deleted', 'stec'));
            } else {
                Admin_Helper::set_message(esc_html__('Unable to delete categories', 'stec'), 'error');
            }
        }

        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case "edit" :

        $category_id = Admin_Helper::get('category_id');
        $category = new Category($category_id);
        
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/categories/addedit.php';

        break;

    case "add" :

        $category_id = false;
        $category = false;
        
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/categories/addedit.php';

        break;

    case "list":
    default:

        $categories = Categories::get_categories();
        
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/categories/list.php';
endswitch;
