<?php

namespace Stachethemes\Stec;

global $plugin_page;

$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

// GET
switch (Admin_Helper::get("task")) {

    case 'reset_filter' :

        $the_filter = Admin_Helper::get('the_filter', null, FILTER_SANITIZE_STRING);

        if ($the_filter) {
            if (in_array($the_filter, array('organizer_id', 'location_id', 'category_id'))) {
                Admin_Helper::session_set($the_filter, null);
            }
        }

        break;

    case 'reset_calendar_session' :
        Admin_Helper::session_set('stec_admin_calendar_id', null);
        break;

    case 'approve' :

        Admin_Helper::verify_nonce('', 'GET');

        $event_id = Admin_Helper::get('event_id', 0, FILTER_VALIDATE_INT);

        $result = Events::approve_event($event_id);

        if ($result) {
            Admin_Helper::set_message(esc_html__('Event approved', 'stec'));
        } else {
            Admin_Helper::set_message(esc_html__('Unable to approve event', 'stec'), 'error');
        }

        break;

    case 'delete' :

        Admin_Helper::verify_nonce('', 'GET');

        $event_id   = Admin_Helper::get('event_id', 0, FILTER_VALIDATE_INT);
        $event_post = new Event_Post($event_id);

        try {
            $event_post->delete_post();
            Admin_Helper::set_message(esc_html__('Event deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage());
        }

        break;
}

// POST handled by inc.task-handler.php
switch (Admin_Helper::post("task")) {

    case 'update' :
    case 'create' :

        break;

    case 'delete_bulk' :

        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case "edit" :

        $event_id    = Admin_Helper::get('event_id');
        $event       = new Event_Post($event_id);
        $calendar_id = $event->get_calid();
        $calendar    = new Calendar_Post($calendar_id);
        $cal_array   = Calendars::get_calendars_list();
        $users       = get_users(array('fields' => array('display_name', 'user_email', 'ID')));

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/events/addedit.php';

        break;

    case "add" :

        $event_id    = false;
        $event       = false;
        $calendar_id = Admin_Helper::get('calendar_id');
        $calendar    = new Calendar_Post($calendar_id);
        $users       = get_users(array('fields' => array('display_name', 'user_email', 'ID')));

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/events/addedit.php';

        break;

    case "list":
    default:

        $calendar_id = Admin_Helper::get('calendar_id');

        if (!$calendar_id && Admin_Helper::session_get('stec_admin_calendar_id')) {
            // Check if session calendar_id is selected
            $calendar_id = Admin_Helper::session_get('stec_admin_calendar_id');
        }

        if ($calendar_id) {
            // Check if calendar exists
            $calendar = new Calendar_Post($calendar_id);
            if (!$calendar->get_id()) {
                $calendar_id = null;
            }
        }

        if (!$calendar_id) {
            $calendars = Calendars::get_admin_calendars();
        } else {
            $calendar = new Calendar_Post($calendar_id);
            $search   = Admin_Helper::get('search', false, FILTER_SANITIZE_STRING);
            $page     = Admin_Helper::get('stec_page', 0, FILTER_VALIDATE_INT);
            $per_page = apply_filters('stec_admin_events_per_page', 20);
            $pages    = 0;

            $organizer_id = Admin_Helper::get('organizer_id', Admin_Helper::session_get('organizer_id', null), FILTER_VALIDATE_INT);
            $location_id  = Admin_Helper::get('location_id', Admin_Helper::session_get('location_id', null), FILTER_VALIDATE_INT);
            $category_id  = Admin_Helper::get('category_id', Admin_Helper::session_get('category_id', null), FILTER_VALIDATE_INT);

            $tax_query = array();

            if ($organizer_id) {
                $organizer = Organizers::get_organizer_by_id($organizer_id);
                if ($organizer) {
                    $organizer_id = $organizer->get_id();
                    $tax_query[]  = array(
                        'taxonomy'         => 'stec_event_organizer',
                        'field'            => 'id',
                        'terms'            => $organizer_id,
                        'include_children' => false
                    );
                }
            }

            if ($location_id) {
                $location = Locations::get_location_by_id($location_id);
                if ($location) {
                    $tax_query[] = array(
                        'taxonomy'         => 'stec_event_location',
                        'field'            => 'id',
                        'terms'            => $location_id,
                        'include_children' => false
                    );
                }
            }

            if ($category_id) {
                $category = Categories::get_category_by_id($category_id);
                if ($category) {
                    $category_id = $category->get_id();
                    $tax_query[] = array(
                        'taxonomy'         => 'stec_event_category',
                        'field'            => 'id',
                        'terms'            => $category_id,
                        'include_children' => false
                    );
                }
            }

            $events = Events::get_admin_events(array(
                        'per_page'  => $per_page,
                        'page'      => $page,
                        'calid'     => $calendar_id,
                        'tax_query' => $tax_query,
                        'query'     => array(
                            's' => trim($search) ? trim($search) : null
                        )
            ));

            $total = Events::get_admin_events_count(array(
                        'calid'     => $calendar_id,
                        'tax_query' => $tax_query,
                        'query'     => array(
                            's' => $search ? $search : null
                        )
            ));

            if ($total > $per_page) {
                $pages = ceil($total / $per_page);
            }

            Admin_Helper::session_set('organizer_id', $organizer_id);
            Admin_Helper::session_set('category_id', $category_id);
            Admin_Helper::session_set('location_id', $location_id);
        }

        Admin_Helper::session_set('stec_admin_calendar_id', $calendar_id);

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/events/list.php';
endswitch;
