<?php

namespace Stachethemes\Stec;
?>
<div class="stachethemes-admin-wrapper">

    <?php echo Admin_Helper::display_message(); ?>

    <?php if (isset($calendar)) : ?>

        <h1 class="stachethemes-admin-flex-head">
            <?php
            esc_html_e('Events Management', 'stec');
            echo " / " . $calendar->get_title();
            Admin_Html::html_button(esc_html__('Switch Calendar', 'stec'), "?page={$plugin_page}&task=reset_calendar_session", true);
            ?>
        </h1>

    <?php else: ?>

        <h1>
            <?php
            esc_html_e('Events Management / Select Calendar', 'stec');
            ?>
        </h1>

    <?php endif ?>

    <?php if (isset($calendars)) : ?>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Select Calendar', 'stec'); ?></h2>

            <?php
            if (empty($calendars)) :


                Admin_Html::html_info(esc_html__('You have no calendars', 'stec'));

                Admin_Html::html_button(esc_html__('Create calendar', 'stec'), '?page=stec_menu__calendars');

            else:

                echo '<ul class="stec-list">';

                foreach ($calendars as $calendar) :

                    $add_text       = '';
                    $aaproval_count = Events::get_aaproval_count($calendar->get_id());
                    if ($aaproval_count > 0) {
                        $add_text = '-- ' . $aaproval_count . ' ' . ($aaproval_count === 1 ? esc_html__('event awaiting approval', 'stec') : esc_html__('events awaiting approval', 'stec')) . ' --';
                    }
                    ?>
                    <li>
                        <div class='calinfo'>
                            <p class='color' style='background: <?php esc_attr_e($calendar->get_color()); ?>'>
                                <i class="<?php esc_attr_e($calendar->get_icon()); ?>"></i>
                            </p>
                            <p class='title'>
                                <a href='<?php echo "?page=stec_menu__events&calendar_id={$calendar->get_id()}"; ?>'>
                                    <?php esc_html_e($calendar->get_title()); ?> (id#<?php echo (int) $calendar->get_id(); ?>)
                                </a> 
                                <span class="stec-text-red"><?php esc_html_e($add_text); ?></span></p>
                        </div>
                        <div class='ctrl'>
                            <a href='<?php echo "?page=stec_menu__events&calendar_id={$calendar->get_id()}"; ?>'><?php esc_html_e('View Events', 'stec'); ?></a>
                        </div>
                    </li>
                    <?php
                endforeach;

                echo '</ul>';

            endif;
            ?>

        </div>

    <?php endif; ?>

    <?php if (isset($events)) : ?>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Add Events', 'stec'); ?></h2>

            <?php
            Admin_Html::html_info(esc_html__('Add new event to this calendar', 'stec'));
            Admin_Html::html_button(esc_html__('Add Event', 'stec'), "?page={$plugin_page}&view=add&calendar_id={$calendar_id}");
            ?>

        </div>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Manage Events', 'stec'); ?></h2>

            <?php
            $terms = array();
            if (isset($organizer)) {
                $terms['organizer_id'] = '<i class="fas fa-user"></i>' . $organizer->get_name();
            }
            if (isset($location)) {
                $terms['location_id'] = '<i class="fas fa-map-marker-alt"></i>' . $location->get_address();
            }
            if (isset($category)) {
                $terms['category_id'] = '<i class="fas fa-folder"></i>' . $category->get_title();
            }
            if (isset($search) && trim($search)) {
                $terms['search'] = '<i class="fas fa-search"></i>' . $search;
            }

            foreach ($terms as $k => $term) {
                $reset_url = esc_url(admin_url("admin.php?page={$plugin_page}&view=list&calendar_id={$calendar_id}&task=reset_filter&the_filter={$k}"));

                echo "<span class='term'>$term<a href='{$reset_url}'><i class='fa fa-times'></i></a></a></span>";
            }
            ?>

            <?php
            if (empty($events)) :

                if (Admin_Helper::isset_get('search')) {
                    Admin_Html::html_info(esc_html__('Your search returned no results', 'stec'));
                    Admin_Html::html_form_start("", "GET");
                    Admin_Html::html_hidden('page', $plugin_page);
                    Admin_Html::html_hidden('calendar_id', $calendar_id);
                    Admin_Html::html_input('search', Admin_Helper::get('search', ''), '', esc_html__('Search event', 'stec'));
                    Admin_Html::html_button('Search', false, true, 'blue-button');
                    if ($organizer_id) {
                        Admin_Html::html_hidden('organizer_id', $organizer_id);
                    }
                    if ($location_id) {
                        Admin_Html::html_hidden('location_id', $location_id);
                    }
                    if ($category_id) {
                        Admin_Html::html_hidden('category_id', $category_id);
                    }
                    Admin_Html::html_form_end();
                } else {
                    Admin_Html::html_info(esc_html__('(No events to display)', 'stec'));
                }

            else :

                Admin_Html::html_info(esc_html__('Manage events list', 'stec'));
                ?>
                <div class="stachethemes-admin-style-flexbox">
                    <div class="stec-list-bulk">
                        <div>
                            <input type="checkbox" name="all_events" value="1" /> <?php esc_html__('Select all events', 'stec'); ?>
                            <p><?php esc_html_e('Selected all events', 'stec'); ?></p>

                            <?php Admin_Html::html_form_start("?page=$plugin_page&calendar_id={$calendar_id}"); ?>

                            <button class='delete-all-items' data-confirm='<?php esc_html_e('Delete all items?', 'stec'); ?>'><p><i class="fas fa-trash"></i><?php esc_html_e('Delete selected events', 'stec'); ?></p></button>

                            <?php
                            Admin_Html::html_hidden('calendar_id', $calendar_id);
                            Admin_Html::html_hidden('task', 'stec_delete_bulk_events');
                            ?>

                            <?php Admin_Html::html_form_end(); ?>

                        </div>   
                    </div>
                    <div>
                        <?php
                        Admin_Html::html_form_start("", "GET");
                        Admin_Html::html_hidden('page', $plugin_page);
                        Admin_Html::html_hidden('calendar_id', $calendar_id);
                        Admin_Html::html_input('search', Admin_Helper::get('search', ''), '', esc_html__('Search event', 'stec'));
                        Admin_Html::html_button('Search', false, true, 'blue-button');
                        if ($organizer_id) {
                            Admin_Html::html_hidden('organizer_id', $organizer_id);
                        }
                        if ($location_id) {
                            Admin_Html::html_hidden('location_id', $location_id);
                        }
                        if ($category_id) {
                            Admin_Html::html_hidden('category_id', $category_id);
                        }
                        Admin_Html::html_form_end();
                        ?>
                    </div>
                </div>
                <?php
                echo '<ul class="stec-list">';

                foreach ($events as $event) :
                    if (!$event instanceof \Stachethemes\Stec\Event_Post) {
                        continue;
                    }
                    ?>
                    <li>
                        <div class='calinfo'>
                            <input type="checkbox" name="eventid" value="<?php echo (int) $event->get_id(); ?>" />
                            <p class='color' style='background: <?php esc_attr_e($event->get_color()) ? esc_attr($event->get_color()) : esc_attr($calendar->get_color()); ?>'>
                                <?php
                                switch ($event->get_icon_type()) :
                                    case 'icon' :
                                        ?><i class="<?php esc_attr_e($event->get_icon()); ?>"></i><?php
                                        break;

                                    case 'date' :
                                        ?><span class="icon_date"><?php esc_html_e($event->get_start_date('j')); ?><span><?php esc_html_e($event->get_start_date('M')); ?></span></span><?php
                                        break;

                                    case 'image':

                                        $icon_image = $event->get_parsed_icon_images();
                                        $icon_image = $icon_image{0};
                                        $icon_image = $icon_image->src;
                                        ?><span class="icon_image" style="background-image:url('<?php echo esc_url($icon_image); ?>')"></span><?php
                                                break;
                                        endswitch;
                                        ?>
                            </p>
                            <p class='title'>
                                <a href='<?php echo "?page={$plugin_page}&view=edit&calendar_id={$calendar_id}&event_id={$event->get_id()}"; ?>'>
                                    <?php esc_html_e($event->get_title()); ?> (id#<?php echo (int) $event->get_id(); ?>)</a> 
                                <?php
                                if ((int) $event->get_approved() === 0) :
                                    echo '<span class="stec-text-red"> -- ' . esc_html__('Awaiting approval', 'stec') . ' -- </span>';
                                endif
                                ?>
                                <span class="stec-event-timespan">
                                    <?php
                                    esc_html_e($event->get_start_date());

                                    if ($event->get_rrule() !== '') {
                                        echo ' (' . esc_html__('Recurring', 'stec') . ')';
                                    }

                                    if ($event->get_recurrence_id() !== '') {
                                        echo ' (' . esc_html__('Recurrence Override: ', 'stec') . $event->get_recurrence_id() . ')';
                                    }
                                    ?>
                                </span>
                            </p>
                        </div>
                        <div class='ctrl'>

                            <?php if ((int) $event->get_approved() === 0) : ?>

                                <a href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&task=approve&view=list&calendar_id={$calendar_id}&event_id={$event->get_id()}")); ?>'><?php esc_html_e('Approve', 'stec'); ?></a>
                                <a href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&view=edit&calendar_id={$calendar_id}&event_id={$event->get_id()}")); ?>'><?php esc_html_e('Review', 'stec'); ?></a>

                            <?php else: ?>

                                <a class="duplicate" href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&task=stec_duplicate_event&view=list&calendar_id={$calendar_id}&event_id={$event->get_id()}")); ?>'><?php esc_html_e('Duplicate', 'stec'); ?></a>
                                <a href='<?php echo esc_url("?page={$plugin_page}&view=edit&calendar_id={$calendar_id}&event_id={$event->get_id()}"); ?>'><?php esc_html_e('Edit', 'stec'); ?></a>

                            <?php endif; ?>

                            <a class='delete-item' data-confirm='<?php esc_html_e('Delete item?', 'stec'); ?>' href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&calendar_id={$calendar_id}&task=delete&event_id={$event->get_id()}")); ?>'><?php esc_html_e('Delete', 'stec'); ?></a>
                        </div>
                    </li>
                    <?php
                endforeach;

                echo '</ul>';
            endif;
            ?>

            <ul class="stec-pagination">
                <?php
                $current_cal_page        = admin_url("admin.php?page=stec_menu__events&calendar_id={$calendar_id}");
                $current_search_param    = null;
                $current_organizer_param = null;
                $current_location_param  = null;

                if (Admin_Helper::isset_get('search')) {
                    $current_search_param = "search=" . Admin_Helper::get('search', null);
                }

                if (Admin_Helper::isset_get('organizer_id')) {
                    $current_search_param = "organizer_id=" . Admin_Helper::get('organizer_id', null, FILTER_VALIDATE_INT);
                }

                if (Admin_Helper::isset_get('location_id')) {
                    $current_location_param = "location_id=" . Admin_Helper::get('location_id', null, FILTER_VALIDATE_INT);
                }

                for ($i = 1; $i <= $pages; $i++) {
                    $page_url = array($current_cal_page, $current_search_param, $current_organizer_param, "stec_page=" . ($i - 1));
                    $page_url = implode('&', array_filter($page_url, 'strlen'));
                    ?>
                    <li><a class="<?php echo (int) ($i - 1) === (int) $page ? 'active' : '' ?>" href="<?php echo esc_url($page_url); ?>"><?php echo (int) $i; ?></a></li>
                    <?php
                }
                ?>
            </ul>
        </div>

    <?php endif; ?>

    <?php
    if (class_exists('SitePress')) :

        $langs   = icl_get_languages('skip_missing=0&orderby=code');
        $options = array();

        foreach ($langs as $lang) {
            $options[$lang['code']] = $lang['native_name'];
        }
        ?>
        <div class="stec-duplicate-select-language">
            <div class="stec-duplicate-select-language-pop">
                <span class="stec-duplicate-select-language-close fa fa-times"></span>
                <?php
                Admin_Html::html_info(esc_html__('Select Language', 'stec'));
                Admin_Html::html_select('', $options, '', false, false);
                Admin_Html::html_button(esc_html__('Duplicate', 'stec'));
                ?>
            </div>
        </div>
        <?php
    endif;
    ?>

</div>