<?php

namespace Stachethemes\Stec;
?>
<div class="stachethemes-admin-section-tab" data-tab="general">
    
     <h2><?php esc_html_e('General', 'stec'); ?></h2>

    <?php
    Admin_Html::html_info(esc_html__('Desired name of the event (Summary)', 'stec'));
    Admin_Html::html_input('summary', $event ? $event->get_title() : null, '', esc_html__('Event Name', 'stec'), true);

    Admin_Html::html_info(esc_html__('Title slug. Must be unique. Leave empty to auto-generate.', 'stec'));
    Admin_Html::html_input('alias', $event ? $event->get_slug() : null, '', esc_html__('Slug', 'stec'), false);

    Admin_Html::html_info(esc_html__('Desired color. By default will use the calendar color.', 'stec'));
    Admin_Html::html_color('event_color', $event ? $event->get_color() : null, $calendar->get_color());

    Admin_Html::html_info(esc_html__('Select event symbol type', 'stec'));
    Admin_Html::html_icon('icon_type', array(
        'icon'  => esc_html__('Icon', 'stec'),
        'date'  => esc_html__('Date', 'stec'),
        'image' => esc_html__('Image', 'stec')
            ), $event ? $event->get_icon_type() : 'icon', true);

    Admin_Html::html_hidden('icon', $event ? $event->get_icon() : $calendar->get_icon());
    ?>
    <div class="stec-icons-list event">
        <ul>
            <?php
            $icons = Admin_Helper::get_icon_list();
            foreach ($icons as $key => $value) {
                ?><li data-icon="<?php esc_attr_e($key) ?>"><i class="<?php esc_attr_e($key) ?>"></i></li><?php
            }
            ?>
        </ul>

        <?php
        Admin_Html::html_info(esc_html__('Add icon from Font-Awesome', 'stec'));
        Admin_Html::html_input('add_fa_icon', '', '', 'Icon class name');
        Admin_Html::html_button(esc_html__('Add to the list', 'stec'), '', true, 'blue-button plus-button add-fa-icon');
        ?>
    </div>


    <div class='stec-icon-image-list'>
        <?php
        Admin_Html::html_add_image('icon_images[]', $event ? $event->get_icon_images() : false, esc_html__('Add Images', 'stec'), false);
        ?>
    </div>


    <?php
    if ($event_id !== false && isset($cal_array) && count($cal_array) > 1) {
        Admin_Html::html_info(esc_html__('Calendar', 'stec'));
        Admin_Html::html_select('calid', $cal_array, $calendar_id, true);
    } else {
        Admin_Html::html_hidden('calid', $calendar_id);
    }

    $calendars = Calendars::get_admin_calendars();
    foreach ($calendars as $cal) :
        Admin_Html::html_hidden("calendar_colors_by_id[" . $cal->get_id() . "]", $cal->get_color());
        Admin_Html::html_hidden("calendar_tz_by_id[" . $cal->get_id() . "]", $cal->get_timezone());
    endforeach;

    Admin_Html::html_info(esc_html__('Front-End Visibility', 'stec'));
    Admin_Html::html_select('visibility', Admin_Helper::event_visibility_list(), $event ? $event->get_visibility() : 'stec_cal_default', true, true);

    Admin_Html::html_info(esc_html__('Back-End Visibility', 'stec'));
    Admin_Html::html_select('back_visibility', Admin_Helper::event_visibility_list(), $event ? $event->get_back_visibility() : 'stec_cal_default', true, true);

    Admin_Html::html_info(esc_html__('Featured', 'stec'));
    Admin_Html::html_select('featured', array(
        0 => esc_html__('No', 'stec'),
        1 => esc_html__('Yes', 'stec'),
        2 => esc_html__('Yes with Background', 'stec')
            ), $event ? $event->get_featured() : 0, true);

    Admin_Html::html_info(esc_html__('Category', 'stec'));
    Admin_Html::html_select('event_categories', Categories::get_categories_list(), $event ? $event->get_categories() : null, false, true);

    Admin_Html::html_info(esc_html__('Starts On', 'stec'));
    ?>
    <div class="stachethemes-admin-section-flex">
        <?php
        if ($event) {
            $start_fulldate = explode(' ', $event->get_start_date());
            $start_fulltime = explode(':', $start_fulldate[1]);

            $end_fulldate = explode(' ', $event->get_end_date());
            $end_fulltime = explode(':', $end_fulldate[1]);
        }

        Admin_Html::html_date('start_date', isset($start_fulldate) ? $start_fulldate[0] : date('Y-m-d'), '', esc_html__('Start Date', 'stec'), true);
        Admin_Html::html_select('start_time_hours', Admin_Helper::get_hours_array(), isset($start_fulltime) ? $start_fulltime[0] : null, true);
        Admin_Html::html_select('start_time_minutes', Admin_Helper::minutes_array(), isset($start_fulltime) ? $start_fulltime[1] : null, true);
        ?>
    </div>

    <?php
    Admin_Html::html_info(esc_html__('Ends On', 'stec'));
    ?>
    <div class="stachethemes-admin-section-flex">
        <?php
        Admin_Html::html_date('end_date', isset($end_fulldate) ? $end_fulldate[0] : null, date('Y-m-d'), esc_html__('End Date', 'stec'), true);
        Admin_Html::html_select('end_time_hours', Admin_Helper::get_hours_array(), isset($end_fulltime) ? $end_fulltime[0] : null, true);
        Admin_Html::html_select('end_time_minutes', Admin_Helper::minutes_array(), isset($end_fulltime) ? $end_fulltime[1] : null, true);
        ?>
    </div>

    <div class="stachethemes-admin-section-flex">
        <?php
        Admin_Html::html_checkbox('all_day', ($event ? ($event->get_all_day() ? 1 : 0) : 0), 0, esc_html__('All Day', 'stec'));
        ?>&nbsp;&nbsp;&nbsp;<?php
        Admin_Html::html_checkbox('hide_end', $event ? $event->get_hide_end() : '', 0, esc_html__('Hide end date', 'stec'));
        ?>
    </div>

    <?php
    if (!$event || !$event->get_recurrence_id()) :
        Admin_Html::html_info(esc_html__('Repeater Scheme ', 'stec'));
        Admin_Html::html_button(esc_html__('Set repeater', 'stec'), '', false, "blue-button set-repeater-button");
        ?>
        <p class="stec-repeater-summary"><?php esc_html_e('Summary: ', 'stec') ?>
            <span>-</span>
        </p>
    <?php endif; ?>

    <?php
    Admin_Html::html_info(esc_html__('Search keywords (optional)', 'stec'));
    Admin_Html::html_input('keywords', $event ? $event->get_keywords() : null, '', esc_html__('Search keywords', 'stec'), false);
    ?>

    <?php
    Admin_Html::html_info(esc_html__('Counter', 'stec'));
    Admin_Html::html_select('counter', array(
        '0' => esc_html__('Disable', 'stec'),
        '1' => esc_html__('Enable', 'stec')
            )
            , $event ? $event->get_counter() : null, true);
    ?>

    <?php
    Admin_Html::html_info(esc_html__('Comments', 'stec'));
    Admin_Html::html_select('comments', array(
        '0' => esc_html__('Disable', 'stec'),
        '1' => esc_html__('Enable', 'stec')
            )
            , $event ? $event->get_comments() : null, true);
    ?>

</div>

<div class="stec-repeater-popup-bg"></div>
<div class="stec-repeater-popup" tabindex="0">
    <div>
        <table>
            <tbody>
                <tr>
                    <th><?php esc_html_e('Repeats:', 'stec') ?></th>
                    <td>
                        <select name="repeat_freq">
                            <option value="0"><?php esc_html_e('No Repeat', 'stec') ?></option>
                            <option value="1"><?php esc_html_e('Daily', 'stec') ?></option>
                            <option value="2"><?php esc_html_e('Weekly', 'stec') ?></option>
                            <option value="3"><?php esc_html_e('Monthly', 'stec') ?></option>
                            <option value="4"><?php esc_html_e('Yearly', 'stec') ?></option>
                        </select>
                    </td>
                </tr>

                <tr class="stec-repeater-popup-weekdays">
                    <th><?php esc_html_e('By Day:', 'stec') ?></th>
                    <td class="stec-repeater-popup-repeat-on">
                        <div>
                            <span>
                                <input name="SU" title="<?php esc_html_e('Sunday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Sunday', 'stec') ?>"><?php esc_html_e('SU', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="MO" title="<?php esc_html_e('Monday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Monday', 'stec') ?>"><?php esc_html_e('MO', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="TU" title="<?php esc_html_e('Tuesday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Tuesday', 'stec') ?>"><?php esc_html_e('TU', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="WE" title="<?php esc_html_e('Wednesday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Wednesday', 'stec') ?>"><?php esc_html_e('WE', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="TH" title="<?php esc_html_e('Thursday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Thursday', 'stec') ?>"><?php esc_html_e('TH', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="FR" title="<?php esc_html_e('Friday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Friday', 'stec') ?>"><?php esc_html_e('FR', 'stec') ?></label>
                            </span>
                            <span>
                                <input name="SA" title="<?php esc_html_e('Saturday', 'stec') ?>" type="checkbox">
                                <label for="" title="<?php esc_html_e('Saturday', 'stec') ?>"><?php esc_html_e('SA', 'stec') ?></label>
                            </span>
                        </div>

                    </td>
                </tr>

            </tbody>
            <tbody>
                <tr>
                    <th><?php esc_html_e('Repeat gap:', 'stec') ?></th>
                    <td>
                        <select name="repeat_gap">
                            <option value=""><?php esc_html_e('No gap', 'stec') ?></option>
                            <?php
                            for ($i = 2; $i < 31; $i++) {
                                ?>
                                <option value="<?php echo (int) $i; ?>"><?php echo (int) $i - 1; ?></option>
                                <?php
                            }
                            ?>
                        </select>

                    </td>
                </tr>
                <tr>
                    <th>
                        <?php esc_html_e('Repeat ends on:', 'stec') ?>
                    </th>
                    <td class="stec-repeater-popup-endson-options">
                        <span>
                            <input id="stec-repeater-popup-repeat-endson-never" name="repeat_endson" value="0" checked="checked" type="radio">
                            <label for="stec-repeater-popup-repeat-endson-never"><?php esc_html_e('Never', 'stec') ?></label>
                        </span>
                        <span>
                            <input id="stec-repeater-popup-repeat-endson-after-n" name="repeat_endson" value="1" type="radio">
                            <label for="stec-repeater-popup-repeat-endson-after-n">
                                <?php esc_html_e('After', 'stec') ?> 
                                <input name="repeat_occurences" size="3" value="" disabled="disabled">
                                <?php esc_html_e('occurences', 'stec') ?>
                            </label>
                        </span>
                        <span>
                            <input id="stec-repeater-popup-repeat-endson-date" name="repeat_endson" value="2" type="radio">
                            <label for="stec-repeater-popup-repeat-endson-date">
                                <?php esc_html_e('On Date', 'stec') ?>
                                <input id="repeat_end_date" name="repeat_ends_on_date" size="10" value="" disabled="disabled" autocomplete="off">
                            </label>
                        </span>
                    </td>
                </tr>
                <tr class="stec-repeater-popup-exdate-menu">
                    <th>
                        <?php esc_html_e('Date Exception:', 'stec') ?>
                    </th>
                    <td class="stec-repeater-popup-exdate-options">
                        <input id="stec-repeater-popup-exdate-datepicker" value="" type="text">
                        <input id="stec-repeater-popup-exdate-datepicker-exdate-value" value="" type="hidden">
                        <a id="stec-add-exdate" href="javascript:void(0);"><?php esc_html_e('Add exdate', 'stec'); ?></a>
                        <ul class="stec-repeater-popup-exdate-datelist">
                            <li class="stec-repeater-popup-exdate-datelist-template">
                                <span>stec_replace_date</span>
                                <span class="stec-repeater-popup-exdate-datelist-submit-value">stec_replace_altdate</span>
                                <a class="stec-remove-exdate" href="javascript:void(0);">
                                    <?php esc_html_e('Remove', 'stec'); ?>
                                </a>
                            </li>
                        </ul>
                    </td>
                </tr>
                <tr>
                    <th><?php esc_html_e('Summary:', 'stec') ?></th>
                    <td class="stec-repeater-popup-repeat-summary">-</td>
                </tr>
            </tbody>
            <tbody class="stec-repeater-popup-repeat-advanced">
                <tr>
                    <th><?php esc_html_e('RRULE String:', 'stec') ?></th>
                    <td>
                        <input type="text" name="advanced_rrule" value="<?php esc_attr_e($event ? $event->get_rrule() : null); ?>" />
                        <input type="hidden" name="is_advanced_rrule" value="<?php esc_attr_e($event ? $event->get_is_advanced_rrule() : 0); ?>" />
                    </td>
                </tr>
            </tbody>
        </table>
    </div>
    <div class="stec-repeater-popup-buttons">
        <?php
        Admin_Html::html_button(esc_html__('Update', 'stec'), '', false, "blue-button");
        Admin_Html::html_button(esc_html__('Cancel', 'stec'), '', true, "blue-button");
        ?>
    </div>
    <?php
    Admin_Html::html_hidden('rrule', $event ? $event->get_rrule() : '');
    Admin_Html::html_hidden('exdate', $event ? $event->get_exdate() : '');
    ?>
</div>