<?php

namespace Stachethemes\Stec;

global $plugin_page;
$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();


// GET
switch (Admin_Helper::get("task")) {

    case 'delete_gdpr_emails' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            DTools::delete_gdpr_emails();
            Admin_Helper::set_message(esc_html__('GDPR emails deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_reminders' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            DTools::delete_reminders();
            Admin_Helper::set_message(esc_html__('Reminders deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'reset_permalinks' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            DTools::reset_permalinks();
            Admin_Helper::set_message(esc_html__('Permalinks have been reset', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_broken_events' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('edit_posts')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            $count = DTools::remove_floating_events();
            Admin_Helper::set_message(sprintf(_n('%s event removed', '%s events removed', $count, 'stec'), $count));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'repair_broken_tickets' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('edit_posts')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            $count = DTools::repair_broken_tickets();
            Admin_Helper::set_message(sprintf(_n('%s ticket repaired', '%s tickets repaired', $count, 'stec'), $count));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_broken_tickets' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('edit_posts')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
            $count = DTools::remove_tickets();
            Admin_Helper::set_message(sprintf(_n('%s ticket removed', '%s tickets removed', $count, 'stec'), $count));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'create_organizer_page' :

        Admin_Helper::verify_nonce('', 'GET');

        try {

            if (false === current_user_can('edit_posts')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            $id = Admin_Helper::get_stec_page('organizer');

            if ('page' === get_post_type((int) $id)) {
                throw new Stec_Exception(esc_html__('Organizer page already exists', 'stec'));
            }

            $args = array(
                'post_title'   => wp_strip_all_tags(esc_html__('Event Organizers', 'stec')),
                'post_type'    => 'page',
                'post_content' => '',
                'post_status'  => 'stec-organizer-page',
                'post_author'  => get_current_user_id(),
            );

            $result = wp_insert_post($args);

            if (is_wp_error($result)) {
                throw new Stec_Exception($result->get_error_message());
            }

            if (!is_numeric($result) || (int) $result <= 0) {
                throw new Stec_Exception(esc_html__('Unable to create organizer page', 'stec'));
            }

            Admin_Helper::set_stec_page('organizer', $result);

            Admin_Helper::set_message(esc_html__('Organizer page created', 'stec'));
            
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }


        break;

    case 'delete_organizer_page' :
        Admin_Helper::verify_nonce('', 'GET');

        try {

            if (false === current_user_can('edit_posts')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            $id = Admin_Helper::get_stec_page('organizer');

            if ('page' !== get_post_type((int) $id)) {
                Admin_Helper::delete_stec_page('organizer');
                throw new Stec_Exception(esc_html__('Organizer page does not exist', 'stec'));
            }

            $result = wp_delete_post((int) $id, true);

            if (false === $result || null === $result) {
                throw new Stec_Exception(esc_html__('Error deleting page', 'stec'));
            }

            Admin_Helper::delete_stec_page('organizer');
            Admin_Helper::set_message(esc_html__('Organizer page deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'reset_mail_template' :
        Admin_Helper::verify_nonce('', 'GET');

        try {

            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            $template = Admin_Helper::get('template', null);

            Settings::reset_admin_settings($plugin_page . '_email_templates', $template . '_subject');
            Settings::reset_admin_settings($plugin_page . '_email_templates', $template . '_content');

            Admin_Helper::set_message(esc_html__('Template Reset', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }
        break;

    case 'reset' :

        Admin_Helper::verify_nonce('', 'GET');

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            Settings::reset_admin_settings($plugin_page);
            Settings::reset_admin_settings($plugin_page . '_event_submit');
            Settings::reset_admin_settings($plugin_page . '_google_captcha');
            Settings::reset_admin_settings($plugin_page . '_privacy_agreement');
            Settings::reset_admin_settings($plugin_page . '_email_notifications');
            Settings::reset_admin_settings($plugin_page . '_email_templates');
            Settings::reset_admin_settings($plugin_page . '_other');

            Admin_Helper::set_message(esc_html__('Settings Reset', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;
}


// POST
switch (Admin_Helper::post("task")) {

    case 'save_mail_template' :

        $template = Admin_Helper::post('template');
        Admin_Helper::verify_nonce("?page={$plugin_page}&view=edit_email&template={$template}");

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            Settings::update_admin_settings($plugin_page . '_email_templates', $template . '_subject');
            Settings::update_admin_settings($plugin_page . '_email_templates', $template . '_content');


            Admin_Helper::set_message(esc_html__('Template updated', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }
        break;
    case 'save' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        try {
            if (false === current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            Settings::update_admin_settings($plugin_page);
            Settings::update_admin_settings($plugin_page . '_event_submit');
            Settings::update_admin_settings($plugin_page . '_google_captcha');
            Settings::update_admin_settings($plugin_page . '_privacy_agreement');
            Settings::update_admin_settings($plugin_page . '_email_notifications');
            Settings::update_admin_settings($plugin_page . '_email_templates');
            Settings::update_admin_settings($plugin_page . '_other');

            Admin_Helper::set_message(esc_html__('Settings updated', 'stec'));
        } catch (Stec_Exception $ex) {

            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case 'edit_email' :
        $template      = Admin_Helper::get('template', null);
        $email_setting = Settings::get_admin_setting($plugin_page . '_email_notifications', $template);
        $template_name = $email_setting['title'];

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/general/email.settings.php';

        break;

    default:
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/general/settings.php';

endswitch;



