<?php

namespace Stachethemes\Stec; ?>

<div class="stachethemes-admin-wrapper">

    <?php echo Admin_Helper::display_message(); ?>

    <?php if (isset($calendar)) : ?>

        <h1 class="stachethemes-admin-flex-head">
            <?php
            esc_html_e('Import Events Management', 'stec');
            echo " / " . $calendar->get_title();
            Admin_Html::html_button(esc_html__('Switch Calendar', 'stec'), "?page={$plugin_page}&task=reset_calendar_session", true);
            ?>
        </h1>

    <?php else: ?>

        <h1>
            <?php
            esc_html_e('Import Events Management / Select Calendar', 'stec');
            ?>
        </h1>

    <?php endif ?>

    <?php if (isset($calendars)) : ?>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Select the calendar on which the events should be imported', 'stec'); ?></h2>

            <?php
            if (empty($calendars)) :

                Admin_Html::html_info(esc_html__('You have no calendars', 'stec'));

                Admin_Html::html_button(esc_html__('Create calendar', 'stec'), '?page=stec_menu__calendars');

            else:

                echo '<ul class="stec-list">';

                foreach ($calendars as $calendar) :
                    ?>
                    <li>
                        <div class='calinfo'>
                            <p class='color' style='background: <?php esc_attr_e($calendar->get_color()); ?>'>
                                <i class="<?php esc_attr_e($calendar->get_icon()); ?>"></i>
                            </p>
                            <p class='title'><a href='<?php echo "?page=stec_menu__import&calendar_id={$calendar->get_id()}"; ?>'><?php esc_html_e($calendar->get_title()); ?> (id#<?php echo (int) $calendar->get_id(); ?>)</a></p>
                        </div>
                        <div class='ctrl'>
                            <a href='<?php echo "?page=stec_menu__import&calendar_id={$calendar->get_id()}"; ?>'><?php esc_html_e('Select', 'stec'); ?></a>
                        </div>
                    </li>
                    <?php
                endforeach;

                echo '</ul>';

            endif;
            ?>

        </div>

    <?php endif; ?>

    <?php if ($calendar_id) : ?>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Import CSV Events', 'stec'); ?></h2>

            <?php
            Admin_Html::html_form_start("?page=$plugin_page&calendar_id={$calendar_id}", "POST", true);

            Admin_Html::html_info(esc_html__('Import from .csv file', 'stec'));
            Admin_Html::html_input('csv_filename', '', '', esc_html__('CSV File', 'stec'), false, "file", 'accept=.csv');

            Admin_Html::html_checkbox('download_images', 1, 1, esc_html__('Download images', 'stec'), false);
            Admin_Html::html_desc(esc_html__('If Images ID is URL it will be downloaded to WP Media.', 'stec'));

            Admin_Html::html_checkbox('create_categories', 1, 1, esc_html__('Create categories', 'stec'), false);
            Admin_Html::html_desc(esc_html__('If Categories ID is text and does not exist, it will be created.', 'stec'));

            Admin_Html::html_checkbox('create_locations', 1, 1, esc_html__('Create locations', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Add the event location to Locations if it does not exist.', 'stec'));

            Admin_Html::html_checkbox('ignore_expired', 1, 1, esc_html__("Don't import expired events", 'stec'), false);
            Admin_Html::html_desc(esc_html__('If checked non-recurring expired events will be ignored.', 'stec'));

            Admin_Html::html_checkbox('overwrite_events', 0, 0, esc_html__('Overwrite events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Existing events will be overwritten.', 'stec'));

            Admin_Html::html_checkbox('delete_removed', 0, 0, esc_html__('Delete events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Delete calendar event if it does not exist in the csv file.', 'stec'));

            Admin_Html::html_hidden('calendar_id', $calendar_id);
            Admin_Html::html_hidden('task', 'import_event_csv');
            Admin_Html::html_button(esc_html__('Import Events', 'stec'));
            Admin_Html::html_button(esc_html__('CSV Settings', 'stec'), "?page=$plugin_page&view=csv_settings", true, 'blue-button');

            Admin_Html::html_form_end();
            ?>

        </div>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Import .ics Events', 'stec'); ?></h2>

            <?php
            Admin_Html::html_form_start("?page=$plugin_page&calendar_id={$calendar_id}", "POST", true);

            Admin_Html::html_info(esc_html__('Import from URL', 'stec'));
            Admin_Html::html_input('ics_url', '', '', esc_html__('URL', 'stec'), false);

            Admin_Html::html_info(esc_html__('Import from .ics file', 'stec'));
            Admin_Html::html_input('ics_filename', '', '', esc_html__('ICS File', 'stec'), false, "file", 'accept=.ics');

            Admin_Html::html_info(esc_html__('Event Icon', 'stec'));
            Admin_Html::html_icon('icon', Admin_Helper::get_icon_list(), $calendar->get_icon(), true);

            Admin_Html::html_checkbox('create_locations', 1, 1, esc_html__('Create locations', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Add the event location to Locations if it does not exist.', 'stec'));

            Admin_Html::html_checkbox('ignore_expired', 1, 1, esc_html__("Don't import expired events", 'stec'), false);
            Admin_Html::html_desc(esc_html__('If checked non-recurring expired events will be ignored.', 'stec'));

            Admin_Html::html_checkbox('overwrite_events', 0, 0, esc_html__('Overwrite events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Existing events will be overwritten.', 'stec'));

            Admin_Html::html_checkbox('delete_removed', 0, 0, esc_html__('Delete events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Delete calendar event if it does not exist in the ics file.', 'stec'));

            Admin_Html::html_hidden('calendar_id', $calendar_id);
            Admin_Html::html_hidden('task', 'import');
            Admin_Html::html_button(esc_html__('Import Events', 'stec'));

            Admin_Html::html_form_end();
            ?>


        </div>

        <h1><?php esc_html_e('Auto-import events from URL', 'stec'); ?></h1>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('Schedule Cronjob', 'stec'); ?></h2>

            <?php
            Admin_Html::html_form_start("?page=$plugin_page&calendar_id={$calendar_id}");

            Admin_Html::html_info(esc_html__('ICS Import URL', 'stec'));
            Admin_Html::html_input('ics_url', '', '', esc_html__('URL', 'stec'), false);

            Admin_Html::html_info(esc_html__('Cronjob Frequency', 'stec'));
            Admin_Html::html_select('ics_cronjob_freq', array(
                '0' => esc_html__('Hourly', 'stec'),
                '1' => esc_html__('Twice Daily', 'stec'),
                '2' => esc_html__('Daily', 'stec'),
                '3' => esc_html__('Weekly', 'stec')
                    ), 0, false);

            Admin_Html::html_info(esc_html__('Event Icon', 'stec'));
            Admin_Html::html_icon('icon', Admin_Helper::get_icon_list(), $calendar->get_icon(), true);

            Admin_Html::html_checkbox('create_locations', 1, 1, esc_html__('Create locations', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Add the event location to Locations if it does not exist.', 'stec'));

            Admin_Html::html_checkbox('ignore_expired', 1, 1, esc_html__("Don't import expired events", 'stec'), false);
            Admin_Html::html_desc(esc_html__('If checked non-recurring expired events will be ignored.', 'stec'));

            Admin_Html::html_checkbox('overwrite_events', 0, 0, esc_html__('Overwrite events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Existing events will be overwritten.', 'stec'));

            Admin_Html::html_checkbox('delete_removed', 0, 0, esc_html__('Delete events', 'stec'), false);
            Admin_Html::html_desc(esc_html__('Delete calendar event if it does not exist in the ics file.', 'stec'));

            Admin_Html::html_hidden('calendar_id', $calendar_id);
            Admin_Html::html_hidden('task', 'create_cronjob');
            Admin_Html::html_button(esc_html__('Create Cronjob', 'stec'));

            Admin_Html::html_form_end();
            ?>

        </div>

        <h1><?php esc_html_e('Manage Cronjob list', 'stec'); ?></h1>

        <div class="stachethemes-admin-section">

            <h2><?php esc_html_e('List of your created cronjobs', 'stec'); ?></h2>

            <?php
            if (empty($cronjobs)) :

                Admin_Html::html_info(esc_html__('(No cronjobs to display)', 'stec'));

            else :
                ?>
                <div class="stec-list-bulk">
                    <div>
                        <input type="checkbox" name="all_cronjobs" value="1" /> <?php esc_html__('Select all jobs', 'stec'); ?>
                        <p><?php esc_html_e('Selected all jobs', 'stec'); ?></p>

                        <?php Admin_Html::html_form_start("?page=$plugin_page&calendar_id={$calendar_id}"); ?>

                        <button class='delete-all-items' data-confirm='<?php esc_html_e('Delete all jobs?', 'stec'); ?>'><p><i class="fas fa-trash"></i><?php esc_html_e('Delete selected jobs', 'stec'); ?></p></button>

                        <?php
                        Admin_Html::html_hidden('calendar_id', $calendar_id);
                        Admin_Html::html_hidden('task', 'delete_bulk');
                        ?>

                        <?php Admin_Html::html_form_end(); ?>
                    </div>   
                </div>
                <?php
                echo '<ul class="stec-list">';

                foreach ($cronjobs as $job) :

                    $importer = $job->get_custom_meta('import');
                    $freq     = $job->get_custom_meta('freq');

                    if (!$importer instanceof Import || is_nan($freq)) {
                        continue;
                    }

                    $calendar = new Calendar_Post($importer->get_calendar_id());
                    $icon     = $importer->get_icon() ? $importer->get_icon() : $calendar->get_icon();
                    ?>
                    <li>
                        <div class='calinfo'>
                            <input type="checkbox" name="id" value="<?php esc_attr_e($job->get_id()); ?>" />
                            <p class='color' style='background: <?php esc_attr_e($calendar->get_color()); ?>'><i class="<?php esc_attr_e($icon); ?>"></i></p>
                            <p class='title'>
                                <?php echo htmlspecialchars($importer->get_ics_url()); ?>
                                (<?php
                                switch ($freq) :
                                    case 0: esc_html_e('hourly', 'stec');
                                        break;
                                    case 1: esc_html_e('twice daily', 'stec');
                                        break;
                                    case 2: esc_html_e('daily', 'stec');
                                        break;
                                    case 3: esc_html_e('once weekly', 'stec');
                                        break;
                                endswitch;

                                if ((int) $importer->get_create_locations() === 1) :
                                    echo ', ' . esc_html__('locations', 'stec');
                                endif;

                                if ((int) $importer->get_overwrite_events() === 1) :
                                    echo ', ' . esc_html__('overwrites', 'stec');
                                endif;

                                if ((int) $importer->get_ignore_expired() === 1) :
                                    echo ', ' . esc_html__('ignores expired', 'stec');
                                endif;

                                if ((int) $importer->get_delete_removed() === 1) :
                                    echo ', ' . esc_html__('deletes obsolete', 'stec');
                                endif;
                                ?>)
                            </p>
                        </div>
                        <div class='ctrl'>
                            <a class='delete-item' data-confirm='<?php esc_html_e('Delete item?', 'stec'); ?>' href='<?php echo esc_url(wp_nonce_url("?page={$plugin_page}&task=delete&id={$job->get_id()}&calendar_id={$calendar_id}")); ?>'><?php esc_html_e('Delete', 'stec'); ?></a>
                        </div>
                    </li>
                    <?php
                endforeach;

                echo '</ul>';
            endif;
            ?>

        </div>

    <?php endif; ?>

</div>

