<?php

namespace Stachethemes\Stec;

global $plugin_page;

$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

// GET
switch (Admin_Helper::get("task")) {

    case 'delete' :
        Admin_Helper::verify_nonce('', 'GET');

        $location_id = Admin_Helper::get('location_id', 0, FILTER_VALIDATE_INT);
        $location    = new Location($location_id);
        $result      = $location->delete();

        if (!is_wp_error($result)) {
            Admin_Helper::set_message(esc_html__('Location deleted', 'stec'));
        } else {
            Admin_Helper::set_message($result->get_error_message(), 'error');
        }

        break;
}

// POST
switch (Admin_Helper::post("task")) {

    case 'update' :
    case 'create' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $location_id = Admin_Helper::post('location_id', null);
        $location    = new Location($location_id);

        $location->set_title(Admin_Helper::post('title', ''));
        $location->set_address(Admin_Helper::post('address', ''));
        $location->set_city(Admin_Helper::post('city', ''));
        $location->set_country(Admin_Helper::post('country', ''));
        $location->set_coordinates(Admin_Helper::post('coordinates', null));
        $location->set_details(Admin_Helper::post('details', ''));
        $location->set_notes(Admin_Helper::post('notes'));
        $location->set_visibility(Admin_Helper::post('visibility'));

        try {

            if ('' === trim($location->get_full_address())) {
                throw new Stec_Exception(esc_html__('Location cannot be empty', 'stec'));
            }

            $result = $location->insert_location();

            if ($result) {
                Admin_Helper::set_message(sprintf(esc_html_x('Location %s', 'created/updated', 'stec'), $location->get_id() ? esc_html__('updated', 'stec') : esc_html__('created', 'stec')));
            } else {
                Admin_Helper::set_message(sprintf(esc_html_x('Error %s location', 'creating/updating', 'stec'), $location->get_id() ? esc_html__('updating', 'stec') : esc_html__('creating', 'stec')), 'error');
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_bulk' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $locations = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        if ($locations) {

            $result = Locations::bulk_delete($locations);

            if ($result) {
                Admin_Helper::set_message(esc_html__('Locations deleted', 'stec'));
            } else {
                Admin_Helper::set_message(esc_html__('Unable to delete locations', 'stec'), 'error');
            }
        }

        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case "edit" :

        $location_id = Admin_Helper::get('location_id');
        $location    = new Location($location_id);
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/locations/addedit.php';

        break;

    case "add" :

        $location_id = false;
        $location    = false;

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/locations/addedit.php';

        break;

    case "list":
    default:

        $locations = Locations::get_locations();
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/locations/list.php';
endswitch;
