<?php

namespace Stachethemes\Stec;

global $plugin_page;

$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

// GET
switch (Admin_Helper::get("task")) {

    case 'delete' :
        Admin_Helper::verify_nonce('', 'GET');

        $organizer_id = Admin_Helper::get('organizer_id', 0, FILTER_VALIDATE_INT);
        $organizer    = new Organizer($organizer_id);
        $result       = $organizer->delete();

        if (!is_wp_error($result)) {
            Admin_Helper::set_message(esc_html__('Organizer deleted', 'stec'));
        } else {
            Admin_Helper::set_message($result->get_error_message(), 'error');
        }

        break;
}

// POST
switch (Admin_Helper::post("task")) {

    case 'update' :
    case 'create' :
        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $organizer_id = Admin_Helper::post('organizer_id', null);
        $organizer    = new Organizer($organizer_id);

        $organizer->set_photo(Admin_Helper::post('photo', '', FILTER_VALIDATE_INT));
        $organizer->set_name(Admin_Helper::post('name', '', FILTER_SANITIZE_STRING));
        $organizer->set_email(Admin_Helper::post('email', '', FILTER_VALIDATE_EMAIL));
        $organizer->set_details(Admin_Helper::post('details', '', FILTER_DEFAULT));
        $organizer->set_social(Admin_Helper::post('social', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY));
        $organizer->set_notes(Admin_Helper::post('notes'));
        $organizer->set_visibility(Admin_Helper::post('visibility'));


        try {
            $result = $organizer->insert_organizer();

            if ($result) {
                Admin_Helper::set_message(sprintf(esc_html_x('Organizer %s', 'created/updated', 'stec'), $organizer->get_id() ? esc_html__('updated', 'stec') : esc_html__('created', 'stec')));
            } else {
                Admin_Helper::set_message(sprintf(esc_html_x('Error %s organizer', 'creating/updating', 'stec'), $organizer->get_id() ? esc_html__('updating', 'stec') : esc_html__('creating', 'stec')), 'error');
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_bulk' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $organizers = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        if ($organizers) {

            $result = Organizers::bulk_delete($organizers);

            if ($result) {
                Admin_Helper::set_message(esc_html__('Organizers deleted', 'stec'));
            } else {
                Admin_Helper::set_message(esc_html__('Unable to delete organizers', 'stec'), 'error');
            }
        }

        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case "edit" :

        $organizer_id = Admin_Helper::get('organizer_id');
        $organizer    = new Organizer($organizer_id);

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/organizers/addedit.php';

        break;

    case "add" :

        $organizer_id = false;
        $organizer    = false;

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/organizers/addedit.php';

        break;

    case "list":
    default:

        $has_organizer_page = Admin_Helper::get_stec_page('organizer');
        $organizers         = Organizers::get_organizers();

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/organizers/list.php';
endswitch;
