<?php

namespace Stachethemes\Stec;

global $plugin_page;

$stachethemes_ec_main = Stachethemes_Event_Calendar::get_instance();

// GET
switch (Admin_Helper::get("task")) {

    case 'delete' :

        Admin_Helper::verify_nonce('', 'GET');

        $ticket_id = Admin_Helper::get('ticket_id', 0, FILTER_VALIDATE_INT);
        $result    = new \WC_Product_Stec_Bookable($ticket_id);

        try {
            if (true === Booking::user_can_manage_ticket($ticket_id)) {
                $result->delete(true);
                Admin_Helper::set_message(esc_html__('Ticket deleted', 'stec'));
            } else {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }


        break;
}

// POST
switch (Admin_Helper::post("task")) {

    case 'update' :
    case 'create' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $ticket_id        = Admin_Helper::post('wc_book_create_ticket_id', false, FILTER_VALIDATE_INT);
        $requires_ticket  = Admin_Helper::post('wc_book_requires_ticket', false, FILTER_VALIDATE_INT);
        $requires_role    = Admin_Helper::post('wc_book_requires_role', false, FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
        $title            = Admin_Helper::post('wc_book_create_ticket_name', false, FILTER_SANITIZE_STRING);
        $price            = Admin_Helper::post('wc_book_create_ticket_price', '', FILTER_VALIDATE_FLOAT);
        $sku              = Admin_Helper::post('wc_book_create_ticket_sku', false, FILTER_SANITIZE_STRING);
        $qty              = Admin_Helper::post('wc_book_create_ticket_qty', false, FILTER_VALIDATE_INT);
        $qty_per_event    = Admin_Helper::post('wc_book_create_ticket_qty_per_event', false, FILTER_VALIDATE_INT);
        $desc             = Admin_Helper::post('wc_book_create_ticket_description', false, FILTER_SANITIZE_STRING);
        $desc_short       = Admin_Helper::post('wc_book_create_ticket_desc_short', false, FILTER_SANITIZE_STRING);
        $attrib_fields    = Admin_Helper::post('wc_book_create_ticket_addinfo_field', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
        $admin_visibility = Admin_Helper::post('wc_book_create_ticket_admin_visibility', 'private', FILTER_SANITIZE_STRING);
        $stop_before      = Admin_Helper::post('wc_book_create_ticket_stop_purchase', false, FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

        try {

            if ($ticket_id) {
                if (false === Booking::user_can_manage_ticket($ticket_id)) {
                    throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
                }
            }

            $product = new \WC_Product_Stec_Bookable($ticket_id);
            $product->set_name($title);
            $product->set_price($price);
            $product->set_regular_price($price);
            $product->set_sku($sku);
            $product->set_manage_stock(true);
            $product->set_stock_quantity($qty);
            $product->set_description($desc);
            $product->set_short_description($desc_short);
            $product->set_quantity_limit_per_event($qty_per_event);
            $product->set_admin_visibility($admin_visibility);


            if ($requires_ticket && $requires_ticket > 0) {
                $product->update_meta_data('stec_requires_ticket', array($requires_ticket));
            } else {
                $product->delete_meta_data('stec_requires_ticket');
            }

            if (is_array($requires_role) && $requires_role) {
                $product->update_meta_data('stec_requires_role', $requires_role);
            } else {
                $product->delete_meta_data('stec_requires_role');
            }

            if (!$qty) {
                $product->set_manage_stock(false);
                $product->set_stock_quantity(0);
                $product->set_stock_status();
            }

            if ($attrib_fields) {
                $attributes = array();
                foreach ($attrib_fields as $attrib_field) {

                    if (trim($attrib_field['name']) === '') {
                        continue;
                    }

                    $attr         = new \WC_Product_Attribute();
                    $attr->set_name($attrib_field['name']);
                    $attr->set_options(explode('|', $attrib_field['value']));
                    $attr->set_visible(true);
                    $attributes[] = $attr;
                }
                $product->set_attributes($attributes);
            } else {
                $product->set_attributes(array());
            }

            if (is_array($stop_before)) {
                $stop_before_units = isset($stop_before['units']) ? $stop_before['units'] : 'hours';
                $stop_before_value = isset($stop_before['value']) ? (int) $stop_before['value'] : 0;

                switch ($stop_before_units) :
                    case 'days' :
                        $product->set_stop_before($stop_before_value * 24 * 3600);
                        break;

                    case 'hours';
                        $product->set_stop_before($stop_before_value * 3600);
                        break;

                    default:
                        $product->set_stop_before(0);
                endswitch;
            }

            $result = $product->save();

            if ($result) {

                Admin_Helper::set_message(sprintf(esc_html_x('Ticket %s', 'created/updated', 'stec'), $ticket_id ? esc_html__('updated', 'stec') : esc_html__('created', 'stec')));
            } else {

                Admin_Helper::set_message(sprintf(esc_html_x('Error %s event', 'creating/updating', 'stec'), $ticket_id ? esc_html__('updating', 'stec') : esc_html__('creating', 'stec')), 'error');
            }
        } catch (\WC_Data_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }

        break;

    case 'delete_bulk' :

        Admin_Helper::verify_nonce("?page={$plugin_page}");

        $tickets = Admin_Helper::post('idlist', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        try {

            if ($tickets) {
                foreach ($tickets as $ticket_id) {
                    $ticket = new \WC_Product_Stec_Bookable($ticket_id);

                    if (true === Booking::user_can_manage_ticket($ticket_id)) {
                        $ticket->delete(true);
                    } else {
                        throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
                    }
                }
            }

            Admin_Helper::set_message(esc_html__('Tickets deleted', 'stec'));
        } catch (Stec_Exception $ex) {
            Admin_Helper::set_message($ex->getMessage(), 'error');
        }


        break;
}

// VIEW
switch (Admin_Helper::get("view")) :

    case "edit" :

        $ticket_id = Admin_Helper::get('ticket_id');
        $ticket    = new \WC_Product_Stec_Bookable($ticket_id);

        $tickets_array_list    = Booking::get_bookable_products_as_array_list();
        $tickets_array_list[0] = esc_html__('No requirements', 'stec');
        unset($tickets_array_list[$ticket_id]);
        ksort($tickets_array_list);


        include $stachethemes_ec_main->get_path('VIEW') . 'admin/tickets/addedit.php';

        break;

    case "add" :

        $ticket_id = false;
        $ticket    = false;

        $tickets_array_list    = Booking::get_bookable_products_as_array_list();
        $tickets_array_list[0] = esc_html__('No requirements', 'stec');
        ksort($tickets_array_list);
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/tickets/addedit.php';

        break;

    case "orders":

        $ticket_id = Admin_Helper::get('ticket_id', 0, FILTER_VALIDATE_INT);
        $orders    = Booking::get_ticket_orders($ticket_id);
        include $stachethemes_ec_main->get_path('VIEW') . 'admin/tickets/orders.php';

        break;
    case "list":
    default:

        $tickets = Booking::get_bookable_products_as_array_list();

        include $stachethemes_ec_main->get_path('VIEW') . 'admin/tickets/list.php';
endswitch;
