<?php

namespace Stachethemes\Stec; ?>
<!DOCTYPE html>
<html>
    <head>
        <style>
            * {
                box-sizing: border-box;
            }
            body {
                margin:0;
                padding:0;
            }
            .left {
                display: flex;
                justify-content: center;
                align-items: center;
                position: fixed;
                height: 100%;
                left:0;
                top:0;
                width: 50%;
                overflow-y: auto;
            }
            .content {
                padding: 120px;
            }
            .right {
                width: 50%;
                position: fixed;
                height: 100%;
                right:0;
                top:0;
            }
            ul { margin:0 ;}
            h1 {
                font-family: Arial;
                font-weight: bold;
                font-size: 36px;
                color:#4e4e4e;
                margin:0;
                padding:0;
                line-height: 1.6;
            }
            .icon {
                position: absolute;
                top:25px;
                left: 25px;
                border-radius: 3px;
                width: 55px;
                height: 55px;
                text-align: center;
            }
            .icon i {
                line-height: 55px;
                font-size: 30px;
                color:#fff;
            }
            .date, .location {
                font-family: Arial;
                font-size: 13px;
                font-weight: normal;
                color: #999da2; 
                margin-top: 5px;
            }
            .date i, .location i {
                min-width: 15px;
                text-align: center;
            }
            .description {
                margin:50px 0;

                font-family: Arial;
                font-weight: normal;
                font-size: 16px;
                color:#999da2;
                line-height: 1.6;
            }
            .link-button {
                width: 100%;
                padding: 15px 10px;
                float: left;
                text-align: center;
                color: #fff;
                text-decoration: none;
                border-radius: 3px;
                font-family: Arial;
                font-size: 13px;
                font-weight: normal;
            }

            .right {
                display: flex;
                justify-content: center;
                align-items: center;
            }

            .clock {
                padding:0;
                margin:0;
                list-style: none;
                position: relative;
                z-index: 3;
            }

            .clock li {
                float: left;
                padding: 0 10px;
                text-align: center;
            }

            .clock li p {
                font-family: Arial;
                font-weight: bold;
                font-size: 72px;
                color:#fff;
                line-height: 1;
                margin:0;
            }

            .clock li span {
                font-family: Arial;
                font-weight: normal;
                font-size: 24px;
                color:#fff;
            }

            .images {
                position: absolute;
                top:0;
                left: 0;
                width: 100%;
                height: 100%;
                list-style: none;
            }

            .images::after {
                content: '';
                position: absolute;
                top:0;
                left: 0;
                width: 100%;
                height: 100%;
                background: rgba(0,0,0,0.3);
                z-index: 3;
            }

            .images li {
                position: absolute;
                top:0;
                left: 0;
                width: 100%;
                height: 100%;
                list-style: none;
                background-size: cover;
                background-position: 50% 50%;
                opacity: 0;
                -webkit-transition: opacity 1.25s ease-in-out;
                transition: opacity 1.25s ease-in-out;
            }

            .images li.transition {
                opacity: 1;
                z-index: 2;
            }

            .images li.active {
                opacity: 1;
                z-index: 1;
            }

            @media only screen and (max-width: 1100px) {
                .left, .right {
                    width: 100%;
                    position: relative;
                    float: left;
                }

                .right {
                    height: 400px;
                }

                .content {
                    padding: 20px;
                    margin-top: 100px;
                }
            }

            @media only screen and (max-width: 700px) {
                .clock li p {
                    font-size: 50px !important;
                }
            }

        </style>
        <?php
        $stec = Stachethemes_Event_Calendar::get_instance();
        ?>
        <link rel='stylesheet'href='<?php echo plugins_url('css/fonts/css/all.min.css', $stec->get_path('ASSETS') . '/.'); ?>' type='text/css' media='all' />
        <title><?php wp_title(''); ?></title>
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <?php
        try {
            if (have_posts()) {
                while (have_posts()) {
                    the_post();
                    $content  = strip_tags(get_the_content());
                    $event_id = (int) ($content);

                    if (!$event_id) {
                        throw new Stec_Exception(esc_html('Please provide a valid event id in the content area', 'stec'));
                    }

                    $event      = new Event_Post($event_id);
                    $now        = new \DateTime('now', new \DateTimeZone($event->get_timezone()));
                    $start_date = new \DateTime($event->get_start_date(), new \DateTimeZone($event->get_timezone()));
                    $total_left = $start_date->getTimestamp() - $now->getTimestamp();
                    $days       = max(0, floor($total_left / 86400));
                    $hours      = max(0, floor($total_left % 86400 / 3600));
                    $minutes    = max(0, floor($total_left % 86400 % 3600 / 60));
                    $seconds    = max(0, floor($total_left % 86400 % 3600 % 60));
                    ?>

                    <div class="icon" style="background-color: <?php esc_attr_e($event->get_color()); ?>">
                        <i class="<?php esc_attr_e($event->get_icon()); ?>"></i>
                    </div>

                    <div class="left">

                        <div class="content">
                            <h1><?php esc_html_e($event->get_title()); ?></h1>

                            <div class="date"><i class="far fa-clock"></i>
                                <?php echo Admin_Helper::get_the_timespan($event, 0, true); ?>
                            </div>

                            <?php if ($event->get_location()) : ?>
                                <div class="location"><i class="fas fa-map-marker-alt"></i>
                                    <?php
                                    $location_data = $event->get_parsed_location();
                                    esc_html_e($location_data->full_address);
                                    ?>
                                </div>
                            <?php endif; ?>

                            <div class="description">
                                <?php echo wpautop(do_shortcode($event->get_description())); ?>
                            </div>

                            <?php if ($event->get_link()) : ?>
                                <a class="link-button" style="background-color: <?php esc_attr_e($event->get_color()); ?>" href="<?php esc_url($event->get_link()); ?>"><?php esc_html_e('Visit Website', 'stec'); ?></a>
                            <?php endif; ?>
                        </div>
                    </div>

                    <div class="right" style="background-color: <?php esc_attr_e($event->get_color()); ?>">

                        <?php
                        $images = $event->get_parsed_images();
                        if ($images) {
                            echo '<ul class="images">';
                            foreach ($images as $image) {
                                echo '<li style="background-image:url(' . $image->src . ')"></li>';
                            }
                            echo '</ul>';
                        }
                        ?>

                        <ul class="clock">
                            <li data-days="<?php echo (int) $days; ?>"><p><?php echo sprintf("%02d", (int) $days); ?></p><span><?php esc_html_e('Days', 'stec'); ?></span></li>
                            <li data-hours="<?php echo (int) $hours; ?>"><p><?php echo sprintf("%02d", (int) $hours); ?></p><span><?php esc_html_e('Hours', 'stec'); ?></span></li>
                            <li data-minutes="<?php echo (int) $minutes; ?>"><p><?php echo sprintf("%02d", (int) $minutes); ?></p><span><?php esc_html_e('Minutes', 'stec'); ?></span></li>
                            <li data-seconds="<?php echo (int) $seconds; ?>"><p><?php echo sprintf("%02d", (int) $seconds); ?></p><span><?php esc_html_e('Seconds', 'stec'); ?></span></li>
                        </ul>
                    </div>

                    <?php
                }
            }
        } catch (Stec_Exception $ex) {
            esc_html_e($ex->getMessage());
        }

        wp_enqueue_script('jquery');
        wp_footer();
        ?>
        <script type="text/javascript">
            (function ($) {

                function digit(number) {
                    return ("0" + number).slice(-2);
                }

                $(function () {

                    var $days = $('.clock').find('li[data-days]');
                    var $hours = $('.clock').find('li[data-hours]');
                    var $minutes = $('.clock').find('li[data-minutes]');
                    var $seconds = $('.clock').find('li[data-seconds]');

                    var clockInterval = setInterval(function () {

                        var seconds = parseInt($seconds.data('seconds'), 10);
                        var minutes = parseInt($minutes.data('minutes'), 10);
                        var hours = parseInt($hours.data('hours'), 10);
                        var days = parseInt($days.data('days'), 10);

                        if (seconds === 0 && minutes === 0 && hours === 0 && days === 0) {
                            clearInterval(clockInterval);
                            return;
                        }

                        seconds--;

                        if (seconds < 0) {
                            seconds = 59;

                            minutes--;

                            if (minutes < 0) {
                                minutes = 59;

                                hours--;

                                if (hours < 0) {
                                    hours = 23;

                                    days = Math.max(0, days--);

                                    $days.data('days', days);
                                    $days.find('p').text(digit(days));

                                }

                                $hours.data('hours', hours);
                                $hours.find('p').text(digit(hours));

                            }

                            $minutes.data('minutes', minutes);
                            $minutes.find('p').text(digit(minutes));
                        }

                        $seconds.data('seconds', seconds);
                        $seconds.find('p').text(digit(seconds));

                    }, 1000);

                    if ($('.images').length > 0) {
                        $('.images').children().first().addClass('active');

                        if ($('.images').children().length > 1) {

                            var imagesInterval = setInterval(function () {

                                var current = $('.images').children().filter('.active');
                                var next = current.next();

                                if (next.length <= 0) {
                                    next = $('.images').children().first();
                                }

                                next.addClass('transition');

                                setTimeout(function () {

                                    current.removeClass('active transition');
                                    next.addClass('active').removeClass('transition');

                                }, 1250);

                            }, 1000 * 6);
                        }
                    }
                });
            })(jQuery);
        </script>
    </body>
</html>