<?php

namespace Stachethemes\Stec;

$stachethemes_ec = Stachethemes_Event_Calendar::get_instance();

do_action('stec_single_before_content');
?>
<div class="stec-layout-single" itemscope itemtype="//schema.org/Event">

    <div class="stec-layout-single-preloader-template">
        <div class="stec-layout-single-preloader"></div>
    </div>

    <div class="stec-layout-single-preview">

        <div class="stec-layout-single-preview-left">

            <?php
            $event_color = $event->get_color();
            if ($event->get_categories() && 1 === (int) Settings::get_admin_setting_value('stec_menu__general', 'use_cat_color')) {
                $categories  = $event->get_parsed_categories();
                $event_color = $categories[0]['color'];
                unset($categories);
            }
            ?>

            <div style="background:<?php esc_attr_e($event_color); ?>" class="stec-layout-single-preview-left-icon <?php
            if ($event->get_icon_type() === 'icon' && $event->get_icon() === 'fa') :
                echo 'stec-layout-single-no-icon';
            endif;
            ?>">

                <?php
                switch ($event->get_icon_type()) :

                    case 'date' :

                        $d   = $event->get_start_date('j', $repeat_offset);
                        $mmm = date_i18n('M', $event->get_start_date('U', $repeat_offset));
                        ?>
                        <span class="stec-layout-single-preview-left-icon-date"><span class="stec-layout-single-preview-left-icon-date-number"><?php esc_html_e($d); ?></span><span class="stec-layout-single-preview-left-icon-date-month"><?php esc_html_e($mmm); ?></span></span>
                        <?php
                        break;

                    case 'image' :

                        if ($event->get_icon_images()) {
                            ?>
                            <div class="stec-layout-single-preview-left-icon-images">
                                <?php
                                foreach ($event->get_parsed_icon_images() as $k => $image) {
                                    ?><div class="stec-layout-single-preview-left-icon-image <?php echo (int) $k === 0 ? 'active' : '' ?>" style="background-image: url(<?php echo esc_url($image->src); ?>)"></div><?php
                                }
                                ?>
                            </div>
                            <?php
                        }
                        break;

                    case 'icon' :
                        ?><i class="<?php esc_attr_e($event->get_icon()); ?>"></i><?php
                        break;

                endswitch;
                ?>

            </div>

            <div class="stec-layout-single-preview-left-text">

                <?php if ($event->get_featured() >= 1) : ?>
                    <p class="stec-layout-single-preview-left-text-featured">
                        <i class="fa fa-star"></i>
                        <span><?php esc_html_e('Featured', 'stec'); ?></span>
                    </p>
                <?php endif; ?>

                <h1 class="stec-layout-single-preview-left-text-title" itemprop="name"><?php esc_html_e($event->get_title()); ?></h1>

                <p class="stec-layout-single-preview-left-text-date stec-get-the-timespan"><?php echo Admin_Helper::get_the_timespan($event, $repeat_offset); ?></p>

                <?php Admin_Helper::meta_schema_datetime_iso8601($event, $repeat_offset); ?>

                <div class="stec-layout-single-preview-separator">
                    <?php if ($event->get_edit_link()) : ?>

                        <a href="<?php echo esc_url($event->get_edit_link()); ?>" class="stec-layout-single-preview-left-edit">
                            <i class="far fa-edit"></i>
                        </a>

                    <?php endif; ?>

                    <?php if (!Admin_Helper::reminder_expired($event, $repeat_offset)) : ?>
                        <a class="stec-layout-single-preview-left-reminder-toggle" href="javascript:void(0);">
                            <i class="fa fa-bell"></i>
                        </a>
                    <?php endif; ?>

                </div>
            </div>
        </div>

        <div class="stec-layout-single-preview-right">
            <?php if ($event->get_edit_link()) : ?>

                <a href="<?php echo esc_url($event->get_edit_link()); ?>" class="stec-layout-single-preview-right-edit">
                    <i class="far fa-edit"></i>
                </a>

            <?php endif; ?>

            <?php if (!Admin_Helper::reminder_expired($event, $repeat_offset)) : ?>

                <div class="stec-layout-single-preview-right-reminder">
                    <i class="fa fa-bell"></i>
                </div>

            <?php endif; ?>
        </div>

    </div>

    <?php if ($event->get_images()) : ?>

        <div class="stec-layout-single-media">

            <div class="stec-layout-single-media-content">

                <?php
                foreach ($event->get_parsed_images() as $image) :
                    ?>
                    <div style="background-image:url(<?php echo esc_url($image->src); ?>);">
                        <img alt="<?php esc_attr_e($image->title); ?>" src="<?php echo esc_url($image->src); ?>">
                        <meta itemprop="image" content="<?php echo esc_url($image->src); ?>" />
                    </div>
                    <?php
                endforeach;
                ?>

            </div>

            <div class="stec-layout-single-media-content-subs">
                <div>
                    <p></p>
                    <span></span>
                </div>
            </div>

            <div class="stec-layout-single-media-controls">
                <div class="stec-layout-single-media-controls-prev stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-left"></i></div>
                <div class="stec-layout-single-media-controls-list-wrap">
                    <ul class="stec-layout-single-media-controls-list">

                        <?php
                        if (($event->get_images())) :
                            foreach ($event->get_parsed_images() as $image) :
                                $image = (object) $image;
                                ?>
                                <li style="background-image: url(<?php echo esc_url($image->thumb); ?>);"></li>
                                <?php
                            endforeach;
                        endif;
                        ?>

                    </ul>
                </div>
                <div class="stec-layout-single-media-controls-next stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-right"></i></div>
            </div>

        </div>

    <?php endif; ?>

    <?php
    if ($event->get_categories() && Settings::get_admin_setting_value('stec_menu__general', 'show_event_tags') === '1') :
        ?>
        <div class="stec-layout-single-tags">
            <?php
            foreach ($event->get_parsed_categories() as $category) {
                ?><span class="stec-layout-single-tag" style="background-color: <?php esc_attr_e($category['color']); ?>"><?php esc_html_e($category['title']) ?></span><?php
            }
            ?>
        </div>
        <?php
    endif;
    ?>

    <?php
    if ($has_tickets) :
        ?>
        <div class="stec-layout-single-section">
            <button type="button" class="stec-style-button stec-layout-single-tickets-default" data-action="open-tickets"><?php esc_html_e('Tickets', 'stec'); ?></button>
        </div>
        <?php
    endif;
    ?>

    <?php if ($event->get_description() !== '') : ?>

        <div class="stec-layout-single-section stec-layout-single-description stec-layout-event-text-fontandcolor" itemprop="description">
            <?php echo Admin_Helper::esc($event->get_description(), 'noscript'); ?>
        </div>

    <?php endif; ?>

    <?php
    if ($event->get_organizers()) :
        ?>            
        <ul class="stec-layout-single-section stec-layout-single-organizers">

            <?php
            $organizers = $event->get_organizers();

            foreach ($organizers as $organizer_id) :

                $organizer = new Organizer((int) $organizer_id);

                if (!$organizer->get_id()) {
                    continue;
                }
                ?>
                <li class="stec-layout-single-organizers-organizer">
                    <div class="stec-layout-single-organizers-organizer-image">
                        <?php if ($organizer->get_photo_src()) : ?>
                            <div class="stec-layout-single-organizers-organizer-image-photo" style="background-image:url(<?php echo esc_url($organizer->get_photo_src()); ?>)"></div>
                        <?php else: ?> 
                            <div class="stec-layout-single-organizers-organizer-image-default"></div>
                        <?php endif; ?>
                    </div>
                    <div class="stec-layout-single-organizers-organizer-content">
                        <div class="stec-layout-single-organizers-organizer-title stec-layout-event-title2-fontandcolor"><?php
                            if ($organizer->get_permalink()) {
                                printf("<a href='%s'>%s</a>", esc_url($organizer->get_permalink()), esc_html($organizer->get_name()));
                            } else {
                                esc_html_e($organizer->get_name());
                            }
                            ?></div>
                        <div class="stec-layout-single-organizers-organizer-sub-title"><?php printf(esc_html__('Organizer of %s', 'stec'), $event->get_title()); ?></div>
                        <div class="stec-layout-single-organizers-organizer-description stec-layout-event-text-fontandcolor"><?php esc_html_e($organizer->get_details()); ?></div>
                        <div class="stec-layout-single-organizers-organizer-social">
                            <?php
                            if ($organizer->get_social()) :
                                foreach ($organizer->get_social() as $social) {
                                    if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                                        printf("<a href='mailto:%s'><i class='%s'></i></a>",
                                                $social['link'],
                                                esc_attr($social['ico'])
                                        );
                                    } else {
                                        printf("<a target='_blank' href='%s'><i class='%s'></i></a>",
                                                esc_url($social['link']),
                                                esc_attr($social['ico'])
                                        );
                                    }
                                }
                            endif;
                            ?>
                        </div>
                    </div>
                </li>
            <?php endforeach; ?>
        </ul>
    <?php endif; ?>

    <div class="stec-layout-single-section stec-style-full-width stec-style-float-left">

        <?php if ($has_tickets) : ?>
            <button type="button" class="stec-style-button stec-style-full-width stec-layout-single-tickets-small" data-action="open-tickets"><?php esc_html_e('Tickets', 'stec'); ?></button>
        <?php endif; ?>

        <div class="stec-style-flexbox-no-media stec-style-full-width stec-style-float-left">

            <?php if ($event->get_link() !== '') : ?>
                <a href="<?php echo esc_url($event->get_link()); ?>" class="stec-layout-single-external-link stec-style-full-width stec-style-button stec-layout-event-btn-fontandcolor" target="_BLANK"><?php esc_html_e('Visit Website', 'stec'); ?></a>
            <?php endif; ?>

            <?php if (false === $rsvp_expired && 1 === (int) $event->get_rsvp() && false === Admin_Helper::user_is_invited($event)) : ?>
                <a class="stec-layout-single-rsvp stec-style-full-width stec-style-button stec-layout-event-btn-fontandcolor" href="javascript:void(0);"><?php esc_html_e('RSVP to Event', 'stec'); ?></a>
            <?php endif; ?>
        </div>
    </div>

    <?php if ((int) $event->get_counter() === 1) : ?>

        <ul class="stec-layout-single-section stec-layout-single-counter">
            <li>
                <p class="stec-layout-single-counter-num">0</p>
                <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Days', 'stec'); ?>" data-singular-label="<?php esc_html_e('Day', 'stec'); ?>">days</p>
            </li>
            <li>
                <p class="stec-layout-single-counter-num">0</p>
                <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Hours', 'stec'); ?>" data-singular-label="<?php esc_html_e('Hour', 'stec'); ?>">hours</p>
            </li>
            <li>
                <p class="stec-layout-single-counter-num">0</p>
                <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Minutes', 'stec'); ?>" data-singular-label="<?php esc_html_e('Minute', 'stec'); ?>">minutes</p>
            </li>
            <li>
                <p class="stec-layout-single-counter-num">0</p>
                <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Seconds', 'stec'); ?>" data-singular-label="<?php esc_html_e('Second', 'stec'); ?>">seconds</p>
            </li>
        </ul>

    <?php endif; ?>

    <?php
    if (Admin_Helper::user_is_invited($event)) :

        $user_status = (int) Admin_Helper::get_user_attendance_status($event, false, $repeat_offset);
        $free_slots  = Rsvp::has_free_slots($event, $event->get_start_date('Y-m-d', $repeat_offset));

        if (1 === $user_status || $free_slots !== false) :
            ?>
            <ul class="stec-layout-single-section stec-layout-single-intro-attendance">
                <li class="stec-layout-single-button-style stec-layout-single-intro-attendance-attend stec-layout-event-btn-fontandcolor <?php
                if ($user_status === 1) {
                    echo 'active';
                }
                ?>"><p><?php esc_html_e('Attend', 'stec'); ?></p></li>

                <li class="stec-layout-single-button-style stec-layout-single-intro-attendance-decline stec-layout-event-btn-fontandcolor <?php
                if ($user_status === 2) {
                    echo 'active';
                }
                ?>"><p><?php esc_html_e('Decline', 'stec'); ?></p></li>
            </ul>

        <?php endif; ?>
    <?php endif; ?>

    <div class="stec-layout-single-section stec-layout-single-event-status">
        <p class="stec-layout-single-event-status-text event-expired stec-layout-event-title2-fontandcolor"><?php esc_html_e('Event expired', 'stec'); ?></p>
        <p class="stec-layout-single-event-status-text event-inprogress stec-layout-event-title2-fontandcolor"><?php esc_html_e('Event is in progress', 'stec'); ?></p>
    </div>

    <div class="stec-single-sections">

        <?php if ($event->get_attachments()) : ?>


            <div class="stec-layout-single-attachments">

                <div class="stec-layout-single-attachments-top">

                    <p class="stec-layout-event-title2-fontandcolor"><i class="far fa-file"></i><?php esc_html_e('Attachments', 'stec'); ?></p>

                    <div class="stec-layout-single-attachments-toggle">
                        <i class="fa fa-plus"></i>
                        <i class="fa fa-minus"></i>
                    </div>

                </div>

                <ul class="stec-layout-single-attachments-list">

                    <?php foreach ($event->get_parsed_attachments() as $file) : ?>

                        <li class="stec-layout-single-attachment">
                            <div>
                                <p class="stec-layout-single-attachment-title stec-layout-event-title2-fontandcolor"><a href="<?php echo esc_url($file->link) ?>"><?php esc_html_e($file->filename); ?></a></p>
                                <p class="stec-layout-single-attachment-desc stec-layout-event-text-fontandcolor"><?php esc_html_e($file->description); ?></p>
                            </div>

                            <div>
                                <a href="<?php echo esc_url($file->link) ?>" class="stec-layout-event-title2-fontandcolor"><?php esc_html_e('Download', 'stec'); ?></a>
                                <p class="stec-layout-single-attachment-size stec-layout-event-text-fontandcolor"><?php esc_html_e($file->size); ?></p>
                            </div>
                        </li>

                    <?php endforeach; ?>
                </ul>
            </div>


        <?php endif; ?>

        <meta itemprop="url" content="<?php echo esc_url($the_permalink); ?>"/>

        <?php
        if (Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1' ||
                Settings::get_admin_setting_value('stec_menu__general', 'show_export_buttons') === '1') :
            ?>  

            <div class="stec-layout-single-share-and-export">

                <div class="stec-layout-single-share">
                    <?php
                    if (Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1') :
                        echo Admin_Helper::get_share_links($the_permalink);
                    endif;
                    ?>
                </div>

                <?php if (Settings::get_admin_setting_value('stec_menu__general', 'show_export_buttons') === '1') : ?>
                    <div class="stec-layout-single-export">
                        <form method="POST"> 
                            <button class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor"><?php esc_html_e('Export to .ICS file', 'stec'); ?></button>
                            <input type="hidden" value="<?php echo (int) $event->get_id(); ?>" name="event_id">
                            <input type="hidden" value="<?php echo (int) $event->get_calid(); ?>" name="calendar_id">
                            <input type="hidden" value="stec_public_export_to_ics" name="task">
                        </form>

                        <a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="<?php echo esc_url($event->get_gcal_link($repeat_offset)); ?>" target="_BLANK" class="stec-layout-event-text-fontandcolor"><?php esc_html_e('Import to Google Calendar', 'stec'); ?></a>
                    </div>
                <?php endif; ?>
            </div>
        <?php endif; ?>

        <?php if ($event->get_location()) : $location = $event->get_parsed_location(); ?>

            <div class="stec-layout-single-location" itemprop="location" itemscope itemtype="//schema.org/Place">

                <div class="stec-layout-single-location-flex">

                    <div class="stec-layout-single-location-left">
                        <p class="stec-layout-event-title2-fontandcolor"><?php esc_html_e('Location', 'stec'); ?></p>
                        <span class="stec-layout-single-location-address stec-layout-event-text-fontandcolor" data-location-coordinates="<?php
                        esc_attr_e($location->coordinates);
                        ?>" itemprop="name"><?php
                                  if ($location->title) {
                                      esc_attr_e(implode(', ', array($location->title, $location->full_address)));
                                  } else {
                                      esc_attr_e($location->full_address);
                                  }
                                  ?></span>
                    </div>

                    <div class="stec-layout-single-location-right">
                        <p class="stec-layout-single-location-directions-title stec-layout-event-title2-fontandcolor"
                           itemprop="address" itemscope itemtype="//schema.org/Text"
                           ><?php esc_html_e('Directions', 'stec'); ?></p>
                    </div>

                </div>

                <div class="stec-layout-single-location-flex">

                    <div class="stec-layout-single-location-left" itemprop="hasMap">
                        <div class="stec-layout-single-location-gmap" itemprop="map" itemtype="//schema.org/Map"></div>
                    </div>

                    <div class="stec-layout-single-location-right">
                        <input class="stec-layout-event-input-fontandcolor" type="text" name="start" placeholder="<?php esc_html_e('Start Location', 'stec'); ?>" />
                        <input class="stec-layout-event-input-fontandcolor" type="text" name="end" placeholder="<?php esc_html_e('Destination', 'stec'); ?>" />
                        <div class="stec-style-flexbox-no-media stec-layout-single-location-directions-buttons">
                            <a href="javascript:void(0);" class="stec-layout-single-button-style stec-layout-single-location-get-direction-btn stec-layout-event-btn-fontandcolor"><?php esc_html_e('Get Directions', 'stec'); ?></a>
                            <a target="_blank" href="javascript:void(0);" class="stec-layout-single-button-style stec-layout-single-location-open-gmap stec-layout-event-btn-fontandcolor"><?php esc_html_e('Open Map', 'stec'); ?></a>
                        </div>
                        <p class="stec-layout-single-location-direction-error stec-layout-event-text-fontandcolor"><?php esc_html_e('Could not find route!', 'stec'); ?></p>
                    </div>

                </div>

                <?php if ($location->details !== '') : ?>

                    <p class="stec-layout-single-location-details-title stec-layout-event-title2-fontandcolor"><?php esc_html_e('Location Details', 'stec'); ?></p>

                    <p class='stec-layout-single-location-details stec-layout-event-text-fontandcolor' itemprop="address">
                        <?php esc_html_e($location->details); ?>
                    </p>

                <?php endif; ?>

            </div>

        <?php endif; ?>

        <?php if (Admin_Helper::event_has_tabs($event)) : ?>

            <div class="stec-layout-single-tabs">

                <ul class="stec-layout-single-tabs-list">

                    <?php do_action('stec_add_event_single_tab'); ?>

                </ul>

                <div class="stec-layout-event-single-tabs-content">

                    <?php do_action('stec_add_event_single_tab_content'); ?>

                </div>

            </div>

        <?php endif ?>

    </div>

    <?php require($stachethemes_ec->get_path('VIEW') . "/front/popup/share.php"); ?>

</div>
<?php
do_action('stec_single_after_content');
